﻿-- Disclaimer
-- Dieser Quellcode ist als Vorlage oder als Ideengeber gedacht. Er kann frei und ohne 
-- Auflagen oder Einschränkungen verwendet oder verändert werden.
-- Jedoch wird keine Garantie übernommen, dass eine Funktionsfähigkeit mit aktuellen und 
-- zukünftigen API-Versionen besteht. Der Autor übernimmt daher keine direkte oder indirekte 
-- Verantwortung, wenn dieser Code gar nicht oder nur fehlerhaft ausgeführt wird.
-- Für Anregungen und Fragen stehe ich jedoch gerne zur Verfügung.
-- Thorsten Kansy, www.dotnetconsulting.eu

-- Datenbank (erneut) anlegen und wechseln
USE [master];
IF EXISTS (SELECT * FROM [sys].[databases] WHERE [name] = 'dotnetconsulting_Videogames')
BEGIN
	ALTER DATABASE [dotnetconsulting_Videogames] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [dotnetconsulting_Videogames];
	PRINT '''dotnetconsulting_Videogames''-Datenbank gelöscht';
END
GO
CREATE DATABASE [dotnetconsulting_Videogames];
GO
USE [dotnetconsulting_Videogames];
PRINT '''dotnetconsulting_Videogames''-Datenbank erstellt und gewechselt';
GO

GO
SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, CONCAT_NULL_YIELDS_NULL, QUOTED_IDENTIFIER ON;

SET NUMERIC_ROUNDABORT OFF;


GO
PRINT N'Erstelle [dbo].[Highscores]...';


GO
CREATE TABLE [dbo].[Highscores] (
    [ID]           INT      IDENTITY (1, 1) NOT NULL,
    [SpielerID]    INT      NOT NULL,
    [VideospielID] INT      NOT NULL,
    [Punkte]       INT      NOT NULL,
    [Zeitpunkt]    DATETIME NOT NULL,
    CONSTRAINT [PK_Highscore] PRIMARY KEY CLUSTERED ([ID] ASC)
);


GO
PRINT N'Erstelle [dbo].[Laender]...';


GO
CREATE TABLE [dbo].[Laender] (
    [ID]   INT          IDENTITY (1, 1) NOT NULL,
    [Land] VARCHAR (50) NULL,
    CONSTRAINT [PK_Laender] PRIMARY KEY CLUSTERED ([ID] ASC)
);


GO
PRINT N'Erstelle [dbo].[Spieler]...';


GO
CREATE TABLE [dbo].[Spieler] (
    [ID]              INT          IDENTITY (1, 1) NOT NULL,
    [HerkunftslandID] INT          NULL,
    [Name]            VARCHAR (50) NOT NULL,
    [Nickname]        VARCHAR (50) NOT NULL,
    [Aktiv]           BIT          NOT NULL,
	[MitgliedSeit]	  DATE         NULL
    CONSTRAINT [PK_Spieler] PRIMARY KEY CLUSTERED ([ID] ASC)
);


GO
PRINT N'Erstelle [dbo].[Videospiele]...';


GO
CREATE TABLE [dbo].[Videospiele] (
    [ID]   INT          IDENTITY (1, 1) NOT NULL,
    [Name] VARCHAR (50) NULL,
    CONSTRAINT [PK_Games] PRIMARY KEY CLUSTERED ([ID] ASC)
);


GO
PRINT N'Erstelle [dbo].[DF_Spieler_Aktiv]...';


GO
ALTER TABLE [dbo].[Spieler]
    ADD CONSTRAINT [DF_Spieler_Aktiv] DEFAULT ((1)) FOR [Aktiv];


GO
PRINT N'Erstelle [dbo].[FK_Highscores_Spieler]...';


GO
ALTER TABLE [dbo].[Highscores] WITH NOCHECK
    ADD CONSTRAINT [FK_Highscores_Spieler] FOREIGN KEY ([SpielerID]) REFERENCES [dbo].[Spieler] ([ID]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Erstelle [dbo].[FK_Highscores_Games]...';


GO
ALTER TABLE [dbo].[Highscores] WITH NOCHECK
    ADD CONSTRAINT [FK_Highscores_Games] FOREIGN KEY ([VideospielID]) REFERENCES [dbo].[Videospiele] ([ID]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Erstelle [dbo].[FK_Spieler_Laender]...';


GO
ALTER TABLE [dbo].[Spieler] WITH NOCHECK
    ADD CONSTRAINT [FK_Spieler_Laender] FOREIGN KEY ([HerkunftslandID]) REFERENCES [dbo].[Laender] ([ID]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Tabellen angelegt.';

PRINT N'Daten einfügen'
BEGIN TRANSACTION
ALTER TABLE [dbo].[Highscores] DROP CONSTRAINT [FK_Highscores_Spieler]
ALTER TABLE [dbo].[Highscores] DROP CONSTRAINT [FK_Highscores_Games]
ALTER TABLE [dbo].[Spieler] DROP CONSTRAINT [FK_Spieler_Laender]
SET IDENTITY_INSERT [dbo].[Laender] ON
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (1, 'Greenland')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (2, 'Tuvalu')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (3, 'Netherlands')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (4, 'Slovenia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (5, 'Macedonia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (6, 'Sierra Leone')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (7, 'Costa Rica')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (8, 'North Korea')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (9, 'Pitcairn')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (10, 'South Africa')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (11, 'Iraq')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (12, 'Croatia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (13, 'Belgium')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (14, 'Jordan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (15, 'Vatican City')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (16, 'New Caledonia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (17, 'Colombia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (18, 'Uzbekistan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (19, 'Comoros')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (20, 'Tajikistan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (21, 'Estonia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (22, 'Congo')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (23, 'Saint Lucia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (24, 'Eire')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (25, 'Kuwait')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (26, 'Afghanistan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (27, 'New Zealand')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (28, 'Dominica')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (29, 'Norway')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (30, 'Somalia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (31, 'Armenia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (32, 'Chile')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (33, 'Belarus')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (34, 'Bangladesh')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (35, 'Niger')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (36, 'Bhutan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (37, 'French Guiana')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (38, 'Burkina Faso')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (39, 'Philippines')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (40, 'Tunisia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (41, 'Virgin Islands')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (42, 'Jamaica')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (43, 'Guinea-Bissau')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (44, 'Saudi Arabia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (45, 'Argentina')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (46, 'Nicaragua')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (47, 'Montserrat')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (48, 'San Marino')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (49, 'Cameroon')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (50, 'Gabon')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (51, 'Timor-Leste')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (52, 'Australia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (53, 'Jersey')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (54, 'Serbia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (55, 'Grenada')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (56, 'Togo')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (57, 'Israel')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (58, 'Romania')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (59, 'Indonesia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (60, 'Senegal')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (61, 'Iran')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (62, 'Burundi')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (63, 'Taiwan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (64, 'Saint Helena')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (65, 'Venezuela')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (66, 'Lebanon')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (67, 'Seychelles')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (68, 'South Korea')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (69, 'Spain')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (70, 'Panama')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (71, 'Puerto Rico')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (72, 'Mozambique')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (73, 'Montenegro')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (74, 'Slovakia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (75, 'Namibia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (76, 'Andorra')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (77, 'Turkey')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (78, 'Ethiopia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (79, 'Germany')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (80, 'Pakistan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (81, 'Ecuador')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (82, 'Yemen')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (83, 'Swaziland')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (84, 'Aruba')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (85, 'Bahrain')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (86, 'France')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (87, 'Kyrgyzstan')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (88, 'Guyana')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (89, 'Mali')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (90, 'Uganda')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (91, 'Western Sahara')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (92, 'Lesotho')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (93, 'China')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (94, 'Kenya')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (95, 'Micronesia')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (96, 'United Kingdom')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (97, 'Italy')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (98, 'Zimbabwe')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (99, 'Fiji')
INSERT INTO [dbo].[Laender] ([ID], [Land]) VALUES (100, 'Vanuatu')
SET IDENTITY_INSERT [dbo].[Laender] OFF
SET IDENTITY_INSERT [dbo].[Spieler] ON
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (1, 82, 'Glenda', 'Glenda034', 0, '20140617')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (2, 56, 'Garry', 'Garry630', 0, '20140622')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (3, 56, 'Josh', 'Josh169', 1, '20160104')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (4, 45, 'Janice', 'Janice663', 0, '20151007')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (5, 28, 'Aisha', 'Aisha698', 0, '20150131')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (6, 47, 'Rick', 'Rick392', 0, '20140516')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (7, 47, 'Levi', 'Levi612', 1, '20160331')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (8, 4, 'Lakesha', 'Lakesha402', 0, '20140308')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (9, 100, 'Bryan', 'Bryan55', 0, '20140219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (10, 32, 'Anthony', 'Anthony004', 0, '20141003')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (11, 85, 'Lorie', 'Lorie227', 1, '20150709')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (12, 4, 'Janette', 'Janette176', 1, '20150107')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (13, 53, 'Roger', 'Roger958', 1, '20150907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (14, 69, 'Maggie', 'Maggie215', 1, '20160116')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (15, 9, 'Janelle', 'Janelle562', 1, '20160514')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (16, 46, 'Kristine', 'Kristine126', 0, '20140228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (17, 99, 'Robin', 'Robin682', 1, '20160512')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (18, 77, 'Erika', 'Erika942', 1, '20151122')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (19, 39, 'Timothy', 'Timothy554', 1, '20160309')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (20, 96, 'Cody', 'Cody813', 0, '20151121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (21, 72, 'Shirley', 'Shirley763', 1, '20140524')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (22, 28, 'Nikki', 'Nikki563', 1, '20140106')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (23, 80, 'Cara', 'Cara179', 0, '20150416')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (24, 46, 'Larry', 'Larry006', 0, '20141228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (25, 23, 'Ronnie', 'Ronnie333', 0, '20140219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (26, 72, 'Jesse', 'Jesse970', 1, '20140822')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (27, 49, 'Cara', 'Cara775', 1, '20140107')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (28, 88, 'Laurie', 'Laurie012', 1, '20150829')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (29, 74, 'Austin', 'Austin651', 1, '20140812')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (30, 68, 'Marisa', 'Marisa095', 0, '20140704')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (31, 22, 'Shawn', 'Shawn302', 0, '20141013')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (32, 90, 'Ricky', 'Ricky590', 0, '20140819')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (33, 84, 'Bethany', 'Bethany423', 1, '20140216')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (34, 65, 'Ivan', 'Ivan992', 1, '20140707')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (35, 22, 'Perry', 'Perry754', 1, '20140510')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (36, 49, 'Nicholas', 'Nicholas086', 0, '20150414')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (37, 72, 'Cari', 'Cari936', 0, '20140514')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (38, 11, 'Kari', 'Kari236', 0, '20150223')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (39, 72, 'Alana', 'Alana653', 0, '20131116')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (40, 91, 'Cindy', 'Cindy201', 1, '20140524')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (41, 48, 'Bobbie', 'Bobbie85', 0, '20160515')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (42, 33, 'Tammy', 'Tammy407', 0, '20140717')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (43, 47, 'Cassie', 'Cassie220', 1, '20140301')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (44, 81, 'Scotty', 'Scotty916', 1, '20150414')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (45, 77, 'Lamar', 'Lamar363', 0, '20140802')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (46, 30, 'Jennie', 'Jennie262', 0, '20150927')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (47, 67, 'Kerri', 'Kerri768', 1, '20140410')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (48, 29, 'Tracey', 'Tracey752', 0, '20160103')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (49, 23, 'Gustavo', 'Gustavo80', 0, '20160130')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (50, 7, 'Jimmie', 'Jimmie142', 1, '20160224')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (51, 70, 'Kent', 'Kent419', 0, '20151115')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (52, 93, 'Rickey', 'Rickey780', 0, '20160126')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (53, 93, 'Norma', 'Norma277', 0, '20140930')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (54, 91, 'Betsy', 'Betsy452', 0, '20150313')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (55, 10, 'Chrystal', 'Chrystal231', 1, '20150831')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (56, 11, 'Loretta', 'Loretta388', 1, '20150113')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (57, 96, 'Dion', 'Dion400', 1, '20130910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (58, 15, 'Veronica', 'Veronica960', 1, '20140924')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (59, 57, 'Rudy', 'Rudy321', 0, '20160206')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (60, 62, 'Felicia', 'Felicia410', 1, '20151201')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (61, NULL, 'Bryan', 'Bryan379', 0, '20150507')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (62, 86, 'Roberta', 'Roberta23', 1, '20160318')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (63, 29, 'Christian', 'Christian551', 0, '20140123')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (64, 96, 'Tanya', 'Tanya244', 1, '20160503')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (65, 39, 'Nicole', 'Nicole0', 1, '20150129')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (66, 29, 'Colby', 'Colby120', 1, '20140308')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (67, 54, 'Lawrence', 'Lawrence440', 1, '20160102')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (68, 29, 'Shad', 'Shad706', 0, '20160418')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (69, 85, 'Elisabeth', 'Elisabeth16', 0, '20160507')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (70, 11, 'Tonia', 'Tonia311', 0, '20131129')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (71, 41, 'Colby', 'Colby991', 0, '20140731')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (72, 14, 'Tommy', 'Tommy701', 0, '20130913')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (73, 62, 'Tyrone', 'Tyrone762', 1, '20160123')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (74, 69, 'Adrienne', 'Adrienne510', 1, '20140329')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (75, 18, 'Tricia', 'Tricia855', 1, '20150510')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (76, 61, 'Lillian', 'Lillian952', 0, '20141007')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (77, 100, 'Raul', 'Raul7', 1, '20131001')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (78, 7, 'Victoria', 'Victoria491', 1, '20131026')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (79, 85, 'Jessica', 'Jessica24', 1, '20151207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (80, 65, 'Jeff', 'Jeff910', 0, '20140114')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (81, 98, 'Gabrielle', 'Gabrielle045', 0, '20141007')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (82, 52, 'Allan', 'Allan026', 0, '20140907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (83, 82, 'Andrew', 'Andrew206', 0, '20150829')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (84, 58, 'Roland', 'Roland250', 0, '20150204')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (85, 68, 'Deborah', 'Deborah208', 0, '20150724')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (86, 31, 'Ethan', 'Ethan356', 1, '20140409')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (87, 47, 'Oliver', 'Oliver1', 1, '20140815')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (88, 22, 'Patrick', 'Patrick173', 0, '20140812')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (89, 24, 'Wendell', 'Wendell799', 1, '20140815')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (90, 45, 'Shauna', 'Shauna097', 0, '20141026')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (91, 2, 'Alberto', 'Alberto408', 0, '20160523')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (92, 82, 'Leon', 'Leon273', 0, '20140919')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (93, 92, 'Spencer', 'Spencer240', 1, '20150807')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (94, 51, 'Anita', 'Anita789', 1, '20150521')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (95, 100, 'Preston', 'Preston678', 1, '20150223')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (96, 66, 'Loren', 'Loren730', 0, '20160412')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (97, 97, 'Bradley', 'Bradley001', 1, '20150805')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (98, 78, 'Deborah', 'Deborah068', 0, '20140128')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (99, 43, 'Clarence', 'Clarence058', 1, '20131008')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (100, 89, 'Marcus', 'Marcus26', 0, '20141226')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (101, 26, 'Susan', 'Susan765', 0, '20150706')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (102, 90, 'Traci', 'Traci669', 0, '20150531')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (103, 89, 'Samantha', 'Samantha973', 0, '20140914')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (104, 79, 'Johnathan', 'Johnathan934', 0, '20150131')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (105, 36, 'Blanca', 'Blanca130', 0, '20150519')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (106, 4, 'Dennis', 'Dennis894', 1, '20150214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (107, 13, 'Yvette', 'Yvette384', 1, '20140907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (108, 50, 'Gina', 'Gina188', 0, '20150222')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (109, 27, 'Garry', 'Garry483', 1, '20140602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (110, 25, 'Aimee', 'Aimee740', 1, '20141024')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (111, 14, 'Anitra', 'Anitra531', 1, '20131220')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (112, 56, 'Melanie', 'Melanie781', 0, '20150817')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (113, 35, 'Jesse', 'Jesse330', 1, '20141017')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (114, 10, 'Marisol', 'Marisol811', 0, '20150128')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (115, 72, 'Derek', 'Derek804', 1, '20140321')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (116, 66, 'Wendell', 'Wendell362', 1, '20160114')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (117, 27, 'Courtney', 'Courtney209', 0, '20160302')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (118, 20, 'Debra', 'Debra409', 0, '20160504')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (119, 50, 'Vincent', 'Vincent326', 1, '20130927')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (120, 32, 'Keisha', 'Keisha675', 1, '20131022')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (121, 25, 'William', 'William180', 1, '20150527')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (122, 62, 'Lea', 'Lea059', 1, '20150104')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (123, NULL, 'Kareem', 'Kareem692', 1, '20151219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (124, 97, 'Beth', 'Beth361', 1, '20140615')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (125, 78, 'Joan', 'Joan156', 1, '20150319')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (126, 20, 'Connie', 'Connie74', 1, '20150428')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (127, 12, 'Michele', 'Michele935', 0, '20150625')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (128, 11, 'Lewis', 'Lewis183', 1, '20130928')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (129, 65, 'Ralph', 'Ralph218', 0, '20140128')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (130, 25, 'Darius', 'Darius810', 1, '20150621')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (131, 86, 'Brandie', 'Brandie445', 1, '20151016')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (132, 91, 'Ian', 'Ian971', 1, '20140504')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (133, 47, 'Tommy', 'Tommy828', 1, '20140722')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (134, 78, 'Dan', 'Dan447', 1, '20150813')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (135, 65, 'Marcella', 'Marcella66', 1, '20140212')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (136, 61, 'Maggie', 'Maggie587', 1, '20140327')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (137, 63, 'Lorraine', 'Lorraine264', 0, '20131225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (138, 88, 'Kate', 'Kate874', 0, '20160529')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (139, 14, 'Robin', 'Robin511', 1, '20140123')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (140, NULL, 'Jeremiah', 'Jeremiah908', 0, '20140804')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (141, 68, 'Ian', 'Ian39', 1, '20150209')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (142, 87, 'Anita', 'Anita343', 1, '20160506')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (143, 79, 'Dean', 'Dean389', 1, '20131128')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (144, 83, 'Tia', 'Tia453', 0, '20140101')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (145, 53, 'Kyle', 'Kyle284', 1, '20140922')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (146, 64, 'Lori', 'Lori747', 0, '20140915')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (147, 32, 'Marlon', 'Marlon826', 1, '20131112')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (148, 81, 'Tasha', 'Tasha341', 1, '20160422')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (149, 93, 'Katina', 'Katina283', 0, '20140917')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (150, 20, 'Theodore', 'Theodore694', 1, '20150412')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (151, 67, 'Earl', 'Earl467', 0, '20160402')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (152, 54, 'Raquel', 'Raquel950', 0, '20140707')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (153, 78, 'Kari', 'Kari902', 0, '20140612')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (154, 11, 'Kelli', 'Kelli732', 1, '20151030')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (155, 12, 'Lashonda', 'Lashonda107', 0, '20140606')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (156, 57, 'Marisa', 'Marisa862', 0, '20150110')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (157, 65, 'Esmeralda', 'Esmeralda986', 1, '20140531')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (158, 57, 'Quentin', 'Quentin512', 0, '20150622')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (159, 100, 'Sonja', 'Sonja961', 0, '20141228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (160, 13, 'Damon', 'Damon374', 0, '20160430')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (161, 64, 'Marcie', 'Marcie769', 0, '20140907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (162, 26, 'Alma', 'Alma805', 1, '20140621')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (163, 2, 'Alan', 'Alan415', 1, '20140425')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (164, 30, 'Kristian', 'Kristian644', 1, '20141028')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (165, 83, 'Latisha', 'Latisha835', 1, '20140907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (166, 96, 'Allison', 'Allison742', 1, '20150302')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (167, 95, 'Natasha', 'Natasha637', 0, '20150729')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (168, 90, 'Jess', 'Jess540', 0, '20150219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (169, 85, 'Chastity', 'Chastity147', 0, '20141101')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (170, 56, 'Gavin', 'Gavin657', 1, '20151101')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (171, 42, 'Lewis', 'Lewis413', 1, '20140214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (172, 43, 'Ana', 'Ana538', 1, '20160502')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (173, 61, 'Bridgett', 'Bridgett041', 1, '20131201')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (174, 92, 'Harry', 'Harry925', 0, '20160319')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (175, 69, 'Marcus', 'Marcus764', 0, '20150315')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (176, 47, 'Johnny', 'Johnny616', 0, '20141203')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (177, 22, 'Sammy', 'Sammy614', 0, '20150906')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (178, 52, 'Ernesto', 'Ernesto508', 0, '20150908')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (179, 95, 'Kent', 'Kent749', 0, '20150524')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (180, 50, 'Elisabeth', 'Elisabeth932', 1, '20150829')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (181, 19, 'Donnie', 'Donnie024', 0, '20150131')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (182, 62, 'Rachel', 'Rachel100', 1, '20141016')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (183, 91, 'Alexandra', 'Alexandra567', 1, '20160410')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (184, 76, 'Miguel', 'Miguel391', 1, '20150529')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (185, 12, 'Dora', 'Dora559', 1, '20160103')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (186, 62, 'Jo', 'Jo67', 0, '20160225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (187, 58, 'Bobbi', 'Bobbi640', 1, '20160101')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (188, 87, 'Xavier', 'Xavier265', 0, '20140505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (189, 38, 'Esmeralda', 'Esmeralda927', 0, '20140418')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (190, 67, 'Garrett', 'Garrett870', 1, '20141125')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (191, 54, 'Ramon', 'Ramon299', 1, '20160211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (192, 16, 'Nikki', 'Nikki431', 1, '20141203')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (193, 7, 'Donald', 'Donald305', 0, '20140617')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (194, 99, 'Gabriel', 'Gabriel541', 0, '20140108')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (195, 34, 'Micah', 'Micah200', 1, '20150120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (196, 52, 'Dwayne', 'Dwayne32', 1, '20140623')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (197, 86, 'Judith', 'Judith72', 1, '20150906')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (198, 12, 'Clinton', 'Clinton829', 1, '20150907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (199, 78, 'Sheryl', 'Sheryl500', 0, '20140925')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (200, 76, 'Whitney', 'Whitney401', 1, '20160210')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (201, 18, 'Irene', 'Irene02', 0, '20140813')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (202, 18, 'Timmy', 'Timmy680', 0, '20160220')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (203, 73, 'Malcolm', 'Malcolm836', 0, '20160521')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (204, 23, 'Miranda', 'Miranda795', 0, '20150520')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (205, 5, 'Randolph', 'Randolph996', 1, '20140330')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (206, 49, 'Helen', 'Helen966', 0, '20140213')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (207, 61, 'Arthur', 'Arthur639', 1, '20160519')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (208, 80, 'Norma', 'Norma646', 1, '20150724')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (209, 11, 'Marlon', 'Marlon941', 1, '20150623')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (210, 53, 'Tameka', 'Tameka501', 0, '20160211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (211, 38, 'Vivian', 'Vivian454', 1, '20150620')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (212, 46, 'Stephen', 'Stephen337', 0, '20151020')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (213, 59, 'Brendan', 'Brendan071', 0, '20140717')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (214, 78, 'Taryn', 'Taryn217', 0, '20141108')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (215, 70, 'Bryan', 'Bryan757', 0, '20131030')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (216, 47, 'Sarah', 'Sarah245', 1, '20150619')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (217, 84, 'Lorie', 'Lorie43', 0, '20140902')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (218, 91, 'Timothy', 'Timothy555', 0, '20141213')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (219, 82, 'Judy', 'Judy866', 0, '20140521')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (220, 76, 'Chester', 'Chester087', 1, '20151110')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (221, 93, 'Dina', 'Dina989', 0, '20140912')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (222, 100, 'Robyn', 'Robyn671', 1, '20150624')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (223, 55, 'Nancy', 'Nancy68', 0, '20140816')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (224, 97, 'Dana', 'Dana47', 1, '20140207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (225, 44, 'Donnie', 'Donnie691', 1, '20150523')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (226, 6, 'April', 'April308', 0, '20140120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (227, 31, 'Hannah', 'Hannah832', 0, '20140613')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (228, 87, 'Brett', 'Brett659', 0, '20141010')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (229, 67, 'Kendall', 'Kendall519', 0, '20131127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (230, 100, 'Connie', 'Connie221', 0, '20140308')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (231, 6, 'Charlie', 'Charlie969', 0, '20140814')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (232, 71, 'Lawanda', 'Lawanda722', 1, '20131202')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (233, 82, 'Brian', 'Brian688', 0, '20160402')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (234, 52, 'Donna', 'Donna149', 0, '20140511')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (235, 52, 'Rochelle', 'Rochelle714', 1, '20140914')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (236, 38, 'Roxanne', 'Roxanne340', 0, '20140309')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (237, 39, 'Cedric', 'Cedric230', 0, '20140427')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (238, 84, 'Cynthia', 'Cynthia335', 1, '20140218')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (239, 73, 'Perry', 'Perry65', 1, '20150411')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (240, 54, 'Brock', 'Brock534', 0, '20150910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (241, 8, 'Alana', 'Alana940', 0, '20140705')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (242, 92, 'Latonya', 'Latonya448', 1, '20151216')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (243, 66, 'Wendy', 'Wendy019', 1, '20150105')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (244, 24, 'Roberta', 'Roberta946', 0, '20140626')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (245, 56, 'Annie', 'Annie690', 1, '20140809')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (246, 75, 'Reginald', 'Reginald255', 1, '20140716')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (247, 59, 'Kristina', 'Kristina473', 0, '20130909')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (248, 19, 'Marsha', 'Marsha157', 0, '20160527')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (249, 89, 'Becky', 'Becky027', 0, '20140610')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (250, 24, 'Norman', 'Norman259', 0, '20160426')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (251, 65, 'Julia', 'Julia772', 1, '20130925')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (252, 94, 'Frankie', 'Frankie964', 1, '20150211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (253, 55, 'Wayne', 'Wayne533', 0, '20140509')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (254, 26, 'Andrea', 'Andrea887', 0, '20140513')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (255, 17, 'Rudy', 'Rudy013', 1, '20150824')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (256, 73, 'Rebekah', 'Rebekah530', 0, '20140715')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (257, 56, 'Carla', 'Carla728', 0, '20140914')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (258, 62, 'Lindsey', 'Lindsey5', 1, '20140504')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (259, 97, 'Deanna', 'Deanna385', 0, '20150417')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (260, 69, 'Rodolfo', 'Rodolfo241', 1, '20140320')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (261, 71, 'Eugene', 'Eugene842', 1, '20141019')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (262, 63, 'Randy', 'Randy002', 1, '20150515')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (263, 65, 'Demetrius', 'Demetrius121', 0, '20141218')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (264, 94, 'April', 'April493', 1, '20140423')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (265, 11, 'Steven', 'Steven030', 1, '20150103')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (266, 91, 'Reginald', 'Reginald418', 0, '20140110')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (267, 17, 'Jerry', 'Jerry727', 1, '20140301')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (268, 79, 'Yesenia', 'Yesenia953', 0, '20150722')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (269, 41, 'Helen', 'Helen50', 1, '20150309')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (270, 24, 'Gabriel', 'Gabriel850', 1, '20160208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (271, 57, 'Paula', 'Paula426', 0, '20140801')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (272, 68, 'Kareem', 'Kareem040', 0, '20160411')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (273, 73, 'Jayson', 'Jayson872', 0, '20131026')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (274, 17, 'Benny', 'Benny405', 0, '20131208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (275, 45, 'Jordan', 'Jordan114', 0, '20140111')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (276, 8, 'Mitchell', 'Mitchell122', 1, '20141017')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (277, 97, 'Bridget', 'Bridget546', 0, '20160415')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (278, 91, 'Dustin', 'Dustin465', 1, '20141218')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (279, 16, 'Arlene', 'Arlene844', 0, '20141024')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (280, 41, 'Jami', 'Jami108', 0, '20150211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (281, 61, 'Jessie', 'Jessie915', 1, '20150917')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (282, 91, 'Hope', 'Hope310', 0, '20141001')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (283, 17, 'Tonia', 'Tonia938', 1, '20141208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (284, 10, 'Jody', 'Jody342', 0, '20160411')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (285, 79, 'Joel', 'Joel336', 0, '20160514')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (286, 32, 'Everett', 'Everett018', 1, '20150513')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (287, 70, 'Robyn', 'Robyn893', 0, '20150120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (288, 43, 'Saul', 'Saul331', 0, '20140503')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (289, 1, 'Tanya', 'Tanya864', 1, '20140701')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (290, 34, 'Jonathan', 'Jonathan81', 1, '20141026')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (291, 50, 'Kristen', 'Kristen195', 1, '20151214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (292, 88, 'Glenda', 'Glenda778', 1, '20160429')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (293, 72, 'Stanley', 'Stanley171', 0, '20151213')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (294, 11, 'Cynthia', 'Cynthia693', 0, '20160112')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (295, 3, 'Andrea', 'Andrea414', 1, '20141001')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (296, 14, 'Marla', 'Marla591', 1, '20150320')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (297, 47, 'Samantha', 'Samantha476', 0, '20151207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (298, 52, 'Antoine', 'Antoine316', 0, '20160131')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (299, 32, 'Cornelius', 'Cornelius145', 0, '20150726')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (300, 32, 'Wallace', 'Wallace578', 0, '20150804')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (301, 39, 'Darcy', 'Darcy898', 1, '20160509')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (302, 24, 'Rita', 'Rita596', 0, '20160317')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (303, 16, 'Travis', 'Travis117', 0, '20150421')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (304, 27, 'Byron', 'Byron281', 0, '20160321')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (305, 82, 'Ruben', 'Ruben664', 0, '20150306')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (306, 96, 'Evan', 'Evan708', 1, '20160503')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (307, 54, 'Nina', 'Nina022', 0, '20130927')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (308, 40, 'Alexandra', 'Alexandra94', 0, '20140328')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (309, 7, 'Dwight', 'Dwight808', 1, '20140725')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (310, 66, 'Jennifer', 'Jennifer574', 0, '20141231')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (311, 55, 'Frances', 'Frances686', 1, '20140415')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (312, 62, 'Sherri', 'Sherri043', 0, '20150114')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (313, 19, 'Ross', 'Ross20', 1, '20131221')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (314, 63, 'George', 'George523', 1, '20140616')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (315, 6, 'Kristy', 'Kristy433', 0, '20141001')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (316, 18, 'Ruth', 'Ruth411', 0, '20160421')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (317, 83, 'Miriam', 'Miriam432', 1, '20150701')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (318, 78, 'Frederick', 'Frederick358', 0, '20160425')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (319, 68, 'Maureen', 'Maureen97', 0, '20150514')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (320, 50, 'Clyde', 'Clyde967', 0, '20140729')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (321, 69, 'Brandi', 'Brandi4', 1, '20150322')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (322, 11, 'Liza', 'Liza332', 1, '20160203')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (323, 87, 'Richard', 'Richard917', 0, '20140901')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (324, 99, 'Nathaniel', 'Nathaniel416', 0, '20141016')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (325, 95, 'Ann', 'Ann159', 1, '20150915')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (326, 57, 'Rodolfo', 'Rodolfo133', 1, '20160120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (327, 64, 'Joanna', 'Joanna42', 1, '20140214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (328, 53, 'Clinton', 'Clinton502', 0, '20150418')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (329, 92, 'Krystal', 'Krystal507', 1, '20160211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (330, 95, 'Lewis', 'Lewis696', 1, '20150723')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (331, 36, 'Roberto', 'Roberto573', 0, '20141025')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (332, 36, 'Janelle', 'Janelle719', 0, '20141007')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (333, 26, 'Angelo', 'Angelo605', 1, '20150904')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (334, 19, 'Tami', 'Tami031', 1, '20140628')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (335, 47, 'Franklin', 'Franklin471', 1, '20131105')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (336, 2, 'Moses', 'Moses051', 0, '20140122')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (337, 78, 'Franklin', 'Franklin344', 0, '20131123')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (338, 54, 'Latisha', 'Latisha442', 1, '20150225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (339, 7, 'Armando', 'Armando812', 1, '20141113')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (340, 10, 'Jody', 'Jody710', 1, '20140926')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (341, 80, 'Lorie', 'Lorie128', 0, '20131106')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (342, 41, 'Shelley', 'Shelley788', 0, '20151119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (343, 23, 'Clarissa', 'Clarissa371', 0, '20150329')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (344, 8, 'Angelica', 'Angelica687', 0, '20150313')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (345, 74, 'Bart', 'Bart096', 0, '20140108')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (346, 5, 'Karen', 'Karen355', 1, '20150216')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (347, 81, 'Tami', 'Tami949', 0, '20150917')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (348, 39, 'Erika', 'Erika849', 0, '20140627')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (349, 36, 'Jenna', 'Jenna373', 0, '20151211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (350, 35, 'Melinda', 'Melinda716', 0, '20140904')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (351, 95, 'Latanya', 'Latanya446', 0, '20131218')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (352, 76, 'Elaine', 'Elaine13', 1, '20131102')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (353, 81, 'Melinda', 'Melinda152', 1, '20150910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (354, 27, 'Alyssa', 'Alyssa873', 1, '20131122')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (355, 56, 'Donnie', 'Donnie458', 0, '20150827')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (356, 16, 'Bernard', 'Bernard549', 1, '20141115')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (357, 19, 'Ramona', 'Ramona624', 1, '20151018')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (358, 57, 'Lonnie', 'Lonnie962', 0, '20160121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (359, 51, 'Tommy', 'Tommy427', 0, '20150201')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (360, 80, 'Daryl', 'Daryl093', 0, '20141225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (361, 17, 'Micheal', 'Micheal196', 1, '20140522')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (362, 55, 'Monique', 'Monique831', 0, '20151003')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (363, 39, 'Hector', 'Hector606', 1, '20160121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (364, 52, 'Johnathan', 'Johnathan084', 1, '20140325')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (365, 54, 'Kelvin', 'Kelvin830', 1, '20150910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (366, 49, 'Jamison', 'Jamison553', 0, '20141016')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (367, 67, 'Tracie', 'Tracie760', 0, '20160407')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (368, 12, 'Tameka', 'Tameka404', 1, '20150927')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (369, 65, 'Peggy', 'Peggy061', 0, '20151022')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (370, 42, 'Maggie', 'Maggie798', 1, '20151101')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (371, 87, 'Christa', 'Christa044', 0, '20150609')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (372, 19, 'Felix', 'Felix73', 1, '20150522')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (373, 40, 'Jolene', 'Jolene882', 0, '20140909')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (374, 22, 'Loren', 'Loren859', 1, '20151112')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (375, 57, 'Colby', 'Colby827', 0, '20160221')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (376, 27, 'Bart', 'Bart394', 1, '20150104')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (377, 30, 'Maribel', 'Maribel085', 0, '20150427')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (378, 47, 'Shirley', 'Shirley883', 0, '20140621')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (379, 17, 'Tabatha', 'Tabatha912', 0, '20130915')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (380, 13, 'Latisha', 'Latisha151', 0, '20151120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (381, 74, 'Eduardo', 'Eduardo820', 1, '20141024')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (382, 18, 'Gregg', 'Gregg023', 0, '20131001')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (383, 20, 'Rick', 'Rick047', 1, '20160227')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (384, 24, 'Latisha', 'Latisha132', 0, '20140716')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (385, 77, 'Pete', 'Pete', 1, '20140110')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (386, 31, 'Marlon', 'Marlon067', 1, '20140304')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (387, 71, 'Noah', 'Noah005', 1, '20131014')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (388, 26, 'Caleb', 'Caleb280', 1, '20140904')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (389, 27, 'Jana', 'Jana611', 0, '20160103')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (390, 52, 'Rickey', 'Rickey349', 1, '20140105')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (391, 41, 'Edward', 'Edward846', 0, '20140618')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (392, 9, 'Elaine', 'Elaine214', 0, '20150124')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (393, 51, 'Jared', 'Jared674', 0, '20141004')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (394, 69, 'Karla', 'Karla127', 0, '20140718')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (395, 1, 'Danielle', 'Danielle735', 0, '20141229')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (396, 93, 'Gloria', 'Gloria266', 1, '20140606')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (397, 78, 'Sherri', 'Sherri980', 1, '20160505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (398, 25, 'Mario', 'Mario194', 1, '20160205')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (399, 19, 'Tabatha', 'Tabatha725', 0, '20140620')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (400, 86, 'Liza', 'Liza847', 1, '20150609')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (401, 91, 'Kristian', 'Kristian267', 0, '20160602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (402, 16, 'Andrew', 'Andrew22', 0, '20151012')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (403, 14, 'Michele', 'Michele629', 0, '20131202')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (404, 53, 'Tera', 'Tera116', 0, '20140528')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (405, 97, 'Tamika', 'Tamika228', 1, '20150211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (406, 23, 'Alfred', 'Alfred229', 0, '20140421')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (407, 33, 'Trent', 'Trent282', 0, '20160226')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (408, 80, 'Donovan', 'Donovan032', 0, '20151215')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (409, 54, 'Nathaniel', 'Nathaniel524', 1, '20140629')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (410, 29, 'Tracey', 'Tracey486', 1, '20150826')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (411, 51, 'Chastity', 'Chastity770', 1, '20150523')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (412, 72, 'Nakia', 'Nakia858', 1, '20160505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (413, 55, 'Tiffany', 'Tiffany774', 0, '20151224')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (414, 83, 'Matthew', 'Matthew59', 0, '20131017')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (415, 58, 'Julia', 'Julia216', 1, '20150809')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (416, 62, 'Christie', 'Christie704', 0, '20141214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (417, 65, 'Benny', 'Benny921', 1, '20131208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (418, 34, 'Terrell', 'Terrell628', 0, '20131008')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (419, 43, 'Gene', 'Gene063', 0, '20150602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (420, NULL, 'Robyn', 'Robyn904', 0, '20160329')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (421, 84, 'Misty', 'Misty838', 1, '20131109')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (422, 20, 'Antonio', 'Antonio901', 0, '20150801')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (423, 35, 'Suzanne', 'Suzanne679', 0, '20160318')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (424, 26, 'Frankie', 'Frankie783', 0, '20151014')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (425, 90, 'Lynette', 'Lynette436', 0, '20160520')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (426, 9, 'Desiree', 'Desiree570', 0, '20140602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (427, 12, 'Janette', 'Janette550', 1, '20140223')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (428, 92, 'Joel', 'Joel677', 1, '20140810')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (429, 12, 'Chadwick', 'Chadwick286', 0, '20151002')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (430, 2, 'Constance', 'Constance853', 1, '20160225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (431, 64, 'Esther', 'Esther585', 1, '20160206')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (432, 4, 'Gordon', 'Gordon158', 1, '20140531')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (433, 95, 'Nichole', 'Nichole080', 1, '20151120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (434, 37, 'Rochelle', 'Rochelle000', 0, '20150820')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (435, 2, 'Roberto', 'Roberto015', 1, '20150129')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (436, 89, 'Miranda', 'Miranda734', 0, '20141217')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (437, 54, 'Mia', 'Mia213', 0, '20140131')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (438, 45, 'Kendall', 'Kendall621', 1, '20151021')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (439, 31, 'Clyde', 'Clyde803', 0, '20150429')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (440, 38, 'Angelina', 'Angelina309', 1, '20140604')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (441, 29, 'Josephine', 'Josephine885', 0, '20140807')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (442, 93, 'Sherry', 'Sherry07', 0, '20131109')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (443, 49, 'Deborah', 'Deborah945', 0, '20160514')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (444, 50, 'Jeffrey', 'Jeffrey931', 1, '20130915')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (445, 73, 'Tracy', 'Tracy71', 0, '20141228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (446, 23, 'Leanne', 'Leanne776', 1, '20160119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (447, 42, 'Sheldon', 'Sheldon065', 1, '20150611')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (448, 34, 'Mathew', 'Mathew9', 0, '20160408')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (449, NULL, 'Omar', 'Omar597', 1, '20150706')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (450, 30, 'Raul', 'Raul718', 1, '20140801')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (451, 30, 'Darryl', 'Darryl552', 0, '20150821')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (452, 24, 'Tameka', 'Tameka707', 1, '20140914')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (453, 37, 'Samuel', 'Samuel69', 0, '20150321')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (454, 37, 'Christina', 'Christina382', 0, '20151227')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (455, 2, 'Faith', 'Faith082', 0, '20150927')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (456, 72, 'Terrell', 'Terrell724', 1, '20141011')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (457, 80, 'Blanca', 'Blanca794', 1, '20140817')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (458, 27, 'Lori', 'Lori943', 0, '20150920')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (459, 63, 'Misti', 'Misti607', 1, '20151227')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (460, 99, 'Michele', 'Michele137', 1, '20151009')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (461, 65, 'Tamara', 'Tamara154', 1, '20160116')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (462, 61, 'Janette', 'Janette979', 0, '20140702')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (463, 43, 'Tania', 'Tania141', 1, '20160420')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (464, 77, 'Miranda', 'Miranda366', 0, '20140822')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (465, 7, 'Teresa', 'Teresa421', 0, '20131218')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (466, 45, 'Michael', 'Michael886', 1, '20150513')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (467, 2, 'Ebony', 'Ebony60', 0, '20150523')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (468, 9, 'Perry', 'Perry269', 1, '20131119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (469, 6, 'Terrence', 'Terrence028', 0, '20150622')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (470, 57, 'Jimmy', 'Jimmy312', 1, '20141223')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (471, 13, 'Jeffery', 'Jeffery33', 1, '20150524')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (472, 48, 'Mindy', 'Mindy025', 0, '20150105')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (473, 2, 'Miranda', 'Miranda153', 1, '20160426')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (474, 43, 'Chad', 'Chad926', 0, '20131231')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (475, 15, 'Leonard', 'Leonard817', 0, '20141015')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (476, 71, 'Bonnie', 'Bonnie975', 0, '20140703')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (477, 72, 'Sally', 'Sally29', 0, '20150808')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (478, 88, 'Whitney', 'Whitney924', 1, '20150512')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (479, 43, 'Sergio', 'Sergio711', 1, '20140429')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (480, 4, 'Lorraine', 'Lorraine181', 1, '20160427')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (481, 78, 'Stacie', 'Stacie112', 0, '20160323')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (482, 12, 'Luis', 'Luis016', 0, '20160413')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (483, 23, 'Beverly', 'Beverly787', 0, '20151029')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (484, 41, 'Jaime', 'Jaime648', 0, '20140416')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (485, NULL, 'Wade', 'Wade36', 0, '20160307')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (486, 55, 'Clay', 'Clay184', 0, '20140704')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (487, 8, 'Natasha', 'Natasha271', 0, '20150209')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (488, 48, 'Barbara', 'Barbara134', 0, '20140602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (489, 53, 'Noel', 'Noel95', 0, '20160227')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (490, 53, 'Cody', 'Cody861', 1, '20130914')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (491, 25, 'Nelson', 'Nelson933', 1, '20150216')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (492, 5, 'Josh', 'Josh077', 1, '20151219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (493, 86, 'Herbert', 'Herbert535', 1, '20140529')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (494, 36, 'Juanita', 'Juanita993', 1, '20130911')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (495, 15, 'Stephen', 'Stephen807', 0, '20130926')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (496, 35, 'Angelica', 'Angelica521', 1, '20141130')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (497, 12, 'Spencer', 'Spencer327', 0, '20140104')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (498, 53, 'Bryant', 'Bryant615', 1, '20140528')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (499, 8, 'Kelli', 'Kelli561', 1, '20140308')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (500, NULL, 'Laura', 'Laura073', 1, '20140305')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (501, 86, 'Annie', 'Annie323', 0, '20140208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (502, 78, 'Rocky', 'Rocky516', 0, '20140919')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (503, 40, 'Gena', 'Gena062', 1, '20150727')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (504, 2, 'Janette', 'Janette851', 1, '20160302')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (505, 75, 'Felix', 'Felix479', 1, '20151219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (506, 57, 'Latonya', 'Latonya475', 0, '20141231')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (507, 66, 'Darren', 'Darren263', 0, '20150610')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (508, 67, 'Vincent', 'Vincent821', 1, '20131027')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (509, 16, 'Terra', 'Terra076', 0, '20150710')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (510, 89, 'Isabel', 'Isabel482', 1, '20141014')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (511, 19, 'Myra', 'Myra634', 1, '20140817')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (512, 44, 'Marla', 'Marla381', 1, '20160324')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (513, 10, 'Leah', 'Leah029', 0, '20150409')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (514, 26, 'Damian', 'Damian365', 1, '20150630')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (515, 56, 'Laura', 'Laura368', 1, '20150404')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (516, 75, 'Erik', 'Erik527', 0, '20140219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (517, 52, 'Fredrick', 'Fredrick645', 1, '20151225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (518, 68, 'Debbie', 'Debbie888', 0, '20150918')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (519, 48, 'Roy', 'Roy066', 1, '20160430')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (520, 65, 'Ty', 'Ty52', 1, '20141017')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (521, 84, 'Jay', 'Jay956', 1, '20140717')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (522, 61, 'Lester', 'Lester109', 1, '20130929')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (523, 59, 'Angelina', 'Angelina968', 1, '20131229')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (524, 70, 'Vanessa', 'Vanessa329', 1, '20150426')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (525, 45, 'Heidi', 'Heidi758', 0, '20140226')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (526, 37, 'Guadalupe', 'Guadalupe298', 0, '20140813')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (527, NULL, 'Desiree', 'Desiree378', 0, '20150709')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (528, 75, 'Ian', 'Ian981', 0, '20160107')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (529, 98, 'Ismael', 'Ismael247', 0, '20151201')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (530, 53, 'Joe', 'Joe049', 0, '20141113')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (531, 43, 'Bridgett', 'Bridgett238', 1, '20160126')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (532, 94, 'Dexter', 'Dexter683', 1, '20140622')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (533, 94, 'Roman', 'Roman995', 0, '20151112')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (534, 34, 'Felipe', 'Felipe913', 0, '20150621')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (535, 80, 'Lamar', 'Lamar204', 1, '20160120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (536, 19, 'Elizabeth', 'Elizabeth144', 1, '20131111')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (537, 69, 'Faith', 'Faith525', 1, '20150717')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (538, 33, 'Brad', 'Brad539', 0, '20160401')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (539, 77, 'Devin', 'Devin094', 1, '20141024')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (540, 89, 'Lakeisha', 'Lakeisha249', 0, '20140318')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (541, 55, 'Kristen', 'Kristen155', 0, '20150521')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (542, 86, 'Mason', 'Mason21', 0, '20160519')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (543, 83, 'Rebecca', 'Rebecca576', 0, '20141229')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (544, 75, 'Alissa', 'Alissa792', 0, '20160109')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (545, 77, 'Seth', 'Seth837', 0, '20140515')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (546, 70, 'Gary', 'Gary848', 1, '20160321')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (547, 95, 'Jessica', 'Jessica320', 1, '20160311')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (548, 23, 'Max', 'Max357', 1, '20131010')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (549, 91, 'Caroline', 'Caroline44', 1, '20141203')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (550, 68, 'Tabitha', 'Tabitha608', 1, '20150731')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (551, 26, 'Joshua', 'Joshua190', 1, '20140709')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (552, 91, 'Jason', 'Jason589', 1, '20141121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (553, 27, 'Kelly', 'Kelly370', 0, '20140606')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (554, 68, 'Pedro', 'Pedro633', 1, '20150909')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (555, 87, 'Naomi', 'Naomi328', 1, '20130910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (556, 27, 'Luke', 'Luke705', 0, '20140814')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (557, 48, 'Jasmine', 'Jasmine974', 1, '20150802')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (558, 79, 'Andrew', 'Andrew987', 1, '20131106')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (559, 51, 'Marisa', 'Marisa565', 1, '20150628')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (560, 26, 'Brandy', 'Brandy285', 1, '20150428')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (561, 76, 'Dena', 'Dena800', 1, '20131118')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (562, 8, 'Miranda', 'Miranda150', 0, '20140322')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (563, 53, 'Enrique', 'Enrique767', 1, '20160405')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (564, 37, 'Kristen', 'Kristen699', 0, '20151023')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (565, 86, 'Evan', 'Evan751', 1, '20160424')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (566, 53, 'Brett', 'Brett27', 0, '20150315')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (567, 61, 'Edgar', 'Edgar818', 1, '20141208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (568, 43, 'Matthew', 'Matthew17', 0, '20140429')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (569, 37, 'Kristian', 'Kristian364', 0, '20140212')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (570, 37, 'Marlene', 'Marlene315', 1, '20160315')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (571, 78, 'Dominick', 'Dominick092', 1, '20140207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (572, 18, 'Samantha', 'Samantha143', 1, '20140223')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (573, 57, 'Nicole', 'Nicole472', 1, '20160210')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (574, 64, 'Sonny', 'Sonny104', 0, '20160314')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (575, 37, 'Gena', 'Gena703', 1, '20160101')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (576, 71, 'Megan', 'Megan868', 0, '20150727')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (577, 10, 'Alicia', 'Alicia193', 1, '20160428')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (578, 29, 'Kendrick', 'Kendrick488', 0, '20140405')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (579, 65, 'Vernon', 'Vernon876', 1, '20140122')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (580, 66, 'Philip', 'Philip617', 0, '20160104')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (581, 6, 'Jim', 'Jim667', 0, '20140619')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (582, 6, 'Barry', 'Barry867', 1, '20141125')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (583, 81, 'Liza', 'Liza140', 0, '20160108')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (584, 11, 'Kerry', 'Kerry313', 1, '20140615')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (585, 43, 'Mary', 'Mary86', 0, '20150805')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (586, 79, 'Latanya', 'Latanya460', 1, '20140616')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (587, 94, 'Lawrence', 'Lawrence655', 0, '20140810')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (588, 70, 'Alice', 'Alice307', 1, '20140522')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (589, 12, 'Wallace', 'Wallace779', 0, '20140402')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (590, 41, 'Cheri', 'Cheri224', 0, '20160601')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (591, 43, 'Joyce', 'Joyce017', 0, '20140114')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (592, 18, 'Elijah', 'Elijah840', 1, '20140927')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (593, 42, 'Scot', 'Scot6', 0, '20140907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (594, 33, 'Summer', 'Summer438', 1, '20131018')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (595, 56, 'Bennie', 'Bennie676', 1, '20160531')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (596, 18, 'Olivia', 'Olivia233', 1, '20131011')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (597, 3, 'Joel', 'Joel494', 1, '20131209')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (598, 36, 'Sergio', 'Sergio083', 0, '20141111')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (599, 77, 'Tonia', 'Tonia777', 0, '20160505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (600, 64, 'Kathy', 'Kathy118', 0, '20140709')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (601, 90, 'Jane', 'Jane334', 0, '20150207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (602, NULL, 'Wallace', 'Wallace12', 0, '20150127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (603, 42, 'Samantha', 'Samantha099', 0, '20140904')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (604, 88, 'Milton', 'Milton514', 0, '20140502')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (605, 25, 'Jamal', 'Jamal56', 0, '20160311')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (606, 35, 'Jose', 'Jose057', 0, '20140109')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (607, 2, 'Rose', 'Rose662', 0, '20160103')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (608, 13, 'Leonard', 'Leonard339', 1, '20140126')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (609, 80, 'Kisha', 'Kisha177', 1, '20150720')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (610, 28, 'Mindy', 'Mindy00', 0, '20160204')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (611, 89, 'Deanna', 'Deanna650', 1, '20160404')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (612, 71, 'Emily', 'Emily042', 0, '20151031')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (613, 97, 'Lindsay', 'Lindsay978', 0, '20141103')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (614, 36, 'Michael', 'Michael088', 0, '20150327')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (615, 78, 'Staci', 'Staci31', 0, '20140602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (616, 6, 'Lindsay', 'Lindsay19', 1, '20151214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (617, 68, 'Ryan', 'Ryan470', 1, '20140211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (618, 70, 'Ronda', 'Ronda98', 0, '20131112')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (619, 43, 'Christie', 'Christie09', 1, '20140505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (620, 77, 'Rose', 'Rose40', 1, '20150409')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (621, 54, 'Philip', 'Philip396', 1, '20150407')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (622, 8, 'Anitra', 'Anitra748', 1, '20150809')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (623, 21, 'Efrain', 'Efrain879', 1, '20151216')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (624, 16, 'Nicole', 'Nicole700', 0, '20140802')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (625, 90, 'Sherri', 'Sherri709', 0, '20160516')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (626, 57, 'Faith', 'Faith075', 0, '20140117')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (627, 50, 'Elias', 'Elias429', 0, '20150421')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (628, 75, 'Darrin', 'Darrin199', 0, '20160119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (629, 20, 'Darryl', 'Darryl990', 1, '20140926')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (630, 71, 'Darcy', 'Darcy403', 1, '20151203')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (631, 94, 'Aaron', 'Aaron008', 1, '20131128')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (632, 90, 'Shelley', 'Shelley439', 0, '20131105')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (633, 46, 'Shauna', 'Shauna668', 0, '20150806')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (634, 17, 'Jeannette', 'Jeannette920', 1, '20151103')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (635, 86, 'Nicolas', 'Nicolas575', 0, '20140919')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (636, 36, 'Antonio', 'Antonio739', 0, '20150502')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (637, 21, 'Jerome', 'Jerome478', 0, '20140816')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (638, 80, 'Ryan', 'Ryan422', 1, '20140526')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (639, 46, 'Tania', 'Tania939', 0, '20150408')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (640, 87, 'Keisha', 'Keisha395', 1, '20150404')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (641, 63, 'Lonnie', 'Lonnie695', 1, '20141107')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (642, 91, 'Tara', 'Tara520', 1, '20141031')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (643, 94, 'Amanda', 'Amanda274', 0, '20150915')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (644, 69, 'Clay', 'Clay293', 1, '20150609')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (645, 26, 'Bradley', 'Bradley113', 0, '20141124')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (646, 44, 'Lorena', 'Lorena069', 0, '20160324')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (647, 90, 'Traci', 'Traci111', 0, '20130911')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (648, 57, 'Dion', 'Dion63', 0, '20141003')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (649, 37, 'Tasha', 'Tasha428', 0, '20160327')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (650, 45, 'Vicki', 'Vicki146', 1, '20140106')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (651, 46, 'Jocelyn', 'Jocelyn631', 1, '20150916')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (652, 3, 'Gabrielle', 'Gabrielle222', 1, '20160221')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (653, 4, 'Tia', 'Tia11', 0, '20140201')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (654, 48, 'Charlotte', 'Charlotte046', 0, '20150827')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (655, 32, 'Erich', 'Erich079', 0, '20151112')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (656, 72, 'Dwayne', 'Dwayne641', 0, '20131022')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (657, 8, 'Sally', 'Sally318', 1, '20160327')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (658, 1, 'Susan', 'Susan251', 0, '20160115')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (659, 18, 'Gregory', 'Gregory261', 0, '20140809')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (660, 82, 'Myron', 'Myron715', 0, '20160604')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (661, 49, 'Philip', 'Philip393', 0, '20150115')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (662, 77, 'Desiree', 'Desiree665', 1, '20151208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (663, 6, 'Ira', 'Ira761', 0, '20141011')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (664, 31, 'Patrice', 'Patrice457', 1, '20131203')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (665, 21, 'Rachael', 'Rachael496', 1, '20140730')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (666, 28, 'Katherine', 'Katherine189', 0, '20140907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (667, 36, 'Leslie', 'Leslie536', 1, '20150602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (668, 90, 'Esther', 'Esther35', 1, '20131231')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (669, 45, 'Milton', 'Milton131', 0, '20131016')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (670, 36, 'Anita', 'Anita947', 0, '20150223')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (671, 90, 'Jeffery', 'Jeffery743', 0, '20160503')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (672, 26, 'Walter', 'Walter899', 0, '20150628')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (673, 80, 'Kari', 'Kari649', 1, '20141220')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (674, 42, 'Teddy', 'Teddy89', 0, '20160316')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (675, 13, 'Ernesto', 'Ernesto619', 0, '20130915')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (676, 29, 'Carlos', 'Carlos697', 0, '20140304')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (677, 51, 'Cary', 'Cary626', 1, '20131127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (678, 55, 'Franklin', 'Franklin57', 1, '20150601')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (679, 38, 'Gilberto', 'Gilberto110', 1, '20160329')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (680, 35, 'Ramiro', 'Ramiro10', 1, '20141217')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (681, 6, 'Virgil', 'Virgil93', 1, '20140118')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (682, 52, 'Nakia', 'Nakia755', 0, '20140702')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (683, 84, 'Emma', 'Emma139', 0, '20131124')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (684, 4, 'Josephine', 'Josephine260', 1, '20140615')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (685, 47, 'Esteban', 'Esteban58', 1, '20131108')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (686, 1, 'Ericka', 'Ericka163', 0, '20151007')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (687, 16, 'Alice', 'Alice345', 0, '20150921')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (688, 19, 'Charlene', 'Charlene495', 1, '20160108')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (689, 34, 'Ruben', 'Ruben136', 0, '20131130')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (690, 54, 'Carolyn', 'Carolyn101', 0, '20141013')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (691, NULL, 'Tonya', 'Tonya620', 1, '20150907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (692, 78, 'Sergio', 'Sergio750', 0, '20150909')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (693, 10, 'Tyler', 'Tyler985', 1, '20151119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (694, 42, 'Hugh', 'Hugh459', 0, '20150718')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (695, 99, 'Sonny', 'Sonny602', 1, '20140228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (696, 4, 'Jason', 'Jason983', 1, '20141106')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (697, 76, 'Jeanine', 'Jeanine353', 0, '20151125')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (698, 19, 'Jeremiah', 'Jeremiah843', 0, '20150615')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (699, 50, 'Betsy', 'Betsy055', 0, '20140521')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (700, 28, 'Jake', 'Jake556', 1, '20141129')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (701, 22, 'Jerrod', 'Jerrod919', 0, '20141031')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (702, 17, 'Lester', 'Lester522', 1, '20151115')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (703, 60, 'Darryl', 'Darryl380', 0, '20150505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (704, 62, 'Esteban', 'Esteban76', 1, '20140505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (705, 26, 'Clinton', 'Clinton30', 0, '20160409')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (706, 3, 'Jocelyn', 'Jocelyn977', 1, '20141002')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (707, 75, 'Ernesto', 'Ernesto035', 1, '20131114')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (708, 34, 'Cheri', 'Cheri604', 0, '20150207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (709, 29, 'Belinda', 'Belinda78', 0, '20151010')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (710, 42, 'Joe', 'Joe036', 0, '20150720')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (711, 42, 'Patrice', 'Patrice963', 0, '20151121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (712, 6, 'Stacy', 'Stacy480', 1, '20131009')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (713, 43, 'Lea', 'Lea528', 0, '20140514')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (714, 26, 'Debbie', 'Debbie212', 0, '20150505')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (715, 98, 'Steve', 'Steve568', 0, '20160501')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (716, 70, 'Alicia', 'Alicia444', 0, '20131210')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (717, 7, 'Tracie', 'Tracie918', 1, '20140117')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (718, 33, 'Darryl', 'Darryl469', 0, '20141218')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (719, 28, 'Marilyn', 'Marilyn45', 1, '20140606')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (720, 36, 'Bridgette', 'Bridgette661', 0, '20150124')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (721, 43, 'Heather', 'Heather625', 0, '20160208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (722, 39, 'Stephanie', 'Stephanie04', 0, '20150813')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (723, 26, 'Alex', 'Alex786', 1, '20130920')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (724, 37, 'Damian', 'Damian595', 1, '20140709')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (725, 83, 'Renee', 'Renee793', 1, '20151026')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (726, 23, 'Gina', 'Gina890', 0, '20140301')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (727, 5, 'Rogelio', 'Rogelio172', 1, '20150411')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (728, 89, 'Kerrie', 'Kerrie746', 0, '20141205')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (729, 96, 'Arlene', 'Arlene773', 1, '20150910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (730, 65, 'Randolph', 'Randolph06', 0, '20141002')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (731, 85, 'Edgar', 'Edgar892', 1, '20140428')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (732, 73, 'Tabatha', 'Tabatha278', 1, '20151210')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (733, 50, 'Ronald', 'Ronald505', 1, '20131004')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (734, 46, 'Amber', 'Amber666', 1, '20140511')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (735, 84, 'Pete', 'Pete814', 0, '20140309')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (736, 45, 'Caroline', 'Caroline816', 0, '20131212')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (737, 11, 'Sandy', 'Sandy922', 0, '20151009')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (738, 51, 'Clifford', 'Clifford498', 0, '20131209')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (739, 57, 'Terrance', 'Terrance295', 0, '20150201')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (740, 72, 'Tanya', 'Tanya219', 0, '20130909')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (741, 5, 'Harold', 'Harold863', 0, '20150305')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (742, 69, 'Greg', 'Greg28', 1, '20150703')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (743, 52, 'Ivan', 'Ivan871', 1, '20160417')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (744, 3, 'Brady', 'Brady352', 1, '20150915')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (745, 44, 'Mia', 'Mia790', 1, '20131102')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (746, 4, 'Trevor', 'Trevor731', 1, '20151207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (747, 6, 'Lori', 'Lori054', 1, '20150810')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (748, 16, 'Raquel', 'Raquel503', 0, '20131220')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (749, 100, 'Catherine', 'Catherine560', 1, '20131010')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (750, 31, 'Maria', 'Maria020', 0, '20140904')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (751, 6, 'Gilbert', 'Gilbert999', 0, '20141229')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (752, 76, 'Kirk', 'Kirk014', 1, '20151215')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (753, 2, 'Shanda', 'Shanda681', 1, '20140423')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (754, 94, 'Ismael', 'Ismael889', 1, '20131118')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (755, 84, 'Christi', 'Christi119', 0, '20151118')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (756, 71, 'Elton', 'Elton387', 0, '20140910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (757, 50, 'Blanca', 'Blanca347', 0, '20150411')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (758, 14, 'Jerome', 'Jerome601', 0, '20150504')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (759, 43, 'Mary', 'Mary930', 0, '20160322')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (760, 1, 'Rodolfo', 'Rodolfo18', 0, '20140105')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (761, 94, 'Elisa', 'Elisa417', 0, '20131003')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (762, 100, 'Mark', 'Mark594', 0, '20150211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (763, 30, 'Gavin', 'Gavin635', 1, '20150211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (764, 53, 'Latisha', 'Latisha434', 0, '20150730')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (765, 27, 'Virginia', 'Virginia464', 1, '20140830')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (766, 71, 'Joanna', 'Joanna791', 1, '20150215')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (767, 95, 'Claude', 'Claude784', 0, '20150407')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (768, 86, 'Iris', 'Iris489', 0, '20140513')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (769, 91, 'Jenna', 'Jenna929', 0, '20150625')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (770, 50, 'Cody', 'Cody976', 0, '20150216')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (771, 73, 'Chrystal', 'Chrystal82', 0, '20140703')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (772, 90, 'Kris', 'Kris248', 1, '20150814')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (773, 35, 'John', 'John102', 0, '20160412')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (774, 59, 'Dana', 'Dana673', 0, '20150825')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (775, 4, 'Molly', 'Molly074', 0, '20151213')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (776, 89, 'Leticia', 'Leticia376', 1, '20160531')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (777, 88, 'Zachary', 'Zachary875', 1, '20140425')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (778, 36, 'Jose', 'Jose564', 1, '20141024')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (779, 94, 'Jodie', 'Jodie232', 1, '20140322')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (780, 19, 'Ralph', 'Ralph441', 1, '20140123')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (781, 12, 'Adriana', 'Adriana759', 0, '20150219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (782, 37, 'Tricia', 'Tricia351', 0, '20160409')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (783, 49, 'Melanie', 'Melanie737', 1, '20140830')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (784, 76, 'Rick', 'Rick372', 0, '20140320')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (785, 88, 'Kurt', 'Kurt011', 0, '20131003')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (786, 29, 'Todd', 'Todd598', 1, '20150812')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (787, 72, 'Karrie', 'Karrie771', 1, '20140604')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (788, 12, 'Dominick', 'Dominick911', 1, '20150605')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (789, 85, 'Herman', 'Herman098', 1, '20131205')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (790, 44, 'Frank', 'Frank461', 0, '20140320')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (791, 79, 'Alberto', 'Alberto170', 0, '20141021')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (792, 91, 'Jana', 'Jana79', 1, '20140906')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (793, 31, 'Fernando', 'Fernando089', 0, '20150409')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (794, 82, 'Dina', 'Dina84', 0, '20131127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (795, 6, 'Kristian', 'Kristian354', 0, '20150526')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (796, 18, 'Ellen', 'Ellen896', 1, '20160422')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (797, 2, 'Darla', 'Darla490', 0, '20160426')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (798, 75, 'Lee', 'Lee982', 0, '20151230')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (799, 4, 'Lloyd', 'Lloyd300', 1, '20131125')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (800, 76, 'Grant', 'Grant484', 1, '20141228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (801, 58, 'Terence', 'Terence064', 1, '20150127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (802, 45, 'Virgil', 'Virgil957', 1, '20141209')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (803, NULL, 'Ellen', 'Ellen162', 0, '20150622')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (804, 67, 'Toby', 'Toby292', 1, '20150801')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (805, 74, 'Louis', 'Louis060', 0, '20160602')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (806, 82, 'Shari', 'Shari745', 1, '20160417')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (807, 19, 'Mason', 'Mason720', 1, '20150715')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (808, 22, 'Jolene', 'Jolene658', 0, '20140212')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (809, 92, 'Clifton', 'Clifton466', 1, '20160327')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (810, 7, 'Duane', 'Duane897', 1, '20150711')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (811, 57, 'Clifford', 'Clifford543', 1, '20140705')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (812, 28, 'Ralph', 'Ralph306', 0, '20131008')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (813, 35, 'Dena', 'Dena726', 0, '20150822')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (814, 36, 'Kendall', 'Kendall900', 0, '20130926')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (815, 87, 'Doris', 'Doris424', 1, '20140120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (816, 39, 'Esmeralda', 'Esmeralda34', 1, '20140714')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (817, 59, 'Dwight', 'Dwight078', 1, '20140303')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (818, 11, 'Dominick', 'Dominick588', 0, '20150510')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (819, 28, 'Hollie', 'Hollie317', 1, '20140721')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (820, 91, 'Wade', 'Wade198', 0, '20140215')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (821, 65, 'Chad', 'Chad937', 1, '20141128')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (822, 8, 'Austin', 'Austin289', 0, '20150729')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (823, 25, 'Rachel', 'Rachel01', 1, '20140702')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (824, 99, 'Moses', 'Moses477', 1, '20140130')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (825, 80, 'Candace', 'Candace369', 0, '20150519')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (826, 84, 'Guy', 'Guy580', 0, '20150815')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (827, 97, 'Alicia', 'Alicia3', 0, '20140524')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (828, 19, 'Salvador', 'Salvador279', 1, '20150331')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (829, 30, 'Alexis', 'Alexis558', 0, '20130928')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (830, 10, 'Dora', 'Dora14', 0, '20160124')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (831, 34, 'Lorie', 'Lorie205', 0, '20151118')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (832, 21, 'Lindsey', 'Lindsey197', 0, '20140211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (833, 98, 'Rochelle', 'Rochelle884', 1, '20160208')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (834, 91, 'Jessica', 'Jessica965', 1, '20140707')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (835, 45, 'Demond', 'Demond586', 0, '20141114')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (836, 60, 'Lynette', 'Lynette211', 1, '20150813')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (837, 19, 'Samantha', 'Samantha515', 0, '20151007')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (838, 69, 'Gerardo', 'Gerardo009', 1, '20140121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (839, 13, 'Kimberley', 'Kimberley492', 0, '20141106')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (840, 78, 'Rachel', 'Rachel77', 1, '20141022')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (841, 87, 'Annette', 'Annette83', 1, '20160224')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (842, 42, 'Cedric', 'Cedric301', 1, '20140511')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (843, 57, 'Vicki', 'Vicki537', 0, '20150226')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (844, 55, 'Karl', 'Karl88', 1, '20150929')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (845, 82, 'Damion', 'Damion566', 0, '20140120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (846, 10, 'Marsha', 'Marsha338', 1, '20141026')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (847, 67, 'Annie', 'Annie124', 1, '20151228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (848, 28, 'Hector', 'Hector652', 0, '20140326')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (849, 64, 'Ashley', 'Ashley148', 0, '20150102')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (850, 16, 'Nelson', 'Nelson05', 1, '20131107')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (851, 15, 'Daphne', 'Daphne839', 0, '20150909')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (852, 94, 'Francis', 'Francis954', 1, '20150324')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (853, 38, 'Jordan', 'Jordan003', 1, '20140930')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (854, 47, 'Wendell', 'Wendell174', 1, '20151102')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (855, 25, 'Javier', 'Javier852', 0, '20140523')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (856, 100, 'Shaun', 'Shaun532', 0, '20131104')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (857, 72, 'Katie', 'Katie834', 1, '20131216')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (858, 30, 'Meghan', 'Meghan689', 0, '20141204')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (859, 4, 'Donna', 'Donna202', 0, '20160212')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (860, 78, 'Theresa', 'Theresa038', 1, '20131110')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (861, 77, 'Sean', 'Sean766', 0, '20131001')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (862, 38, 'Anne', 'Anne297', 1, '20140329')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (863, 74, 'Kenneth', 'Kenneth325', 1, '20151211')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (864, 49, 'Alisha', 'Alisha622', 1, '20150624')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (865, 20, 'Kent', 'Kent856', 0, '20150228')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (866, 10, 'Clyde', 'Clyde276', 1, '20140420')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (867, 99, 'Gail', 'Gail399', 0, '20131207')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (868, 15, 'Tameka', 'Tameka907', 0, '20150530')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (869, 5, 'Tiffany', 'Tiffany997', 0, '20140907')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (870, 52, 'Faith', 'Faith08', 0, '20150522')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (871, 27, 'Jon', 'Jon998', 0, '20150109')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (872, 41, 'Jeannie', 'Jeannie61', 1, '20150910')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (873, 100, 'Esteban', 'Esteban322', 0, '20150511')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (874, 8, 'Stanley', 'Stanley463', 0, '20140607')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (875, 46, 'Latasha', 'Latasha738', 0, '20150925')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (876, 16, 'Lorie', 'Lorie425', 0, '20150303')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (877, 96, 'Kristie', 'Kristie87', 0, '20150717')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (878, 18, 'Regina', 'Regina513', 0, '20140818')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (879, 44, 'Willie', 'Willie03', 1, '20141105')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (880, 23, 'Brad', 'Brad733', 0, '20150906')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (881, 20, 'Gabriel', 'Gabriel160', 1, '20131130')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (882, 31, 'Ryan', 'Ryan186', 0, '20160107')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (883, 37, 'Tonia', 'Tonia377', 1, '20150428')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (884, 17, 'Theodore', 'Theodore928', 1, '20150222')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (885, 65, 'Shelly', 'Shelly923', 1, '20140817')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (886, 31, 'Dale', 'Dale857', 1, '20141021')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (887, 79, 'Tyson', 'Tyson721', 0, '20150324')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (888, 98, 'Peter', 'Peter825', 0, '20150629')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (889, 85, 'Jeannette', 'Jeannette288', 1, '20140829')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (890, 47, 'Eduardo', 'Eduardo959', 1, '20150604')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (891, 74, 'Kevin', 'Kevin268', 0, '20150514')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (892, 98, 'Kelli', 'Kelli75', 1, '20131006')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (893, 33, 'Jodie', 'Jodie988', 1, '20150524')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (894, 44, 'Nora', 'Nora618', 0, '20150804')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (895, 67, 'Marla', 'Marla191', 0, '20131113')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (896, 63, 'Robyn', 'Robyn642', 1, '20160121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (897, 22, 'Cindy', 'Cindy729', 0, '20131019')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (898, 69, 'Frederick', 'Frederick48', 1, '20140611')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (899, 74, 'Bradford', 'Bradford744', 0, '20140126')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (900, 100, 'Tracey', 'Tracey223', 1, '20140406')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (901, 77, 'Thomas', 'Thomas64', 1, '20140225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (902, 23, 'Terence', 'Terence481', 0, '20160525')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (903, 63, 'Jay', 'Jay581', 0, '20150816')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (904, 54, 'Lloyd', 'Lloyd46', 0, '20150708')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (905, 95, 'Ivan', 'Ivan984', 0, '20151128')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (906, 5, 'Penny', 'Penny406', 0, '20131201')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (907, 3, 'Rose', 'Rose167', 0, '20141119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (908, 85, 'Brendan', 'Brendan557', 0, '20140905')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (909, 6, 'Enrique', 'Enrique972', 1, '20151012')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (910, 45, 'Joel', 'Joel881', 0, '20131217')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (911, 17, 'Alexandra', 'Alexandra806', 1, '20150603')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (912, 22, 'Cari', 'Cari468', 1, '20140705')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (913, 60, 'Kelly', 'Kelly2', 1, '20140403')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (914, 38, 'Tyler', 'Tyler226', 0, '20160421')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (915, 21, 'Terry', 'Terry123', 1, '20140417')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (916, 98, 'Johnny', 'Johnny599', 0, '20150722')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (917, 3, 'Leroy', 'Leroy824', 0, '20151010')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (918, 4, 'Yolanda', 'Yolanda235', 0, '20141011')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (919, 40, 'Staci', 'Staci600', 1, '20140824')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (920, 65, 'Darius', 'Darius877', 1, '20160528')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (921, 23, 'Timothy', 'Timothy670', 0, '20140312')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (922, 12, 'Dexter', 'Dexter15', 1, '20140809')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (923, 21, 'Gene', 'Gene258', 0, '20150214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (924, 55, 'Dustin', 'Dustin632', 0, '20150627')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (925, 11, 'Brandi', 'Brandi275', 0, '20151120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (926, 19, 'Jeanne', 'Jeanne346', 0, '20140901')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (927, 20, 'Wade', 'Wade548', 0, '20160112')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (928, 23, 'Melody', 'Melody571', 1, '20150708')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (929, 100, 'Lashonda', 'Lashonda603', 1, '20140610')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (930, 63, 'Clifford', 'Clifford25', 0, '20140417')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (931, 20, 'Jerome', 'Jerome712', 1, '20131121')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (932, 86, 'Kareem', 'Kareem234', 1, '20141127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (933, 49, 'Lewis', 'Lewis547', 1, '20140814')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (934, 40, 'Cassie', 'Cassie518', 1, '20160526')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (935, 40, 'Kari', 'Kari168', 1, '20140109')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (936, 19, 'Deanna', 'Deanna610', 1, '20141124')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (937, 40, 'Ted', 'Ted296', 0, '20160130')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (938, 37, 'Latoya', 'Latoya272', 1, '20140417')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (939, 99, 'Rene', 'Rene187', 0, '20141002')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (940, NULL, 'Vanessa', 'Vanessa430', 1, '20160226')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (941, 39, 'Mitchell', 'Mitchell252', 0, '20150111')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (942, 59, 'Charity', 'Charity314', 1, '20140310')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (943, 11, 'Jennifer', 'Jennifer039', 1, '20151225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (944, 28, 'Rose', 'Rose819', 1, '20150411')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (945, 63, 'Demond', 'Demond70', 0, '20151213')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (946, 37, 'Dwight', 'Dwight723', 1, '20160202')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (947, 57, 'Katie', 'Katie741', 0, '20140516')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (948, 76, 'Anita', 'Anita584', 0, '20160404')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (949, 54, 'Lorraine', 'Lorraine105', 1, '20131029')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (950, 91, 'Maggie', 'Maggie756', 0, '20151218')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (951, 37, 'Elizabeth', 'Elizabeth192', 1, '20131012')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (952, 60, 'Clay', 'Clay053', 1, '20150701')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (953, 80, 'Robyn', 'Robyn450', 1, '20131111')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (954, 89, 'Leo', 'Leo125', 1, '20160516')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (955, 51, 'Lashonda', 'Lashonda583', 1, '20160513')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (956, 44, 'Cecil', 'Cecil052', 1, '20140127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (957, 99, 'Johnathan', 'Johnathan397', 1, '20151225')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (958, 25, 'Cory', 'Cory801', 1, '20140517')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (959, 62, 'Norman', 'Norman203', 1, '20160328')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (960, 76, 'Bradley', 'Bradley091', 1, '20140424')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (961, 21, 'Luke', 'Luke090', 0, '20160401')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (962, 77, 'Terra', 'Terra037', 1, '20140408')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (963, 3, 'Howard', 'Howard713', 1, '20160119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (964, 63, 'Miranda', 'Miranda287', 0, '20140601')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (965, 65, 'James', 'James906', 0, '20160403')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (966, 84, 'Marshall', 'Marshall944', 1, '20140324')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (967, 8, 'Katina', 'Katina660', 1, '20140129')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (968, 84, 'Leah', 'Leah572', 1, '20141126')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (969, 60, 'Rachelle', 'Rachelle656', 1, '20150619')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (970, 94, 'Theresa', 'Theresa360', 0, '20140608')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (971, 66, 'Annette', 'Annette103', 0, '20150725')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (972, 52, 'Christian', 'Christian509', 1, '20141024')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (973, 41, 'Joey', 'Joey802', 0, '20150531')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (974, 56, 'Richard', 'Richard256', 1, '20140217')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (975, 89, 'Julius', 'Julius526', 1, '20150301')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (976, 94, 'Tracie', 'Tracie592', 0, '20140210')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (977, 77, 'Karrie', 'Karrie529', 0, '20140921')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (978, 52, 'Rachel', 'Rachel647', 0, '20140620')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (979, 6, 'Lamont', 'Lamont994', 0, '20140312')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (980, 37, 'Marcia', 'Marcia860', 1, '20151217')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (981, 30, 'Douglas', 'Douglas880', 1, '20150329')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (982, 66, 'Anita', 'Anita753', 1, '20131125')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (983, 20, 'Teddy', 'Teddy033', 0, '20150414')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (984, 62, 'Geoffrey', 'Geoffrey577', 0, '20140127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (985, 80, 'Emily', 'Emily845', 1, '20150127')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (986, 11, 'Shawna', 'Shawna161', 0, '20131214')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (987, 3, 'Kathryn', 'Kathryn239', 1, '20141221')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (988, 2, 'Samantha', 'Samantha736', 0, '20150312')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (989, 31, 'Erica', 'Erica319', 0, '20151119')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (990, 72, 'Alice', 'Alice304', 1, '20151219')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (991, 22, 'Christie', 'Christie106', 1, '20160229')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (992, 30, 'Patrice', 'Patrice809', 0, '20160111')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (993, 20, 'Latonya', 'Latonya627', 1, '20150405')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (994, 51, 'Stuart', 'Stuart010', 0, '20140616')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (995, 77, 'Christie', 'Christie545', 1, '20150401')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (996, 51, 'Herman', 'Herman412', 1, '20150120')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (997, 82, 'Everett', 'Everett485', 0, '20140317')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (998, 89, 'Maggie', 'Maggie717', 1, '20140324')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (999, 32, 'Luz', 'Luz499', 1, '20160204')
INSERT INTO [dbo].[Spieler] ([ID], [HerkunftslandID], [Name], [Nickname], [Aktiv], [MitgliedSeit]) VALUES (1000, 42, 'Laurie', 'Laurie348', 0, '20150603')
SET IDENTITY_INSERT [dbo].[Spieler] OFF
SET IDENTITY_INSERT [dbo].[Videospiele] ON
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (1, 'Game of Trupebistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (2, 'Game of Qwicador')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (3, 'Game of Thrunipaquar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (4, 'Game of Inzapefax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (5, 'Game of Uperollan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (6, 'Game of Monvenplin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (7, 'Game of Endwerpower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (8, 'Game of Rebanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (9, 'Game of Lomrobover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (10, 'Game of Qwinipilor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (11, 'Game of Retuman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (12, 'Game of Emmunazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (13, 'Game of Lompebinar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (14, 'Game of Monquestilin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (15, 'Game of Intinaquor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (16, 'Game of Lomcadazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (17, 'Game of Rapcadantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (18, 'Game of Grohupanar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (19, 'Game of Frokilistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (20, 'Game of Winwerpegax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (21, 'Game of Tupkililin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (22, 'Game of Monsipadin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (23, 'Game of Frosipamin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (24, 'Game of Grocadazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (25, 'Game of Unwerpin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (26, 'Game of Surmunantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (27, 'Game of Tupsipex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (28, 'Game of Parrobackax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (29, 'Game of Zeesapplazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (30, 'Game of Endcadex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (31, 'Game of Hapjubador')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (32, 'Game of Upcadantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (33, 'Game of Emzapax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (34, 'Game of Mondudower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (35, 'Game of Gromunamor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (36, 'Game of Tupcadedex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (37, 'Game of Qwibanentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (38, 'Game of Tipzapanan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (39, 'Game of Monquestazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (40, 'Game of Endquestower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (41, 'Game of Doprobicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (42, 'Game of Inbananantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (43, 'Game of Frodimegicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (44, 'Game of Tiprobedistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (45, 'Game of Tuptinedgicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (46, 'Game of Suptinamover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (47, 'Game of Hapzapar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (48, 'Game of Vartanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (49, 'Game of Emmunover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (50, 'Game of Supdimar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (51, 'Game of Klitinover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (52, 'Game of Zeepebor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (53, 'Game of Trududplantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (54, 'Game of Endmunin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (55, 'Game of Aderonax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (56, 'Game of Qwihupover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (57, 'Game of Parbanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (58, 'Game of Adcadewar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (59, 'Game of Zeetumower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (60, 'Game of Zeedudicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (61, 'Game of Tuprobar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (62, 'Game of Rebanimantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (63, 'Game of Winerax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (64, 'Game of Thruglibor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (65, 'Game of Frovenor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (66, 'Game of Adbanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (67, 'Game of Thrumunar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (68, 'Game of Inerollar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (69, 'Game of Tupsapedgicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (70, 'Game of Refropex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (71, 'Game of Truvenanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (72, 'Game of Lomrobentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (73, 'Game of Barbanin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (74, 'Game of Qwizapewentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (75, 'Game of Growerommower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (76, 'Game of Redudaquower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (77, 'Game of Grojubex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (78, 'Game of Barsipicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (79, 'Game of Supglibax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (80, 'Game of Lommunor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (81, 'Game of Surglibicor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (82, 'Game of Dopfropollazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (83, 'Game of Varjubicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (84, 'Game of Tupnipover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (85, 'Game of Dopglibefantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (86, 'Game of Adsipin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (87, 'Game of Monzapamicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (88, 'Game of Intinewax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (89, 'Game of Lomtininex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (90, 'Game of Qwiglibentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (91, 'Game of Adrobinan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (92, 'Game of Tipdimopicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (93, 'Game of Monbaninentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (94, 'Game of Qwimunantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (95, 'Game of Updimollazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (96, 'Game of Thrumuninor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (97, 'Game of Varbanackor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (98, 'Game of Updudewover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (99, 'Game of Lomwericator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (100, 'Game of Barglibupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (101, 'Game of Lompickilantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (102, 'Game of Varnipover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (103, 'Game of Grobanin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (104, 'Game of Refropicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (105, 'Game of Thrutanadistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (106, 'Game of Groquestor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (107, 'Game of Zeeglibegin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (108, 'Game of Upfropaquentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (109, 'Game of Varzapex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (110, 'Game of Supcadedgover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (111, 'Game of Wintinantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (112, 'Game of Bareregazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (113, 'Game of Emcadepax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (114, 'Game of Kliweredgan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (115, 'Game of Surdiman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (116, 'Game of Grosapicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (117, 'Game of Emjubar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (118, 'Game of Ciptinover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (119, 'Game of Dopdudower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (120, 'Game of Endrobadicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (121, 'Game of Lompebollax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (122, 'Game of Happebexazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (123, 'Game of Raphupedazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (124, 'Game of Enddudar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (125, 'Game of Groerax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (126, 'Game of Rapmunommentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (127, 'Game of Tipwerpewex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (128, 'Game of Endwerepan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (129, 'Game of Parvenewan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (130, 'Game of Undudower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (131, 'Game of Rapsapax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (132, 'Game of Upcadower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (133, 'Game of Tipfropan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (134, 'Game of Adsipantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (135, 'Game of Zeetumower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (136, 'Game of Parpickower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (137, 'Game of Tupsapepan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (138, 'Game of Gronipower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (139, 'Game of Emdimar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (140, 'Game of Surfropentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (141, 'Game of Grodimentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (142, 'Game of Zeecadinar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (143, 'Game of Endbanaquover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (144, 'Game of Trutumex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (145, 'Game of Barsipegan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (146, 'Game of Surhupedgentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (147, 'Game of Zeenipaquicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (148, 'Game of Endrobewax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (149, 'Game of Tupquestackor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (150, 'Game of Parnipackentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (151, 'Game of Redudackazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (152, 'Game of Aderover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (153, 'Game of Barhupex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (154, 'Game of Qwidimor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (155, 'Game of Froglibinantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (156, 'Game of Winkilantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (157, 'Game of Hapdimadax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (158, 'Game of Addimefower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (159, 'Game of Rezapazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (160, 'Game of Trusipantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (161, 'Game of Reglibaquor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (162, 'Game of Embanplicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (163, 'Game of Grojubicantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (164, 'Game of Endquestegentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (165, 'Game of Zeefropamor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (166, 'Game of Ciptanewax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (167, 'Game of Trumunaman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (168, 'Game of Emnipicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (169, 'Game of Refropicor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (170, 'Game of Qwierazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (171, 'Game of Endglibicar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (172, 'Game of Resapower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (173, 'Game of Uppebantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (174, 'Game of Varfropower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (175, 'Game of Reglibantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (176, 'Game of Cipwericator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (177, 'Game of Klikilar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (178, 'Game of Dopsipentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (179, 'Game of Tiptanower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (180, 'Game of Parglibefentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (181, 'Game of Kliquestupan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (182, 'Game of Dopzapupentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (183, 'Game of Winwerican')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (184, 'Game of Supbanentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (185, 'Game of Varrobar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (186, 'Game of Monglibar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (187, 'Game of Qwitumin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (188, 'Game of Thruerollan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (189, 'Game of Cipvenicin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (190, 'Game of Tupdimplantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (191, 'Game of Klisapupicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (192, 'Game of Monnipazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (193, 'Game of Winkilower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (194, 'Game of Klidimegan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (195, 'Game of Untinar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (196, 'Game of Dopcadimentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (197, 'Game of Klidimex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (198, 'Game of Parquestan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (199, 'Game of Hapmunin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (200, 'Game of Insipimover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (201, 'Game of Winnipantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (202, 'Game of Winmunollax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (203, 'Game of Barzaponar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (204, 'Game of Tipwerpexentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (205, 'Game of Trupickadax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (206, 'Game of Trusapower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (207, 'Game of Monsapadicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (208, 'Game of Hapdudar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (209, 'Game of Varquestentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (210, 'Game of Raptumeficator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (211, 'Game of Trufropover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (212, 'Game of Thrutanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (213, 'Game of Cipsapedgistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (214, 'Game of Dopnipupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (215, 'Game of Thrutanewor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (216, 'Game of Tuptinonover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (217, 'Game of Monvenewantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (218, 'Game of Parpebicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (219, 'Game of Tippebamor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (220, 'Game of Inmunistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (221, 'Game of Groglibupistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (222, 'Game of Infropover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (223, 'Game of Cipdimicicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (224, 'Game of Emdudepar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (225, 'Game of Frozapommin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (226, 'Game of Endmunewex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (227, 'Game of Thrusapor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (228, 'Game of Cipsapegex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (229, 'Game of Frofropin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (230, 'Game of Dopfropazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (231, 'Game of Surnipar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (232, 'Game of Suptumamistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (233, 'Game of Adnipover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (234, 'Game of Addudedantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (235, 'Game of Qwipebover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (236, 'Game of Emhupazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (237, 'Game of Rezapar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (238, 'Game of Parquestepin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (239, 'Game of Rapweredower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (240, 'Game of Grotinantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (241, 'Game of Qwibanan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (242, 'Game of Trufropefantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (243, 'Game of Parwerpor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (244, 'Game of Qwisipover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (245, 'Game of Barpebexower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (246, 'Game of Endmunedistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (247, 'Game of Barericator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (248, 'Game of Barcadantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (249, 'Game of Klimunover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (250, 'Game of Monwerexor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (251, 'Game of Grotinax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (252, 'Game of Klitanadax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (253, 'Game of Grosipantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (254, 'Game of Truzapin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (255, 'Game of Adnipicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (256, 'Game of Surwerplar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (257, 'Game of Dopdimexan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (258, 'Game of Hapzapplentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (259, 'Game of Upglibex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (260, 'Game of Suptumentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (261, 'Game of Adzapax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (262, NULL)
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (263, 'Game of Supdimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (264, 'Game of Recadopower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (265, 'Game of Inhupedower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (266, 'Game of Unjubadin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (267, 'Game of Grodimommex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (268, 'Game of Endbanupin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (269, 'Game of Cipjuban')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (270, 'Game of Adnipilistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (271, 'Game of Growerpackin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (272, 'Game of Supzapplover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (273, 'Game of Zeesapover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (274, 'Game of Cipkilaman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (275, 'Game of Happebanower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (276, 'Game of Zeerobax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (277, 'Game of Remunaquax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (278, 'Game of Klitinax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (279, 'Game of Upsapedazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (280, 'Game of Rekilimin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (281, 'Game of Haptanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (282, 'Game of Thruwerpicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (283, 'Game of Grocadamor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (284, 'Game of Unsapaquan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (285, 'Game of Frokilicex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (286, 'Game of Truglibar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (287, 'Game of Repickistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (288, 'Game of Rapvenan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (289, 'Game of Lomcadackantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (290, 'Game of Surpebollazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (291, 'Game of Lompickommentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (292, 'Game of Grobanin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (293, 'Game of Rapnipommantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (294, 'Game of Klisipistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (295, 'Game of Gropickex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (296, 'Game of Barnipackor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (297, 'Game of Adtanistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (298, 'Game of Sursipedgor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (299, 'Game of Varfropentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (300, 'Game of Groweradazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (301, 'Game of Cipfropover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (302, 'Game of Wineranar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (303, 'Game of Frodimewantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (304, 'Game of Dopjubicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (305, 'Game of Pardimentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (306, 'Game of Supbanomman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (307, NULL)
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (308, 'Game of Lomweropax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (309, 'Game of Zeepickicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (310, 'Game of Dophupex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (311, 'Game of Klitumopistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (312, 'Game of Tupcadantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (313, 'Game of Barpebicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (314, 'Game of Parfropistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (315, 'Game of Tipkilewex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (316, 'Game of Zeedimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (317, 'Game of Endjubax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (318, 'Game of Resapedgantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (319, 'Game of Updudistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (320, 'Game of Monsipazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (321, 'Game of Rapquestax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (322, 'Game of Varjubegentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (323, 'Game of Varsapicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (324, 'Game of Unkiledgistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (325, 'Game of Tiptanadover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (326, 'Game of Qwikilamentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (327, 'Game of Lomhupex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (328, 'Game of Parpickegover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (329, 'Game of Qwimunefin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (330, 'Game of Haptinicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (331, 'Game of Lombanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (332, 'Game of Tupdimicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (333, 'Game of Lomnipar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (334, 'Game of Upnipefazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (335, 'Game of Bardimefistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (336, 'Game of Lomtumicicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (337, 'Game of Tipdudin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (338, 'Game of Adcadanax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (339, 'Game of Tupquestefistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (340, 'Game of Adwerpin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (341, 'Game of Qwitanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (342, 'Game of Retanefan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (343, 'Game of Lombanexin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (344, 'Game of Tipcadexower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (345, 'Game of Supsipaquentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (346, 'Game of Surtumollistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (347, 'Game of Supnipantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (348, 'Game of Adquestimin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (349, 'Game of Parwerpex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (350, 'Game of Raptanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (351, 'Game of Vartumentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (352, 'Game of Dopdiminicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (353, 'Game of Haperadar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (354, 'Game of Endnipedgor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (355, 'Game of Grotumar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (356, 'Game of Grozapewazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (357, 'Game of Thruhupewin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (358, NULL)
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (359, 'Game of Zeewericator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (360, 'Game of Varmunegistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (361, 'Game of Redudopistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (362, 'Game of Unquestackicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (363, 'Game of Qwimunicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (364, 'Game of Rapglibower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (365, 'Game of Cipcadollazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (366, 'Game of Endsapamover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (367, 'Game of Frofropommower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (368, 'Game of Doptumamax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (369, 'Game of Varzapicazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (370, 'Game of Hapfropilower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (371, 'Game of Vartinegicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (372, 'Game of Gropickimover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (373, 'Game of Adglibover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (374, 'Game of Supcadover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (375, NULL)
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (376, 'Game of Hapsapegazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (377, 'Game of Endwerpopazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (378, 'Game of Tippickonax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (379, 'Game of Growerower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (380, 'Game of Surtanollentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (381, 'Game of Requestedgex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (382, 'Game of Lomkilin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (383, 'Game of Emtumedgan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (384, 'Game of Upzapin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (385, 'Game of Surquestegantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (386, 'Game of Supzapexor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (387, 'Game of Inbanilower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (388, 'Game of Qwipebinax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (389, 'Game of Undimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (390, 'Game of Tuptinower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (391, 'Game of Tuppebower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (392, 'Game of Truzapar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (393, 'Game of Thrunipistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (394, 'Game of Rapkilax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (395, 'Game of Sursipommex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (396, 'Game of Advenewax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (397, 'Game of Winrobaquover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (398, 'Game of Undimackex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (399, 'Game of Endglibor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (400, 'Game of Doptanicex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (401, 'Game of Qwirobefax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (402, 'Game of Thrucadin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (403, 'Game of Haptanax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (404, 'Game of Partumimor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (405, 'Game of Lomsaponentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (406, 'Game of Parglibex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (407, 'Game of Tupdudin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (408, 'Game of Montumupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (409, 'Game of Hapdudex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (410, 'Game of Supglibaquistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (411, 'Game of Surdudicor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (412, 'Game of Inquestan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (413, 'Game of Dopglibonax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (414, 'Game of Dopsipedar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (415, 'Game of Tiperar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (416, 'Game of Updimazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (417, 'Game of Enddimover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (418, 'Game of Winbanistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (419, 'Game of Unglibicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (420, 'Game of Dopdudex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (421, 'Game of Tipzapor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (422, 'Game of Uppebex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (423, 'Game of Unfropan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (424, 'Game of Upwerexentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (425, 'Game of Inglibex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (426, 'Game of Haphupollin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (427, 'Game of Gromunex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (428, 'Game of Varrobepex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (429, 'Game of Varpebimover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (430, 'Game of Fropickin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (431, 'Game of Thruglibazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (432, 'Game of Wintumimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (433, 'Game of Varbanedgantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (434, 'Game of Enddimackazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (435, 'Game of Frohupewex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (436, 'Game of Hapmunimor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (437, NULL)
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (438, 'Game of Frojubistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (439, 'Game of Klitumor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (440, 'Game of Lompebazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (441, 'Game of Tuptinex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (442, 'Game of Thrufropor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (443, 'Game of Retumaquentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (444, 'Game of Grozapinex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (445, 'Game of Klimunilex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (446, 'Game of Lomdimor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (447, 'Game of Tuptumistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (448, 'Game of Tupfropefex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (449, 'Game of Cipcadupentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (450, 'Game of Recadax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (451, 'Game of Cipwerewor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (452, 'Game of Zeedudax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (453, 'Game of Qwitinilax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (454, 'Game of Endcadackicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (455, 'Game of Endjubinazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (456, 'Game of Lomdimiman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (457, 'Game of Unjubover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (458, 'Game of Dopweror')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (459, 'Game of Winsaponover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (460, 'Game of Wintumicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (461, 'Game of Bartinor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (462, 'Game of Klitanackower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (463, 'Game of Uptanackicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (464, 'Game of Unzapex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (465, 'Game of Ciptumin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (466, 'Game of Endnipor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (467, 'Game of Monzapistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (468, 'Game of Frojubefower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (469, 'Game of Dopdudamantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (470, 'Game of Uppickilar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (471, 'Game of Zeesapar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (472, 'Game of Frodimentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (473, 'Game of Varpebex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (474, 'Game of Lomdimimor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (475, 'Game of Endkilonower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (476, 'Game of Tipdimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (477, 'Game of Klidudazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (478, 'Game of Rehupackazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (479, 'Game of Frowerpentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (480, 'Game of Froweran')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (481, 'Game of Rejubentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (482, 'Game of Rewerover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (483, 'Game of Inbanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (484, 'Game of Truvenantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (485, 'Game of Klirobexazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (486, 'Game of Hapfropackar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (487, 'Game of Reerommazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (488, 'Game of Lomrobin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (489, 'Game of Tipsapefax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (490, 'Game of Parerax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (491, 'Game of Unnipentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (492, 'Game of Froerommistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (493, 'Game of Unkilor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (494, 'Game of Tuphupower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (495, 'Game of Endnipicazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (496, 'Game of Inrobollazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (497, 'Game of Indimewover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (498, 'Game of Qwiwerpommax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (499, 'Game of Grotinollar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (500, 'Game of Adtanupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (501, 'Game of Upbanan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (502, 'Game of Tipsipadower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (503, 'Game of Dopglibefar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (504, 'Game of Unsapilentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (505, 'Game of Vareristor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (506, 'Game of Monmunan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (507, 'Game of Rapvenex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (508, 'Game of Hapwerimar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (509, 'Game of Haptinimar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (510, 'Game of Surkilicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (511, 'Game of Redimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (512, 'Game of Klisipistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (513, 'Game of Dopnipegicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (514, 'Game of Winfropomman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (515, 'Game of Winvenistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (516, 'Game of Klicadar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (517, 'Game of Lomwerpin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (518, 'Game of Hapdimentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (519, 'Game of Emmunopan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (520, 'Game of Insipentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (521, 'Game of Raptinex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (522, 'Game of Unvenex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (523, 'Game of Emmunar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (524, 'Game of Haprobar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (525, 'Game of Enddimicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (526, 'Game of Monzapegistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (527, 'Game of Montinicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (528, 'Game of Zeetanewazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (529, 'Game of Uphupopin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (530, 'Game of Partinimin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (531, 'Game of Endjubover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (532, 'Game of Klipebor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (533, 'Game of Varwerpedgentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (534, 'Game of Parwerpimover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (535, 'Game of Emkilistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (536, 'Game of Emquestentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (537, 'Game of Inrobedgax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (538, 'Game of Cipwerupar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (539, 'Game of Supwerpegicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (540, NULL)
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (541, 'Game of Rapglibower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (542, 'Game of Barsipover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (543, 'Game of Hapbanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (544, 'Game of Surcadazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (545, 'Game of Dopdudan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (546, 'Game of Emsipedgin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (547, 'Game of Tipdimin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (548, 'Game of Lomdimomman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (549, 'Game of Refropedgistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (550, 'Game of Varglibower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (551, 'Game of Supglibinex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (552, 'Game of Endfropewentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (553, 'Game of Barrobicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (554, 'Game of Bardimegicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (555, 'Game of Rewerpantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (556, 'Game of Trujubefistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (557, 'Game of Rapquesticator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (558, 'Game of Lomtinex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (559, 'Game of Froquestentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (560, 'Game of Lomerollentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (561, 'Game of Surbanadistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (562, 'Game of Adkilommazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (563, 'Game of Unquestomman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (564, 'Game of Sursipax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (565, 'Game of Lomdimupistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (566, 'Game of Tuptanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (567, 'Game of Hapnipentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (568, 'Game of Uppickicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (569, 'Game of Frohupegantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (570, 'Game of Untanantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (571, 'Game of Uneran')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (572, 'Game of Tupvenicax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (573, 'Game of Grosapantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (574, 'Game of Frotanegin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (575, 'Game of Dopfropopantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (576, 'Game of Injubepentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (577, 'Game of Supwerpamex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (578, 'Game of Unwerpor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (579, 'Game of Grosapan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (580, 'Game of Klisapplar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (581, 'Game of Survenamicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (582, 'Game of Trumunex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (583, 'Game of Thrupebazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (584, 'Game of Klimunan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (585, 'Game of Dopdimonower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (586, 'Game of Grofropimicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (587, 'Game of Supwericator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (588, 'Game of Monfropegentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (589, 'Game of Inkilar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (590, 'Game of Varvenommicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (591, 'Game of Klinipupex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (592, 'Game of Thrujubantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (593, 'Game of Unwerupax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (594, 'Game of Zeecadin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (595, 'Game of Rapdudan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (596, 'Game of Unwerpantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (597, 'Game of Zeenipex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (598, 'Game of Klinipar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (599, 'Game of Monsapistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (600, 'Game of Frorobantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (601, 'Game of Monjubor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (602, 'Game of Zeedimor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (603, 'Game of Thrudimin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (604, 'Game of Trufropar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (605, 'Game of Endtinanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (606, 'Game of Ciphupollicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (607, 'Game of Raphupackax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (608, 'Game of Rekilollistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (609, 'Game of Ciptanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (610, 'Game of Monweronantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (611, 'Game of Varbanicantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (612, 'Game of Trusapewin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (613, 'Game of Dopbanonazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (614, 'Game of Hapquestower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (615, 'Game of Uptanexex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (616, 'Game of Parwerpor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (617, 'Game of Endhupicower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (618, 'Game of Tipvenackax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (619, 'Game of Klidimentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (620, 'Game of Klinipeficator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (621, 'Game of Endtanilicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (622, 'Game of Doppebentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (623, 'Game of Parwerin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (624, 'Game of Hapquestegin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (625, 'Game of Bardudegazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (626, 'Game of Thruhupollax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (627, 'Game of Thrutumilentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (628, 'Game of Uphupexax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (629, 'Game of Retanewax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (630, 'Game of Varkilommantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (631, 'Game of Adnipin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (632, 'Game of Inkilover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (633, 'Game of Tipfropantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (634, 'Game of Monfropin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (635, 'Game of Rapwerewar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (636, 'Game of Indimopistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (637, 'Game of Supquesticin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (638, 'Game of Requestupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (639, 'Game of Untanar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (640, 'Game of Barnipin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (641, 'Game of Monkilupicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (642, 'Game of Hapglibower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (643, 'Game of Varnipan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (644, 'Game of Supzapantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (645, 'Game of Adpickex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (646, 'Game of Rapbanilor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (647, 'Game of Varwerupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (648, 'Game of Surrobedgan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (649, 'Game of Endpebover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (650, 'Game of Tipglibedgantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (651, 'Game of Emjubollor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (652, 'Game of Klierover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (653, 'Game of Cipweredgar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (654, 'Game of Varquestepantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (655, 'Game of Ciptananicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (656, 'Game of Qwijubefin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (657, 'Game of Wineran')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (658, 'Game of Upsapover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (659, 'Game of Thruzapistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (660, 'Game of Tuptinollicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (661, 'Game of Supjubar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (662, 'Game of Frovenepower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (663, 'Game of Thrunipinin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (664, 'Game of Endsipanor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (665, 'Game of Emzapistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (666, 'Game of Thrutinepazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (667, 'Game of Tupwerpollax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (668, 'Game of Qwisapower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (669, 'Game of Lompickimazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (670, 'Game of Zeepebopicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (671, 'Game of Barkilentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (672, 'Game of Varbanilower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (673, 'Game of Tupnipadex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (674, 'Game of Haprobin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (675, 'Game of Suptumicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (676, 'Game of Haptuman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (677, 'Game of Tiphupommicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (678, 'Game of Uncadupentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (679, 'Game of Zeejubin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (680, 'Game of Upquesticator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (681, 'Game of Emdimanantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (682, 'Game of Dopbanantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (683, 'Game of Uppebexazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (684, 'Game of Zeepickimazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (685, 'Game of Partanan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (686, 'Game of Adfropantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (687, 'Game of Qwifropazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (688, 'Game of Varhupedistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (689, 'Game of Advenin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (690, 'Game of Tiphupupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (691, 'Game of Adsiponazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (692, 'Game of Grotanantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (693, 'Game of Doptanax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (694, 'Game of Suptanex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (695, 'Game of Lomsipentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (696, 'Game of Uppebex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (697, 'Game of Hapmunupan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (698, 'Game of Hapniponar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (699, 'Game of Inrobadistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (700, 'Game of Barhupor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (701, 'Game of Suptanar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (702, 'Game of Vardudollistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (703, 'Game of Enddudupistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (704, 'Game of Varnipackex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (705, 'Game of Emtumefower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (706, 'Game of Klipickinax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (707, 'Game of Insapadex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (708, 'Game of Unsipax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (709, 'Game of Klidudin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (710, 'Game of Wintumilicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (711, 'Game of Thrujubex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (712, 'Game of Wintumackex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (713, 'Game of Truweran')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (714, 'Game of Adglibackantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (715, 'Game of Tupwerpantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (716, 'Game of Intanin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (717, 'Game of Emnipamover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (718, 'Game of Rappebower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (719, 'Game of Tuppebin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (720, 'Game of Grobanedentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (721, 'Game of Tupquestilantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (722, 'Game of Cipsapepor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (723, 'Game of Barpickilantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (724, 'Game of Innipar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (725, 'Game of Parhupinover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (726, 'Game of Thrunipentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (727, 'Game of Zeekilantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (728, 'Game of Inrobar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (729, 'Game of Monhupover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (730, 'Game of Rapsipepan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (731, 'Game of Emsapimistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (732, 'Game of Zeefropan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (733, 'Game of Cipquestonower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (734, 'Game of Barhupinicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (735, 'Game of Thrupickefor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (736, 'Game of Haprobilan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (737, 'Game of Frorobollistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (738, 'Game of Tuptinex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (739, 'Game of Hapkilin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (740, 'Game of Tupkilollax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (741, 'Game of Gropickentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (742, 'Game of Lomdimollicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (743, 'Game of Thrujubower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (744, 'Game of Unfropazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (745, 'Game of Rapquestower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (746, 'Game of Parnipantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (747, 'Game of Rapsipommistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (748, 'Game of Trutanan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (749, 'Game of Supnipower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (750, 'Game of Parvenin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (751, 'Game of Tipquestommentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (752, 'Game of Lomnipewin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (753, 'Game of Tipdudonicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (754, 'Game of Surwerex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (755, 'Game of Surwerpicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (756, 'Game of Monwerpor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (757, 'Game of Tipsipegin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (758, 'Game of Thrudimanantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (759, 'Game of Undudicantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (760, 'Game of Supmunentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (761, 'Game of Zeejubinazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (762, 'Game of Barkilin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (763, 'Game of Cippickower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (764, 'Game of Remunex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (765, 'Game of Surerexax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (766, 'Game of Aderilin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (767, 'Game of Dopsipommantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (768, 'Game of Qwihupover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (769, 'Game of Endquestaman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (770, 'Game of Emkilamicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (771, 'Game of Windudan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (772, 'Game of Barwerpexantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (773, 'Game of Surglibicazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (774, 'Game of Hapdudefex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (775, 'Game of Winzapex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (776, 'Game of Qwijubentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (777, 'Game of Untanower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (778, 'Game of Revenicantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (779, 'Game of Lomjubewan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (780, 'Game of Supsapistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (781, 'Game of Thrupickewar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (782, 'Game of Dopwerex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (783, 'Game of Adkilor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (784, 'Game of Rekilepar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (785, 'Game of Truvenamazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (786, 'Game of Untumin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (787, 'Game of Froquestax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (788, 'Game of Winsapupan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (789, 'Game of Endzapax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (790, 'Game of Zeenipplin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (791, 'Game of Remunedgistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (792, 'Game of Winpickazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (793, 'Game of Ciptuman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (794, 'Game of Resipistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (795, 'Game of Zeetumex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (796, 'Game of Zeebanicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (797, 'Game of Admunower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (798, 'Game of Vartinazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (799, 'Game of Trutumepin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (800, 'Game of Trurobex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (801, 'Game of Suptumollazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (802, 'Game of Zeetumistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (803, 'Game of Hapsapackicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (804, 'Game of Fropebex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (805, 'Game of Kliwericator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (806, 'Game of Hapwerpax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (807, 'Game of Inquestopower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (808, 'Game of Barcadackar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (809, 'Game of Groglibedover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (810, 'Game of Barmunupover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (811, 'Game of Untumower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (812, 'Game of Trusipewover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (813, 'Game of Rapjubadin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (814, 'Game of Tupglibollentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (815, 'Game of Frodimopar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (816, 'Game of Adjubimicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (817, 'Game of Frorobollin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (818, 'Game of Frokilax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (819, 'Game of Qwierupax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (820, 'Game of Qwivenican')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (821, 'Game of Qwipickantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (822, 'Game of Suptinentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (823, 'Game of Parnipin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (824, 'Game of Dopzapan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (825, 'Game of Aderistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (826, 'Game of Endmunadar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (827, 'Game of Groglibicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (828, 'Game of Trukilackicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (829, 'Game of Gropebazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (830, 'Game of Hapzapewax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (831, 'Game of Frokilower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (832, 'Game of Thrukilentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (833, 'Game of Supjubazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (834, 'Game of Bareredax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (835, 'Game of Klisapazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (836, 'Game of Partumadistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (837, 'Game of Cipsapedgin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (838, 'Game of Empebexower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (839, 'Game of Dopquestaquar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (840, 'Game of Lomsipin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (841, 'Game of Parfropupistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (842, 'Game of Barnipaman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (843, 'Game of Doppickaquantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (844, 'Game of Truerplantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (845, 'Game of Barpebadantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (846, 'Game of Thrusapiman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (847, 'Game of Fromunollar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (848, 'Game of Dopwerpegex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (849, 'Game of Barwerpollantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (850, 'Game of Dopwerex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (851, 'Game of Grozapedgicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (852, 'Game of Haprobicentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (853, 'Game of Emquestamicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (854, 'Game of Winwerefex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (855, 'Game of Enderazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (856, 'Game of Lommunaman')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (857, 'Game of Qwijubedgazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (858, 'Game of Qwikilopor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (859, 'Game of Cipquestanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (860, 'Game of Thruvenefor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (861, 'Game of Unsipax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (862, 'Game of Lomdudador')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (863, 'Game of Trurobackover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (864, 'Game of Grofropanor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (865, 'Game of Supsapar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (866, 'Game of Dopkilar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (867, 'Game of Ciptinantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (868, 'Game of Barsapefower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (869, 'Game of Hapwerpewan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (870, 'Game of Parfropistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (871, 'Game of Adhupepin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (872, 'Game of Adzapexistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (873, 'Game of Inpickower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (874, 'Game of Rapweredan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (875, 'Game of Hapmunor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (876, 'Game of Doppickar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (877, 'Game of Tupglibegex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (878, 'Game of Qwivenaquower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (879, 'Game of Upcadin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (880, 'Game of Trutanax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (881, 'Game of Emwerepazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (882, 'Game of Doprobor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (883, 'Game of Adpebilower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (884, 'Game of Zeevenewentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (885, 'Game of Zeetinin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (886, 'Game of Mondudinex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (887, 'Game of Montinommex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (888, 'Game of Reeror')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (889, 'Game of Emhupanistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (890, 'Game of Klitinan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (891, 'Game of Unzapewentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (892, 'Game of Ciperistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (893, 'Game of Haperamower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (894, 'Game of Thruglibamover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (895, 'Game of Raptumin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (896, 'Game of Thrutanentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (897, 'Game of Klicadepazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (898, 'Game of Tipsipinax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (899, 'Game of Surfropefantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (900, 'Game of Klisipin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (901, 'Game of Remunantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (902, 'Game of Ciperupentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (903, 'Game of Unhupor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (904, 'Game of Surtanantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (905, 'Game of Cipdudedgantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (906, 'Game of Rapbanentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (907, 'Game of Partinimistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (908, 'Game of Repebollor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (909, 'Game of Rapmunupazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (910, 'Game of Mondimewan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (911, 'Game of Qwipebplicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (912, 'Game of Ciprobilex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (913, 'Game of Survenin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (914, 'Game of Parfropedan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (915, 'Game of Dopnipplentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (916, 'Game of Refropewax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (917, 'Game of Trupebinentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (918, 'Game of Winrobilicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (919, 'Game of Barkililan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (920, 'Game of Frotinax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (921, 'Game of Trusipedgicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (922, 'Game of Monvenistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (923, 'Game of Supzapicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (924, 'Game of Surdimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (925, 'Game of Frotanollex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (926, 'Game of Surdudimower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (927, 'Game of Frocadar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (928, 'Game of Barpebin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (929, 'Game of Monsipax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (930, 'Game of Lomtanicover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (931, 'Game of Adglibefower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (932, 'Game of Thrunipicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (933, 'Game of Qwipebazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (934, 'Game of Dophupantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (935, 'Game of Gronipistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (936, 'Game of Surhupilentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (937, 'Game of Indiminan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (938, 'Game of Endkilicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (939, 'Game of Truwerefower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (940, 'Game of Parfropanex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (941, 'Game of Adrobefex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (942, 'Game of Varglibor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (943, 'Game of Dopjubar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (944, 'Game of Qwikilan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (945, 'Game of Unwerinax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (946, 'Game of Reglibilex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (947, 'Game of Winpickax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (948, 'Game of Lommunazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (949, 'Game of Vartinin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (950, 'Game of Cipdudadantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (951, 'Game of Superantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (952, 'Game of Thruwerpan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (953, 'Game of Doprobollantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (954, 'Game of Truhupefan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (955, 'Game of Frohupommistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (956, 'Game of Monsapinin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (957, NULL)
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (958, 'Game of Trurobollicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (959, 'Game of Emquestax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (960, 'Game of Surtinor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (961, 'Game of Vargliban')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (962, 'Game of Barhupan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (963, 'Game of Doptanazz')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (964, 'Game of Winrobantor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (965, 'Game of Trubanover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (966, 'Game of Adfropistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (967, 'Game of Klisipegan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (968, 'Game of Partinover')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (969, 'Game of Indudin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (970, 'Game of Cipmunefan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (971, 'Game of Tupwerollor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (972, 'Game of Tipsapupin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (973, 'Game of Partumedgentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (974, 'Game of Endtumicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (975, 'Game of Truzapplar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (976, 'Game of Intanan')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (977, 'Game of Zeevenistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (978, 'Game of Dopsapollicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (979, 'Game of Surquestanicator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (980, 'Game of Varbanilar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (981, 'Game of Frozapedower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (982, 'Game of Trukilaquower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (983, 'Game of Thruquestadax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (984, 'Game of Dopjubegistor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (985, 'Game of Emglibax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (986, 'Game of Adzapar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (987, 'Game of Winhupin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (988, 'Game of Inquestonentor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (989, 'Game of Frotanexax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (990, 'Game of Unwerar')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (991, 'Game of Barglibin')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (992, 'Game of Monweristor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (993, 'Game of Tupdimimex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (994, 'Game of Varquestedax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (995, 'Game of Winmunollor')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (996, 'Game of Varsipex')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (997, 'Game of Thrupickedower')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (998, 'Game of Emwerax')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (999, 'Game of Monericator')
INSERT INTO [dbo].[Videospiele] ([ID], [Name]) VALUES (1000, 'Game of Unsapazz')
SET IDENTITY_INSERT [dbo].[Videospiele] OFF
SET IDENTITY_INSERT [dbo].[Highscores] ON
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1, 727, 727, 758577969, '19710811 20:02:23.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2, 818, 818, 2002514178, '19680823 02:43:13.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3, 769, 769, 1675563779, '19670619 03:08:02.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4, 559, 559, 805094670, '19870524 22:11:24.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5, 207, 207, 699328205, '19780628 13:23:51.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6, 4, 4, 1612865305, '20040122 05:52:55.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7, 907, 907, 327478103, '19610117 22:33:44.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8, 443, 443, 1161860433, '20050412 16:47:54.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9, 978, 978, 941191225, '19851014 12:41:19.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (10, 274, 274, 1888059364, '19590618 10:01:49.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (11, 292, 292, 1305154149, '20060827 01:15:02.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (12, 468, 468, 538033552, '19730526 00:08:11.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (13, 633, 633, 2140997306, '19561105 08:52:38.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (14, 470, 470, 1624908051, '19561122 22:09:13.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (15, 983, 983, 670600707, '19790706 03:31:02.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (16, 31, 31, 1390128226, '19610531 17:42:39.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (17, 863, 863, 1614417824, '20060110 04:37:11.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (18, 996, 996, 1814874233, '19580331 04:03:41.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (19, 678, 678, 1627193684, '19970908 08:26:52.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (20, 315, 315, 1993828585, '19661014 03:03:54.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (21, 817, 817, 1941558272, '19790621 01:57:36.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (22, 849, 849, 176740239, '19570210 04:58:38.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (23, 992, 992, 1598593144, '19990111 08:34:23.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (24, 33, 33, 689530861, '19951111 21:29:52.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (25, 700, 700, 1546087475, '19630613 22:31:23.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (26, 527, 527, 1888294590, '19931026 13:19:31.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (27, 935, 935, 214637909, '19651022 03:02:47.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (28, 688, 688, 1928013423, '19960317 13:17:49.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (29, 547, 547, 1287055675, '20040112 08:00:47.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (30, 82, 82, 854128486, '19581208 13:06:19.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (31, 188, 188, 1790211801, '19541026 03:18:26.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (32, 454, 454, 459801310, '19950515 05:56:10.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (33, 298, 298, 1165331949, '19691205 14:41:02.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (34, 989, 989, 1402214770, '19590407 20:19:37.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (35, 643, 643, 1438861656, '19740516 14:51:38.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (36, 763, 763, 656606580, '20061117 07:16:44.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (37, 16, 16, 2145816653, '19620620 19:01:31.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (38, 382, 382, 356493676, '19960513 13:42:41.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (39, 344, 344, 1877613727, '19631220 09:01:01.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (40, 958, 958, 438240258, '19961020 16:53:55.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (41, 506, 506, 1178978005, '19711125 01:44:49.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (42, 716, 716, 959347524, '19581012 23:26:51.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (43, 119, 119, 280700440, '20061118 01:22:55.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (44, 10, 10, 1549887033, '19990227 10:42:45.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (45, 908, 908, 785814492, '19821117 19:43:35.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (46, 795, 795, 328228237, '19631121 12:41:00.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (47, 338, 338, 836476983, '19550208 16:04:01.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (48, 458, 458, 6872229, '19820615 08:34:10.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (49, 147, 147, 2123504164, '19871210 03:24:52.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (50, 222, 222, 1640294754, '20010707 23:31:08.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (51, 411, 411, 933327390, '20070702 14:47:59.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (52, 719, 719, 1250009568, '20060621 03:59:02.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (53, 620, 620, 827494829, '19661213 10:09:32.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (54, 488, 488, 1151460013, '19801111 03:02:25.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (55, 195, 195, 1536695702, '19631014 07:01:04.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (56, 879, 879, 42097888, '19670702 15:03:45.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (57, 826, 826, 867082094, '19770811 18:08:50.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (58, 736, 736, 1066603193, '19790807 05:38:09.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (59, 859, 859, 1714996551, '19761211 23:40:01.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (60, 680, 680, 952480090, '19920901 00:04:19.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (61, 625, 625, 198529027, '19910403 02:50:07.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (62, 219, 219, 683862542, '19721101 09:15:54.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (63, 896, 896, 542480589, '19540402 08:47:07.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (64, 897, 897, 1585891228, '19791107 23:35:00.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (65, 87, 87, 1185333846, '19570824 06:43:23.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (66, 839, 839, 334569219, '19640414 19:18:51.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (67, 171, 171, 688987715, '19560621 09:27:08.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (68, 645, 645, 615788154, '20050730 12:33:01.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (69, 827, 827, 1980994553, '19900709 07:17:34.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (70, 220, 220, 1640995533, '19800817 20:14:17.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (71, 1000, 1000, 911124806, '20061210 22:41:08.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (72, 482, 482, 28738078, '19620830 14:54:29.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (73, 653, 653, 1095198106, '20020410 19:02:40.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (74, 720, 720, 1652114485, '19531119 15:32:57.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (75, 810, 810, 1874315142, '20021119 01:19:05.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (76, 101, 101, 1618396342, '19730909 02:30:44.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (77, 730, 730, 138563265, '19580325 03:35:42.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (78, 52, 52, 627977657, '20071113 21:51:37.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (79, 126, 126, 683474931, '19651226 01:46:17.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (80, 906, 906, 1885535069, '20070723 09:50:22.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (81, 190, 190, 1700472765, '19910918 21:15:30.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (82, 477, 477, 430949724, '19910508 18:28:36.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (83, 541, 541, 600094123, '19610409 09:52:57.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (84, 326, 326, 2098478176, '19550708 08:29:39.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (85, 672, 672, 836708433, '19590607 22:18:20.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (86, 469, 469, 200561402, '19560505 23:19:23.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (87, 834, 834, 1380669748, '19810602 19:46:38.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (88, 75, 75, 311362098, '19970123 11:38:37.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (89, 794, 794, 77004773, '20030624 13:18:33.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (90, 765, 765, 709436668, '19591115 23:47:53.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (91, 938, 938, 946269808, '19820408 13:07:53.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (92, 295, 295, 1611273209, '19901113 13:23:21.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (93, 523, 523, 1013315617, '19720602 14:02:40.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (94, 664, 664, 2033478524, '19790419 08:56:41.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (95, 333, 333, 2089985907, '19580721 14:03:47.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (96, 287, 287, 1082306763, '20070122 16:28:54.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (97, 821, 821, 966711601, '19570713 14:39:44.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (98, 223, 223, 1174913742, '19800112 01:47:54.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (99, 187, 187, 921031011, '19940707 03:59:22.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (100, 69, 69, 1822684800, '19710806 00:24:43.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (101, 61, 61, 614971403, '19600602 03:13:52.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (102, 694, 694, 1452728038, '19570711 03:30:59.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (103, 631, 631, 513002154, '19991207 01:16:36.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (104, 928, 928, 2016136429, '19600424 07:10:34.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (105, 50, 50, 47081712, '20020728 00:49:59.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (106, 929, 929, 750579763, '19971102 23:53:29.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (107, 67, 67, 1573684799, '19570313 08:56:22.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (108, 901, 901, 587069696, '19660124 18:36:35.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (109, 679, 679, 724176076, '19851223 01:43:19.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (110, 94, 94, 202553052, '19980203 04:30:19.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (111, 150, 150, 749266200, '19620410 11:28:02.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (112, 107, 107, 124108956, '19770927 20:17:13.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (113, 610, 610, 1747225506, '19660813 03:51:52.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (114, 954, 954, 384598962, '19770521 13:49:38.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (115, 491, 491, 707813045, '19531215 02:48:49.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (116, 149, 149, 2074482720, '20071125 08:21:30.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (117, 19, 19, 507912546, '19640725 23:22:57.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (118, 570, 570, 1810209513, '20060925 00:17:28.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (119, 226, 226, 482856414, '19730603 01:16:40.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (120, 619, 619, 1338157734, '19540421 07:23:59.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (121, 6, 6, 378802708, '19901111 23:32:13.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (122, 361, 361, 1864021456, '19670528 21:48:31.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (123, 851, 851, 2036117970, '19550206 23:14:27.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (124, 63, 63, 456938375, '19830825 07:29:41.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (125, 282, 282, 1613921088, '20060511 07:06:23.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (126, 705, 705, 2147179609, '19690128 09:17:46.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (127, 959, 959, 1389683644, '19980329 00:51:49.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (128, 34, 34, 1633993807, '19751224 10:05:59.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (129, 388, 388, 189653508, '20030501 01:29:09.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (130, 93, 93, 554958985, '20031027 08:50:10.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (131, 281, 281, 1206822221, '19690227 11:51:00.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (132, 507, 507, 971592739, '19860313 01:47:48.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (133, 532, 532, 634686615, '19660510 17:52:15.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (134, 58, 58, 1730890834, '20020522 01:21:34.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (135, 131, 131, 46170728, '20000221 06:36:30.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (136, 497, 497, 1213457556, '19870311 17:44:30.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (137, 847, 847, 1700704256, '19990601 17:12:00.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (138, 614, 614, 1362600334, '19531217 02:42:22.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (139, 105, 105, 508741198, '19601210 07:39:39.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (140, 743, 743, 937139138, '19690803 22:24:50.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (141, 402, 402, 860369450, '20070222 14:23:01.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (142, 934, 934, 1961122740, '19680510 01:10:03.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (143, 131, 131, 660319768, '19640202 09:55:17.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (144, 25, 25, 1641148285, '19580522 08:48:13.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (145, 616, 616, 2140551036, '20050806 12:19:50.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (146, 831, 831, 442853834, '19570712 16:50:40.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (147, 686, 686, 99776148, '19770403 09:31:29.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (148, 118, 118, 1252606196, '20030112 00:13:01.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (149, 173, 173, 684232068, '19780506 06:07:52.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (150, 185, 185, 862607229, '19580826 05:42:16.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (151, 113, 113, 797820533, '19950630 17:05:56.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (152, 251, 251, 1610387637, '19581018 14:22:16.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (153, 998, 998, 1378338691, '19590812 00:31:14.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (154, 569, 569, 1855595507, '19930318 20:35:22.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (155, 64, 64, 1623770486, '19880923 00:52:29.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (156, 264, 264, 387768637, '20010104 05:25:20.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (157, 843, 843, 1290888344, '19550605 04:16:32.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (158, 154, 154, 1664675121, '19690414 17:46:54.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (159, 646, 646, 298804667, '19611214 00:04:11.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (160, 517, 517, 1273861864, '19860630 15:32:13.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (161, 971, 971, 53850858, '20070806 23:01:39.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (162, 78, 78, 2101039862, '19890319 22:50:23.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (163, 513, 513, 1562033304, '19700925 17:07:25.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (164, 156, 156, 1104077030, '19900225 16:53:09.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (165, 814, 814, 1454121452, '19811207 16:39:19.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (166, 35, 35, 1590790676, '19840128 09:40:14.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (167, 576, 576, 985433354, '19640521 02:24:58.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (168, 553, 553, 535507908, '19720323 02:30:17.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (169, 673, 673, 1809505610, '19850330 07:13:08.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (170, 995, 995, 382980690, '19741005 09:04:18.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (171, 302, 302, 116187079, '19610626 15:09:30.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (172, 65, 65, 1324397087, '19630810 20:40:34.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (173, 466, 466, 126737482, '19981016 16:37:48.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (174, 484, 484, 1053752141, '19561031 02:51:49.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (175, 217, 217, 2090402413, '19550226 17:00:58.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (176, 72, 72, 668574289, '19750705 22:22:10.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (177, 231, 231, 2124101144, '19560426 11:53:13.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (178, 151, 151, 571070420, '20040916 14:26:14.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (179, 447, 447, 357118192, '19860117 19:09:50.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (180, 452, 452, 1080191961, '20011030 14:15:43.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (181, 20, 20, 1034940127, '19991026 23:46:16.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (182, 389, 389, 623681807, '20030317 00:38:48.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (183, 21, 21, 863355476, '20050819 03:58:06.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (184, 203, 203, 577405041, '19970905 19:46:52.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (185, 914, 914, 833606281, '19610429 15:44:13.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (186, 30, 30, 1346132483, '19630513 21:28:44.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (187, 510, 510, 510228225, '19790803 01:16:33.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (188, 963, 963, 2037059331, '19810220 21:16:52.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (189, 994, 994, 1753293467, '19660830 00:29:04.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (190, 17, 17, 1668814699, '20070216 01:11:09.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (191, 654, 654, 2030073495, '19841007 13:27:58.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (192, 279, 279, 1223933427, '19830217 23:25:06.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (193, 968, 968, 1924811758, '20000103 02:38:11.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (194, 588, 588, 1485387415, '19820611 16:07:25.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (195, 773, 773, 1332594385, '19691227 23:23:11.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (196, 325, 325, 99689278, '19701206 15:32:38.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (197, 421, 421, 1459930983, '20050814 08:02:38.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (198, 975, 975, 768258369, '19811206 17:02:08.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (199, 886, 886, 1434390449, '19840613 16:08:54.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (200, 973, 973, 1202729489, '19740711 02:39:36.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (201, 256, 256, 450094043, '20010220 14:25:42.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (202, 133, 133, 2102944323, '19580112 07:01:12.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (203, 898, 898, 2119544738, '19701015 16:59:52.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (204, 179, 179, 354536749, '19560801 21:03:33.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (205, 883, 883, 1621912131, '20050228 14:32:46.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (206, 545, 545, 260095435, '19841120 20:25:22.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (207, 786, 786, 526762454, '19680101 21:44:27.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (208, 515, 515, 1593169314, '19551012 21:39:25.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (209, 353, 353, 535190113, '19910122 03:34:24.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (210, 23, 23, 483536915, '19660322 02:30:19.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (211, 34, 34, 2033548154, '19970910 17:32:07.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (212, 692, 692, 1344253640, '19580917 13:50:18.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (213, 122, 122, 1000053599, '19910327 22:37:03.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (214, 55, 55, 3891300, '19680213 09:48:28.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (215, 136, 136, 449648564, '19940903 15:45:56.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (216, 582, 582, 151466301, '19570523 22:22:50.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (217, 262, 262, 1632678184, '19910731 18:52:16.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (218, 311, 311, 181599037, '19810118 21:20:32.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (219, 243, 243, 120721686, '19811030 01:08:55.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (220, 785, 785, 1237608610, '19710728 19:10:35.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (221, 134, 134, 411133069, '19570628 08:23:41.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (222, 613, 613, 897607828, '19910330 05:08:06.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (223, 560, 560, 694868679, '19580726 02:01:12.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (224, 656, 656, 953267827, '19860211 06:01:59.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (225, 342, 342, 230683694, '19971227 19:36:19.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (226, 24, 24, 1877743367, '19860508 15:44:23.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (227, 98, 98, 707082661, '19710807 18:02:23.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (228, 878, 878, 310512087, '19690508 00:30:23.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (229, 711, 711, 779886513, '19941103 03:28:38.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (230, 893, 893, 2042715243, '19920322 01:28:20.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (231, 652, 652, 1046980847, '19771120 14:19:31.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (232, 201, 201, 1644172996, '19820520 18:51:04.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (233, 265, 265, 1518697205, '19730405 22:17:19.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (234, 475, 475, 530236754, '19640710 16:30:14.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (235, 196, 196, 253999566, '19530909 23:50:01.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (236, 888, 888, 748957429, '19941015 16:45:04.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (237, 492, 492, 1224007175, '19850602 07:38:56.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (238, 638, 638, 1387189235, '20020118 06:54:33.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (239, 320, 320, 1743382131, '20000707 05:14:06.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (240, 369, 369, 1889968338, '19840607 19:18:51.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (241, 245, 245, 1631946946, '20030511 23:44:17.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (242, 18, 18, 1579195320, '19761022 03:37:07.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (243, 611, 611, 109495466, '19980129 17:42:28.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (244, 2, 2, 643971588, '19530610 21:58:45.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (245, 984, 984, 817451102, '19620608 07:39:01.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (246, 962, 962, 1171851057, '19790121 23:37:40.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (247, 158, 158, 1359782011, '19991123 00:48:02.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (248, 195, 195, 272302541, '19841120 16:49:43.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (249, 92, 92, 1867915459, '19951008 00:21:29.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (250, 191, 191, 479591898, '19650807 01:00:20.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (251, 124, 124, 1475589695, '19870507 20:40:21.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (252, 111, 111, 1071696316, '19770727 10:42:05.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (253, 733, 733, 1924120000, '19530207 15:53:13.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (254, 102, 102, 103748863, '19651117 20:58:19.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (255, 840, 840, 86774088, '19700113 18:15:55.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (256, 166, 166, 891766829, '19621125 09:17:35.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (257, 573, 573, 982123673, '20070619 05:00:00.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (258, 242, 242, 1956402322, '19920902 10:57:52.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (259, 838, 838, 2086465423, '19660805 01:27:13.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (260, 860, 860, 36840303, '19711024 22:48:22.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (261, 448, 448, 1734930403, '19660417 02:22:58.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (262, 45, 45, 1418651838, '20060825 21:14:04.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (263, 804, 804, 1044388111, '19681208 18:10:47.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (264, 461, 461, 1144981561, '20061101 02:06:03.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (265, 341, 341, 1039434633, '19960430 12:10:47.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (266, 779, 779, 2022253358, '19680423 22:41:02.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (267, 427, 427, 1858482057, '19930612 15:32:58.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (268, 648, 648, 199770763, '19790918 06:06:49.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (269, 71, 71, 827724572, '19670606 09:58:27.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (270, 609, 609, 1819777983, '20071118 23:00:52.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (271, 90, 90, 2053249722, '19791221 14:43:54.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (272, 624, 624, 654704697, '19970712 11:57:43.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (273, 991, 991, 2006036138, '19880713 16:06:26.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (274, 874, 874, 2101040968, '20050523 05:50:03.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (275, 540, 540, 12175684, '19760317 19:26:18.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (276, 138, 138, 794550117, '19880905 04:46:33.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (277, 244, 244, 714431754, '20010227 11:25:37.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (278, 558, 558, 299532425, '19580215 04:02:27.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (279, 169, 169, 1321846892, '19760905 22:22:57.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (280, 380, 380, 387500352, '19711207 19:58:39.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (281, 867, 867, 698259943, '19940615 14:56:21.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (282, 84, 84, 1644089146, '19940916 01:12:39.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (283, 787, 787, 854225145, '19810731 00:08:53.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (284, 520, 520, 1351450389, '19820330 02:28:17.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (285, 830, 830, 2040408900, '19571116 04:47:58.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (286, 542, 542, 467319032, '19530427 03:37:26.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (287, 130, 130, 1449206149, '19820413 02:57:51.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (288, 163, 163, 336569726, '19600520 01:19:00.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (289, 635, 635, 207795094, '20020626 22:14:19.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (290, 554, 554, 153079542, '19981015 14:32:25.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (291, 20, 20, 747690221, '19950428 11:45:04.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (292, 250, 250, 708043777, '20000928 20:41:03.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (293, 801, 801, 1963565871, '19880616 05:27:19.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (294, 264, 264, 12139527, '19810914 06:25:43.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (295, 922, 922, 1827844611, '19710222 16:55:52.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (296, 47, 47, 282915029, '20040915 02:30:12.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (297, 193, 193, 896383157, '19601114 09:26:20.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (298, 178, 178, 931698963, '19990330 15:36:24.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (299, 662, 662, 745231600, '19650209 09:47:57.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (300, 206, 206, 2089971301, '20020213 08:57:58.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (301, 535, 535, 168306306, '19930810 08:04:41.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (302, 187, 187, 1773702271, '19730307 18:41:13.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (303, 774, 774, 1490060097, '19700618 06:51:16.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (304, 483, 483, 702791173, '19951119 01:20:36.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (305, 254, 254, 785960343, '19930816 10:16:25.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (306, 439, 439, 1472142758, '19971025 08:42:38.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (307, 120, 120, 1523393926, '19970112 18:35:39.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (308, 260, 260, 829024172, '19550917 10:03:09.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (309, 563, 563, 1324514375, '19970901 01:32:01.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (310, 703, 703, 1075611581, '19890509 13:29:21.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (311, 641, 641, 153056019, '19690926 21:51:58.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (312, 16, 16, 2104170472, '20020505 00:57:33.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (313, 148, 148, 947098157, '19681228 12:34:54.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (314, 48, 48, 28235203, '20020828 05:28:34.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (315, 189, 189, 1451075744, '19850510 07:52:01.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (316, 123, 123, 566179189, '19790801 01:10:19.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (317, 121, 121, 833323794, '19780126 21:05:10.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (318, 284, 284, 985484063, '19940910 15:42:29.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (319, 103, 103, 1999619062, '20011216 21:18:05.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (320, 800, 800, 1816473324, '19960105 08:33:21.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (321, 232, 232, 168854090, '19940110 19:43:10.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (322, 156, 156, 527994823, '19860124 14:13:57.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (323, 13, 13, 788777538, '19850323 07:52:30.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (324, 446, 446, 500207410, '19760820 19:26:02.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (325, 92, 92, 2078032902, '19650724 11:15:48.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (326, 255, 255, 961477075, '19870323 18:02:51.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (327, 824, 824, 474526435, '19620127 06:30:23.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (328, 531, 531, 1250545882, '19591031 09:40:42.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (329, 952, 952, 1433564531, '19870401 12:54:11.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (330, 184, 184, 1056289810, '19790702 16:56:05.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (331, 946, 946, 915438375, '19801022 19:20:13.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (332, 852, 852, 1610083015, '19541201 19:49:13.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (333, 64, 64, 1578057554, '20010104 18:14:29.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (334, 467, 467, 175218643, '19820723 13:24:59.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (335, 845, 845, 395709566, '19860428 11:53:57.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (336, 357, 357, 2118734250, '19740410 20:15:07.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (337, 552, 552, 1889791520, '19770331 18:21:23.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (338, 305, 305, 1260016032, '19930910 22:48:17.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (339, 419, 419, 2119658557, '19960812 16:11:51.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (340, 391, 391, 2145821219, '19680121 20:05:19.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (341, 422, 422, 1157185510, '19640413 09:01:46.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (342, 663, 663, 804093659, '19790728 16:54:42.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (343, 601, 601, 1335611872, '19700917 23:46:59.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (344, 932, 932, 1813129844, '19660218 08:44:57.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (345, 913, 913, 1793852007, '19820119 16:40:26.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (346, 300, 300, 350280636, '20001223 10:47:31.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (347, 681, 681, 1277892176, '19841002 08:06:08.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (348, 343, 343, 472289469, '19931018 23:58:44.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (349, 334, 334, 1759069640, '20040505 22:33:41.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (350, 320, 320, 1385392340, '19990724 15:45:33.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (351, 175, 175, 172895771, '19790821 05:25:02.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (352, 910, 910, 1954261862, '19740305 17:43:50.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (353, 284, 284, 1421148063, '20050412 18:18:19.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (354, 17, 17, 972578845, '19770204 01:14:36.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (355, 950, 950, 1886276974, '19610203 13:14:47.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (356, 271, 271, 1489080522, '19600717 17:50:43.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (357, 215, 215, 1492062843, '19951215 10:48:43.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (358, 328, 328, 1823380135, '20011027 11:25:14.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (359, 189, 189, 2048588714, '19830428 14:04:48.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (360, 217, 217, 348223364, '19590527 16:13:34.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (361, 145, 145, 1541634978, '19880929 02:12:15.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (362, 589, 589, 93630499, '19900316 08:54:56.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (363, 909, 909, 488285605, '19760616 21:08:58.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (364, 122, 122, 174980167, '19710303 08:35:55.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (365, 757, 757, 1861887072, '19610718 18:09:08.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (366, 789, 789, 1161172627, '19671028 02:02:44.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (367, 307, 307, 2050295532, '19540501 06:43:03.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (368, 103, 103, 426087969, '19940606 23:09:55.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (369, 138, 138, 319823890, '19690502 17:34:36.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (370, 238, 238, 288729161, '19640131 11:36:53.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (371, 571, 571, 510813017, '19550503 06:48:56.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (372, 183, 183, 497184656, '19920516 22:16:52.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (373, 865, 865, 714594544, '20060127 23:30:37.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (374, 240, 240, 982844194, '19861127 01:12:46.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (375, 378, 378, 839988809, '19840923 23:31:34.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (376, 593, 593, 1906771259, '19670617 02:48:28.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (377, 94, 94, 2054065504, '19680509 14:26:58.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (378, 81, 81, 316468219, '19720203 23:07:32.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (379, 534, 534, 1033056595, '20020801 02:45:35.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (380, 242, 242, 1257401103, '19720731 00:28:17.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (381, 160, 160, 1331816553, '19550622 09:56:42.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (382, 481, 481, 1642698008, '19760409 06:31:39.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (383, 155, 155, 1464098283, '19630626 11:07:01.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (384, 153, 153, 831193402, '19950907 21:08:37.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (385, 986, 986, 2116265549, '20071111 11:31:02.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (386, 37, 37, 2064914839, '19590821 01:36:22.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (387, 95, 95, 307348547, '19570820 01:30:54.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (388, 35, 35, 722704859, '19761201 16:59:53.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (389, 518, 518, 1067126304, '19740620 00:10:11.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (390, 574, 574, 455600997, '19781012 18:03:14.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (391, 308, 308, 628865674, '19860426 09:26:24.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (392, 225, 225, 1746389004, '19830604 06:56:08.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (393, 117, 117, 1840335743, '19630517 08:43:28.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (394, 247, 247, 1580976385, '19900319 23:58:17.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (395, 776, 776, 1237122680, '19870423 17:53:04.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (396, 833, 833, 978038093, '19820128 00:06:50.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (397, 755, 755, 1464357034, '19560211 19:45:44.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (398, 241, 241, 79299243, '20070717 15:11:03.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (399, 176, 176, 2034555874, '19570803 14:35:48.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (400, 124, 124, 148033537, '19670326 14:37:42.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (401, 180, 180, 1409294283, '19990826 04:04:27.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (402, 578, 578, 381067304, '19980427 08:56:13.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (403, 77, 77, 1086413861, '19770619 06:24:08.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (404, 285, 285, 558729572, '19930406 10:56:47.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (405, 229, 229, 953352217, '19970324 08:56:37.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (406, 401, 401, 1384799669, '19950405 07:08:09.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (407, 45, 45, 1432574756, '19760205 18:13:12.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (408, 135, 135, 633243686, '19710518 17:05:33.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (409, 485, 485, 1743341075, '20000512 01:43:01.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (410, 336, 336, 479940299, '20010819 10:26:19.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (411, 608, 608, 505712280, '20000309 03:23:45.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (412, 416, 416, 665714297, '19761111 11:41:10.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (413, 46, 46, 1591287447, '20070328 08:39:39.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (414, 297, 297, 1083699013, '19630929 13:36:31.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (415, 368, 368, 1699784041, '19561206 06:16:52.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (416, 135, 135, 1198534340, '19650622 19:40:35.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (417, 526, 526, 1959563765, '19790921 21:47:55.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (418, 756, 756, 328197122, '19881009 00:00:20.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (419, 375, 375, 1692114363, '19710421 21:04:53.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (420, 721, 721, 1919111805, '19541129 16:32:45.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (421, 457, 457, 635245149, '19630311 00:31:50.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (422, 86, 86, 689004291, '19850529 13:43:10.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (423, 586, 586, 1054819180, '19721218 22:59:44.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (424, 41, 41, 581888757, '19980720 23:31:22.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (425, 778, 778, 1606623887, '19851006 02:10:29.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (426, 347, 347, 1772308594, '19791128 23:52:47.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (427, 144, 144, 1843585671, '19820101 13:33:24.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (428, 708, 708, 1937079102, '19681110 23:32:20.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (429, 465, 465, 1258945368, '20070805 07:19:41.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (430, 206, 206, 1250441684, '19550828 23:24:44.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (431, 259, 259, 1496241668, '19640506 07:02:44.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (432, 424, 424, 1955022137, '19681022 23:15:54.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (433, 80, 80, 1694358168, '19670704 08:31:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (434, 410, 410, 509183292, '19880508 12:07:53.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (435, 816, 816, 569240757, '19801204 20:23:49.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (436, 216, 216, 597603044, '19650224 01:00:29.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (437, 752, 752, 2086769656, '20061021 00:07:30.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (438, 235, 235, 1001649139, '19780321 00:10:13.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (439, 924, 924, 647316247, '20060615 12:12:00.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (440, 585, 585, 380121229, '19650809 04:22:52.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (441, 210, 210, 1796117599, '19910715 07:23:10.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (442, 53, 53, 739868987, '19940403 08:25:21.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (443, 550, 550, 1505529643, '19840721 15:16:51.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (444, 161, 161, 1094351772, '19801030 13:43:51.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (445, 966, 966, 602821450, '19860804 14:39:29.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (446, 444, 444, 1813925181, '19620615 21:45:14.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (447, 431, 431, 198298164, '19840307 22:16:29.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (448, 439, 439, 88362959, '20070817 19:06:57.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (449, 56, 56, 807626459, '19860414 17:41:25.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (450, 311, 311, 128902525, '19741101 22:12:28.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (451, 308, 308, 1301990197, '19550510 02:18:54.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (452, 269, 269, 1857293850, '19750504 19:45:23.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (453, 866, 866, 1571276129, '19890328 18:06:09.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (454, 455, 455, 1310613875, '19550905 22:03:02.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (455, 667, 667, 1416768067, '19570116 14:05:51.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (456, 95, 95, 1245447424, '19670330 14:21:17.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (457, 993, 993, 713489311, '19780510 09:56:28.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (458, 689, 689, 1297292891, '19861129 06:52:46.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (459, 52, 52, 1461868162, '19600702 08:46:18.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (460, 205, 205, 1779472667, '19700427 09:03:50.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (461, 701, 701, 1647477749, '19660404 17:34:45.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (462, 296, 296, 1774880380, '19600327 18:40:51.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (463, 435, 435, 151330298, '19711013 09:45:51.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (464, 940, 940, 184246103, '19970915 02:18:17.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (465, 519, 519, 1421769423, '19900416 03:23:35.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (466, 150, 150, 1302002148, '19840518 04:07:50.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (467, 511, 511, 1028525591, '19620512 03:09:25.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (468, 385, 385, 503509655, '19711120 20:31:46.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (469, 375, 375, 1159577543, '19901024 17:12:41.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (470, 521, 521, 845165173, '19991012 05:16:23.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (471, 384, 384, 1859246051, '19660901 19:33:05.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (472, 291, 291, 946065053, '19540704 21:37:41.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (473, 396, 396, 2123191570, '19900425 11:48:20.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (474, 791, 791, 795753077, '19580913 16:42:01.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (475, 729, 729, 324107930, '19710517 09:09:35.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (476, 405, 405, 1475854809, '19761206 16:06:29.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (477, 536, 536, 1762713496, '19531108 22:26:18.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (478, 425, 425, 2072066315, '20000218 09:15:52.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (479, 75, 75, 1839881052, '19641217 08:51:53.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (480, 335, 335, 978382519, '19760423 04:25:14.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (481, 917, 917, 882126504, '20030922 01:36:17.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (482, 777, 777, 1235893431, '19820516 18:26:26.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (483, 73, 73, 1214443088, '19900729 05:50:55.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (484, 327, 327, 1297415430, '19851004 09:07:13.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (485, 237, 237, 2029212621, '19530421 21:05:49.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (486, 66, 66, 532723620, '19970101 11:17:20.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (487, 278, 278, 2074076354, '19870727 05:09:44.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (488, 180, 180, 113522338, '19641029 10:28:41.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (489, 140, 140, 33821252, '19840422 22:02:01.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (490, 722, 722, 692362204, '19660908 06:02:32.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (491, 362, 362, 1707392423, '19941016 15:04:00.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (492, 251, 251, 709791021, '19721122 17:14:44.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (493, 183, 183, 2100965847, '19700919 15:23:55.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (494, 42, 42, 583506788, '19890219 03:08:10.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (495, 884, 884, 1344461156, '20070506 10:48:06.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (496, 655, 655, 1710184234, '19840419 12:42:18.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (497, 485, 485, 779257296, '19750622 22:39:29.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (498, 151, 151, 1306545108, '19941105 12:58:34.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (499, 642, 642, 651720866, '19980716 10:20:16.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (500, 782, 782, 38742481, '19550320 10:31:38.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (501, 142, 142, 1321361735, '19530204 18:35:48.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (502, 46, 46, 531988280, '19650419 01:44:43.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (503, 29, 29, 1964990385, '19691025 01:54:16.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (504, 301, 301, 1258095123, '19940703 12:25:02.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (505, 258, 258, 687362074, '19610303 02:39:23.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (506, 18, 18, 905527714, '19531106 04:41:13.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (507, 167, 167, 1118826689, '19930108 07:57:03.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (508, 479, 479, 1099728008, '19830717 01:24:08.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (509, 1, 1, 1980562027, '19920420 12:53:27.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (510, 360, 360, 1039792964, '19880209 21:59:22.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (511, 6, 6, 423994317, '19660522 15:54:58.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (512, 568, 568, 105768822, '19860322 00:40:36.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (513, 615, 615, 741731157, '19741213 22:00:53.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (514, 717, 717, 1851281626, '19920310 04:21:03.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (515, 967, 967, 1025232868, '19740805 07:27:32.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (516, 925, 925, 432457595, '19911118 23:08:19.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (517, 147, 147, 435819568, '19770829 12:49:56.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (518, 457, 457, 1277239917, '19940108 00:38:38.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (519, 704, 704, 75740340, '19970527 05:12:27.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (520, 347, 347, 1245737744, '20010413 16:06:14.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (521, 627, 627, 2080444666, '20041021 22:58:05.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (522, 60, 60, 1533462721, '20050712 16:40:44.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (523, 499, 499, 1341367026, '19950729 22:29:44.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (524, 657, 657, 1761770016, '19770216 11:10:08.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (525, 933, 933, 1690437329, '19571227 14:12:22.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (526, 321, 321, 318345156, '20011009 02:18:20.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (527, 109, 109, 967444972, '19911015 06:13:46.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (528, 474, 474, 1407203346, '19540307 08:40:09.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (529, 7, 7, 854934765, '19590510 05:53:54.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (530, 508, 508, 1596653798, '19950126 20:27:16.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (531, 169, 169, 1773774323, '19540617 17:26:59.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (532, 927, 927, 1432892301, '19670103 09:27:44.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (533, 784, 784, 1441218727, '19540805 22:55:35.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (534, 293, 293, 1676520848, '19620930 22:20:14.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (535, 141, 141, 559256795, '19760319 09:49:25.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (536, 871, 871, 198657542, '19910605 23:51:34.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (537, 177, 177, 1074394852, '19650723 16:32:09.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (538, 118, 118, 1164167750, '20040125 17:25:53.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (539, 567, 567, 2139077534, '19770804 06:27:49.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (540, 70, 70, 821735540, '20070615 16:24:36.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (541, 279, 279, 880123048, '19561225 03:20:16.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (542, 304, 304, 259042664, '19570111 03:45:36.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (543, 726, 726, 1763831418, '19800709 21:35:13.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (544, 382, 382, 280363814, '20040313 00:02:39.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (545, 288, 288, 935290933, '19530419 14:07:33.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (546, 21, 21, 771936518, '19610728 14:23:24.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (547, 261, 261, 1013883812, '20051209 19:13:51.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (548, 349, 349, 76638954, '19860711 11:02:51.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (549, 530, 530, 276574874, '19670718 19:40:37.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (550, 960, 960, 2115706769, '19680618 11:39:47.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (551, 969, 969, 133296068, '19590822 23:52:22.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (552, 244, 244, 937983621, '19890613 22:00:51.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (553, 529, 529, 367883113, '20050610 07:46:06.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (554, 246, 246, 1322460801, '20050402 18:14:02.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (555, 164, 164, 152555354, '19580528 11:33:33.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (556, 274, 274, 195288758, '19550726 01:55:04.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (557, 404, 404, 1917599421, '19770920 03:14:58.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (558, 890, 890, 1963466465, '20000906 14:44:07.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (559, 602, 602, 987998700, '19890707 22:12:39.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (560, 937, 937, 1472029363, '19670525 00:21:02.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (561, 352, 352, 1696386711, '19700121 22:27:26.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (562, 376, 376, 1582205210, '19911104 23:16:53.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (563, 531, 531, 1351052163, '19770309 19:30:14.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (564, 231, 231, 1337934030, '20050324 16:26:11.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (565, 110, 110, 1634677917, '19860826 04:32:22.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (566, 13, 13, 229995044, '20070519 06:27:14.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (567, 220, 220, 1737791677, '19840817 01:45:00.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (568, 323, 323, 2143307426, '19650315 23:40:38.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (569, 291, 291, 8461628, '19681222 18:31:37.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (570, 96, 96, 2056674548, '19910301 07:35:57.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (571, 102, 102, 930548581, '19790429 06:36:09.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (572, 512, 512, 1926348860, '19810803 19:24:03.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (573, 267, 267, 471260098, '19690606 18:10:48.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (574, 174, 174, 941719994, '19971211 12:47:51.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (575, 9, 9, 727330314, '19970420 12:45:57.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (576, 144, 144, 1053224387, '20001011 19:12:29.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (577, 480, 480, 1120539871, '19780103 19:05:30.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (578, 162, 162, 697300757, '19990517 22:27:05.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (579, 494, 494, 1680134177, '19761030 21:02:34.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (580, 912, 912, 239730479, '20040530 23:48:58.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (581, 873, 873, 906587581, '20031030 07:04:29.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (582, 523, 523, 938936380, '19580406 19:10:54.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (583, 42, 42, 3715335, '19940819 12:59:31.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (584, 948, 948, 2143709700, '19981120 18:14:06.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (585, 106, 106, 409181338, '19880606 20:34:54.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (586, 516, 516, 492544267, '19730102 19:26:07.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (587, 546, 546, 165090034, '19690725 01:41:37.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (588, 823, 823, 1979855088, '19570504 04:41:33.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (589, 276, 276, 949879694, '19570506 10:46:41.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (590, 528, 528, 495189662, '19730826 07:54:21.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (591, 12, 12, 1914666673, '19660916 20:36:35.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (592, 265, 265, 1639330840, '19721115 01:48:01.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (593, 160, 160, 36352069, '19670721 19:13:13.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (594, 630, 630, 1647193899, '19630314 06:06:47.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (595, 218, 218, 2097704735, '19900525 17:57:09.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (596, 297, 297, 453887493, '19730219 04:03:22.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (597, 569, 569, 2090591602, '19871103 08:15:21.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (598, 74, 74, 982152920, '19830417 05:09:02.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (599, 129, 129, 37168970, '19700719 19:30:16.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (600, 328, 328, 1449618430, '19531201 07:40:19.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (601, 497, 497, 354696652, '19841210 12:38:24.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (602, 79, 79, 1956138734, '19930926 19:33:12.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (603, 152, 152, 156830613, '19700720 16:31:13.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (604, 980, 980, 929327530, '19840518 12:04:40.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (605, 266, 266, 913454825, '19970219 05:03:37.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (606, 535, 535, 159885539, '19860118 04:28:18.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (607, 2, 2, 550086019, '19730108 03:50:03.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (608, 423, 423, 34270994, '19600630 18:58:15.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (609, 114, 114, 155128740, '19601214 05:28:04.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (610, 12, 12, 100368651, '19650815 16:21:03.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (611, 275, 275, 1832234514, '19850721 06:49:34.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (612, 219, 219, 506767467, '19860505 04:47:53.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (613, 397, 397, 956376371, '19761107 17:41:33.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (614, 691, 691, 1512999517, '19930205 22:23:41.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (615, 383, 383, 1025786333, '19710725 17:16:32.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (616, 268, 268, 1635919210, '19641018 03:16:31.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (617, 660, 660, 239574489, '20050317 10:17:22.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (618, 105, 105, 2064219417, '19860419 01:16:07.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (619, 503, 503, 2129805197, '19691018 19:51:17.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (620, 563, 563, 780342058, '19660824 09:06:15.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (621, 222, 222, 353733596, '19901026 04:45:37.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (622, 554, 554, 562637407, '19800415 21:03:26.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (623, 466, 466, 882009094, '19601110 12:53:57.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (624, 788, 788, 1586900074, '20030428 10:37:16.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (625, 628, 628, 863191959, '19770615 10:59:22.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (626, 354, 354, 112138292, '19590615 04:48:07.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (627, 459, 459, 1057787237, '19670115 00:10:49.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (628, 797, 797, 414213907, '19590327 12:04:00.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (629, 149, 149, 952147970, '19600719 18:50:42.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (630, 712, 712, 508173724, '19770302 08:42:34.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (631, 829, 829, 801105904, '19651210 10:57:07.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (632, 261, 261, 2006781810, '20020918 13:56:28.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (633, 303, 303, 1436738911, '19811030 02:56:48.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (634, 964, 964, 1783406551, '19751201 13:22:15.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (635, 675, 675, 1355236458, '19720621 11:10:33.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (636, 748, 748, 679885766, '19630203 11:31:17.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (637, 115, 115, 1852320174, '19950917 02:39:41.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (638, 737, 737, 514404258, '19630404 00:54:51.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (639, 682, 682, 112527954, '19541002 13:10:29.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (640, 519, 519, 1514722390, '19550521 16:06:35.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (641, 108, 108, 968774749, '20071225 15:36:04.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (642, 598, 598, 551918767, '19620125 06:43:22.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (643, 870, 870, 861965056, '20040519 23:13:29.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (644, 602, 602, 1266063431, '19990920 18:25:37.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (645, 128, 128, 425254537, '19960205 01:04:47.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (646, 392, 392, 1042979036, '19880513 15:48:42.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (647, 944, 944, 1464571015, '20040107 08:06:28.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (648, 57, 57, 1020881600, '19820613 20:49:32.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (649, 539, 539, 1880642083, '19990819 12:50:14.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (650, 44, 44, 264208650, '19780808 14:40:37.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (651, 637, 637, 559900633, '19751204 17:46:00.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (652, 565, 565, 536617083, '19540718 06:59:14.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (653, 590, 590, 678444758, '19660630 09:17:45.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (654, 540, 540, 1333500034, '19561126 10:24:01.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (655, 919, 919, 389064926, '19710205 02:54:42.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (656, 276, 276, 1506738487, '19570827 15:34:57.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (657, 942, 942, 1956186696, '19851117 06:39:14.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (658, 364, 364, 1267560634, '19750324 05:53:56.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (659, 352, 352, 2111443789, '19591205 01:05:18.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (660, 580, 580, 151128242, '19900908 00:15:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (661, 351, 351, 1587000305, '19720105 04:17:29.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (662, 22, 22, 359104859, '19661015 15:46:02.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (663, 152, 152, 217390422, '20071212 00:07:32.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (664, 258, 258, 428613397, '19911013 20:45:30.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (665, 493, 493, 1668758630, '20070905 05:23:56.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (666, 280, 280, 1432877656, '19901101 16:53:06.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (667, 86, 86, 1806136393, '19570707 01:51:04.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (668, 434, 434, 546239830, '19531209 04:19:17.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (669, 190, 190, 1230997225, '19730818 11:13:07.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (670, 317, 317, 2084730776, '19610622 05:45:15.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (671, 482, 482, 735917385, '19770131 00:36:50.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (672, 448, 448, 2126701446, '19941215 09:22:30.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (673, 611, 611, 1310754044, '19840718 12:05:38.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (674, 157, 157, 76065114, '19670601 03:44:41.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (675, 62, 62, 199204930, '19660829 17:30:10.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (676, 312, 312, 1344006712, '19811001 22:02:14.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (677, 93, 93, 1675325958, '19831126 21:49:57.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (678, 522, 522, 1027105842, '19690219 18:28:20.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (679, 350, 350, 1656622509, '19600324 09:32:29.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (680, 403, 403, 459173901, '19970131 19:10:39.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (681, 434, 434, 1428001134, '19630607 20:41:38.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (682, 119, 119, 1994286221, '19920802 03:21:16.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (683, 230, 230, 1281839639, '19670807 23:13:45.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (684, 130, 130, 2142238835, '19891211 04:10:04.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (685, 76, 76, 1003277077, '19540330 14:56:33.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (686, 58, 58, 1568484022, '19640526 03:57:53.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (687, 734, 734, 1736990489, '19890322 04:38:42.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (688, 555, 555, 572409999, '19771204 16:32:47.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (689, 502, 502, 1007189590, '19571027 10:27:33.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (690, 400, 400, 1521957177, '19671026 19:35:35.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (691, 806, 806, 1811025060, '19850318 04:52:02.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (692, 127, 127, 2056911224, '19730626 20:45:44.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (693, 468, 468, 1018894177, '19560430 23:49:33.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (694, 770, 770, 2056657491, '19720124 12:54:58.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (695, 626, 626, 165549878, '19910516 11:49:05.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (696, 422, 422, 701689435, '19940312 23:50:02.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (697, 426, 426, 489922364, '19620213 06:35:49.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (698, 629, 629, 68128457, '19650808 02:27:58.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (699, 943, 943, 2051183301, '20000115 13:01:40.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (700, 710, 710, 2020639469, '19580406 08:11:41.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (701, 418, 418, 691142157, '19831108 13:57:37.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (702, 467, 467, 277602294, '20030130 03:47:11.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (703, 88, 88, 2127345895, '19611027 09:36:25.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (704, 292, 292, 456844199, '19910228 07:04:59.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (705, 616, 616, 530954912, '19540709 14:03:46.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (706, 713, 713, 1475260749, '19881219 08:23:30.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (707, 290, 290, 596318187, '19771229 18:53:35.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (708, 685, 685, 1003204294, '20070129 05:33:03.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (709, 394, 394, 433633835, '19980329 16:53:51.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (710, 904, 904, 1378458540, '19970505 04:52:27.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (711, 665, 665, 1060676981, '19810930 17:45:00.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (712, 882, 882, 1166908648, '19690928 12:10:54.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (713, 565, 565, 2088058820, '19671231 20:21:27.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (714, 949, 949, 744504873, '19701102 05:54:38.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (715, 503, 503, 1778353009, '19800403 03:33:51.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (716, 125, 125, 214685222, '19870604 00:56:12.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (717, 792, 792, 313514319, '20070309 16:32:16.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (718, 213, 213, 977344941, '19710101 19:57:28.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (719, 166, 166, 1129868269, '19900717 05:48:56.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (720, 547, 547, 933191710, '19960412 01:26:02.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (721, 546, 546, 1683317390, '19540813 12:14:23.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (722, 915, 915, 854511075, '19870802 09:18:17.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (723, 599, 599, 342088089, '20040211 17:51:44.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (724, 617, 617, 1975418247, '19581024 15:37:31.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (725, 702, 702, 941111167, '19840405 00:53:13.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (726, 650, 650, 851819114, '19560807 03:51:05.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (727, 673, 673, 685063280, '19910817 09:32:57.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (728, 694, 694, 1375450515, '19650625 23:10:39.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (729, 489, 489, 96848884, '19840115 15:55:35.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (730, 976, 976, 1695075983, '19800617 17:40:07.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (731, 669, 669, 972028470, '19720819 15:26:25.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (732, 429, 429, 1407973868, '19710421 19:21:58.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (733, 393, 393, 1646048598, '19770204 05:26:39.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (734, 271, 271, 810730145, '19541220 01:20:48.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (735, 268, 268, 1034567331, '19660327 05:13:01.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (736, 899, 899, 554133109, '19680507 16:54:27.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (737, 417, 417, 414159899, '19831007 17:44:51.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (738, 604, 604, 1593274529, '19840608 16:08:46.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (739, 862, 862, 986493218, '19880604 14:06:18.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (740, 658, 658, 1426863825, '19730410 06:33:03.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (741, 182, 182, 712292460, '19940527 09:04:17.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (742, 49, 49, 746292001, '19900220 23:05:39.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (743, 613, 613, 1564862065, '19870906 23:38:56.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (744, 143, 143, 1131962077, '19680623 05:35:05.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (745, 351, 351, 1979216774, '19940125 01:50:35.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (746, 586, 586, 1911556006, '19680620 16:41:07.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (747, 562, 562, 200074090, '19580628 00:24:17.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (748, 436, 436, 1669667636, '19930628 17:54:54.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (749, 705, 705, 390159607, '19991022 09:21:07.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (750, 683, 683, 2125747267, '19561213 10:52:53.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (751, 294, 294, 1233285174, '19950103 07:17:45.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (752, 682, 682, 664668230, '19990211 10:38:21.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (753, 463, 463, 2057753139, '19830121 20:39:02.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (754, 496, 496, 1665211163, '19561005 11:35:38.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (755, 164, 164, 1445740315, '19560824 19:57:17.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (756, 842, 842, 2136355355, '20040407 04:39:20.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (757, 316, 316, 1505725439, '20061218 09:55:27.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (758, 414, 414, 570971077, '19560104 17:58:53.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (759, 740, 740, 170632677, '19591126 06:11:45.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (760, 120, 120, 661376280, '20051201 10:12:40.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (761, 174, 174, 2043883521, '20050503 22:50:33.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (762, 533, 533, 1585990514, '19650706 19:42:55.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (763, 197, 197, 1712983521, '19760115 13:37:27.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (764, 711, 711, 654819813, '19701012 23:13:43.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (765, 438, 438, 988848262, '19770916 13:26:02.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (766, 78, 78, 1532893837, '19630612 22:23:02.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (767, 766, 766, 1711816458, '20000823 06:44:15.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (768, 234, 234, 1505960441, '19660112 19:00:38.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (769, 171, 171, 528084726, '19570809 00:41:36.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (770, 604, 604, 1438205311, '19641127 10:39:24.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (771, 709, 709, 983464002, '19560828 07:11:21.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (772, 813, 813, 503302184, '19751002 00:23:29.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (773, 260, 260, 1105146685, '19900729 05:51:09.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (774, 803, 803, 2060262684, '19700408 23:15:52.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (775, 267, 267, 1994784729, '19541117 16:21:45.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (776, 357, 357, 1697132923, '19720621 13:08:43.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (777, 465, 465, 2124604893, '20071125 09:39:21.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (778, 246, 246, 1954375555, '19880821 12:16:38.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (779, 406, 406, 1028295907, '19720928 13:46:56.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (780, 587, 587, 909925097, '19681016 08:12:06.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (781, 88, 88, 1731636428, '20060209 03:26:48.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (782, 104, 104, 1770449255, '20060217 15:38:02.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (783, 715, 715, 1921047920, '19730903 13:49:32.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (784, 259, 259, 1869963698, '19800203 05:02:42.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (785, 639, 639, 402646425, '19930614 10:22:22.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (786, 8, 8, 324932608, '19810708 04:48:03.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (787, 277, 277, 1842519226, '19960329 22:42:55.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (788, 622, 622, 1947765440, '19730502 17:51:01.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (789, 532, 532, 1971359380, '20060426 05:23:30.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (790, 691, 691, 12506939, '19691219 00:33:41.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (791, 671, 671, 1335808848, '19690520 06:58:59.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (792, 224, 224, 1535950838, '19801003 23:45:57.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (793, 398, 398, 1590866832, '19770715 09:57:01.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (794, 552, 552, 1868124219, '19900704 03:53:37.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (795, 81, 81, 230803511, '19751207 07:42:30.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (796, 398, 398, 1905792487, '19811121 13:21:21.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (797, 853, 853, 1508058319, '19670207 09:28:12.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (798, 215, 215, 169849153, '19691126 00:25:12.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (799, 262, 262, 1801137301, '19981006 16:09:03.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (800, 498, 498, 174547432, '19830816 03:27:33.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (801, 159, 159, 318962376, '19751028 09:56:52.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (802, 781, 781, 1246741967, '20000614 05:23:38.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (803, 221, 221, 373607890, '19881120 17:13:18.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (804, 665, 665, 2002789394, '19871125 22:41:42.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (805, 522, 522, 786319609, '19530418 03:41:32.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (806, 72, 72, 2088835894, '19720404 06:54:16.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (807, 515, 515, 1476424433, '19610717 04:47:11.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (808, 661, 661, 526643593, '19651015 21:23:10.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (809, 405, 405, 180249770, '19541119 19:13:53.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (810, 668, 668, 1935582846, '19920304 06:48:33.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (811, 324, 324, 1644211143, '20040512 18:59:59.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (812, 228, 228, 1060784915, '19710429 21:38:49.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (813, 717, 717, 2014448974, '19910407 04:11:57.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (814, 384, 384, 129340571, '19990208 21:59:39.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (815, 600, 600, 1167911069, '20071022 06:45:52.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (816, 70, 70, 2039868782, '20070316 07:12:30.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (817, 125, 125, 1059704365, '19991102 05:53:23.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (818, 91, 91, 1554686400, '20031212 08:34:44.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (819, 501, 501, 1549309972, '19850429 12:51:21.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (820, 793, 793, 1157041951, '20040310 08:37:58.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (821, 718, 718, 2127246764, '19750314 23:40:06.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (822, 576, 576, 1336690237, '19800423 12:53:14.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (823, 549, 549, 1799380215, '19670918 13:37:08.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (824, 776, 776, 247201182, '19950821 00:07:55.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (825, 828, 828, 1155683650, '20030710 03:40:24.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (826, 409, 409, 11431651, '19831124 07:25:23.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (827, 572, 572, 596625590, '20060318 14:26:28.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (828, 669, 669, 1317309299, '19530126 01:57:32.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (829, 621, 621, 1191410556, '20020502 15:33:21.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (830, 869, 869, 1428803412, '19801226 03:00:23.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (831, 68, 68, 1521497570, '19580516 03:40:31.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (832, 338, 338, 41368092, '19901231 09:14:08.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (833, 339, 339, 1442209808, '19971114 20:07:34.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (834, 710, 710, 1265660255, '19970212 10:19:22.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (835, 428, 428, 479110049, '20001218 22:15:13.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (836, 1, 1, 1681241812, '19580828 22:03:10.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (837, 970, 970, 170026756, '19700330 22:24:43.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (838, 431, 431, 41267707, '19931008 15:07:49.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (839, 738, 738, 17097166, '19791019 01:21:10.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (840, 200, 200, 721533631, '19740313 03:28:06.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (841, 82, 82, 233212851, '19721223 00:00:52.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (842, 592, 592, 1015946304, '19830616 01:19:45.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (843, 393, 393, 1077136690, '19710614 22:37:08.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (844, 773, 773, 918455931, '19670222 22:34:56.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (845, 542, 542, 1077372436, '19730807 05:33:42.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (846, 736, 736, 1087514474, '20060120 09:20:10.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (847, 432, 432, 2032604157, '19970628 19:34:00.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (848, 802, 802, 249045912, '19860605 11:57:21.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (849, 706, 706, 290792489, '19900911 21:07:44.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (850, 367, 367, 860124615, '19760922 00:29:59.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (851, 116, 116, 639565482, '19900119 07:27:58.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (852, 916, 916, 1985333843, '19710226 00:53:27.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (853, 714, 714, 786675455, '19920728 11:33:51.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (854, 115, 115, 89592959, '20020728 07:31:05.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (855, 781, 781, 2075708514, '19610604 03:47:26.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (856, 676, 676, 1599767191, '20030705 21:03:38.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (857, 539, 539, 1964170839, '19850927 15:46:30.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (858, 370, 370, 733722827, '20010401 17:05:22.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (859, 838, 838, 342227117, '19920517 19:01:18.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (860, 38, 38, 1365805955, '19770524 20:16:08.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (861, 561, 561, 846335770, '19740116 16:27:47.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (862, 355, 355, 1225595731, '19610622 02:49:38.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (863, 742, 742, 671857627, '19710615 05:49:48.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (864, 364, 364, 1841931268, '19811123 16:13:29.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (865, 226, 226, 1985918926, '19861021 03:08:02.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (866, 181, 181, 1193422390, '19660514 09:45:51.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (867, 857, 857, 782745493, '19810614 01:31:15.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (868, 889, 889, 2027919685, '20020221 17:52:34.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (869, 543, 543, 399672735, '20060219 23:44:25.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (870, 815, 815, 2133761992, '20020224 08:42:42.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (871, 76, 76, 1315339634, '19891217 08:47:47.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (872, 446, 446, 874886390, '19590125 14:45:33.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (873, 840, 840, 1593474656, '19770224 07:13:34.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (874, 301, 301, 583756250, '19640218 09:23:15.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (875, 670, 670, 363525013, '19840319 08:37:06.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (876, 377, 377, 467319989, '19990928 22:20:21.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (877, 437, 437, 1822908930, '19611110 14:16:06.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (878, 290, 290, 1189282216, '19530727 15:02:11.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (879, 14, 14, 1592944036, '19750114 18:34:13.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (880, 106, 106, 1411884432, '20050618 18:20:14.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (881, 442, 442, 1788294420, '19940527 11:51:23.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (882, 826, 826, 2123033412, '19721014 02:29:06.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (883, 304, 304, 778742992, '19700516 04:49:47.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (884, 493, 493, 677941136, '19780811 15:03:21.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (885, 753, 753, 913728650, '19981206 19:32:24.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (886, 403, 403, 1409251023, '19950320 02:47:04.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (887, 591, 591, 300991150, '20060603 21:40:16.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (888, 646, 646, 2122348969, '20030419 12:36:29.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (889, 757, 757, 134017301, '19880911 06:31:55.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (890, 413, 413, 970859258, '20050615 01:11:35.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (891, 753, 753, 189158348, '19801130 06:16:40.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (892, 332, 332, 856737550, '19761228 05:19:21.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (893, 836, 836, 911884258, '19540524 20:06:31.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (894, 461, 461, 1413729959, '19550528 05:05:01.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (895, 472, 472, 2041205001, '19530225 11:00:18.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (896, 26, 26, 349729360, '19640703 21:11:13.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (897, 534, 534, 1020373894, '19650101 19:29:56.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (898, 24, 24, 2101067525, '19970721 06:23:38.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (899, 601, 601, 1439492907, '19880503 19:26:54.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (900, 891, 891, 706047616, '19600326 19:47:50.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (901, 864, 864, 1078658593, '19770809 07:48:55.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (902, 544, 544, 315341873, '20030507 01:41:25.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (903, 362, 362, 1627092071, '19880414 12:12:56.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (904, 764, 764, 748830357, '19960717 12:25:02.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (905, 866, 866, 1096030578, '19941005 15:42:12.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (906, 541, 541, 570386313, '19831226 16:42:24.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (907, 583, 583, 887647767, '20011231 17:39:53.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (908, 890, 890, 1249139316, '19810611 02:10:36.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (909, 798, 798, 1987407154, '19710511 22:53:59.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (910, 476, 476, 1145571548, '19690905 05:27:05.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (911, 322, 322, 1244365636, '19940826 06:47:32.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (912, 249, 249, 86295907, '19850304 00:44:19.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (913, 272, 272, 553466317, '19790319 22:31:09.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (914, 861, 861, 787105585, '19941129 00:41:04.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (915, 985, 985, 1817827428, '19901228 08:24:45.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (916, 426, 426, 479645878, '19540405 13:58:40.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (917, 647, 647, 1268950310, '19990205 21:59:51.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (918, 557, 557, 3750030, '20001102 14:46:26.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (919, 411, 411, 908623628, '19901217 20:41:05.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (920, 751, 751, 1617451055, '19990803 00:20:57.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (921, 429, 429, 229151215, '19671210 12:05:34.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (922, 597, 597, 1345295323, '19860224 12:54:46.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (923, 767, 767, 992257681, '19660613 11:20:39.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (924, 313, 313, 1272740233, '19530907 22:32:49.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (925, 100, 100, 1972125355, '19740328 02:26:02.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (926, 3, 3, 1418488347, '19651221 03:28:25.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (927, 872, 872, 106661581, '19570903 18:39:02.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (928, 500, 500, 1985980717, '19741001 02:08:32.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (929, 11, 11, 304885787, '19631224 22:22:56.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (930, 528, 528, 1054346664, '19720916 11:25:52.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (931, 84, 84, 20218571, '19870928 02:50:24.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (932, 74, 74, 1454895667, '19720421 07:52:28.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (933, 54, 54, 1782822890, '19730324 19:35:16.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (934, 551, 551, 1908574680, '19671021 22:46:22.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (935, 572, 572, 1414467724, '19801110 10:31:18.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (936, 302, 302, 1773399903, '19990120 10:09:57.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (937, 889, 889, 591477120, '19920630 14:16:10.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (938, 718, 718, 1502131620, '19811028 16:24:45.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (939, 473, 473, 169595067, '19911105 23:21:09.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (940, 378, 378, 1641173025, '19671213 02:49:59.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (941, 879, 879, 1898901971, '20010318 09:07:11.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (942, 204, 204, 125672059, '19771224 19:29:39.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (943, 573, 573, 1592932415, '19841209 13:42:29.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (944, 766, 766, 1081217961, '19720108 01:04:49.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (945, 116, 116, 1329006436, '19631021 18:24:03.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (946, 214, 214, 566832688, '20030928 05:32:21.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (947, 428, 428, 1805422097, '20051010 06:48:11.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (948, 249, 249, 84341528, '19670625 19:25:27.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (949, 514, 514, 2054820316, '19720529 20:40:15.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (950, 373, 373, 1102688747, '20061121 21:01:37.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (951, 880, 880, 2080363734, '19730527 23:11:22.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (952, 686, 686, 1523149806, '19720229 04:43:30.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (953, 478, 478, 537108652, '19590805 09:42:32.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (954, 699, 699, 2078737918, '19960930 19:05:57.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (955, 388, 388, 1887549477, '20000416 07:42:16.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (956, 689, 689, 1146295228, '19990614 18:54:09.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (957, 425, 425, 457946595, '19891124 14:20:46.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (958, 50, 50, 222226632, '19870808 13:40:06.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (959, 114, 114, 1825355701, '19750423 09:59:00.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (960, 663, 663, 1860386520, '19811016 12:13:46.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (961, 809, 809, 478677006, '19790424 22:03:22.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (962, 216, 216, 162932238, '19800402 15:31:20.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (963, 606, 606, 1613948658, '19700619 03:47:40.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (964, 8, 8, 1578393273, '20060824 11:28:06.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (965, 526, 526, 813010468, '19891208 02:36:41.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (966, 430, 430, 2122280279, '19750507 22:55:03.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (967, 235, 235, 853270821, '19641211 05:09:02.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (968, 73, 73, 1967166124, '19640529 23:44:46.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (969, 225, 225, 1194578021, '19670119 14:09:45.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (970, 768, 768, 1028838816, '19991207 22:14:48.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (971, 303, 303, 1323064524, '19691004 23:42:29.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (972, 525, 525, 1360428854, '19700411 05:35:06.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (973, 931, 931, 1053003595, '19611229 15:25:17.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (974, 653, 653, 560161638, '19760106 17:51:05.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (975, 666, 666, 1236744173, '20060516 15:13:46.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (976, 704, 704, 532965087, '19971215 16:35:54.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (977, 868, 868, 1400298987, '19540318 23:12:16.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (978, 48, 48, 42379805, '20020605 10:15:49.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (979, 755, 755, 804143548, '19971117 04:08:45.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (980, 856, 856, 1458544375, '19780629 20:53:41.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (981, 61, 61, 322494823, '19680311 20:40:13.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (982, 209, 209, 1603700884, '19980310 23:13:35.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (983, 762, 762, 2129050784, '19550505 05:28:16.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (984, 146, 146, 774774716, '19650201 01:21:18.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (985, 911, 911, 626591703, '20070421 12:14:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (986, 349, 349, 711298948, '19680730 22:06:54.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (987, 325, 325, 245285342, '19650917 22:09:55.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (988, 112, 112, 1352923217, '19840623 00:29:19.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (989, 593, 593, 552348857, '19750706 15:04:06.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (990, 805, 805, 847045783, '19890528 15:37:09.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (991, 253, 253, 943297517, '19620226 19:49:11.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (992, 598, 598, 695255187, '19571124 00:36:04.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (993, 964, 964, 1816447917, '19590709 06:25:44.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (994, 98, 98, 1611871379, '19630121 11:07:22.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (995, 332, 332, 1016615828, '19960819 04:46:37.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (996, 799, 799, 624845988, '19731216 17:35:50.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (997, 957, 957, 1785741475, '19600708 15:41:59.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (998, 339, 339, 414213033, '19860417 02:14:23.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (999, 399, 399, 1908052132, '19900129 22:28:08.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1000, 605, 605, 756408117, '19960614 19:28:59.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1001, 19, 19, 311496081, '19911019 00:23:18.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1002, 289, 289, 1056044266, '19940514 07:03:25.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1003, 750, 750, 1217420171, '19530607 05:15:41.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1004, 664, 664, 1426746170, '19800621 22:25:27.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1005, 309, 309, 1368745082, '19900217 21:19:08.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1006, 355, 355, 322851051, '19560217 17:36:16.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1007, 564, 564, 2122761321, '19630303 13:18:12.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1008, 203, 203, 1909648021, '19910729 15:51:27.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1009, 342, 342, 988721350, '19980517 03:52:10.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1010, 207, 207, 138639445, '19550503 15:06:21.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1011, 709, 709, 256521046, '19980329 19:41:53.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1012, 731, 731, 529818780, '19950621 04:04:57.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1013, 199, 199, 315874754, '19970920 09:31:20.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1014, 784, 784, 1090295671, '20041023 13:05:19.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1015, 795, 795, 1937565587, '19660807 15:33:33.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1016, 877, 877, 1617710610, '19890212 22:49:47.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1017, 847, 847, 1368316858, '19771130 10:03:04.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1018, 321, 321, 1937973714, '19580519 02:26:21.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1019, 97, 97, 1095848038, '20070504 23:13:36.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1020, 972, 972, 1758415432, '19740511 04:29:47.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1021, 960, 960, 1661556929, '19620821 00:45:08.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1022, 973, 973, 1498766739, '19880619 04:39:43.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1023, 918, 918, 2091475261, '19961123 08:40:40.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1024, 820, 820, 56415190, '19850822 22:32:36.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1025, 489, 489, 683611869, '19901012 02:25:36.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1026, 750, 750, 1848278765, '19641111 23:06:25.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1027, 989, 989, 1968743476, '19631004 23:09:21.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1028, 739, 739, 4349233, '20061208 04:17:55.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1029, 676, 676, 631391895, '19870519 13:04:27.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1030, 806, 806, 1419597848, '19850531 21:37:56.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1031, 472, 472, 1943153036, '19900610 00:53:55.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1032, 14, 14, 1160606317, '19751201 20:57:53.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1033, 649, 649, 890885863, '19650507 11:47:40.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1034, 223, 223, 636885499, '19540605 08:39:45.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1035, 837, 837, 1032165148, '19940910 20:30:22.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1036, 916, 916, 1763078837, '19811028 13:36:48.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1037, 603, 603, 983066503, '19971004 17:57:53.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1038, 543, 543, 473325595, '19821112 07:02:48.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1039, 584, 584, 846085981, '19821214 04:02:10.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1040, 556, 556, 1955759767, '20040304 18:38:13.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1041, 697, 697, 1573737405, '19580531 08:48:41.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1042, 796, 796, 787709623, '19820219 06:18:27.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1043, 450, 450, 281171138, '19940205 20:37:08.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1044, 732, 732, 56229069, '19730305 23:57:44.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1045, 365, 365, 1565657378, '19990228 19:55:16.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1046, 787, 787, 224436767, '19740907 15:44:13.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1047, 359, 359, 1071092483, '19680304 15:04:43.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1048, 3, 3, 1012670179, '19620914 17:20:24.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1049, 898, 898, 499423195, '20040614 19:33:49.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1050, 730, 730, 1294928936, '19600707 05:56:50.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1051, 367, 367, 719391846, '20040116 07:32:45.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1052, 287, 287, 235385589, '19550221 13:15:37.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1053, 322, 322, 1406232196, '19861213 03:00:46.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1054, 943, 943, 2115769708, '19680921 17:58:21.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1055, 6, 6, 397572029, '19861026 18:43:51.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1056, 27, 27, 2029392790, '19560501 19:43:34.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1057, 79, 79, 616588118, '20050619 22:22:44.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1058, 931, 931, 799897624, '19751015 06:43:05.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1059, 499, 499, 1538293640, '19970909 19:59:50.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1060, 334, 334, 1156550418, '19780408 22:12:42.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1061, 859, 859, 2115830907, '20000514 18:26:54.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1062, 930, 930, 1004204200, '19640325 09:00:17.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1063, 618, 618, 1916171630, '19570313 02:46:59.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1064, 181, 181, 1881349415, '19651217 06:14:14.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1065, 314, 314, 56452122, '19721122 14:12:25.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1066, 458, 458, 1729202675, '19750429 22:43:59.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1067, 30, 30, 2066176505, '19830214 16:17:16.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1068, 282, 282, 1759794438, '19960418 00:51:34.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1069, 341, 341, 1519490176, '19630213 16:34:56.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1070, 764, 764, 1656989652, '19921009 01:56:44.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1071, 44, 44, 557020477, '19990511 04:51:53.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1072, 558, 558, 1563044691, '20030118 03:00:15.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1073, 85, 85, 200984060, '19830604 22:24:11.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1074, 286, 286, 700477313, '19560302 04:35:22.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1075, 656, 656, 1784473276, '19681211 19:41:05.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1076, 797, 797, 627320727, '19880630 18:26:41.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1077, 2, 2, 1614320688, '20020513 08:02:35.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1078, 648, 648, 1137267988, '19760920 08:42:56.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1079, 456, 456, 687405990, '19940624 02:37:19.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1080, 741, 741, 1070292890, '19690204 10:41:22.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1081, 494, 494, 1304156066, '20040908 08:01:42.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1082, 768, 768, 126907568, '19540121 05:48:56.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1083, 404, 404, 815847739, '19550626 08:44:05.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1084, 427, 427, 1457518924, '19791113 07:07:37.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1085, 854, 854, 1840626290, '19890515 14:05:10.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1086, 862, 862, 1287041921, '19880419 11:38:17.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1087, 945, 945, 338379711, '19961218 17:57:06.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1088, 994, 994, 818872791, '20040814 17:49:18.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1089, 300, 300, 144177380, '19750809 13:55:54.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1090, 902, 902, 512106169, '19910513 00:46:48.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1091, 348, 348, 514345710, '19840509 15:14:04.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1092, 906, 906, 530889959, '19741222 11:14:47.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1093, 650, 650, 756977780, '19930303 11:02:58.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1094, 21, 21, 1211173166, '19570907 05:49:28.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1095, 821, 821, 287123469, '19561022 00:11:19.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1096, 553, 553, 1805269940, '20020306 23:48:24.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1097, 399, 399, 563295471, '19771210 03:31:27.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1098, 269, 269, 1671618256, '19810220 14:22:53.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1099, 998, 998, 1327780014, '19530414 09:45:18.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1100, 677, 677, 1468150182, '19761101 21:11:16.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1101, 920, 920, 1236799821, '19990723 23:26:57.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1102, 354, 354, 2077795088, '19791116 14:13:08.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1103, 358, 358, 1336206743, '20070801 00:45:27.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1104, 98, 98, 405911600, '19640906 17:37:05.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1105, 310, 310, 1342211350, '19690226 01:04:56.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1106, 478, 478, 1227143694, '19531230 04:32:30.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1107, 123, 123, 2063552634, '19790918 14:51:26.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1108, 855, 855, 2061659108, '19831013 22:25:24.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1109, 298, 298, 1393076351, '20071010 22:17:16.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1110, 502, 502, 1743704258, '20060427 00:17:09.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1111, 874, 874, 933532406, '19611219 11:40:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1112, 254, 254, 105867343, '19810930 13:39:47.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1113, 970, 970, 501806042, '19890422 04:05:46.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1114, 758, 758, 559789819, '19810806 09:18:27.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1115, 897, 897, 269616687, '19810730 14:10:59.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1116, 71, 71, 1734662094, '19990425 12:37:57.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1117, 622, 622, 977173178, '19611001 00:16:11.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1118, 52, 52, 941205113, '19850429 19:39:22.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1119, 66, 66, 1253842066, '19840802 16:09:04.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1120, 740, 740, 1352801587, '19920804 02:34:08.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1121, 727, 727, 341819882, '19860511 04:46:52.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1122, 391, 391, 873115227, '19860701 22:27:58.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1123, 850, 850, 550315570, '19730910 10:55:40.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1124, 470, 470, 1125716547, '19791003 15:48:08.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1125, 737, 737, 1269242616, '19610602 10:42:39.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1126, 343, 343, 664799573, '19770520 17:37:05.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1127, 846, 846, 2053839425, '19621117 15:57:17.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1128, 69, 69, 1528649409, '19780928 16:34:42.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1129, 484, 484, 1945864491, '20070511 04:24:03.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1130, 370, 370, 2056557245, '19741006 19:52:40.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1131, 100, 100, 2129296207, '19630723 14:55:13.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1132, 366, 366, 591806015, '19740323 17:39:41.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1133, 407, 407, 438610742, '19760608 22:57:37.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1134, 780, 780, 201209816, '19700823 05:26:02.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1135, 822, 822, 1722801162, '19770714 11:14:25.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1136, 942, 942, 60097684, '19771024 17:48:34.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1137, 441, 441, 1199991631, '19540622 05:03:29.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1138, 561, 561, 800038197, '19981018 15:07:00.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1139, 997, 997, 1861538029, '19630918 06:02:40.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1140, 858, 858, 1714113736, '19880517 09:32:39.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1141, 100, 100, 880460160, '19610802 20:46:51.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1142, 368, 368, 643556758, '19660308 19:31:42.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1143, 176, 176, 1265133651, '20041103 19:32:02.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1144, 126, 126, 1520632276, '19770413 13:38:45.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1145, 559, 559, 488388933, '19820525 13:05:48.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1146, 456, 456, 5240054, '19550811 01:34:16.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1147, 199, 199, 920972935, '19930831 07:09:01.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1148, 96, 96, 1986670568, '19640729 01:44:30.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1149, 127, 127, 1590660062, '19840207 15:38:28.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1150, 735, 735, 2076661392, '19650628 11:33:21.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1151, 331, 331, 2111120927, '19930606 23:32:12.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1152, 191, 191, 1227274611, '20000811 07:52:04.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1153, 330, 330, 535770265, '20040719 22:13:49.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1154, 55, 55, 277521188, '19680909 07:11:10.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1155, 137, 137, 884920835, '19980623 16:24:23.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1156, 678, 678, 539419184, '19660123 00:58:59.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1157, 744, 744, 1347366833, '19590309 03:17:28.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1158, 808, 808, 176073130, '19801029 08:57:19.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1159, 76, 76, 388040700, '19560407 06:54:25.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1160, 415, 415, 523256379, '19991008 07:27:51.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1161, 632, 632, 1500748835, '19990211 12:35:13.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1162, 430, 430, 635802554, '19531221 22:16:43.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1163, 957, 957, 947467097, '19840603 18:01:21.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1164, 500, 500, 1178464977, '19860105 02:24:35.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1165, 78, 78, 2083517858, '19951006 09:21:56.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1166, 936, 936, 1845662604, '19950927 18:01:17.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1167, 137, 137, 1548129732, '19580424 14:42:10.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1168, 696, 696, 1919612861, '19710831 22:39:43.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1169, 560, 560, 1705310737, '19570123 22:04:02.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1170, 959, 959, 211720295, '19561006 20:22:25.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1171, 319, 319, 1089712839, '19971104 07:34:28.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1172, 779, 779, 554776956, '19701213 09:09:10.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1173, 896, 896, 650162949, '19740813 13:36:42.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1174, 921, 921, 1007521689, '20040317 06:36:24.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1175, 233, 233, 635817461, '19840103 05:47:16.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1176, 629, 629, 1069665370, '19730305 09:15:10.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1177, 366, 366, 2129489351, '19890827 02:55:55.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1178, 412, 412, 777478222, '20040528 21:16:55.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1179, 101, 101, 1811968767, '20050510 02:42:20.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1180, 53, 53, 2095675043, '19581024 09:08:22.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1181, 963, 963, 1027014769, '19910919 10:28:03.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1182, 34, 34, 878641387, '20060420 14:12:47.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1183, 464, 464, 1615373257, '20020821 00:56:14.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1184, 177, 177, 5830894, '19941113 16:50:41.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1185, 953, 953, 1474919713, '19890430 20:20:27.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1186, 865, 865, 1786367416, '19701211 07:03:08.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1187, 545, 545, 706336981, '19770902 19:25:52.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1188, 659, 659, 193031584, '19600930 15:46:27.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1189, 644, 644, 354218713, '19800229 13:01:39.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1190, 762, 762, 1415062598, '19640622 10:15:26.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1191, 748, 748, 1681320685, '19710819 14:31:05.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1192, 566, 566, 573525784, '19810822 20:06:10.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1193, 947, 947, 1727333244, '19950714 08:12:35.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1194, 894, 894, 800442469, '19711210 22:34:49.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1195, 7, 7, 379162300, '19970404 20:57:26.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1196, 671, 671, 1500113979, '19600812 22:30:07.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1197, 51, 51, 1481269372, '19891004 01:30:21.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1198, 878, 878, 6785433, '19711031 17:07:26.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1199, 951, 951, 953086454, '19890708 04:16:48.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1200, 666, 666, 1951394714, '19940827 19:04:31.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1201, 248, 248, 1679517473, '20050417 10:52:06.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1202, 504, 504, 1725767542, '19750101 08:29:18.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1203, 350, 350, 2077334117, '19600706 03:40:27.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1204, 335, 335, 6171149, '19800503 19:16:03.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1205, 506, 506, 774471924, '19750824 03:58:53.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1206, 966, 966, 920631982, '19750626 14:23:02.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1207, 128, 128, 1194796011, '19781230 18:15:22.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1208, 775, 775, 743406298, '19530505 15:37:24.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1209, 62, 62, 642135804, '19920906 01:23:54.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1210, 634, 634, 2098787165, '19780421 07:09:13.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1211, 272, 272, 1068023596, '19840527 22:03:03.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1212, 103, 103, 338055812, '19621010 06:00:33.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1213, 977, 977, 693302559, '19830621 06:14:58.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1214, 870, 870, 77545368, '20050613 21:46:02.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1215, 894, 894, 1591071082, '19610119 20:59:47.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1216, 143, 143, 9404162, '20001022 22:22:25.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1217, 807, 807, 1362227525, '19590502 21:20:29.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1218, 108, 108, 5703854, '19970721 01:10:39.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1219, 410, 410, 526764348, '20040905 06:04:07.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1220, 802, 802, 636292320, '19771027 02:18:47.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1221, 633, 633, 1490600062, '19710125 22:35:24.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1222, 495, 495, 1142621518, '20050723 22:55:43.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1223, 627, 627, 1967611892, '19990702 09:38:04.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1224, 97, 97, 1025602060, '20000802 11:48:35.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1225, 109, 109, 1741255263, '19930217 05:51:19.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1226, 153, 153, 1873768409, '19690315 11:28:18.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1227, 571, 571, 37233589, '19830601 00:56:35.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1228, 192, 192, 934898758, '19550904 15:01:53.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1229, 608, 608, 1121494060, '19591214 09:38:58.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1230, 595, 595, 749326320, '19760523 07:17:08.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1231, 190, 190, 1899123290, '19910601 07:44:49.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1232, 59, 59, 1101616268, '19701028 09:48:28.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1233, 537, 537, 879325265, '19671122 17:00:06.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1234, 112, 112, 1431086295, '19630914 07:37:48.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1235, 992, 992, 646176543, '19610707 22:16:16.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1236, 846, 846, 1521971857, '19690919 01:58:45.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1237, 210, 210, 1537834609, '19981016 10:32:19.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1238, 760, 760, 1861569753, '19750420 05:40:10.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1239, 801, 801, 1704623464, '19720323 12:51:48.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1240, 881, 881, 1681553382, '19661106 05:57:25.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1241, 333, 333, 1569033703, '19991211 12:47:46.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1242, 723, 723, 1923267909, '19770501 03:48:28.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1243, 138, 138, 77207127, '19760124 14:22:33.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1244, 60, 60, 1916428594, '19541111 05:52:25.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1245, 637, 637, 1131658241, '19880125 12:12:23.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1246, 641, 641, 223611622, '19611110 08:30:32.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1247, 745, 745, 529054230, '20060303 13:51:11.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1248, 184, 184, 1227857503, '19980129 10:26:32.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1249, 194, 194, 1847704853, '19631122 01:35:01.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1250, 56, 56, 1388892393, '20040611 22:35:00.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1251, 640, 640, 576802490, '19540414 01:09:38.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1252, 171, 171, 441899155, '20000315 00:19:41.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1253, 501, 501, 376481151, '19750224 11:03:19.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1254, 162, 162, 1976245766, '19640912 01:58:01.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1255, 230, 230, 41119383, '19760802 20:29:06.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1256, 754, 754, 1422616321, '20070924 11:56:23.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1257, 488, 488, 383725642, '19830701 22:51:14.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1258, 175, 175, 35131227, '19671203 15:51:52.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1259, 719, 719, 225253112, '19950316 13:24:44.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1260, 905, 905, 1801141217, '19590611 16:30:34.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1261, 995, 995, 421288286, '20000125 13:26:26.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1262, 222, 222, 133621712, '19760428 03:13:29.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1263, 982, 982, 1803548359, '20010522 05:58:26.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1264, 763, 763, 982184670, '19690416 18:06:45.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1265, 153, 153, 2038916163, '19941119 23:24:24.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1266, 248, 248, 496340452, '19660801 12:14:34.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1267, 813, 813, 2058451961, '20021117 13:00:27.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1268, 77, 77, 970783873, '19721007 22:37:10.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1269, 314, 314, 1452429488, '19961207 23:29:46.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1270, 220, 220, 1895803352, '19990502 19:01:01.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1271, 28, 28, 2016462461, '19550108 11:50:06.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1272, 91, 91, 417103460, '19920112 15:40:19.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1273, 788, 788, 1886890980, '19780430 12:18:51.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1274, 841, 841, 1780359025, '19901101 00:06:41.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1275, 433, 433, 1086161903, '19851115 13:31:00.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1276, 386, 386, 1091580202, '20010925 18:46:55.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1277, 253, 253, 1912513506, '19820701 08:33:10.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1278, 373, 373, 179981952, '19970822 03:45:38.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1279, 517, 517, 1331208594, '19650709 23:36:12.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1280, 827, 827, 568488007, '19840409 21:20:46.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1281, 253, 253, 1145966509, '19710111 21:37:07.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1282, 27, 27, 1099527523, '19930501 14:30:02.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1283, 440, 440, 1431302765, '19991013 13:26:51.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1284, 54, 54, 2058148686, '19630703 11:03:57.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1285, 891, 891, 1769043918, '19750210 06:07:30.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1286, 132, 132, 563836529, '19990318 07:25:07.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1287, 566, 566, 1981458011, '20030902 22:45:09.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1288, 459, 459, 446045011, '19560312 15:02:04.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1289, 903, 903, 914812870, '19950210 11:08:41.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1290, 965, 965, 473865707, '19611013 10:19:52.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1291, 720, 720, 1566204920, '19940320 03:07:30.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1292, 36, 36, 1548125678, '19831114 18:40:26.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1293, 453, 453, 96337538, '19880203 16:15:26.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1294, 139, 139, 700319294, '19651013 20:21:13.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1295, 186, 186, 1389063749, '19741011 16:30:58.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1296, 181, 181, 1187721874, '19760815 09:34:17.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1297, 955, 955, 1256695566, '19831209 21:50:02.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1298, 45, 45, 1834269995, '19591010 20:15:47.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1299, 274, 274, 1973424680, '19671222 06:28:01.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1300, 95, 95, 160865012, '19740626 23:57:48.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1301, 316, 316, 1029651838, '19661223 19:30:05.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1302, 252, 252, 1198609102, '19860527 15:14:01.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1303, 932, 932, 1976842899, '19540222 10:56:46.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1304, 392, 392, 1357694597, '19720721 06:33:59.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1305, 968, 968, 809221868, '20020605 10:44:54.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1306, 219, 219, 477556936, '19700401 09:29:19.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1307, 575, 575, 307316897, '19741116 12:00:50.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1308, 421, 421, 1153200924, '19920424 10:56:37.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1309, 172, 172, 2088799801, '19861027 12:27:00.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1310, 132, 132, 940024508, '19821107 01:42:11.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1311, 24, 24, 1984483228, '19780327 03:23:13.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1312, 57, 57, 1313096105, '19931108 19:05:36.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1313, 490, 490, 1626925548, '19550527 16:15:40.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1314, 230, 230, 2124811056, '19631207 16:03:57.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1315, 82, 82, 1281602013, '19960529 18:53:27.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1316, 577, 577, 98956050, '19590926 22:56:24.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1317, 449, 449, 1978031018, '19840715 13:46:37.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1318, 950, 950, 1804703350, '19901120 18:54:28.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1319, 307, 307, 853479497, '20020306 11:59:15.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1320, 216, 216, 277191281, '20030903 15:59:17.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1321, 5, 5, 898220364, '19701128 13:29:25.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1322, 60, 60, 172026585, '19990907 21:58:22.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1323, 71, 71, 174598953, '19850827 11:28:28.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1324, 155, 155, 1467187167, '19880226 13:09:53.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1325, 55, 55, 903061592, '19580213 15:53:31.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1326, 69, 69, 1659446298, '19790223 17:09:39.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1327, 895, 895, 1559283095, '19561209 23:24:52.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1328, 574, 574, 308014356, '19650718 15:57:37.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1329, 291, 291, 2060070360, '19690121 07:35:42.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1330, 505, 505, 2013298052, '19620403 03:56:43.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1331, 661, 661, 119770181, '19701018 20:56:53.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1332, 313, 313, 337476867, '19750922 12:17:22.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1333, 241, 241, 1265730940, '19820901 21:17:36.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1334, 193, 193, 316504782, '19990112 23:38:23.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1335, 257, 257, 1146724425, '19700419 01:50:41.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1336, 744, 744, 1654316402, '19920816 06:23:06.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1337, 909, 909, 1824550835, '19920310 03:33:15.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1338, 978, 978, 277434750, '19800325 06:52:51.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1339, 178, 178, 2137681902, '19690127 00:19:02.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1340, 379, 379, 1522056938, '19571111 20:38:10.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1341, 937, 937, 1043410318, '19770502 19:24:16.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1342, 110, 110, 586845198, '19640511 11:48:32.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1343, 148, 148, 485028290, '19770516 02:35:03.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1344, 855, 855, 1201612742, '19650406 09:26:30.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1345, 819, 819, 768578397, '19910719 06:56:39.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1346, 240, 240, 645806433, '19530512 08:01:54.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1347, 619, 619, 1263313145, '19810621 02:24:46.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1348, 59, 59, 1239617239, '19770228 00:11:29.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1349, 372, 372, 233225748, '19770516 01:27:46.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1350, 695, 695, 1043442080, '19680116 17:34:34.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1351, 346, 346, 1990184068, '20000130 19:47:39.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1352, 67, 67, 693972986, '20010118 02:55:33.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1353, 134, 134, 1526600335, '19650805 08:16:32.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1354, 910, 910, 1822739763, '19720418 14:28:44.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1355, 512, 512, 1292224853, '19560730 10:28:23.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1356, 982, 982, 1309461885, '19750416 21:31:42.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1357, 35, 35, 597163995, '19531001 03:45:32.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1358, 583, 583, 1522734545, '19731226 21:46:53.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1359, 860, 860, 860520706, '19670608 14:40:28.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1360, 778, 778, 1722613887, '19760412 17:35:15.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1361, 462, 462, 145098221, '19660423 10:04:26.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1362, 417, 417, 1537854255, '19620501 20:03:13.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1363, 10, 10, 1458485255, '19760404 03:20:54.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1364, 810, 810, 1843842754, '19770727 08:30:17.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1365, 555, 555, 2069220313, '19650120 04:45:17.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1366, 94, 94, 432813877, '19550706 15:05:50.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1367, 904, 904, 2134249326, '19640309 21:47:59.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1368, 990, 990, 1184287664, '19640513 16:37:16.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1369, 462, 462, 1292290781, '20010819 14:13:16.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1370, 101, 101, 1942794059, '19561014 12:30:20.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1371, 538, 538, 200277292, '19750718 19:23:09.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1372, 307, 307, 328983937, '19570423 06:53:46.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1373, 43, 43, 804418675, '19741025 18:35:26.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1374, 221, 221, 107466619, '19970425 15:21:04.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1375, 714, 714, 131766405, '19790504 20:35:00.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1376, 227, 227, 1409666246, '19590721 01:40:52.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1377, 47, 47, 984742420, '19750425 19:23:19.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1378, 99, 99, 2135210219, '19730524 02:01:57.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1379, 651, 651, 1307540702, '20040809 06:13:13.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1380, 385, 385, 1026742022, '19571005 07:51:36.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1381, 975, 975, 2077156764, '20050903 14:44:52.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1382, 292, 292, 1549934348, '19871011 23:13:23.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1383, 988, 988, 282902676, '19960303 14:29:50.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1384, 885, 885, 2056239260, '19540106 14:01:08.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1385, 381, 381, 1949505949, '19700303 08:09:03.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1386, 64, 64, 936636933, '19770929 18:01:19.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1387, 163, 163, 380956201, '19630218 04:00:50.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1388, 237, 237, 1991029411, '19630517 06:48:52.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1389, 308, 308, 1196721886, '19950616 13:10:00.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1390, 275, 275, 701383756, '20030102 04:18:58.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1391, 306, 306, 754811833, '19911117 02:37:33.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1392, 798, 798, 508392722, '19710316 05:46:22.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1393, 929, 929, 2058720757, '19651018 16:12:22.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1394, 170, 170, 836672614, '20001015 06:43:47.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1395, 320, 320, 1691702017, '19990721 10:33:43.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1396, 642, 642, 1246104964, '19680102 23:21:03.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1397, 871, 871, 1065588233, '19960206 08:27:37.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1398, 53, 53, 40733630, '20071019 18:04:45.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1399, 699, 699, 746250794, '19530318 04:41:13.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1400, 366, 366, 843700990, '19890113 15:50:49.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1401, 99, 99, 1697400310, '19970303 10:13:55.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1402, 25, 25, 1548722701, '19700208 09:47:30.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1403, 368, 368, 326883321, '19830711 09:58:12.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1404, 747, 747, 1807118067, '19730802 12:57:25.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1405, 11, 11, 1617227332, '20000629 22:11:25.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1406, 337, 337, 2073207880, '19951010 12:53:53.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1407, 420, 420, 634270445, '19790327 08:20:07.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1408, 318, 318, 77534848, '19760410 16:55:27.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1409, 79, 79, 249933857, '20001214 17:53:43.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1410, 414, 414, 1133826211, '20050109 08:47:30.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1411, 166, 166, 327417759, '20071222 02:08:23.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1412, 378, 378, 1400414517, '19880509 01:43:35.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1413, 947, 947, 129532439, '19770518 15:33:39.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1414, 223, 223, 1223655959, '19620904 06:48:52.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1415, 445, 445, 1874408031, '19780809 02:50:22.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1416, 54, 54, 1944952667, '19860712 10:51:03.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1417, 410, 410, 786808382, '19740227 05:33:22.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1418, 335, 335, 1376154493, '19750330 13:19:45.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1419, 508, 508, 1055624950, '19600527 00:21:14.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1420, 50, 50, 635343404, '19950422 10:43:58.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1421, 739, 739, 444868338, '20000517 11:05:00.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1422, 212, 212, 943751101, '19531025 14:59:07.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1423, 172, 172, 937624510, '19761127 03:27:16.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1424, 615, 615, 1114731782, '20060817 09:54:17.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1425, 319, 319, 368549880, '19721127 09:52:47.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1426, 568, 568, 1881724968, '19570505 13:36:46.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1427, 999, 999, 360389821, '19990705 14:41:23.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1428, 381, 381, 1760831282, '19820109 11:51:39.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1429, 702, 702, 1842453271, '20051005 04:47:20.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1430, 188, 188, 45064667, '19640502 21:13:57.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1431, 226, 226, 1045799262, '19710613 17:13:15.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1432, 416, 416, 1749626089, '19750707 01:18:33.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1433, 303, 303, 1139039327, '19730308 21:20:44.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1434, 387, 387, 1577374810, '19680727 14:22:24.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1435, 679, 679, 107742200, '19680802 21:37:41.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1436, 18, 18, 581541507, '19880727 04:57:22.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1437, 208, 208, 73720991, '19570403 13:15:10.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1438, 298, 298, 2020941405, '19750803 01:32:24.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1439, 314, 314, 894711339, '19610708 15:49:43.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1440, 588, 588, 395936556, '19820523 19:15:09.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1441, 460, 460, 384346405, '20060705 09:41:12.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1442, 961, 961, 1507420232, '19941108 11:05:23.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1443, 382, 382, 448744843, '19700601 12:55:08.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1444, 43, 43, 290066503, '19980606 04:22:29.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1445, 340, 340, 1473080557, '20030111 02:10:34.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1446, 236, 236, 1676059872, '19560711 00:53:58.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1447, 749, 749, 1858752699, '20011027 14:12:42.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1448, 149, 149, 294562148, '19560215 09:23:29.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1449, 204, 204, 841623706, '19750310 03:53:24.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1450, 159, 159, 383845656, '19660109 23:42:40.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1451, 623, 623, 48094448, '20011104 17:47:40.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1452, 844, 844, 78774678, '19731109 19:07:52.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1453, 113, 113, 1104232512, '20000525 17:43:31.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1454, 724, 724, 1110276417, '19651125 17:57:14.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1455, 355, 355, 2001735480, '19960828 04:45:48.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1456, 262, 262, 1209348620, '19960509 19:14:47.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1457, 550, 550, 1016746968, '20010313 06:20:13.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1458, 288, 288, 1474561354, '19841123 00:03:55.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1459, 174, 174, 839172515, '19530906 03:04:38.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1460, 836, 836, 440749771, '19960226 08:34:38.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1461, 833, 833, 1939988886, '20040405 22:12:30.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1462, 442, 442, 1046295547, '20050316 20:28:27.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1463, 358, 358, 977140570, '19780706 12:08:06.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1464, 139, 139, 1446334530, '19890814 20:39:46.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1465, 125, 125, 452475699, '19860730 15:34:15.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1466, 509, 509, 1753687211, '19850618 00:49:49.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1467, 399, 399, 2102808328, '19680303 04:22:51.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1468, 243, 243, 536976527, '19611023 01:11:15.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1469, 281, 281, 658536129, '20020131 09:11:11.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1470, 229, 229, 1321226338, '19980111 21:53:00.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1471, 660, 660, 201879175, '19820410 21:35:52.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1472, 863, 863, 368769804, '20060727 04:00:57.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1473, 299, 299, 147831407, '19660922 21:30:02.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1474, 708, 708, 1373805544, '19860103 05:06:04.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1475, 516, 516, 1094847586, '19961017 01:02:45.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1476, 725, 725, 273943539, '19580710 23:59:37.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1477, 224, 224, 1460235110, '19910526 02:03:59.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1478, 23, 23, 232568484, '19860622 23:04:46.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1479, 725, 725, 413241326, '19560807 07:43:42.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1480, 447, 447, 90328612, '19690520 08:58:49.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1481, 154, 154, 1425270504, '19630708 23:24:04.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1482, 40, 40, 1912865550, '19960521 05:17:53.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1483, 685, 685, 1242564214, '19591103 07:58:14.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1484, 317, 317, 1241907743, '19920926 05:04:40.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1485, 507, 507, 1076982836, '19700628 03:26:16.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1486, 154, 154, 1423789499, '20050802 22:05:22.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1487, 922, 922, 1224166370, '19830210 07:35:02.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1488, 380, 380, 1506263086, '19600414 03:23:29.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1489, 875, 875, 760562883, '19791129 09:36:35.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1490, 832, 832, 1021428005, '19800325 02:22:53.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1491, 296, 296, 1982561667, '19950515 22:37:08.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1492, 57, 57, 516752047, '19800510 13:11:15.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1493, 169, 169, 1260517352, '19741127 22:27:46.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1494, 760, 760, 1994277330, '19730512 07:15:05.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1495, 882, 882, 797046650, '19860215 21:02:39.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1496, 1, 1, 1488431796, '19540421 13:12:44.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1497, 450, 450, 1591476203, '19690505 22:57:16.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1498, 58, 58, 450653087, '19881016 16:15:21.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1499, 521, 521, 1705387214, '19641107 12:48:14.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1500, 146, 146, 618251434, '19700727 01:20:44.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1501, 15, 15, 763080295, '19960508 20:31:06.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1502, 193, 193, 831978270, '19930105 13:01:26.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1503, 707, 707, 336121370, '19620114 00:12:18.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1504, 884, 884, 1831268156, '19850225 06:00:24.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1505, 735, 735, 1100474642, '19911001 02:06:47.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1506, 324, 324, 1841801123, '20030410 13:46:42.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1507, 367, 367, 35570906, '19600218 21:37:50.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1508, 835, 835, 1013470894, '19670524 12:37:26.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1509, 209, 209, 2122674750, '19770207 16:54:29.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1510, 46, 46, 1902127961, '19910219 04:46:11.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1511, 486, 486, 1018215679, '19571216 17:10:48.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1512, 818, 818, 1883656641, '19670922 07:15:26.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1513, 505, 505, 15319086, '19810418 16:20:12.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1514, 431, 431, 710570838, '19920417 18:05:49.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1515, 218, 218, 964714101, '19850821 09:10:34.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1516, 346, 346, 162425620, '19601116 16:54:36.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1517, 132, 132, 1095126756, '19980515 12:30:13.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1518, 173, 173, 692887722, '19931009 07:03:26.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1519, 939, 939, 114175413, '19720218 17:13:29.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1520, 911, 911, 2017203012, '19881226 17:28:52.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1521, 214, 214, 985531824, '19550509 17:14:47.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1522, 251, 251, 530662218, '19601120 00:59:21.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1523, 687, 687, 1908693228, '19891123 15:34:40.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1524, 584, 584, 1507775513, '19741109 06:48:17.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1525, 339, 339, 781877186, '19550830 23:15:51.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1526, 344, 344, 1438232490, '19541202 08:24:37.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1527, 383, 383, 64833961, '19840830 05:33:11.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1528, 90, 90, 2041806269, '20010512 14:14:56.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1529, 201, 201, 1624764384, '20071117 08:03:25.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1530, 824, 824, 1784537611, '19950629 11:50:00.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1531, 318, 318, 1703205582, '19970306 01:02:20.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1532, 869, 869, 454650894, '19550810 09:48:37.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1533, 396, 396, 1608121014, '19740816 10:16:32.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1534, 194, 194, 911501580, '19940111 06:22:57.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1535, 4, 4, 2058527128, '19810110 12:27:42.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1536, 272, 272, 1411857778, '19780702 10:22:37.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1537, 322, 322, 711677279, '19870509 05:05:35.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1538, 39, 39, 1382359706, '19820907 01:57:50.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1539, 180, 180, 1202757538, '19531228 02:57:52.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1540, 37, 37, 1860246200, '19750319 13:39:33.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1541, 161, 161, 1665034851, '19980616 00:27:32.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1542, 446, 446, 493575656, '19680919 18:57:36.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1543, 283, 283, 1875832474, '19910306 10:28:59.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1544, 581, 581, 385539273, '19961009 04:50:23.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1545, 173, 173, 865599392, '19750410 09:12:04.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1546, 358, 358, 1003140095, '19800824 15:21:41.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1547, 426, 426, 849282713, '20070121 20:51:02.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1548, 464, 464, 1785902499, '19900811 04:21:56.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1549, 738, 738, 1720809573, '19950920 22:04:31.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1550, 451, 451, 1517846061, '19780402 04:08:03.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1551, 152, 152, 1004917270, '19631207 00:42:31.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1552, 123, 123, 1388535140, '19830727 15:53:50.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1553, 518, 518, 1134774063, '19690829 23:01:52.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1554, 363, 363, 1111538761, '19831111 19:19:21.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1555, 636, 636, 1897528270, '19680320 08:05:57.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1556, 487, 487, 1657034140, '19880619 19:31:45.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1557, 587, 587, 2128770692, '19560213 21:26:46.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1558, 759, 759, 1268484355, '19950306 17:24:00.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1559, 254, 254, 1452813009, '19820629 06:44:32.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1560, 199, 199, 1279286322, '19891030 17:42:10.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1561, 402, 402, 1603971809, '19710812 08:13:31.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1562, 408, 408, 1801305257, '19661008 07:22:53.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1563, 227, 227, 631955088, '19670707 04:34:00.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1564, 849, 849, 2036702565, '19890811 05:04:28.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1565, 589, 589, 1995826500, '20011215 03:43:51.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1566, 170, 170, 783958460, '19550510 07:22:11.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1567, 38, 38, 1534810800, '20010204 20:58:53.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1568, 841, 841, 1832275211, '19950407 09:57:14.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1569, 701, 701, 1970024518, '19640408 05:38:07.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1570, 433, 433, 351152712, '19600220 22:47:56.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1571, 415, 415, 1547399607, '19810710 11:49:01.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1572, 783, 783, 1286419627, '19680908 10:32:22.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1573, 205, 205, 1232737400, '19921013 04:05:33.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1574, 799, 799, 1284229768, '20041201 03:33:55.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1575, 920, 920, 2110166744, '19980712 17:01:20.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1576, 451, 451, 1971126934, '19940818 22:17:11.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1577, 481, 481, 1477383605, '19770915 14:30:22.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1578, 165, 165, 1233772531, '20010213 10:44:16.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1579, 845, 845, 155493783, '20070731 21:36:13.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1580, 59, 59, 1485173086, '19830105 07:54:09.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1581, 533, 533, 712148103, '19551110 11:33:34.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1582, 176, 176, 473680342, '20020119 01:11:14.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1583, 923, 923, 258570030, '19580823 16:10:25.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1584, 828, 828, 1332264017, '19820818 03:55:22.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1585, 452, 452, 1272388107, '19840724 11:07:29.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1586, 247, 247, 404153123, '19660811 09:08:30.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1587, 991, 991, 887119533, '19931210 10:46:45.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1588, 211, 211, 1696750332, '19981004 14:57:11.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1589, 14, 14, 2134323660, '19781024 22:17:00.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1590, 569, 569, 1157243658, '20000722 16:55:57.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1591, 327, 327, 64146978, '19750520 12:55:50.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1592, 167, 167, 831786006, '20000303 11:41:34.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1593, 310, 310, 791481698, '19530311 19:13:18.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1594, 786, 786, 883812345, '19720227 09:15:42.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1595, 195, 195, 63787023, '19560808 05:26:02.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1596, 690, 690, 1248458986, '19840908 17:04:39.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1597, 808, 808, 684483093, '19540317 14:23:35.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1598, 549, 549, 884057530, '19540727 05:24:31.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1599, 513, 513, 360263263, '20021025 01:06:32.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1600, 263, 263, 638242694, '19721202 01:49:53.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1601, 463, 463, 1239972011, '19870719 18:22:47.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1602, 579, 579, 931366843, '19641017 10:53:48.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1603, 550, 550, 449557594, '19790505 22:43:49.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1604, 347, 347, 1698717705, '19880731 23:16:34.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1605, 654, 654, 1945654907, '20040921 16:19:01.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1606, 872, 872, 117533275, '20030329 14:10:09.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1607, 722, 722, 11205929, '19981013 11:48:16.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1608, 85, 85, 21161747, '19720927 19:27:56.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1609, 816, 816, 408797793, '19930501 02:18:01.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1610, 212, 212, 638118913, '19810801 09:48:46.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1611, 332, 332, 200208420, '19631006 05:01:22.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1612, 321, 321, 51169614, '19621230 10:42:30.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1613, 75, 75, 1074543814, '19950805 19:47:47.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1614, 496, 496, 810877537, '20070623 22:50:22.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1615, 536, 536, 535417807, '19861222 06:08:35.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1616, 261, 261, 1724436856, '19770723 07:02:16.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1617, 359, 359, 1188877029, '19610215 15:12:27.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1618, 211, 211, 1998287813, '19921118 00:38:38.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1619, 830, 830, 594177325, '19580118 17:56:59.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1620, 843, 843, 995872480, '19880506 18:35:20.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1621, 381, 381, 73048865, '19690529 20:35:26.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1622, 466, 466, 832023875, '19550505 06:01:42.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1623, 436, 436, 66981897, '19690614 11:40:14.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1624, 452, 452, 1908934543, '19710916 12:42:09.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1625, 112, 112, 244682238, '19921002 09:52:05.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1626, 340, 340, 24444269, '19890508 00:07:26.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1627, 277, 277, 940828141, '19680630 15:19:03.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1628, 36, 36, 704258883, '19730817 18:49:51.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1629, 528, 528, 1450356022, '20010425 22:07:53.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1630, 770, 770, 1933845600, '19661103 23:56:40.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1631, 917, 917, 17602191, '19930604 07:53:36.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1632, 756, 756, 862690543, '19760221 09:05:51.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1633, 312, 312, 781096093, '20060421 09:37:44.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1634, 143, 143, 821263324, '19870830 19:46:19.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1635, 794, 794, 1617334567, '20030618 13:31:21.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1636, 958, 958, 1407573390, '19990123 00:39:46.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1637, 988, 988, 179861434, '19750917 02:27:24.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1638, 562, 562, 737448179, '19780121 16:53:51.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1639, 628, 628, 2124398685, '19851125 11:36:20.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1640, 200, 200, 1523885178, '19890426 20:57:19.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1641, 500, 500, 989590809, '19751028 21:20:14.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1642, 33, 33, 1770890340, '19740315 15:18:48.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1643, 965, 965, 1066788805, '19580606 12:39:10.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1644, 498, 498, 714610260, '20050324 12:28:14.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1645, 186, 186, 455255623, '19891018 11:54:34.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1646, 552, 552, 766205760, '19650522 02:13:20.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1647, 516, 516, 1742171745, '19570730 15:53:47.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1648, 284, 284, 1128342046, '19530917 20:22:55.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1649, 146, 146, 1376986549, '19930307 03:07:06.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1650, 155, 155, 226031692, '19870115 22:23:45.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1651, 331, 331, 1702108605, '19760725 01:52:11.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1652, 451, 451, 548730921, '19690428 13:44:56.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1653, 89, 89, 1189940308, '20040707 11:27:31.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1654, 348, 348, 2069073910, '19670621 06:31:11.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1655, 20, 20, 1983444354, '19560706 05:14:26.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1656, 319, 319, 2118165811, '19850317 12:21:05.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1657, 108, 108, 1456553816, '20030504 23:13:34.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1658, 645, 645, 1768173128, '19600820 10:01:39.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1659, 137, 137, 1622844735, '20031029 13:24:28.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1660, 83, 83, 1956193141, '19680517 16:11:35.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1661, 439, 439, 968710083, '19870929 22:51:05.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1662, 603, 603, 1603564201, '19780226 16:58:24.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1663, 184, 184, 1134635575, '19790603 21:20:03.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1664, 545, 545, 458277765, '19790629 02:21:20.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1665, 520, 520, 729717167, '19960227 01:55:09.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1666, 475, 475, 1017749152, '19950814 19:55:31.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1667, 129, 129, 965493782, '19640909 01:04:45.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1668, 984, 984, 434668247, '19601207 00:01:27.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1669, 208, 208, 257008436, '19570106 09:19:01.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1670, 33, 33, 27904521, '19951129 05:28:49.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1671, 531, 531, 2061117766, '19541107 04:34:52.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1672, 15, 15, 1180661971, '19910124 22:18:36.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1673, 417, 417, 67475198, '19591225 13:02:17.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1674, 333, 333, 736202840, '19760922 20:59:40.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1675, 5, 5, 321402455, '19650710 21:15:06.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1676, 587, 587, 1352822265, '20030316 05:16:37.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1677, 206, 206, 456395880, '20041127 17:22:31.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1678, 651, 651, 1511306706, '19720321 23:11:59.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1679, 578, 578, 1304545411, '19891128 00:47:35.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1680, 987, 987, 674005804, '19800514 07:38:45.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1681, 91, 91, 850060482, '19841009 08:13:39.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1682, 177, 177, 1537747794, '19671014 18:56:08.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1683, 364, 364, 45747128, '19880611 15:42:44.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1684, 636, 636, 1459891633, '19670411 23:44:07.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1685, 323, 323, 1101073490, '19980410 22:04:29.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1686, 264, 264, 2118887427, '19770206 18:08:39.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1687, 555, 555, 923079985, '19790815 21:38:19.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1688, 638, 638, 1617529053, '19911102 03:06:33.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1689, 239, 239, 1276419321, '19840225 14:31:53.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1690, 493, 493, 365880845, '19710404 01:10:16.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1691, 87, 87, 1423425256, '20030920 01:26:11.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1692, 774, 774, 231487827, '19680128 16:25:44.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1693, 411, 411, 770970151, '19820325 03:29:22.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1694, 334, 334, 1080997305, '19700711 19:24:45.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1695, 469, 469, 530018150, '19540729 22:06:13.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1696, 225, 225, 907278539, '20050831 04:21:49.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1697, 664, 664, 163217272, '20021011 17:58:45.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1698, 232, 232, 1091609925, '19861208 10:17:50.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1699, 976, 976, 1234578612, '19620127 10:33:04.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1700, 269, 269, 1727729610, '20020305 15:59:02.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1701, 441, 441, 573011596, '19530913 01:08:34.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1702, 486, 486, 25306987, '20040822 15:52:19.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1703, 564, 564, 704940722, '20040910 11:03:54.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1704, 337, 337, 1103567395, '20050407 21:38:17.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1705, 895, 895, 1993985200, '19850311 17:48:53.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1706, 615, 615, 957650184, '19930630 03:33:35.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1707, 386, 386, 1324247792, '19620506 00:42:39.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1708, 309, 309, 920707363, '19801015 14:27:51.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1709, 134, 134, 1779710197, '19541212 09:16:04.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1710, 374, 374, 1048663079, '19610421 23:57:48.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1711, 628, 628, 1486308904, '19880418 18:58:33.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1712, 400, 400, 2079025235, '19840214 00:01:34.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1713, 349, 349, 943040675, '19780922 09:14:04.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1714, 454, 454, 1549543262, '19760617 05:45:43.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1715, 421, 421, 450708051, '19910808 07:57:56.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1716, 807, 807, 1466835839, '19721217 03:54:57.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1717, 328, 328, 732406066, '19970916 01:15:40.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1718, 437, 437, 252974687, '19650222 21:35:55.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1719, 758, 758, 434995456, '19890319 04:16:50.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1720, 495, 495, 1782502323, '19720121 07:46:30.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1721, 652, 652, 154320309, '19681230 22:17:12.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1722, 304, 304, 2018941354, '19801106 21:58:12.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1723, 680, 680, 1277954623, '19841011 09:29:34.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1724, 61, 61, 1755344119, '19931004 08:08:44.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1725, 691, 691, 1463826772, '20000310 04:02:37.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1726, 675, 675, 193072976, '19941009 12:09:07.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1727, 338, 338, 246009013, '19611102 18:49:14.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1728, 179, 179, 1247618806, '19970614 17:37:32.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1729, 353, 353, 69267865, '19750225 22:53:26.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1730, 495, 495, 1398215610, '19671109 16:53:16.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1731, 447, 447, 1638178486, '19530605 11:17:52.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1732, 956, 956, 646589702, '19701222 07:04:41.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1733, 172, 172, 509459937, '19630224 12:38:55.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1734, 306, 306, 32113889, '19950924 08:48:33.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1735, 170, 170, 668786656, '19790902 06:30:10.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1736, 690, 690, 1118979243, '19880216 16:21:19.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1737, 77, 77, 1898909343, '19710202 07:52:14.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1738, 536, 536, 840673903, '19910305 18:04:27.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1739, 389, 389, 277008979, '19900130 05:55:13.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1740, 926, 926, 1650298075, '19571012 18:30:54.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1741, 385, 385, 568564955, '19671005 17:26:00.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1742, 591, 591, 1364941127, '20061030 07:10:28.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1743, 163, 163, 849777013, '19891121 17:50:28.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1744, 179, 179, 404125043, '19751106 14:34:04.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1745, 258, 258, 52460067, '19901215 06:11:42.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1746, 236, 236, 1630419399, '19720807 01:24:37.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1747, 17, 17, 1136347098, '19970507 07:11:40.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1748, 30, 30, 228161266, '19581008 21:43:39.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1749, 442, 442, 396340707, '19861203 11:26:48.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1750, 371, 371, 1169549532, '19890811 18:11:16.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1751, 521, 521, 110644694, '19601216 03:06:09.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1752, 295, 295, 720152919, '19900819 20:22:49.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1753, 656, 656, 711353103, '20050920 06:00:59.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1754, 923, 923, 2096708546, '19980106 02:46:34.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1755, 438, 438, 731569567, '19860306 17:41:50.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1756, 126, 126, 964193168, '19801105 03:10:22.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1757, 168, 168, 308890467, '19721112 06:22:45.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1758, 489, 489, 1898070541, '19631209 02:55:09.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1759, 642, 642, 942657985, '19580129 17:35:40.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1760, 26, 26, 1739128557, '19980602 05:39:46.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1761, 111, 111, 977132907, '19840828 08:44:21.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1762, 887, 887, 1255290945, '19721120 07:05:06.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1763, 790, 790, 1256430822, '19580821 15:34:24.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1764, 674, 674, 1330025861, '19950201 16:22:47.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1765, 655, 655, 2064448263, '19601117 12:39:56.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1766, 949, 949, 295645448, '19700429 11:53:52.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1767, 850, 850, 824355715, '19731221 11:22:39.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1768, 856, 856, 689637396, '19901229 00:25:31.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1769, 294, 294, 1511152109, '20041014 23:15:32.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1770, 692, 692, 2000502560, '19560622 15:36:37.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1771, 12, 12, 1816960346, '19541115 20:02:43.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1772, 369, 369, 576076674, '19590715 07:11:12.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1773, 549, 549, 195063018, '19830123 15:40:41.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1774, 731, 731, 1753712874, '19840607 09:45:56.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1775, 196, 196, 371331979, '19570418 14:20:30.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1776, 767, 767, 707687290, '19700302 15:06:43.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1777, 435, 435, 1537944901, '19981216 04:07:43.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1778, 68, 68, 658621811, '19611206 18:55:30.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1779, 377, 377, 1444705943, '19551022 01:57:02.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1780, 712, 712, 531970623, '19800804 02:38:00.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1781, 594, 594, 768246327, '20050603 04:57:26.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1782, 649, 649, 925518801, '19560126 21:05:35.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1783, 62, 62, 985975059, '19630714 06:10:43.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1784, 375, 375, 663847790, '19930716 04:42:09.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1785, 66, 66, 333090372, '19591125 13:47:07.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1786, 238, 238, 1483510834, '19701016 14:55:02.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1787, 759, 759, 715016247, '19730402 01:03:41.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1788, 687, 687, 871380057, '19730217 09:52:21.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1789, 553, 553, 1851609027, '19620721 15:06:42.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1790, 394, 394, 388209273, '20061027 03:19:48.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1791, 728, 728, 1141669434, '19680406 09:58:33.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1792, 202, 202, 1817244585, '19600226 04:57:04.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1793, 63, 63, 1517358664, '19860205 00:28:47.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1794, 228, 228, 797060044, '19990830 00:15:27.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1795, 370, 370, 1797190960, '19810213 09:46:33.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1796, 611, 611, 1729183154, '20021114 10:20:53.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1797, 371, 371, 975032093, '20010311 15:36:03.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1798, 981, 981, 643310219, '20021020 01:27:41.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1799, 524, 524, 1529947944, '19671221 01:54:07.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1800, 458, 458, 1141393676, '19730818 18:17:50.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1801, 472, 472, 1634987049, '20060816 14:01:57.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1802, 525, 525, 729060230, '19590511 06:46:09.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1803, 557, 557, 1736115899, '19611223 22:28:06.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1804, 432, 432, 966810771, '19830613 19:50:11.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1805, 568, 568, 1052134346, '19870927 22:08:33.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1806, 623, 623, 16560603, '19540604 19:54:56.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1807, 51, 51, 1325070471, '19600728 04:42:07.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1808, 482, 482, 1694046096, '19740415 20:15:03.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1809, 462, 462, 92706820, '19930920 12:26:03.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1810, 396, 396, 1236367690, '19690105 02:35:06.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1811, 259, 259, 1239465666, '19891119 23:02:39.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1812, 397, 397, 23425103, '19631208 11:27:15.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1813, 212, 212, 857518063, '19610218 00:58:07.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1814, 999, 999, 1539376826, '19850626 14:57:39.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1815, 600, 600, 1646511677, '20001229 00:42:19.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1816, 687, 687, 1183457300, '19810803 11:38:46.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1817, 278, 278, 41521347, '19620511 00:54:22.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1818, 805, 805, 1871529986, '19730204 10:52:15.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1819, 208, 208, 99988775, '19880310 02:35:40.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1820, 599, 599, 743810721, '19980131 21:44:53.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1821, 9, 9, 868233129, '20050126 10:50:10.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1822, 392, 392, 1077551211, '19531104 01:30:18.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1823, 186, 186, 1786842078, '19810610 09:18:48.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1824, 107, 107, 750779560, '20051218 19:55:44.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1825, 49, 49, 381775419, '19960317 05:38:03.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1826, 529, 529, 1258871225, '20020919 15:05:38.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1827, 722, 722, 1876280222, '19810608 06:42:25.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1828, 877, 877, 803192483, '19910526 15:25:14.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1829, 256, 256, 1077113945, '19560424 23:59:23.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1830, 397, 397, 1762004639, '19620604 03:59:36.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1831, 760, 760, 1483558445, '19761220 23:30:40.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1832, 374, 374, 38026409, '20040226 02:40:01.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1833, 646, 646, 1886966585, '20011216 17:01:22.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1834, 573, 573, 687386860, '19900304 07:39:11.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1835, 792, 792, 541308526, '19811218 12:36:02.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1836, 946, 946, 912950369, '19990124 22:11:17.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1837, 721, 721, 2047212213, '20020201 22:37:28.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1838, 267, 267, 430181675, '19880130 10:20:32.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1839, 246, 246, 693074043, '19581020 06:33:35.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1840, 839, 839, 1478495193, '19580623 17:52:10.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1841, 31, 31, 487484164, '19630322 10:12:55.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1842, 84, 84, 1575713966, '20061217 04:48:38.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1843, 249, 249, 2072293431, '19870529 21:26:17.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1844, 395, 395, 989835460, '20010715 12:31:36.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1845, 794, 794, 21289972, '19691208 04:17:09.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1846, 670, 670, 989133353, '19570430 22:31:18.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1847, 273, 273, 1298237017, '20070107 03:58:57.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1848, 440, 440, 841651404, '19530812 09:31:07.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1849, 698, 698, 1927448392, '20060830 23:33:07.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1850, 892, 892, 1134767783, '20070712 22:07:47.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1851, 198, 198, 2071736400, '19930707 09:26:31.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1852, 41, 41, 690203202, '19810205 04:43:48.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1853, 854, 854, 1282579294, '19670812 22:52:01.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1854, 836, 836, 746923936, '19771118 04:09:14.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1855, 748, 748, 1796248334, '19760722 07:27:40.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1856, 835, 835, 1943583926, '20051013 12:31:38.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1857, 620, 620, 2043300071, '19851129 21:27:20.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1858, 955, 955, 1314380141, '19640105 02:32:21.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1859, 790, 790, 1632245835, '19950328 14:12:07.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1860, 924, 924, 2134342155, '19930107 07:02:54.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1861, 501, 501, 1160571919, '19801226 13:12:30.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1862, 299, 299, 1097101077, '19770303 13:16:53.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1863, 42, 42, 626026613, '19701222 20:15:40.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1864, 523, 523, 1991265448, '19840419 08:26:27.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1865, 346, 346, 1436112055, '20000115 03:04:26.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1866, 820, 820, 429921061, '19930923 20:22:37.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1867, 981, 981, 1188781861, '19691221 18:25:52.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1868, 993, 993, 18636707, '19781216 17:18:55.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1869, 851, 851, 833875536, '19560710 02:21:36.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1870, 276, 276, 5696541, '19541206 02:31:02.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1871, 47, 47, 1703834784, '19870701 13:01:17.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1872, 581, 581, 1056352693, '19820410 14:33:49.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1873, 676, 676, 1898661649, '19670419 04:18:39.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1874, 812, 812, 1851108654, '19820918 08:43:29.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1875, 604, 604, 737603192, '19871113 11:31:58.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1876, 278, 278, 339729647, '20060209 06:01:31.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1877, 803, 803, 968459234, '19740607 04:04:01.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1878, 849, 849, 1529275524, '19871125 20:47:11.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1879, 511, 511, 776208095, '19890111 15:38:35.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1880, 120, 120, 288072485, '19911118 07:06:45.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1881, 867, 867, 800671216, '20030912 11:06:41.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1882, 158, 158, 154292483, '19801027 21:11:18.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1883, 400, 400, 797056929, '19920924 15:52:06.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1884, 751, 751, 287733913, '19561013 01:51:36.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1885, 23, 23, 1875025302, '19761126 18:33:04.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1886, 581, 581, 1694029073, '20031114 18:46:52.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1887, 124, 124, 235470795, '19890717 03:48:00.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1888, 275, 275, 1373704600, '19770129 12:52:08.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1889, 723, 723, 1776553839, '19660814 00:11:31.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1890, 51, 51, 913506632, '19840306 00:04:23.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1891, 135, 135, 1931466841, '19660226 00:43:56.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1892, 762, 762, 1013187015, '19910129 20:05:07.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1893, 814, 814, 309226392, '20001102 20:08:24.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1894, 796, 796, 1256985917, '19731011 23:30:40.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1895, 918, 918, 1554339521, '19850627 04:39:39.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1896, 72, 72, 2115314595, '19940117 20:56:01.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1897, 971, 971, 1704143727, '19881226 08:32:58.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1898, 419, 419, 465113230, '19560210 12:59:49.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1899, 27, 27, 1043620725, '19540702 09:27:10.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1900, 191, 191, 2002744947, '19840315 07:54:31.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1901, 785, 785, 528705562, '19950510 04:05:25.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1902, 245, 245, 1861790416, '19810122 10:40:02.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1903, 626, 626, 1972875104, '20050201 07:09:30.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1904, 612, 612, 1330344702, '20040925 21:02:05.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1905, 522, 522, 2052232916, '19730112 09:06:29.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1906, 618, 618, 1722465063, '19640329 18:52:41.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1907, 210, 210, 761851586, '19661021 00:25:08.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1908, 384, 384, 136495747, '19921123 14:08:31.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1909, 683, 683, 2017053287, '19980104 16:37:16.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1910, 834, 834, 466794703, '19780612 01:26:08.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1911, 679, 679, 1471813149, '19840509 08:27:37.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1912, 70, 70, 1700312800, '20060104 00:40:08.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1913, 733, 733, 1081184518, '19821223 10:53:46.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1914, 752, 752, 414227309, '19560511 07:05:22.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1915, 374, 374, 1717756185, '19970426 19:56:13.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1916, 621, 621, 1254622205, '19530301 16:01:12.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1917, 745, 745, 1037590063, '19920607 21:24:47.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1918, 631, 631, 1792860250, '19670312 18:24:03.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1919, 684, 684, 386115214, '19600524 13:53:22.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1920, 460, 460, 1165041892, '20040301 08:17:34.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1921, 211, 211, 707656023, '19570310 16:34:36.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1922, 483, 483, 1068163100, '20001121 05:33:44.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1923, 285, 285, 1202330270, '19650505 17:07:23.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1924, 641, 641, 1603334269, '19800505 02:17:13.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1925, 564, 564, 1557053583, '19961009 01:47:05.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1926, 810, 810, 1904140697, '20040531 16:56:10.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1927, 741, 741, 887278716, '19890606 18:25:24.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1928, 512, 512, 1993016729, '20010707 04:47:58.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1929, 507, 507, 1191110827, '20050323 04:03:50.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1930, 524, 524, 1427164118, '20011025 14:35:56.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1931, 198, 198, 2005806716, '19700214 21:28:33.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1932, 514, 514, 717632903, '19710428 15:04:11.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1933, 37, 37, 1010089986, '19860527 11:20:01.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1934, 680, 680, 1443710732, '19571030 07:44:03.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1935, 635, 635, 2023621696, '20040710 03:01:19.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1936, 40, 40, 816489379, '19750822 00:26:39.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1937, 647, 647, 162284077, '19830925 14:01:48.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1938, 712, 712, 712089584, '19951229 18:50:00.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1939, 727, 727, 203640628, '19910930 16:45:06.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1940, 548, 548, 2042866993, '19650830 23:40:23.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1941, 796, 796, 750787878, '19900125 18:21:43.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1942, 315, 315, 1081885080, '20040822 13:39:29.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1943, 239, 239, 1990820861, '19870124 20:14:52.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1944, 777, 777, 277061319, '19960302 22:45:23.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1945, 707, 707, 1002644120, '20071027 15:36:46.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1946, 353, 353, 3583966, '19680222 00:03:48.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1947, 113, 113, 1493751171, '19610207 09:11:20.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1948, 130, 130, 685387043, '19970625 13:03:02.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1949, 662, 662, 2048237363, '19840616 03:34:27.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1950, 596, 596, 901328183, '19850428 12:38:26.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1951, 907, 907, 808258629, '19540812 23:31:14.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1952, 930, 930, 914734444, '19741017 14:08:54.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1953, 695, 695, 1553980743, '20030917 23:06:26.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1954, 192, 192, 810352638, '19580502 01:09:36.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1955, 165, 165, 160387154, '19800106 15:19:55.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1956, 772, 772, 2020480619, '20020617 22:31:41.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1957, 842, 842, 205048369, '19680919 17:32:39.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1958, 894, 894, 1871116592, '19770930 04:52:17.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1959, 696, 696, 242995393, '19601218 19:15:00.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1960, 585, 585, 1535159615, '19760812 16:10:19.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1961, 718, 718, 2090124594, '19821023 00:27:16.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1962, 26, 26, 749580074, '19730414 19:37:09.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1963, 750, 750, 1900786829, '19950902 10:04:40.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1964, 393, 393, 250129525, '20040311 02:01:19.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1965, 537, 537, 2119398139, '19610428 03:57:34.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1966, 761, 761, 833286850, '19660112 17:23:25.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1967, 578, 578, 1291833681, '19720810 13:20:07.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1968, 853, 853, 893492602, '19780224 03:09:42.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1969, 325, 325, 1539103529, '19591101 04:04:02.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1970, 783, 783, 573715921, '19740906 19:29:34.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1971, 117, 117, 1963869736, '19770606 01:59:24.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1972, 575, 575, 2142757479, '20040609 02:34:46.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1973, 599, 599, 1176765111, '19830612 01:38:57.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1974, 250, 250, 1202340003, '20020923 14:12:59.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1975, 605, 605, 752661588, '19670205 13:55:50.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1976, 202, 202, 1917675155, '19600718 02:55:06.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1977, 357, 357, 1386889574, '19661029 09:18:51.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1978, 754, 754, 1594109696, '19770303 18:21:59.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1979, 876, 876, 189828756, '19800708 10:40:26.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1980, 593, 593, 93452460, '19810819 01:43:17.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1981, 341, 341, 163093244, '19960424 07:44:49.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1982, 772, 772, 145720654, '19991212 05:22:22.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1983, 577, 577, 1611353067, '19700122 01:13:31.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1984, 13, 13, 491228351, '19721125 15:25:23.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1985, 657, 657, 272559341, '20000315 15:04:42.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1986, 318, 318, 157665897, '20030430 07:19:39.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1987, 506, 506, 50439979, '19750810 15:57:45.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1988, 197, 197, 906251349, '19810126 10:10:25.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1989, 943, 943, 1613550224, '19851023 16:24:07.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1990, 726, 726, 1944578149, '19550124 04:29:37.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1991, 857, 857, 868637368, '19591203 06:54:00.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1992, 148, 148, 528077492, '19581227 09:09:30.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1993, 265, 265, 9477205, '19880111 23:34:59.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1994, 985, 985, 217612097, '19680219 00:34:47.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1995, 976, 976, 14251521, '19901107 23:13:06.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1996, 902, 902, 1452191532, '19691013 22:44:33.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1997, 869, 869, 631996825, '19611222 03:34:49.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1998, 749, 749, 1493395054, '19901114 18:13:32.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (1999, 289, 289, 1031276324, '19871106 18:47:49.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2000, 469, 469, 1976645715, '19941015 22:13:20.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2001, 764, 764, 312960944, '20030713 10:43:40.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2002, 231, 231, 1858620633, '19901214 06:01:38.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2003, 15, 15, 835416953, '19900826 08:59:00.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2004, 263, 263, 1649091421, '19621011 08:04:52.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2005, 398, 398, 1052695220, '19601123 10:39:02.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2006, 128, 128, 809407437, '19580304 20:56:27.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2007, 946, 946, 611946735, '19990507 12:29:57.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2008, 401, 401, 2088915477, '19531226 08:53:26.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2009, 740, 740, 75277887, '19990326 11:13:45.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2010, 473, 473, 1162117572, '19720621 12:24:49.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2011, 791, 791, 4926583, '19880820 13:12:49.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2012, 277, 277, 696913198, '19990718 23:10:39.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2013, 944, 944, 843437716, '20050323 18:11:50.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2014, 582, 582, 1114450719, '19870501 17:31:43.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2015, 902, 902, 214795423, '19880419 08:25:29.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2016, 913, 913, 1077172427, '19901110 19:04:54.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2017, 342, 342, 945515452, '19560324 18:23:38.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2018, 474, 474, 1944195246, '19751008 18:39:17.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2019, 925, 925, 473585978, '19561227 10:56:37.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2020, 345, 345, 347069350, '19651228 20:37:55.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2021, 979, 979, 1287679405, '19980604 01:25:40.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2022, 884, 884, 1065233647, '19990715 03:09:41.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2023, 476, 476, 888227952, '20000502 10:45:34.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2024, 972, 972, 1892160675, '19571008 23:34:25.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2025, 289, 289, 661684036, '19671006 12:35:34.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2026, 530, 530, 1568761181, '19710611 16:49:53.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2027, 948, 948, 1975483187, '19690530 02:59:46.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2028, 952, 952, 743253271, '19680424 01:04:10.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2029, 28, 28, 610807547, '19641116 14:59:53.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2030, 490, 490, 1629505959, '20050423 15:11:16.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2031, 391, 391, 1394512366, '20060726 23:20:17.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2032, 270, 270, 750656455, '19831214 15:05:19.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2033, 106, 106, 1544782989, '19970426 23:34:09.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2034, 561, 561, 1872040080, '19930922 12:27:05.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2035, 498, 498, 447656536, '19860205 14:59:36.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2036, 25, 25, 2146683551, '19580513 01:43:11.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2037, 704, 704, 763648647, '19620418 20:23:22.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2038, 453, 453, 1681870130, '19600413 17:08:39.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2039, 775, 775, 529291307, '19650103 04:46:20.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2040, 887, 887, 907921471, '19950112 18:08:14.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2041, 632, 632, 108815040, '19561224 18:49:41.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2042, 86, 86, 815987384, '19740816 07:04:18.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2043, 720, 720, 747270353, '19891101 22:56:39.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2044, 577, 577, 278308251, '19660505 03:59:18.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2045, 286, 286, 858956018, '19740604 15:16:47.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2046, 583, 583, 2123632750, '19680516 07:43:20.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2047, 781, 781, 1351315600, '19611004 14:57:39.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2048, 914, 914, 1455756461, '19530913 06:03:16.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2049, 448, 448, 648987410, '19871031 16:09:17.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2050, 560, 560, 1187719191, '20071228 04:08:11.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2051, 668, 668, 272631352, '19660723 04:20:55.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2052, 932, 932, 1540346473, '19940315 08:55:31.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2053, 741, 741, 178902422, '19861119 07:16:55.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2054, 876, 876, 1916832145, '19741109 22:09:54.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2055, 89, 89, 1620193345, '20040512 20:47:40.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2056, 49, 49, 336628328, '19561230 07:33:58.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2057, 45, 45, 1341466056, '19980812 19:16:03.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2058, 30, 30, 1785019748, '19851118 09:24:35.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2059, 809, 809, 352217068, '20030104 19:29:17.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2060, 734, 734, 2094309776, '19970613 17:49:08.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2061, 651, 651, 2066693174, '19961004 17:56:41.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2062, 296, 296, 298265073, '19840114 11:25:46.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2063, 928, 928, 473800651, '19970207 17:53:32.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2064, 449, 449, 2035035894, '20011201 20:02:29.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2065, 356, 356, 1151059578, '19731125 13:04:32.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2066, 757, 757, 103778954, '19570907 22:07:30.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2067, 739, 739, 1825316371, '19550324 23:36:29.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2068, 579, 579, 2124752203, '19640702 05:44:44.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2069, 925, 925, 1702639781, '19540327 02:32:07.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2070, 623, 623, 929932306, '19821209 06:42:18.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2071, 165, 165, 919101390, '19810725 22:41:32.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2072, 514, 514, 1648420494, '20031211 01:14:59.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2073, 418, 418, 2073001387, '19631006 13:52:56.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2074, 823, 823, 101112972, '19691214 16:48:22.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2075, 483, 483, 587068334, '19620105 01:48:13.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2076, 822, 822, 413124544, '19761019 18:21:21.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2077, 975, 975, 211615550, '19620913 04:13:02.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2078, 746, 746, 1699144893, '19861230 06:46:15.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2079, 868, 868, 947320453, '19910506 08:17:37.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2080, 283, 283, 1383509987, '20070522 04:52:14.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2081, 657, 657, 1516082911, '19620908 01:52:14.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2082, 990, 990, 1438833041, '19680916 20:56:30.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2083, 467, 467, 1187033871, '19880623 08:54:53.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2084, 158, 158, 2067033284, '19641119 10:51:42.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2085, 999, 999, 199883447, '19911003 10:50:20.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2086, 838, 838, 1817595450, '19650514 14:24:45.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2087, 373, 373, 1288110580, '20050124 07:48:23.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2088, 874, 874, 1466153625, '19750619 01:24:15.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2089, 693, 693, 897510605, '19970511 15:39:25.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2090, 183, 183, 1111678135, '19820207 07:25:37.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2091, 922, 922, 66298382, '19670930 06:27:07.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2092, 737, 737, 300789901, '19840530 10:58:46.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2093, 73, 73, 360889183, '19650408 21:39:21.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2094, 178, 178, 52886334, '19750722 10:57:12.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2095, 861, 861, 1004195313, '20060409 00:11:33.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2096, 348, 348, 63392707, '19801210 07:23:54.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2097, 652, 652, 961723059, '19850707 13:10:45.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2098, 562, 562, 445762950, '19951201 02:54:10.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2099, 793, 793, 12066616, '20000608 14:54:46.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2100, 538, 538, 1165722487, '19690927 17:09:17.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2101, 548, 548, 614524717, '19660223 08:06:50.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2102, 987, 987, 373669706, '20050403 09:12:55.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2103, 101, 101, 1486634899, '20070619 03:31:09.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2104, 336, 336, 1999691029, '19571123 09:26:58.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2105, 933, 933, 1321996979, '19790605 05:26:38.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2106, 961, 961, 126030635, '19700812 03:05:56.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2107, 570, 570, 608527536, '19830610 19:02:35.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2108, 612, 612, 1018934508, '19691206 14:28:32.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2109, 454, 454, 2031738460, '19651105 20:21:40.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2110, 758, 758, 1681354433, '19800725 02:26:19.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2111, 240, 240, 2128243539, '20020418 03:20:56.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2112, 242, 242, 496771856, '19640814 12:29:47.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2113, 968, 968, 362099853, '20020715 01:06:57.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2114, 3, 3, 886022760, '20030815 14:37:03.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2115, 95, 95, 677730880, '19871007 15:56:54.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2116, 270, 270, 1725735191, '19810118 21:00:11.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2117, 105, 105, 1551965198, '20030723 02:30:53.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2118, 422, 422, 238042675, '19850322 07:01:50.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2119, 852, 852, 238966715, '19630302 09:12:08.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2120, 69, 69, 1728903448, '19610714 22:39:46.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2121, 944, 944, 2124921172, '19600813 14:19:07.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2122, 65, 65, 137347565, '19871005 22:12:14.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2123, 511, 511, 1126840432, '19750220 14:05:19.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2124, 141, 141, 339466927, '19800216 19:06:29.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2125, 548, 548, 973248555, '19680312 00:15:10.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2126, 48, 48, 386821114, '20050224 11:42:45.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2127, 68, 68, 1867826283, '19910904 03:35:37.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2128, 793, 793, 1090626115, '19960316 02:55:43.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2129, 360, 360, 1896626055, '19710906 16:36:49.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2130, 89, 89, 873217743, '19890124 18:24:18.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2131, 912, 912, 1246902211, '19990414 05:10:35.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2132, 915, 915, 557874485, '19741013 01:18:51.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2133, 971, 971, 1480966599, '19940722 15:51:29.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2134, 285, 285, 922006042, '19740809 15:08:19.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2135, 640, 640, 747105413, '19940329 20:45:23.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2136, 131, 131, 1293944226, '19650605 16:39:19.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2137, 661, 661, 1863100363, '19690330 17:25:20.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2138, 554, 554, 1379951440, '19830801 23:27:54.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2139, 830, 830, 2093473260, '19930616 05:25:03.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2140, 616, 616, 1967433617, '19740222 07:44:24.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2141, 852, 852, 2031315018, '19891203 19:22:09.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2142, 724, 724, 2106058552, '19880219 17:19:51.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2143, 54, 54, 865684429, '19620814 05:02:35.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2144, 798, 798, 1201675160, '19691018 13:13:06.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2145, 463, 463, 1050333986, '19871022 04:04:40.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2146, 29, 29, 141553163, '19560216 17:57:20.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2147, 880, 880, 339738208, '19891115 09:51:49.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2148, 846, 846, 1252554624, '19690824 07:02:17.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2149, 637, 637, 406890128, '19951014 19:00:17.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2150, 744, 744, 1824329063, '19780322 03:11:21.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2151, 157, 157, 845087645, '19850520 04:53:01.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2152, 494, 494, 303211379, '19530418 06:08:55.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2153, 517, 517, 1500282607, '19851001 17:42:02.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2154, 658, 658, 98461673, '19911126 16:15:00.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2155, 690, 690, 1649607552, '19620215 02:50:10.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2156, 527, 527, 1778327270, '19860520 09:54:36.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2157, 579, 579, 66668254, '19830212 19:07:35.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2158, 890, 890, 2058439085, '19800503 08:24:39.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2159, 682, 682, 1982047952, '19970912 09:11:48.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2160, 490, 490, 1292703305, '19820409 17:43:53.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2161, 302, 302, 2045243702, '19861207 23:30:18.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2162, 980, 980, 672227502, '19950326 16:06:51.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2163, 1000, 1000, 1662513060, '20060331 21:51:42.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2164, 644, 644, 710534673, '19841011 01:57:21.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2165, 156, 156, 1150568443, '19890411 21:15:43.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2166, 1, 1, 795069011, '19800707 02:02:05.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2167, 470, 470, 1889796183, '19780122 20:38:18.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2168, 129, 129, 1176339656, '19801206 09:58:37.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2169, 48, 48, 606236409, '19620519 17:51:39.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2170, 36, 36, 677273026, '19750504 23:17:35.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2171, 935, 935, 1133299998, '19641022 03:34:50.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2172, 842, 842, 979717699, '19721222 03:02:59.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2173, 736, 736, 519461356, '19991006 01:36:47.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2174, 424, 424, 1148161647, '19970107 01:27:44.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2175, 868, 868, 124322444, '19790811 03:17:37.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2176, 111, 111, 1423475834, '19800623 20:59:16.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2177, 979, 979, 783327923, '19780222 17:09:39.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2178, 800, 800, 944096660, '19580506 00:52:13.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2179, 389, 389, 2055322064, '20000426 15:01:48.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2180, 122, 122, 935496039, '19650125 06:17:50.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2181, 870, 870, 1373986441, '19680412 01:50:56.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2182, 28, 28, 612123404, '19750111 20:33:20.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2183, 163, 163, 133571203, '19530603 07:55:26.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2184, 80, 80, 1661413743, '20010616 13:25:28.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2185, 140, 140, 1914157884, '19560907 13:31:17.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2186, 5, 5, 458639058, '19981227 23:01:40.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2187, 5, 5, 1443872607, '19970110 07:36:49.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2188, 260, 260, 1460311922, '19550827 23:36:29.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2189, 388, 388, 1535305452, '19650625 12:18:09.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2190, 271, 271, 1734715502, '19601110 10:50:47.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2191, 658, 658, 935710813, '19900422 21:31:43.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2192, 998, 998, 1127047992, '19961126 09:00:40.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2193, 390, 390, 227822709, '19931118 14:16:06.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2194, 97, 97, 1293683977, '19640309 11:41:10.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2195, 80, 80, 337100410, '19950317 08:14:05.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2196, 818, 818, 1814123863, '20020909 03:15:17.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2197, 423, 423, 906687899, '19891119 19:28:08.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2198, 728, 728, 755638991, '19851102 03:05:13.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2199, 42, 42, 1275353526, '19880707 15:57:22.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2200, 864, 864, 1085619255, '19600417 02:02:35.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2201, 821, 821, 1088274108, '19860123 21:19:02.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2202, 83, 83, 1049426338, '19611210 23:53:11.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2203, 640, 640, 1702327329, '19600407 13:10:37.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2204, 675, 675, 469878669, '19730612 19:42:42.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2205, 360, 360, 82111132, '19660531 23:36:31.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2206, 601, 601, 788497203, '19650530 01:50:01.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2207, 848, 848, 31955555, '19610713 04:32:07.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2208, 769, 769, 997682883, '19960923 16:14:17.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2209, 763, 763, 80751565, '19650418 12:57:23.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2210, 13, 13, 391964686, '19890823 05:50:54.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2211, 202, 202, 25712761, '19610328 16:44:57.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2212, 940, 940, 1551629228, '19771010 18:15:22.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2213, 767, 767, 1743578412, '19880106 17:22:50.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2214, 930, 930, 1509694320, '19850819 02:53:58.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2215, 655, 655, 1429994846, '19661229 15:52:56.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2216, 714, 714, 1180401877, '19641127 02:56:58.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2217, 630, 630, 1585072484, '19941024 08:11:25.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2218, 911, 911, 27965252, '19571016 08:26:14.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2219, 24, 24, 525839880, '19810203 12:26:04.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2220, 625, 625, 1425778316, '19980414 22:14:02.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2221, 627, 627, 1922041191, '19910626 18:25:16.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2222, 614, 614, 480035297, '19750529 21:01:49.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2223, 387, 387, 1218028610, '19680613 21:40:28.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2224, 620, 620, 129557153, '19540710 07:00:51.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2225, 710, 710, 967421916, '19930111 01:45:52.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2226, 198, 198, 905986686, '19751125 18:34:10.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2227, 776, 776, 1105776207, '19870202 12:46:53.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2228, 556, 556, 687977986, '19880729 13:55:37.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2229, 547, 547, 544254574, '19541024 17:13:38.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2230, 28, 28, 56894014, '19841201 00:02:19.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2231, 491, 491, 1458258742, '19980804 01:31:07.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2232, 886, 886, 1407288668, '20000421 14:04:25.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2233, 125, 125, 1862197000, '19771028 08:54:50.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2234, 924, 924, 2094525135, '19930110 12:59:13.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2235, 938, 938, 1253385828, '19870101 08:58:48.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2236, 127, 127, 819874753, '19590504 01:57:45.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2237, 584, 584, 871040071, '19671007 07:22:42.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2238, 222, 222, 286549445, '19871221 12:12:43.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2239, 843, 843, 517629999, '19880117 13:48:57.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2240, 891, 891, 807143956, '19990410 11:41:16.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2241, 929, 929, 167569196, '19900617 18:29:32.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2242, 986, 986, 1155173853, '19530419 15:22:31.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2243, 965, 965, 1205553212, '19980510 10:39:06.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2244, 135, 135, 1863201371, '19831102 06:27:15.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2245, 886, 886, 84841232, '20070814 18:05:50.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2246, 960, 960, 1822394578, '19650714 03:16:21.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2247, 379, 379, 502742390, '19611121 15:37:49.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2248, 189, 189, 359067218, '19610402 11:22:08.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2249, 133, 133, 803404559, '20050310 19:48:13.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2250, 343, 343, 1200554653, '19830421 05:17:06.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2251, 786, 786, 1408710227, '19601116 19:03:52.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2252, 266, 266, 164268832, '19850204 11:01:53.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2253, 814, 814, 737533250, '19570929 14:39:08.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2254, 585, 585, 1591655352, '19980324 17:43:20.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2255, 681, 681, 263323044, '19761021 07:37:18.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2256, 288, 288, 1529827582, '19630829 00:17:13.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2257, 509, 509, 653167035, '20010628 02:12:43.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2258, 31, 31, 1054202595, '19580930 06:09:45.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2259, 229, 229, 739062635, '19880531 17:56:50.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2260, 996, 996, 1633994599, '19980706 05:02:21.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2261, 801, 801, 251967034, '19860426 13:03:08.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2262, 104, 104, 1899446721, '19801213 14:36:30.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2263, 177, 177, 876991994, '19941201 23:00:39.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2264, 711, 711, 92114592, '19880517 12:55:03.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2265, 88, 88, 58347167, '19990119 04:19:46.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2266, 756, 756, 899934136, '19681206 02:40:31.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2267, 232, 232, 564193431, '20071213 09:20:31.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2268, 188, 188, 1126600116, '20021226 04:23:37.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2269, 261, 261, 488895666, '20060817 17:55:09.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2270, 305, 305, 1577452454, '19600828 13:55:11.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2271, 306, 306, 58126931, '20050219 19:34:15.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2272, 20, 20, 822957452, '19591104 19:58:41.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2273, 187, 187, 1724976878, '19770928 18:37:17.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2274, 939, 939, 559845499, '19891026 00:44:48.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2275, 672, 672, 593214590, '19601029 03:44:29.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2276, 457, 457, 976232861, '19910310 03:02:44.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2277, 769, 769, 658610413, '19850118 10:22:42.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2278, 129, 129, 2006244956, '19920811 15:13:12.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2279, 184, 184, 1525099755, '19541125 12:55:01.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2280, 416, 416, 1111651141, '19800630 22:29:30.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2281, 778, 778, 219015109, '19670105 02:56:20.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2282, 883, 883, 565945885, '19781103 01:24:45.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2283, 858, 858, 232188802, '19910520 18:07:23.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2284, 707, 707, 1531061282, '19790705 11:56:32.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2285, 209, 209, 497579004, '19770115 04:58:26.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2286, 118, 118, 1526577085, '19660630 14:29:13.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2287, 280, 280, 1010082989, '19950724 23:25:17.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2288, 144, 144, 1460537335, '19870805 15:11:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2289, 197, 197, 291204210, '19690323 05:21:54.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2290, 594, 594, 505170038, '19760506 08:41:35.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2291, 185, 185, 2026725106, '19651104 23:45:02.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2292, 837, 837, 777891941, '19620108 01:12:56.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2293, 142, 142, 816246557, '20070721 18:15:53.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2294, 533, 533, 1313079876, '19970713 08:46:35.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2295, 252, 252, 272699539, '19651215 22:38:07.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2296, 167, 167, 1325235513, '19620706 03:53:02.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2297, 461, 461, 1956347212, '19660518 16:21:52.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2298, 591, 591, 1735698025, '19621224 21:44:02.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2299, 22, 22, 167785160, '19921013 02:07:29.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2300, 667, 667, 1279470509, '19910909 15:25:19.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2301, 257, 257, 1471949093, '19650731 17:55:49.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2302, 278, 278, 634184498, '19661126 11:14:12.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2303, 596, 596, 775842075, '19620815 17:26:58.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2304, 800, 800, 1786792612, '19970714 05:53:02.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2305, 22, 22, 588651260, '19860228 09:42:50.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2306, 775, 775, 407149368, '19540113 23:05:10.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2307, 702, 702, 1268578014, '19600509 16:24:36.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2308, 665, 665, 1137571156, '19751204 13:54:20.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2309, 150, 150, 554313468, '19900527 13:58:50.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2310, 194, 194, 1271661108, '19930813 04:34:34.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2311, 825, 825, 1709761891, '19860809 13:54:30.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2312, 305, 305, 1107212558, '19611117 10:59:31.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2313, 825, 825, 1212677563, '19561105 17:14:44.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2314, 841, 841, 1752784565, '19601201 19:27:19.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2315, 771, 771, 544644517, '19840702 02:06:00.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2316, 9, 9, 424192650, '19600624 11:38:22.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2317, 38, 38, 427305493, '19970725 20:29:06.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2318, 954, 954, 1090499479, '19680530 11:04:06.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2319, 244, 244, 473650237, '19640503 07:56:37.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2320, 957, 957, 2114243545, '19850722 13:50:32.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2321, 221, 221, 1905272747, '19810514 03:15:14.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2322, 631, 631, 847428244, '19730510 18:09:01.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2323, 973, 973, 1158361242, '19861005 23:01:42.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2324, 216, 216, 1774085638, '19830414 09:13:34.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2325, 900, 900, 1570552331, '20021023 14:10:08.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2326, 255, 255, 1828391947, '19960213 18:21:19.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2327, 292, 292, 1744934183, '19600415 01:42:48.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2328, 792, 792, 860143133, '19880317 09:50:41.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2329, 215, 215, 354087429, '19761111 02:06:40.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2330, 559, 559, 2098645710, '20060424 23:51:27.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2331, 916, 916, 1069440084, '19771023 10:40:52.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2332, 789, 789, 1916206067, '19750126 12:38:40.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2333, 35, 35, 198810051, '20071029 13:05:42.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2334, 595, 595, 1047243413, '19710317 21:29:41.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2335, 204, 204, 1753670384, '19671202 04:33:41.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2336, 771, 771, 785013782, '19530209 15:42:07.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2337, 826, 826, 797365429, '19690321 07:57:47.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2338, 754, 754, 1686665857, '20011105 12:14:20.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2339, 871, 871, 480398261, '20010506 02:13:41.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2340, 697, 697, 259668472, '19760103 05:53:16.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2341, 948, 948, 2078536914, '19590221 22:04:37.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2342, 320, 320, 993791512, '19720821 09:30:56.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2343, 567, 567, 1579292381, '20050310 01:12:40.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2344, 837, 837, 1389717256, '19830810 00:47:20.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2345, 227, 227, 1652837985, '19970926 18:47:04.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2346, 159, 159, 519796240, '19561219 12:45:31.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2347, 52, 52, 1328331839, '19580305 07:58:12.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2348, 386, 386, 430150952, '19990820 22:48:34.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2349, 804, 804, 900945916, '19660112 06:40:35.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2350, 296, 296, 524628608, '19580624 10:59:45.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2351, 340, 340, 447939460, '19721210 07:23:55.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2352, 743, 743, 494009388, '20051218 05:17:45.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2353, 338, 338, 515591465, '20060822 13:47:24.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2354, 807, 807, 687297847, '19600324 12:16:57.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2355, 363, 363, 777908480, '19630429 12:10:05.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2356, 157, 157, 1132690214, '20000321 23:46:47.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2357, 761, 761, 1948016675, '19880220 12:12:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2358, 141, 141, 264550598, '19870503 08:13:23.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2359, 823, 823, 1017402165, '20020920 15:42:53.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2360, 956, 956, 206918851, '19980114 15:21:31.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2361, 709, 709, 1510932877, '20010929 21:22:22.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2362, 214, 214, 103212997, '19800222 06:33:54.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2363, 773, 773, 1457509243, '20070122 11:47:39.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2364, 674, 674, 1132828083, '19920201 14:07:23.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2365, 234, 234, 2127661276, '19681021 17:53:41.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2366, 920, 920, 552035990, '19640715 01:15:49.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2367, 225, 225, 865945643, '19620118 21:53:48.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2368, 58, 58, 65425572, '19930820 19:45:02.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2369, 795, 795, 86608864, '20001230 14:53:37.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2370, 478, 478, 1271552197, '19840523 10:23:52.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2371, 863, 863, 399715173, '19990405 03:40:35.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2372, 984, 984, 1589329257, '20030919 08:14:45.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2373, 256, 256, 694482686, '19750124 08:50:25.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2374, 980, 980, 888789645, '19960430 02:03:21.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2375, 61, 61, 1638549652, '19790601 15:45:54.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2376, 40, 40, 1712643118, '19610923 17:44:17.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2377, 93, 93, 939141098, '19760301 16:56:21.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2378, 47, 47, 2115487350, '19560227 22:14:21.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2379, 901, 901, 1401415472, '19930717 14:26:29.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2380, 918, 918, 824669915, '19981105 01:24:37.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2381, 287, 287, 691854236, '19901212 10:23:07.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2382, 755, 755, 1140988055, '19680825 02:54:14.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2383, 182, 182, 1002608883, '19750307 03:10:06.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2384, 853, 853, 1902432466, '19710521 15:06:55.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2385, 108, 108, 543789153, '19860516 16:27:31.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2386, 888, 888, 402212965, '19791105 05:23:07.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2387, 178, 178, 111306861, '19760604 22:51:15.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2388, 123, 123, 859565503, '20000807 00:48:45.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2389, 934, 934, 1920715672, '19601119 09:19:36.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2390, 819, 819, 137538103, '19750911 04:46:53.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2391, 784, 784, 1192032479, '19721220 03:29:37.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2392, 182, 182, 1558794984, '19891117 23:44:23.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2393, 274, 274, 1426265432, '20070214 20:31:26.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2394, 234, 234, 542680860, '19560422 10:52:09.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2395, 200, 200, 632866931, '19820404 08:30:53.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2396, 659, 659, 621081033, '19861230 15:30:43.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2397, 433, 433, 801960925, '19730729 21:43:16.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2398, 318, 318, 1830172771, '19660207 11:05:16.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2399, 140, 140, 1275047165, '19671020 06:53:39.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2400, 344, 344, 2062608095, '19860315 17:31:15.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2401, 443, 443, 1909993589, '20021130 14:51:42.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2402, 86, 86, 739889750, '19720714 12:13:09.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2403, 936, 936, 1534625667, '20060820 07:54:56.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2404, 582, 582, 876142566, '19890820 20:16:42.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2405, 449, 449, 1594217571, '19670321 07:19:09.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2406, 872, 872, 509990942, '19770322 23:09:09.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2407, 829, 829, 598192317, '19831125 20:27:19.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2408, 645, 645, 1210358690, '19630425 11:44:03.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2409, 407, 407, 2072755298, '19881022 20:31:02.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2410, 112, 112, 1048115169, '19540806 18:25:48.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2411, 329, 329, 64032436, '19770121 06:50:25.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2412, 286, 286, 1049220459, '19841224 13:58:08.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2413, 257, 257, 423804003, '20011015 17:46:53.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2414, 143, 143, 1411189600, '19561116 14:18:16.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2415, 592, 592, 1119557137, '19600204 04:58:24.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2416, 415, 415, 2114817786, '19860205 18:28:08.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2417, 341, 341, 2142888813, '19571219 03:23:47.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2418, 692, 692, 1043684930, '19880903 20:40:44.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2419, 243, 243, 64828732, '19580918 21:39:51.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2420, 606, 606, 1608106555, '19961216 12:30:33.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2421, 369, 369, 1311556004, '19960209 02:24:33.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2422, 308, 308, 1047331635, '19690612 21:05:02.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2423, 239, 239, 966159359, '19851002 10:25:26.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2424, 716, 716, 900169772, '19780422 10:06:43.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2425, 598, 598, 1953769631, '19640604 06:54:15.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2426, 363, 363, 735311635, '19620519 03:56:11.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2427, 908, 908, 1185552338, '20040803 11:43:18.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2428, 376, 376, 2123746695, '19711004 21:58:16.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2429, 97, 97, 304965442, '19670127 17:32:27.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2430, 896, 896, 113986777, '20000601 00:15:11.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2431, 270, 270, 307775156, '19960224 20:01:00.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2432, 88, 88, 1434022798, '19630124 05:51:04.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2433, 313, 313, 167969915, '19960509 15:20:41.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2434, 114, 114, 1495480693, '19600504 20:55:14.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2435, 376, 376, 1500190003, '20031205 10:32:54.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2436, 491, 491, 1251434803, '19710531 22:09:57.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2437, 703, 703, 1758858396, '19700104 18:59:17.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2438, 895, 895, 1545388969, '19930716 06:53:24.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2439, 354, 354, 818347306, '19570303 07:47:45.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2440, 32, 32, 1898520508, '19620224 17:18:21.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2441, 806, 806, 151164750, '20031210 08:55:48.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2442, 936, 936, 934237519, '19660211 11:07:12.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2443, 879, 879, 387955705, '19641016 04:17:42.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2444, 3, 3, 476015659, '19590416 14:53:48.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2445, 43, 43, 2121451885, '20060819 21:56:27.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2446, 372, 372, 1777442000, '19951226 13:31:28.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2447, 173, 173, 1136830719, '19960202 05:57:30.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2448, 19, 19, 443115676, '20030401 06:13:10.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2449, 154, 154, 1912167348, '20040318 05:53:45.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2450, 921, 921, 1883222408, '20040226 14:02:45.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2451, 116, 116, 1761261533, '19820112 12:06:55.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2452, 290, 290, 1536311293, '19921124 01:02:32.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2453, 630, 630, 607868102, '19600522 13:25:25.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2454, 340, 340, 629550711, '19781102 18:23:05.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2455, 452, 452, 1955241447, '19980203 15:06:42.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2456, 619, 619, 774689281, '19860620 17:46:39.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2457, 258, 258, 1797937712, '19941012 01:47:43.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2458, 126, 126, 1663383116, '19810430 21:48:12.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2459, 935, 935, 1186046488, '19780319 13:22:26.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2460, 279, 279, 2031698795, '19571103 03:22:57.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2461, 224, 224, 1403971686, '19800818 11:25:50.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2462, 981, 981, 1625091807, '19650221 22:29:03.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2463, 66, 66, 1181665603, '20040327 18:11:36.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2464, 365, 365, 86919363, '19960522 20:15:51.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2465, 780, 780, 927850481, '19660611 22:24:48.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2466, 815, 815, 1120061012, '19661230 00:59:21.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2467, 407, 407, 1794755582, '19960815 22:37:26.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2468, 227, 227, 2074385002, '19940613 20:51:39.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2469, 192, 192, 1014139342, '19601212 03:45:21.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2470, 149, 149, 1083858539, '19960904 06:48:26.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2471, 1000, 1000, 1615060813, '19690201 23:28:50.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2472, 413, 413, 2055726068, '19720603 20:30:23.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2473, 368, 368, 1324040939, '19840704 12:52:59.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2474, 873, 873, 190515546, '20040724 04:21:29.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2475, 34, 34, 493753102, '20010106 03:34:45.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2476, 322, 322, 126299452, '19821229 15:17:21.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2477, 632, 632, 1719677422, '19570828 16:47:20.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2478, 160, 160, 2070622046, '19790619 19:31:38.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2479, 146, 146, 1244387495, '19790902 12:10:15.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2480, 505, 505, 39519929, '19760608 17:22:46.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2481, 270, 270, 1522921844, '19740415 21:58:41.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2482, 875, 875, 1336885331, '19540506 05:30:08.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2483, 864, 864, 954800853, '19750718 16:04:31.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2484, 74, 74, 408989895, '19701201 03:29:42.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2485, 390, 390, 655309196, '19710521 12:08:15.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2486, 819, 819, 1257934057, '19560402 18:58:27.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2487, 390, 390, 125826178, '19550904 16:25:39.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2488, 360, 360, 621057181, '19700708 20:57:36.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2489, 955, 955, 276245595, '19700331 05:48:32.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2490, 831, 831, 373202593, '19700617 16:46:14.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2491, 913, 913, 1982011832, '19840818 20:22:14.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2492, 144, 144, 1082981486, '19960904 21:11:05.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2493, 331, 331, 461800436, '19680429 17:30:57.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2494, 571, 571, 920691827, '20070430 04:24:47.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2495, 102, 102, 1722013847, '19890708 05:52:30.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2496, 304, 304, 1103677570, '19911019 10:26:44.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2497, 969, 969, 1184362207, '19691115 16:55:35.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2498, 372, 372, 125654558, '19760525 08:01:51.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2499, 459, 459, 2073031453, '20001103 16:28:59.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2500, 210, 210, 310232890, '19920821 20:57:06.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2501, 917, 917, 1874408034, '20070511 14:54:01.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2502, 635, 635, 1446805935, '19540822 09:05:51.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2503, 607, 607, 871102626, '19950421 02:27:48.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2504, 196, 196, 1079961166, '19600714 23:05:18.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2505, 909, 909, 1510275960, '19880125 14:33:54.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2506, 273, 273, 815715167, '19620811 15:36:31.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2507, 544, 544, 115066282, '19610523 12:09:33.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2508, 387, 387, 2061074722, '19631029 09:03:55.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2509, 442, 442, 154276933, '19851026 14:48:20.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2510, 978, 978, 1931399950, '19680206 23:49:20.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2511, 899, 899, 652983048, '19811024 00:59:19.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2512, 463, 463, 1651997090, '19680425 06:16:04.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2513, 82, 82, 714617696, '19540830 09:37:56.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2514, 334, 334, 69416172, '19541011 19:59:40.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2515, 158, 158, 313655754, '19910720 05:24:16.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2516, 140, 140, 1289732567, '19790416 05:55:42.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2517, 844, 844, 2126616619, '19970806 06:24:32.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2518, 238, 238, 10676921, '19860428 14:41:54.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2519, 610, 610, 1969745257, '19780103 08:07:35.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2520, 247, 247, 1703904361, '19630312 03:26:20.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2521, 618, 618, 1233448506, '19640427 08:33:41.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2522, 395, 395, 1542744492, '19790209 01:39:50.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2523, 337, 337, 1834577285, '19770316 15:03:07.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2524, 636, 636, 410804183, '19980626 10:59:07.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2525, 738, 738, 1422148205, '19650117 10:26:11.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2526, 677, 677, 233108629, '19800529 02:17:45.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2527, 305, 305, 1875952076, '19570205 02:59:26.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2528, 590, 590, 744268430, '19850306 08:28:11.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2529, 406, 406, 1225927368, '19680117 22:28:58.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2530, 287, 287, 1979288367, '19620321 17:08:00.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2531, 228, 228, 98087490, '19660213 22:21:45.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2532, 87, 87, 1398901667, '19890404 08:45:28.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2533, 876, 876, 434390893, '19860815 03:02:38.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2534, 461, 461, 551129203, '19950110 15:13:09.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2535, 22, 22, 818832297, '19770128 02:28:45.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2536, 476, 476, 1505334512, '19720825 12:52:49.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2537, 698, 698, 2021404707, '19670115 03:02:19.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2538, 239, 239, 1940791910, '19680104 16:59:12.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2539, 235, 235, 709787544, '19901105 12:55:47.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2540, 474, 474, 287523128, '19611010 20:31:44.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2541, 262, 262, 1120866883, '20021110 06:48:54.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2542, 812, 812, 736230017, '19991106 07:21:43.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2543, 226, 226, 562988, '19920911 06:09:16.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2544, 155, 155, 1556643069, '19550223 05:59:11.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2545, 993, 993, 379166330, '19960823 15:46:54.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2546, 199, 199, 1099385477, '19690426 14:06:09.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2547, 624, 624, 13493903, '19950107 11:33:09.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2548, 424, 424, 196209282, '19660720 21:31:17.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2549, 323, 323, 183613935, '19681011 23:00:31.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2550, 964, 964, 846420580, '19630325 23:56:48.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2551, 540, 540, 1153061183, '20020314 04:02:12.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2552, 109, 109, 1758961899, '19910720 02:13:41.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2553, 139, 139, 811101545, '20060521 23:54:15.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2554, 354, 354, 1695026796, '19900826 13:02:39.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2555, 708, 708, 2090927549, '19810427 12:23:24.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2556, 648, 648, 396245015, '19810402 13:14:10.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2557, 715, 715, 886466916, '19850606 18:02:44.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2558, 942, 942, 1592935084, '20041024 15:28:41.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2559, 435, 435, 1130272995, '20030627 12:39:06.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2560, 462, 462, 1744510567, '19600829 12:16:09.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2561, 179, 179, 436244261, '19580707 12:37:05.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2562, 491, 491, 1387534810, '19840318 03:41:22.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2563, 224, 224, 1857601870, '20031011 10:34:56.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2564, 937, 937, 1032835669, '19760807 21:40:19.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2565, 469, 469, 799319465, '19541225 01:27:35.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2566, 719, 719, 1536568627, '20000720 16:13:51.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2567, 11, 11, 515738150, '19890910 03:13:26.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2568, 326, 326, 322127212, '19670819 18:24:47.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2569, 845, 845, 621024026, '19850912 17:30:55.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2570, 797, 797, 196787175, '19711125 16:31:26.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2571, 695, 695, 900500261, '19650402 02:53:22.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2572, 824, 824, 774804917, '19820803 13:25:19.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2573, 688, 688, 655882992, '20030623 01:46:06.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2574, 812, 812, 264765076, '19690326 11:35:51.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2575, 377, 377, 455349043, '19680430 20:37:25.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2576, 537, 537, 290867467, '19720621 01:11:58.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2577, 128, 128, 1275511817, '19940529 19:30:34.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2578, 919, 919, 1150422740, '19750123 09:03:55.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2579, 350, 350, 659223825, '19541103 19:12:12.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2580, 295, 295, 171215540, '20030922 20:20:02.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2581, 327, 327, 301427034, '19930521 14:44:36.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2582, 513, 513, 624167004, '19980718 05:28:09.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2583, 405, 405, 1734917369, '19690526 09:27:39.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2584, 127, 127, 2030082427, '19571025 22:32:10.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2585, 515, 515, 1470481608, '19680107 13:05:48.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2586, 136, 136, 1532482312, '19820727 20:08:03.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2587, 977, 977, 617647905, '19901113 08:51:12.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2588, 71, 71, 328417167, '20031219 13:59:58.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2589, 861, 861, 18705093, '19660916 02:49:44.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2590, 303, 303, 1962920528, '20031027 13:14:34.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2591, 566, 566, 1847589758, '19950320 18:50:05.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2592, 315, 315, 1339167709, '19700322 11:33:38.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2593, 859, 859, 755229798, '19720709 04:20:05.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2594, 152, 152, 710354451, '19830310 00:39:38.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2595, 785, 785, 4640699, '19560406 07:54:38.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2596, 970, 970, 2125521548, '19710826 03:07:32.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2597, 429, 429, 2043566489, '20040125 20:46:47.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2598, 783, 783, 512356896, '19690204 08:28:57.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2599, 317, 317, 1456812897, '19530302 04:31:36.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2600, 426, 426, 16860578, '20040202 23:33:03.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2601, 659, 659, 1849926069, '19870816 10:52:43.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2602, 220, 220, 1662637387, '19800521 17:08:21.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2603, 982, 982, 1654817416, '19881106 04:00:22.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2604, 492, 492, 1248469052, '20041116 06:29:05.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2605, 449, 449, 362827807, '20051222 21:03:25.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2606, 803, 803, 1965276289, '19720811 14:36:52.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2607, 723, 723, 437254340, '19960127 00:27:35.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2608, 603, 603, 1127193133, '19900226 12:18:24.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2609, 79, 79, 527959579, '19750428 16:25:14.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2610, 420, 420, 402712681, '19611107 11:11:26.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2611, 247, 247, 251047928, '19941103 17:43:36.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2612, 197, 197, 953866433, '19630516 08:58:48.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2613, 580, 580, 1222341284, '20021222 20:16:29.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2614, 56, 56, 638160793, '20071004 16:19:04.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2615, 402, 402, 150574075, '19750409 21:31:33.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2616, 67, 67, 578219403, '19671220 07:08:56.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2617, 276, 276, 1833822245, '19671024 18:13:42.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2618, 213, 213, 1496376423, '19981217 12:02:46.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2619, 615, 615, 459882933, '19610802 08:34:30.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2620, 475, 475, 1874756157, '19800529 05:19:31.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2621, 734, 734, 1804762076, '19620908 07:22:38.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2622, 351, 351, 535407191, '19930921 13:13:27.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2623, 313, 313, 1012749226, '19531204 15:35:02.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2624, 161, 161, 635694050, '19891116 04:16:20.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2625, 432, 432, 2143687741, '19840905 21:41:20.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2626, 70, 70, 2037476659, '20030110 15:19:44.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2627, 92, 92, 287891578, '19630126 09:05:14.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2628, 201, 201, 517174506, '19730416 01:06:28.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2629, 634, 634, 1647608490, '19651221 03:41:12.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2630, 825, 825, 1416897781, '20040904 17:29:52.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2631, 624, 624, 1058112369, '19760526 04:25:11.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2632, 839, 839, 58466781, '19780425 11:01:37.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2633, 498, 498, 1923790475, '19741124 04:32:19.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2634, 587, 587, 2132443204, '19580930 19:39:08.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2635, 263, 263, 718927162, '19690207 09:27:18.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2636, 271, 271, 1103185093, '19651231 21:36:14.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2637, 912, 912, 1487696031, '19620912 01:27:46.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2638, 448, 448, 975592188, '19720709 02:58:34.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2639, 731, 731, 1316240971, '19591103 01:28:44.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2640, 233, 233, 616779875, '20030527 22:28:55.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2641, 11, 11, 1053842541, '19940630 19:40:28.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2642, 7, 7, 663507402, '19530917 20:32:48.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2643, 485, 485, 654185435, '19810823 21:34:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2644, 372, 372, 2033532570, '19571109 15:38:17.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2645, 986, 986, 1046649824, '19611224 19:30:58.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2646, 523, 523, 493760788, '19841105 09:51:17.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2647, 437, 437, 1577998157, '19750330 15:17:08.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2648, 620, 620, 148327913, '19721004 12:01:00.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2649, 927, 927, 1704636834, '19601201 03:08:05.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2650, 752, 752, 2094706873, '19960417 00:45:42.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2651, 517, 517, 1395522012, '19561102 08:53:50.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2652, 159, 159, 162439257, '19580209 08:44:00.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2653, 316, 316, 111360502, '19600705 23:54:26.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2654, 420, 420, 1975373796, '19801002 23:12:04.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2655, 551, 551, 944180102, '19940528 16:10:25.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2656, 566, 566, 247148680, '20011123 21:39:15.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2657, 425, 425, 1761462718, '19790619 01:33:19.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2658, 649, 649, 970512569, '20061220 23:44:01.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2659, 379, 379, 641945705, '19730313 08:47:44.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2660, 443, 443, 18191771, '19551214 05:43:41.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2661, 277, 277, 740155447, '19620718 05:31:38.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2662, 633, 633, 1048005656, '19751013 23:21:06.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2663, 481, 481, 1540848045, '19770309 08:37:11.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2664, 241, 241, 1341597267, '19780822 22:55:21.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2665, 39, 39, 121686309, '19930613 06:46:15.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2666, 509, 509, 380629343, '19690712 06:41:58.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2667, 619, 619, 1522863934, '19960621 04:26:29.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2668, 608, 608, 1667563173, '19970325 00:37:21.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2669, 361, 361, 1479386707, '19910828 06:51:46.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2670, 399, 399, 1031038163, '19620409 23:55:18.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2671, 486, 486, 647953909, '19580410 05:41:09.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2672, 495, 495, 670934480, '20030416 15:15:07.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2673, 658, 658, 1748174638, '19920215 10:34:01.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2674, 345, 345, 2002680624, '19660307 10:05:35.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2675, 715, 715, 499085280, '19931128 09:39:02.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2676, 288, 288, 542806402, '19611222 10:49:49.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2677, 150, 150, 1228903611, '19650129 15:38:43.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2678, 56, 56, 1655254570, '20040124 23:56:44.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2679, 283, 283, 1433225263, '19801123 08:45:22.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2680, 63, 63, 444247560, '20071031 11:50:03.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2681, 603, 603, 76232384, '19801014 00:02:35.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2682, 583, 583, 969932870, '19980422 21:04:13.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2683, 639, 639, 1760672819, '19650516 21:58:23.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2684, 905, 905, 271600780, '19770904 02:55:30.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2685, 530, 530, 92525105, '20001103 08:36:02.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2686, 266, 266, 633221054, '19710417 19:41:10.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2687, 681, 681, 2114970048, '19701021 11:07:11.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2688, 389, 389, 1244324497, '20020220 05:20:15.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2689, 663, 663, 1197266796, '19720504 03:28:42.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2690, 451, 451, 746902285, '19750316 11:48:02.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2691, 147, 147, 368671577, '19940714 20:02:55.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2692, 572, 572, 838588131, '19630922 01:43:45.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2693, 441, 441, 1825708148, '20070427 18:10:50.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2694, 228, 228, 49632655, '19561120 19:45:02.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2695, 772, 772, 1039649921, '19931110 16:57:16.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2696, 926, 926, 1422679883, '19710918 20:19:21.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2697, 926, 926, 242118503, '19840710 11:55:37.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2698, 659, 659, 769576110, '19560102 22:39:23.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2699, 74, 74, 1947600972, '19720528 08:52:02.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2700, 326, 326, 966140646, '20000613 12:48:59.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2701, 951, 951, 399681338, '19960516 05:24:48.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2702, 479, 479, 1169227018, '19860104 14:39:47.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2703, 900, 900, 1786211137, '19890206 05:09:14.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2704, 444, 444, 1726960372, '20060823 03:12:46.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2705, 205, 205, 819730036, '19910108 19:04:33.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2706, 352, 352, 328077099, '19610718 17:38:42.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2707, 93, 93, 416809323, '19731224 22:09:58.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2708, 441, 441, 302956147, '20020430 13:48:50.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2709, 175, 175, 143331953, '19941104 13:33:01.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2710, 546, 546, 1315919123, '19850607 05:20:35.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2711, 526, 526, 303045157, '19891026 06:00:32.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2712, 104, 104, 1035717479, '19830525 01:36:34.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2713, 78, 78, 1193353341, '19790128 14:58:38.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2714, 742, 742, 787202962, '19730512 20:57:40.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2715, 265, 265, 1597192833, '19830301 05:41:05.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2716, 883, 883, 444199050, '19720326 08:27:25.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2717, 213, 213, 1271086465, '19630531 01:22:43.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2718, 111, 111, 1538069679, '20070705 05:41:41.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2719, 724, 724, 1243151982, '19851018 14:19:18.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2720, 685, 685, 1234079766, '19750227 11:05:04.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2721, 141, 141, 1566672629, '20060920 19:34:46.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2722, 501, 501, 1917999509, '20020430 23:06:14.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2723, 629, 629, 1021033071, '19771121 21:08:38.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2724, 581, 581, 161317829, '19690614 19:03:43.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2725, 395, 395, 56454289, '19750926 03:52:23.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2726, 142, 142, 1551039954, '20020719 03:57:23.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2727, 688, 688, 833513581, '20031219 21:04:17.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2728, 484, 484, 1330155784, '19880327 14:48:59.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2729, 301, 301, 1892540034, '19800425 17:08:18.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2730, 541, 541, 13504553, '19750313 13:19:40.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2731, 840, 840, 1273718024, '19850612 22:54:12.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2732, 519, 519, 2025319527, '19620128 16:59:20.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2733, 654, 654, 1669793524, '19840829 15:04:41.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2734, 927, 927, 142807020, '19880610 19:56:23.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2735, 945, 945, 1174261714, '19640617 06:25:15.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2736, 55, 55, 510537897, '20021009 09:22:48.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2737, 480, 480, 929336245, '19620317 15:54:58.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2738, 139, 139, 1195334482, '19660923 18:45:36.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2739, 542, 542, 66644290, '19940826 07:50:56.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2740, 394, 394, 1203876313, '19920418 14:57:20.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2741, 731, 731, 1179965752, '20050422 21:31:11.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2742, 232, 232, 1457092951, '19760621 21:18:20.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2743, 477, 477, 640681631, '19600418 15:47:13.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2744, 765, 765, 599108811, '19941126 22:08:06.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2745, 362, 362, 1190798597, '19930520 05:47:30.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2746, 325, 325, 970550518, '19640221 18:26:20.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2747, 534, 534, 1216293104, '19920823 10:45:06.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2748, 121, 121, 1008266763, '19861216 21:13:09.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2749, 255, 255, 1818728224, '19810921 14:03:57.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2750, 667, 667, 713812446, '19740818 08:29:51.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2751, 403, 403, 1510949283, '19610421 18:56:38.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2752, 329, 329, 1239997507, '19850105 06:13:55.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2753, 268, 268, 1155831349, '19780316 08:20:04.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2754, 977, 977, 649031773, '20050330 21:46:58.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2755, 336, 336, 1518968701, '20010922 17:14:12.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2756, 725, 725, 36908617, '20020115 06:18:34.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2757, 207, 207, 1929991111, '19610213 17:31:09.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2758, 969, 969, 1735081109, '19720825 10:05:30.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2759, 39, 39, 57029416, '19831128 23:20:23.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2760, 16, 16, 15664901, '19960621 15:07:09.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2761, 315, 315, 443829541, '19650729 20:34:24.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2762, 282, 282, 1240146815, '19930928 07:20:59.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2763, 662, 662, 254395145, '19750105 20:40:31.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2764, 8, 8, 414929962, '19720825 00:13:20.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2765, 477, 477, 2058734625, '20071225 06:26:23.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2766, 267, 267, 597112013, '19800928 13:01:12.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2767, 259, 259, 1943339362, '20060924 10:31:52.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2768, 324, 324, 611996380, '19980818 19:02:15.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2769, 683, 683, 146492395, '19611126 14:32:24.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2770, 280, 280, 1943551678, '19880521 20:18:17.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2771, 732, 732, 1783200743, '19630110 16:32:26.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2772, 496, 496, 814701771, '20040905 06:37:06.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2773, 635, 635, 468952440, '19651109 21:50:44.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2774, 543, 543, 599481080, '20010630 23:21:57.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2775, 772, 772, 1952660915, '19771105 13:33:53.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2776, 622, 622, 1297380335, '19830401 22:16:25.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2777, 203, 203, 631214525, '19950113 07:19:00.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2778, 505, 505, 1539476649, '19901225 23:00:31.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2779, 753, 753, 1536299061, '19840125 13:16:13.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2780, 445, 445, 1422886519, '19640805 09:26:02.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2781, 319, 319, 982876873, '19621126 17:12:17.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2782, 18, 18, 721490597, '19700103 23:51:07.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2783, 294, 294, 1779393412, '19590708 00:45:01.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2784, 464, 464, 1940899228, '19580909 08:38:27.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2785, 416, 416, 300461075, '19661122 18:23:02.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2786, 617, 617, 1400447133, '19530608 16:40:16.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2787, 701, 701, 870131532, '19911114 08:39:15.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2788, 378, 378, 173834363, '19870531 17:17:43.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2789, 208, 208, 2115111469, '19940918 02:42:10.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2790, 747, 747, 1072150256, '19700602 00:06:41.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2791, 77, 77, 1979658625, '19940826 15:51:38.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2792, 834, 834, 127288241, '19970722 05:49:27.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2793, 556, 556, 268654947, '19901025 19:25:13.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2794, 732, 732, 1211449663, '20060305 16:43:47.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2795, 248, 248, 1789978029, '19790921 18:22:55.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2796, 15, 15, 985295536, '19640726 14:10:11.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2797, 715, 715, 60543118, '19540618 00:37:49.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2798, 291, 291, 1740002077, '19950824 15:33:52.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2799, 857, 857, 2006363229, '19941202 15:41:43.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2800, 332, 332, 697134388, '19650822 16:46:17.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2801, 591, 591, 1794246253, '19650529 07:54:28.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2802, 113, 113, 792013834, '20020104 15:42:51.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2803, 46, 46, 441443162, '19780121 06:40:45.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2804, 613, 613, 1615439610, '20010501 17:45:39.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2805, 229, 229, 1860814, '19640808 15:57:24.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2806, 435, 435, 1316653587, '19640815 12:19:31.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2807, 191, 191, 441716832, '19720227 08:23:11.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2808, 193, 193, 2090962476, '19840914 08:58:06.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2809, 500, 500, 199632557, '19800526 08:13:04.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2810, 131, 131, 1832640085, '19710624 18:57:46.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2811, 136, 136, 1392632052, '19600103 22:59:33.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2812, 365, 365, 1105525699, '19780218 18:30:38.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2813, 316, 316, 1095631297, '19960730 20:49:17.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2814, 789, 789, 1862395772, '19720425 13:54:20.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2815, 792, 792, 1782175112, '19860727 21:54:52.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2816, 310, 310, 513839930, '20030725 20:43:17.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2817, 132, 132, 1928764005, '19870325 06:35:57.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2818, 815, 815, 1732405886, '19690429 12:02:03.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2819, 602, 602, 1469558782, '19581004 05:50:18.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2820, 673, 673, 1541173817, '20070719 13:46:07.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2821, 699, 699, 1337107593, '19961114 04:21:56.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2822, 693, 693, 26588292, '19720426 17:14:08.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2823, 307, 307, 1065447729, '19561201 16:20:04.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2824, 625, 625, 1809091527, '19990626 14:25:43.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2825, 643, 643, 1150677220, '20010925 04:26:39.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2826, 755, 755, 911471371, '19730620 10:42:59.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2827, 730, 730, 1157382125, '19661112 11:11:53.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2828, 252, 252, 170698914, '19670906 05:06:56.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2829, 356, 356, 305543213, '19741011 04:59:01.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2830, 947, 947, 1820435680, '19660412 23:23:41.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2831, 502, 502, 1928099205, '19811015 21:38:36.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2832, 114, 114, 1580625938, '20070521 15:45:08.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2833, 804, 804, 1634535106, '20040123 07:18:47.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2834, 511, 511, 1597130910, '19710605 20:29:55.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2835, 563, 563, 1532537613, '20070309 01:31:34.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2836, 589, 589, 1164986691, '19681121 01:29:25.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2837, 2, 2, 2006314049, '19991213 17:10:22.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2838, 483, 483, 362777737, '19660307 06:59:22.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2839, 683, 683, 151547019, '20020130 16:41:03.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2840, 242, 242, 1587522620, '19550410 06:03:30.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2841, 830, 830, 1445206384, '19890411 08:17:05.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2842, 412, 412, 967481153, '19600301 23:41:08.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2843, 165, 165, 298742711, '19600106 09:04:38.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2844, 369, 369, 958551042, '19760327 07:44:23.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2845, 588, 588, 1049642655, '19630531 01:07:08.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2846, 161, 161, 662264294, '19690708 21:21:00.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2847, 882, 882, 1578660043, '19531223 09:00:10.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2848, 747, 747, 618540011, '19710703 05:30:52.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2849, 741, 741, 1636731793, '19720809 18:48:55.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2850, 709, 709, 965252251, '19840226 21:39:38.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2851, 547, 547, 739612244, '19850503 07:34:14.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2852, 425, 425, 1584426711, '19930217 12:35:45.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2853, 650, 650, 1161695382, '19891121 09:43:33.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2854, 521, 521, 1821270432, '19950516 01:55:36.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2855, 472, 472, 1912538964, '19761007 12:24:20.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2856, 6, 6, 23555062, '20010131 14:40:19.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2857, 383, 383, 1393355949, '19980203 23:22:46.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2858, 231, 231, 845980162, '19860727 16:15:02.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2859, 133, 133, 821500774, '20070806 13:19:00.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2860, 98, 98, 1507095980, '19990218 05:14:25.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2861, 418, 418, 2076373546, '20051003 01:07:38.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2862, 12, 12, 1828267393, '19570624 03:16:14.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2863, 897, 897, 1573550631, '19621206 03:59:05.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2864, 575, 575, 882421028, '19670214 08:49:23.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2865, 51, 51, 1005862744, '19970907 21:19:10.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2866, 606, 606, 1907251647, '19600814 07:14:24.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2867, 50, 50, 745988550, '19820126 11:11:50.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2868, 576, 576, 2051466699, '19780226 00:19:21.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2869, 411, 411, 691511615, '19730216 12:22:45.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2870, 497, 497, 1515023127, '19700906 20:25:26.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2871, 233, 233, 1353497921, '19560813 03:32:55.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2872, 938, 938, 459794361, '19740118 23:36:35.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2873, 99, 99, 216420615, '19750329 19:21:00.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2874, 811, 811, 757192089, '19560626 23:46:47.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2875, 502, 502, 1787927758, '19710410 05:29:01.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2876, 560, 560, 1800106648, '19890915 04:40:48.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2877, 708, 708, 594553892, '19650422 08:09:29.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2878, 26, 26, 1038418309, '19880905 08:35:41.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2879, 137, 137, 1349683043, '19890127 13:18:34.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2880, 594, 594, 1887065723, '19850320 07:05:38.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2881, 808, 808, 725289357, '19720629 01:42:48.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2882, 749, 749, 1640163978, '20030512 05:41:00.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2883, 145, 145, 253224883, '20030126 10:18:01.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2884, 557, 557, 1069539331, '19980814 07:19:23.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2885, 841, 841, 1678339996, '19881209 15:49:27.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2886, 746, 746, 420787922, '19960827 09:02:51.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2887, 848, 848, 1683851596, '19700701 06:01:34.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2888, 119, 119, 746247521, '19610910 05:23:11.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2889, 415, 415, 1677922493, '20020828 08:05:35.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2890, 790, 790, 1988933907, '19590206 10:51:15.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2891, 83, 83, 861468726, '19781017 02:06:38.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2892, 103, 103, 1423922342, '19660520 00:55:20.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2893, 700, 700, 994006613, '19660731 17:40:22.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2894, 62, 62, 1677948434, '19551214 11:26:38.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2895, 954, 954, 1587985692, '19570707 04:55:52.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2896, 821, 821, 934943448, '19841019 05:00:50.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2897, 851, 851, 1201029534, '20050326 19:42:03.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2898, 281, 281, 113226615, '20001121 00:15:15.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2899, 895, 895, 2135692865, '19860719 10:25:13.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2900, 703, 703, 1844522760, '19551017 17:52:43.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2901, 419, 419, 998422551, '19950612 10:09:10.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2902, 510, 510, 608261177, '19831027 08:40:48.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2903, 589, 589, 1988289273, '20060515 17:08:07.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2904, 900, 900, 24769055, '19541204 22:23:28.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2905, 423, 423, 826255129, '19800716 18:06:43.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2906, 614, 614, 414437396, '19640415 07:57:38.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2907, 806, 806, 29536839, '19701122 17:14:44.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2908, 990, 990, 653142369, '19860528 09:56:46.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2909, 995, 995, 667018363, '19770205 20:26:34.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2910, 330, 330, 2100445542, '19950122 07:43:32.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2911, 700, 700, 637414390, '19881012 06:30:49.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2912, 540, 540, 1403286521, '19620601 14:47:05.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2913, 597, 597, 325776307, '20050629 02:56:27.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2914, 380, 380, 1331825376, '19750520 06:13:21.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2915, 503, 503, 1385096744, '19790823 03:31:36.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2916, 766, 766, 1533532957, '19550525 19:26:37.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2917, 285, 285, 1613672229, '19620528 16:58:12.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2918, 900, 900, 1418709595, '19720422 20:39:41.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2919, 847, 847, 1876986803, '19860306 16:59:56.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2920, 622, 622, 1880797575, '19540112 12:16:19.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2921, 250, 250, 1346679986, '19980213 01:12:50.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2922, 815, 815, 494714416, '19730519 05:48:38.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2923, 873, 873, 945064646, '19830630 16:13:46.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2924, 327, 327, 473888195, '19670112 01:31:30.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2925, 413, 413, 612190700, '19991120 18:18:47.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2926, 67, 67, 883871341, '19980326 02:37:34.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2927, 779, 779, 1702030285, '19600621 05:56:12.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2928, 311, 311, 1527943380, '19720416 07:54:22.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2929, 931, 931, 1290700118, '20061220 18:50:54.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2930, 485, 485, 920543472, '19940621 00:28:11.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2931, 595, 595, 990831814, '19920620 08:58:44.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2932, 347, 347, 30295295, '19720531 07:54:14.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2933, 421, 421, 872467970, '19550218 22:10:27.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2934, 168, 168, 1646524944, '19860413 19:25:50.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2935, 646, 646, 1121810908, '19971007 20:56:28.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2936, 860, 860, 649776313, '19960901 17:02:00.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2937, 728, 728, 888494973, '20041226 12:32:53.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2938, 839, 839, 774332154, '20010222 11:54:32.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2939, 535, 535, 327597191, '19710129 13:12:39.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2940, 597, 597, 1835423039, '19770827 07:51:48.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2941, 311, 311, 517216514, '19781006 15:48:06.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2942, 96, 96, 1413891720, '19920202 20:04:48.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2943, 848, 848, 1688513525, '19920803 08:56:37.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2944, 611, 611, 1542125498, '19600807 00:22:02.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2945, 494, 494, 1717945186, '19780426 04:20:25.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2946, 639, 639, 1596295458, '19690518 11:19:33.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2947, 510, 510, 1443301948, '19681119 21:58:52.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2948, 811, 811, 344168359, '19990313 11:27:01.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2949, 934, 934, 1427184105, '19840422 07:55:01.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2950, 885, 885, 1799388675, '19550213 09:29:14.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2951, 828, 828, 1674527200, '19750525 12:02:37.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2952, 962, 962, 313818229, '19851204 23:02:21.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2953, 537, 537, 1456822857, '19670918 07:15:18.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2954, 938, 938, 2031885237, '19850707 22:08:54.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2955, 829, 829, 100976565, '19650902 16:39:53.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2956, 802, 802, 922255575, '19750125 04:20:31.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2957, 935, 935, 2034666508, '19530429 16:27:02.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2958, 217, 217, 1349126850, '19920504 15:36:36.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2959, 648, 648, 92878148, '19630114 04:04:41.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2960, 989, 989, 2049298280, '19900422 15:29:08.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2961, 76, 76, 905751384, '19561026 02:01:26.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2962, 979, 979, 1449080351, '20060807 09:20:22.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2963, 444, 444, 841466109, '19870607 15:05:51.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2964, 933, 933, 485595139, '19900817 19:58:22.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2965, 767, 767, 1632130430, '19550804 22:44:47.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2966, 742, 742, 1907221813, '19690514 22:36:35.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2967, 822, 822, 249281035, '19600413 16:36:38.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2968, 931, 931, 1026388357, '19720318 07:30:36.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2969, 445, 445, 236090895, '19850811 09:16:52.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2970, 844, 844, 878772655, '19901220 10:29:35.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2971, 512, 512, 568000719, '19580529 06:53:44.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2972, 849, 849, 631267955, '19690404 05:03:40.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2973, 446, 446, 1443334290, '19540326 07:27:01.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2974, 781, 781, 806543782, '19610711 09:08:08.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2975, 595, 595, 300239611, '19830407 20:28:12.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2976, 499, 499, 955197260, '19590112 05:08:01.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2977, 234, 234, 1234368263, '19881014 20:52:00.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2978, 486, 486, 675687376, '19751124 15:51:43.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2979, 362, 362, 795026789, '19960916 21:11:04.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2980, 962, 962, 1621929707, '19830706 06:59:14.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2981, 301, 301, 1865242461, '19820507 04:38:42.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2982, 384, 384, 144320390, '19690409 18:29:11.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2983, 716, 716, 1655024767, '19951224 23:59:20.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2984, 63, 63, 2137357161, '20041107 09:21:40.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2985, 785, 785, 1605189690, '19720128 12:25:33.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2986, 209, 209, 204831793, '19590718 09:56:23.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2987, 768, 768, 803314665, '19570914 00:38:56.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2988, 477, 477, 1452873704, '19770813 00:01:33.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2989, 83, 83, 1936295599, '19730812 02:45:57.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2990, 801, 801, 1304881551, '19750914 16:35:56.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2991, 983, 983, 1076128389, '19960506 00:34:58.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2992, 145, 145, 563400776, '19650824 20:56:16.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2993, 930, 930, 1228940514, '19910210 07:49:51.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2994, 204, 204, 550664703, '19881007 21:43:31.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2995, 383, 383, 202418117, '19731102 05:50:21.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2996, 934, 934, 376243286, '19800229 06:27:44.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2997, 254, 254, 1158982062, '19570829 04:58:57.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2998, 386, 386, 1664849214, '19541218 12:58:14.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (2999, 275, 275, 875105974, '19580104 01:37:15.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3000, 777, 777, 278100690, '19611213 05:43:50.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3001, 610, 610, 1865829465, '19620204 18:42:55.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3002, 880, 880, 740689074, '20040902 14:28:15.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3003, 967, 967, 772155701, '19660803 02:10:08.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3004, 966, 966, 1765102445, '20010502 21:25:25.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3005, 868, 868, 1609125635, '20040916 02:35:30.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3006, 248, 248, 1654429429, '19590221 06:58:57.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3007, 544, 544, 71948327, '19840911 15:35:19.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3008, 518, 518, 962631962, '19590310 22:07:09.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3009, 893, 893, 836040892, '19550403 01:40:41.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3010, 608, 608, 1617565136, '19590823 11:31:51.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3011, 187, 187, 1092189216, '19940411 07:28:30.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3012, 864, 864, 2041448297, '19850605 00:35:18.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3013, 284, 284, 324585257, '20030817 18:25:31.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3014, 960, 960, 1335937138, '19870710 21:05:26.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3015, 999, 999, 998158962, '19601217 10:50:26.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3016, 36, 36, 84960868, '19950718 07:32:15.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3017, 253, 253, 723484858, '19630815 09:21:01.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3018, 437, 437, 1879452388, '19900730 05:44:11.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3019, 803, 803, 609406679, '19710722 07:32:49.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3020, 249, 249, 1233789978, '20040117 12:48:24.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3021, 312, 312, 1327815236, '19640831 21:57:38.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3022, 359, 359, 625438714, '19900831 16:35:04.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3023, 15, 15, 1244649951, '20060806 04:44:39.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3024, 121, 121, 1824149679, '19621128 16:40:55.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3025, 243, 243, 29637119, '20020815 09:54:37.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3026, 440, 440, 480001806, '20001003 09:57:27.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3027, 994, 994, 1347112199, '19860221 21:13:48.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3028, 974, 974, 190379439, '19730617 09:43:30.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3029, 590, 590, 392054963, '19950908 03:17:47.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3030, 412, 412, 1798916013, '19560208 14:00:28.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3031, 26, 26, 1656397343, '19540517 00:09:04.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3032, 29, 29, 581646673, '19861029 07:53:46.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3033, 943, 943, 1443082159, '19701121 14:14:28.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3034, 705, 705, 143698205, '19871006 15:27:14.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3035, 863, 863, 781579063, '19740601 12:16:19.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3036, 885, 885, 1677261248, '19850902 14:10:26.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3037, 444, 444, 1381480825, '19550908 16:19:02.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3038, 168, 168, 618224502, '20020823 02:33:54.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3039, 656, 656, 1779108956, '19530222 06:46:09.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3040, 771, 771, 1756427653, '19651208 05:26:35.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3041, 827, 827, 559957353, '19821104 18:21:03.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3042, 729, 729, 855150287, '20060706 02:31:46.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3043, 496, 496, 1101434561, '19750513 22:20:51.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3044, 764, 764, 1919473358, '19661221 15:14:05.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3045, 701, 701, 1667193037, '19890604 09:07:26.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3046, 195, 195, 181209979, '19631202 23:59:39.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3047, 875, 875, 615422074, '19700107 02:30:45.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3048, 712, 712, 1948917808, '19560803 10:44:24.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3049, 613, 613, 536696041, '19801022 10:53:05.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3050, 949, 949, 1291345727, '19860416 22:18:06.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3051, 371, 371, 1708379807, '19690717 21:06:42.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3052, 329, 329, 231054022, '19750129 23:14:45.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3053, 789, 789, 2140958869, '19910529 05:25:24.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3054, 799, 799, 1570593339, '19611217 12:48:51.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3055, 52, 52, 369466921, '20050412 07:46:11.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3056, 928, 928, 1066949732, '19790606 02:07:50.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3057, 366, 366, 129485212, '20060127 09:43:36.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3058, 479, 479, 1744834798, '19560905 09:29:13.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3059, 238, 238, 1357556492, '20060917 11:30:48.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3060, 889, 889, 113741681, '19561214 16:38:03.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3061, 117, 117, 1837079631, '19801229 09:45:08.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3062, 804, 804, 1079012332, '19640328 05:51:49.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3063, 447, 447, 208157038, '19710702 18:49:22.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3064, 825, 825, 1817112715, '20070222 11:40:13.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3065, 266, 266, 715442352, '19580409 11:22:46.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3066, 866, 866, 1673486151, '19600613 23:34:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3067, 855, 855, 910583492, '19670716 10:20:50.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3068, 746, 746, 1771062453, '19681112 02:58:17.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3069, 89, 89, 1505882423, '19660209 08:49:07.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3070, 585, 585, 514887005, '19830415 20:39:59.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3071, 529, 529, 312661077, '19921109 15:13:13.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3072, 910, 910, 440068336, '19681222 06:47:06.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3073, 965, 965, 1160501072, '19900607 22:58:01.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3074, 662, 662, 1114361564, '19580815 02:06:14.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3075, 358, 358, 622180439, '19740315 18:27:21.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3076, 757, 757, 1615336187, '19660225 19:25:52.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3077, 471, 471, 1721835557, '19680420 18:14:13.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3078, 834, 834, 54015374, '19920816 13:30:33.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3079, 663, 663, 1220157448, '19810625 07:33:29.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3080, 525, 525, 505854885, '19910914 09:54:57.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3081, 39, 39, 277579690, '19830928 07:26:41.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3082, 941, 941, 244662725, '19820722 10:29:24.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3083, 414, 414, 1294311595, '20000824 22:50:25.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3084, 919, 919, 935975529, '19620525 04:59:40.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3085, 820, 820, 1917120419, '19940723 16:53:45.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3086, 436, 436, 2096911990, '19870417 00:54:18.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3087, 728, 728, 619089706, '20030602 02:28:21.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3088, 740, 740, 1571552800, '19611206 01:25:37.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3089, 188, 188, 1041578258, '19900625 07:41:02.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3090, 382, 382, 1185593994, '20021226 10:08:29.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3091, 409, 409, 421976666, '19870806 04:26:49.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3092, 474, 474, 192734915, '20070102 06:49:48.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3093, 66, 66, 1235997573, '20031205 15:03:06.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3094, 471, 471, 1670404994, '20040309 14:08:06.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3095, 19, 19, 1101120595, '19921208 19:41:26.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3096, 643, 643, 1289424535, '19710810 12:29:13.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3097, 337, 337, 1438458347, '19880105 07:42:24.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3098, 10, 10, 1986964875, '19760320 10:40:37.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3099, 950, 950, 96984307, '19610914 03:51:18.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3100, 973, 973, 447432657, '19811222 09:32:41.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3101, 836, 836, 619900898, '20040518 13:38:49.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3102, 293, 293, 852016247, '19540225 23:32:28.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3103, 800, 800, 1617920523, '19980624 01:55:17.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3104, 688, 688, 1216370377, '20050708 14:13:05.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3105, 240, 240, 465356970, '20010606 07:04:52.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3106, 95, 95, 2084121012, '19530727 14:19:35.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3107, 968, 968, 541517612, '19920823 00:16:44.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3108, 718, 718, 276933008, '19851015 03:19:10.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3109, 828, 828, 642981859, '19951003 18:21:29.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3110, 41, 41, 1985652686, '19920216 12:20:19.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3111, 104, 104, 1386597487, '19640216 07:53:36.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3112, 809, 809, 130227380, '19660613 20:13:03.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3113, 694, 694, 980429897, '19830313 01:55:44.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3114, 928, 928, 1133978332, '19680105 01:35:50.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3115, 782, 782, 588866270, '19810318 09:11:21.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3116, 379, 379, 945514498, '20060608 23:15:44.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3117, 638, 638, 394175771, '19710803 07:01:23.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3118, 865, 865, 1967620289, '19620208 13:49:41.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3119, 811, 811, 1604024791, '19650802 18:46:28.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3120, 674, 674, 1467997377, '19781223 10:28:28.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3121, 859, 859, 140164896, '19650111 21:06:22.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3122, 21, 21, 477680527, '19580811 08:55:12.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3123, 991, 991, 1865624157, '19860519 19:17:14.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3124, 963, 963, 744159993, '19710213 22:40:37.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3125, 914, 914, 1534697502, '20030909 16:13:09.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3126, 509, 509, 981446090, '19931107 08:23:24.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3127, 967, 967, 1868821483, '19730116 15:44:00.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3128, 516, 516, 61873425, '19940330 08:49:54.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3129, 989, 989, 800046092, '19730905 06:24:47.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3130, 563, 563, 1024487003, '19550807 05:58:07.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3131, 829, 829, 1130146646, '19860220 11:43:28.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3132, 697, 697, 1986345047, '20000131 07:33:35.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3133, 592, 592, 1933997930, '19831204 09:39:14.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3134, 251, 251, 1382199826, '20070703 22:00:47.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3135, 385, 385, 1547514553, '19950427 20:16:08.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3136, 134, 134, 486129376, '19820803 07:54:13.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3137, 50, 50, 503610707, '19920128 08:34:06.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3138, 387, 387, 715470810, '20030216 08:37:19.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3139, 684, 684, 1558484320, '19681217 21:54:47.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3140, 200, 200, 677023436, '19931228 02:34:09.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3141, 637, 637, 168217705, '20020824 00:37:34.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3142, 667, 667, 190326230, '19700818 23:09:10.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3143, 438, 438, 117540454, '19740304 07:04:07.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3144, 616, 616, 322422967, '20060508 19:20:42.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3145, 948, 948, 2070227006, '19911111 02:14:52.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3146, 272, 272, 839140748, '19740223 08:13:46.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3147, 196, 196, 384556481, '19761023 04:54:04.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3148, 487, 487, 2136693243, '19881201 13:27:16.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3149, 543, 543, 639677004, '19751224 04:56:45.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3150, 621, 621, 747618942, '19940701 20:25:41.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3151, 436, 436, 2140426125, '19530108 05:27:49.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3152, 689, 689, 1744518070, '19781127 17:52:42.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3153, 726, 726, 792284808, '19630819 15:54:09.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3154, 329, 329, 1589723378, '19900921 17:22:50.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3155, 988, 988, 1123673861, '19691211 12:26:18.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3156, 799, 799, 37963539, '19981009 04:43:36.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3157, 237, 237, 1857631939, '19751009 04:15:13.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3158, 805, 805, 1624774103, '19800511 03:14:40.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3159, 355, 355, 1011416487, '19541030 21:59:55.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3160, 649, 649, 322571793, '19600730 22:41:28.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3161, 556, 556, 213596357, '19880709 22:35:35.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3162, 812, 812, 1822352744, '20060525 15:26:49.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3163, 692, 692, 1127652756, '19650209 20:54:23.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3164, 92, 92, 996865539, '19930227 12:23:22.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3165, 744, 744, 1950828672, '19581228 00:36:51.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3166, 743, 743, 1753348275, '19720116 00:20:01.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3167, 633, 633, 568987426, '19900710 10:33:48.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3168, 294, 294, 1946452615, '19830909 03:54:56.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3169, 671, 671, 1466136757, '19800908 05:19:41.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3170, 774, 774, 1481832724, '20060815 01:17:07.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3171, 778, 778, 1747358571, '19670513 14:41:38.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3172, 906, 906, 1295172786, '19760615 22:24:03.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3173, 273, 273, 588678773, '20010220 15:54:53.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3174, 32, 32, 360473827, '19790805 16:12:19.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3175, 638, 638, 1607892347, '19840430 09:50:53.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3176, 536, 536, 2072994308, '20020525 21:57:12.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3177, 593, 593, 788738308, '19920608 01:51:05.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3178, 966, 966, 605985629, '19880110 23:56:32.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3179, 357, 357, 16641544, '19870404 20:25:45.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3180, 816, 816, 1192274560, '19820718 07:59:23.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3181, 644, 644, 750642177, '19700116 03:29:20.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3182, 539, 539, 948303633, '19920215 02:16:44.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3183, 888, 888, 781409323, '19710408 03:53:09.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3184, 61, 61, 1085547702, '19870306 09:59:06.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3185, 465, 465, 615367416, '19550731 00:30:17.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3186, 848, 848, 1822064180, '19600326 17:50:45.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3187, 391, 391, 185035591, '19890614 15:39:26.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3188, 686, 686, 1002958942, '20010313 16:16:23.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3189, 941, 941, 1963147686, '19900724 09:34:28.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3190, 349, 349, 1689093817, '20040718 15:32:31.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3191, 321, 321, 549071837, '19591027 03:39:00.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3192, 549, 549, 1098281437, '19640919 05:19:26.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3193, 29, 29, 1518666179, '20010419 10:37:24.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3194, 75, 75, 1311049082, '19610520 23:13:19.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3195, 866, 866, 325197642, '19580621 03:58:34.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3196, 721, 721, 883545536, '20040330 09:10:44.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3197, 476, 476, 1666181011, '19580710 02:14:47.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3198, 765, 765, 1306867614, '19890103 18:40:45.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3199, 797, 797, 1770839274, '20000512 18:43:50.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3200, 982, 982, 267786269, '19721027 01:54:51.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3201, 532, 532, 1853718568, '19910815 21:39:57.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3202, 324, 324, 26532439, '20010213 00:59:08.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3203, 602, 602, 1321395419, '19610326 08:07:34.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3204, 172, 172, 1469786926, '19770510 03:39:37.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3205, 802, 802, 489059627, '19800220 05:44:02.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3206, 167, 167, 1213925260, '19840724 07:03:46.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3207, 33, 33, 1102492857, '19851006 01:57:48.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3208, 348, 348, 407409129, '19920113 23:41:49.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3209, 162, 162, 67248196, '19590525 07:31:57.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3210, 877, 877, 1382056314, '19750718 14:29:06.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3211, 945, 945, 797581573, '19590504 02:52:31.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3212, 59, 59, 107212615, '19950928 04:18:41.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3213, 9, 9, 886460573, '20010205 06:48:54.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3214, 722, 722, 167343950, '19800413 14:00:32.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3215, 257, 257, 38181198, '19980714 19:12:08.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3216, 103, 103, 659487625, '20040524 20:18:50.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3217, 681, 681, 81890863, '19880218 11:33:39.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3218, 90, 90, 1703377037, '19851206 10:55:16.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3219, 433, 433, 1819050404, '19900801 11:53:04.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3220, 678, 678, 366542699, '20061002 06:46:06.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3221, 763, 763, 1146293661, '19900802 08:40:34.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3222, 4, 4, 1412090771, '19970427 18:17:24.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3223, 84, 84, 297061943, '20010214 18:20:27.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3224, 567, 567, 280459126, '19840220 13:47:10.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3225, 650, 650, 1771980997, '19991020 21:38:07.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3226, 106, 106, 1857315350, '19550825 09:22:12.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3227, 198, 198, 1307361273, '19830226 11:46:39.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3228, 344, 344, 304564709, '19921003 16:41:33.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3229, 133, 133, 894592776, '19740215 12:13:44.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3230, 832, 832, 2116875383, '19880131 00:40:08.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3231, 852, 852, 332240625, '19961116 19:42:24.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3232, 168, 168, 945501701, '19560605 07:10:20.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3233, 784, 784, 1057849652, '19950830 05:12:41.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3234, 96, 96, 658815369, '19670609 18:30:54.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3235, 81, 81, 1300511037, '19981202 10:19:25.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3236, 396, 396, 2080240396, '19761202 02:30:11.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3237, 669, 669, 1684611818, '19831122 18:09:10.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3238, 702, 702, 1898667369, '20011128 00:13:32.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3239, 908, 908, 175832388, '19600116 04:15:03.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3240, 760, 760, 1986056682, '19790105 17:26:31.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3241, 8, 8, 866994070, '19820620 15:52:56.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3242, 558, 558, 100362984, '20050531 15:57:36.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3243, 19, 19, 355374150, '19941021 08:44:26.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3244, 205, 205, 959889351, '19680107 19:05:22.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3245, 846, 846, 125420660, '19980318 12:40:00.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3246, 214, 214, 1347326330, '19720128 23:20:18.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3247, 63, 63, 101525107, '19710813 22:30:31.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3248, 985, 985, 1276749556, '19740106 20:36:51.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3249, 734, 734, 1092750036, '19701106 04:01:11.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3250, 102, 102, 771236183, '19620126 07:39:43.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3251, 4, 4, 1589717891, '19690210 21:37:05.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3252, 220, 220, 1656749194, '19800802 15:19:30.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3253, 600, 600, 1769787361, '20060604 06:47:40.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3254, 91, 91, 1989383810, '20010810 11:57:44.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3255, 994, 994, 1231036271, '19900326 17:14:16.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3256, 454, 454, 280449249, '20020419 03:22:32.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3257, 542, 542, 480358387, '20071229 06:38:41.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3258, 713, 713, 959124607, '19850202 18:20:23.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3259, 85, 85, 101468100, '19850720 06:01:29.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3260, 831, 831, 1585598796, '19770628 20:21:50.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3261, 289, 289, 1826342689, '19540529 19:17:06.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3262, 376, 376, 1657898889, '20010101 09:16:14.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3263, 598, 598, 2087498280, '19701129 11:28:05.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3264, 200, 200, 907331540, '19790424 06:51:48.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3265, 520, 520, 839202173, '19860830 18:46:42.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3266, 510, 510, 1183645525, '19551129 19:42:11.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3267, 612, 612, 1419367579, '19530129 23:05:59.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3268, 528, 528, 1961628132, '19860830 12:18:00.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3269, 685, 685, 486386081, '19890512 03:41:07.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3270, 696, 696, 1858196449, '19740813 16:41:56.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3271, 596, 596, 1193325702, '19730704 02:09:40.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3272, 153, 153, 147663481, '19940322 11:20:07.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3273, 117, 117, 1366366101, '19781122 06:40:13.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3274, 955, 955, 1867452322, '19640727 18:26:33.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3275, 331, 331, 1344958761, '19770414 14:53:09.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3276, 577, 577, 1449592848, '19930302 16:42:57.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3277, 953, 953, 329990845, '19550709 09:32:58.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3278, 73, 73, 842374324, '19860208 23:15:05.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3279, 747, 747, 249610772, '19931204 01:07:10.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3280, 350, 350, 1618367225, '19800922 22:17:48.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3281, 30, 30, 521463756, '19920111 10:51:40.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3282, 110, 110, 1709016063, '19620220 15:09:48.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3283, 745, 745, 543380325, '19741129 12:40:22.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3284, 240, 240, 1332875543, '19650121 04:34:01.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3285, 427, 427, 174873301, '19711221 03:03:03.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3286, 14, 14, 1016217656, '19690417 04:22:53.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3287, 551, 551, 1062972842, '19580101 00:22:40.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3288, 169, 169, 65175749, '20020118 17:14:57.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3289, 582, 582, 520782853, '19850315 01:16:37.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3290, 70, 70, 2069453555, '20040814 06:56:52.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3291, 120, 120, 2046919610, '19761203 19:51:30.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3292, 237, 237, 2101302549, '20061018 23:48:46.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3293, 25, 25, 1248433421, '19690511 18:12:02.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3294, 120, 120, 2119775978, '19900719 07:53:12.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3295, 144, 144, 1844462641, '19770809 22:09:24.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3296, 862, 862, 951929590, '19890618 06:36:42.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3297, 175, 175, 55886595, '19870704 04:29:31.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3298, 609, 609, 1594884902, '19680630 01:52:38.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3299, 672, 672, 317493832, '19890509 00:18:39.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3300, 17, 17, 222305534, '19950420 16:57:48.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3301, 735, 735, 590308687, '19711231 00:14:19.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3302, 820, 820, 639664813, '19921213 10:12:30.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3303, 75, 75, 1648087304, '19920826 16:55:43.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3304, 29, 29, 786984375, '20040324 11:19:15.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3305, 992, 992, 760468563, '19960724 05:30:03.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3306, 639, 639, 442278389, '19821121 10:16:57.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3307, 695, 695, 1520185101, '19540912 08:39:17.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3308, 787, 787, 970278711, '19941108 12:21:06.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3309, 768, 768, 336954579, '19770428 21:04:35.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3310, 217, 217, 567939771, '20050123 00:31:18.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3311, 162, 162, 968784889, '19991012 17:49:34.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3312, 111, 111, 2041799773, '19741120 07:23:35.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3313, 211, 211, 1194345345, '19990301 17:13:13.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3314, 261, 261, 534320309, '19571028 07:43:41.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3315, 932, 932, 2138518219, '19930617 09:30:09.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3316, 724, 724, 1826056115, '20000407 04:07:18.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3317, 163, 163, 575335186, '19790203 20:35:52.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3318, 499, 499, 1935240112, '19581109 06:54:03.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3319, 526, 526, 1620613212, '19600505 03:48:44.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3320, 27, 27, 971675477, '19700516 14:23:49.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3321, 431, 431, 474380175, '20051127 19:19:31.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3322, 36, 36, 1931738631, '19590111 05:51:02.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3323, 468, 468, 927999529, '19540618 11:01:23.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3324, 574, 574, 227500720, '19920926 20:44:15.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3325, 910, 910, 395649385, '20050908 20:50:25.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3326, 221, 221, 1913415960, '19740915 02:20:53.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3327, 738, 738, 1887266288, '19771111 17:08:05.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3328, 912, 912, 963323745, '19960504 22:47:00.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3329, 151, 151, 1308099254, '19941217 20:17:08.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3330, 522, 522, 2055681309, '19951027 08:16:27.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3331, 1, 1, 1500378239, '19580831 12:13:26.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3332, 555, 555, 1314280580, '19950108 07:40:19.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3333, 204, 204, 1191716396, '20041001 22:56:25.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3334, 278, 278, 1024118511, '20060815 08:09:21.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3335, 458, 458, 1559421160, '19610924 22:03:29.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3336, 45, 45, 975541720, '20070128 00:39:10.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3337, 180, 180, 2004279714, '19770625 22:14:04.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3338, 107, 107, 955119245, '19780906 01:21:06.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3339, 987, 987, 696391332, '19760628 23:03:57.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3340, 901, 901, 24079500, '19970128 16:46:05.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3341, 733, 733, 1139824154, '19670806 19:43:36.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3342, 693, 693, 375137182, '19710222 12:01:33.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3343, 575, 575, 1813656029, '19770922 20:10:21.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3344, 893, 893, 1641333461, '19880220 00:45:18.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3345, 854, 854, 403823183, '19571103 13:07:17.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3346, 600, 600, 463595023, '19550115 12:27:54.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3347, 330, 330, 1056921766, '19600820 06:35:33.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3348, 974, 974, 2003222724, '19640715 10:28:20.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3349, 832, 832, 1647663893, '20050130 22:23:03.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3350, 985, 985, 1929941028, '19850503 18:02:06.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3351, 281, 281, 475841027, '19630516 10:00:49.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3352, 306, 306, 815001732, '19810825 21:35:27.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3353, 961, 961, 222128516, '19610225 22:03:53.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3354, 850, 850, 2014331336, '19711225 09:54:50.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3355, 332, 332, 1058912339, '19970522 21:38:17.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3356, 392, 392, 55522829, '19651220 18:23:17.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3357, 793, 793, 737248106, '19910628 23:11:07.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3358, 394, 394, 1743423364, '20071129 20:11:28.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3359, 94, 94, 1432681199, '20060715 14:28:49.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3360, 343, 343, 1214996185, '19741110 03:09:09.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3361, 730, 730, 243502480, '19580110 17:08:52.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3362, 46, 46, 1143671567, '19660509 09:52:16.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3363, 538, 538, 277446094, '20071121 16:03:57.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3364, 242, 242, 278792355, '19890702 12:48:08.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3365, 878, 878, 1441429307, '19990526 12:17:52.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3366, 887, 887, 209684137, '19571005 10:09:14.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3367, 854, 854, 862315031, '19780217 08:27:10.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3368, 647, 647, 51079558, '20000716 09:03:51.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3369, 504, 504, 463738003, '20040202 09:40:11.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3370, 291, 291, 736650321, '19940520 08:50:59.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3371, 687, 687, 985660799, '19751014 05:53:13.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3372, 745, 745, 723616733, '19530531 19:14:46.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3373, 892, 892, 478548366, '19900512 07:50:05.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3374, 456, 456, 102036545, '19710406 11:02:38.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3375, 212, 212, 1917846406, '19551011 18:40:13.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3376, 497, 497, 817121744, '19871007 07:17:57.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3377, 356, 356, 1983057527, '19890419 09:40:40.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3378, 236, 236, 1138897442, '19740428 10:16:04.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3379, 721, 721, 1037278847, '19871125 07:36:57.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3380, 127, 127, 450903996, '20030826 08:22:30.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3381, 164, 164, 767044856, '19670126 19:45:20.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3382, 258, 258, 851331402, '19660430 06:39:44.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3383, 287, 287, 2103347931, '19990404 00:23:57.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3384, 944, 944, 1102788289, '19620817 02:15:02.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3385, 114, 114, 1559939631, '19850613 22:15:37.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3386, 964, 964, 1095372332, '19850104 14:37:58.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3387, 196, 196, 211515730, '19861113 09:36:26.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3388, 87, 87, 1148706658, '19851009 13:01:35.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3389, 963, 963, 665456352, '19620326 10:31:03.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3390, 16, 16, 649184747, '20031005 03:40:12.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3391, 904, 904, 2113263124, '19680802 01:28:02.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3392, 345, 345, 1793872656, '19770727 02:02:35.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3393, 381, 381, 753556955, '19800221 10:52:16.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3394, 397, 397, 508793452, '19540820 19:54:48.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3395, 299, 299, 984630081, '19920119 23:37:13.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3396, 597, 597, 2070710281, '19540331 09:51:20.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3397, 284, 284, 1877858730, '19710402 19:57:35.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3398, 424, 424, 668429196, '19960322 07:22:13.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3399, 660, 660, 244916688, '19960925 12:27:25.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3400, 691, 691, 105867614, '19530130 02:58:02.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3401, 958, 958, 1414368899, '19841127 04:00:44.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3402, 774, 774, 904495997, '19680203 21:31:41.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3403, 729, 729, 1349957886, '19680612 00:48:09.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3404, 402, 402, 413879794, '19630914 13:32:04.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3405, 99, 99, 1691930333, '19620722 12:08:52.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3406, 678, 678, 568795235, '19621216 14:46:03.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3407, 5, 5, 2035657954, '19990415 13:45:21.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3408, 91, 91, 2095832722, '19971121 11:01:14.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3409, 71, 71, 270034910, '19690316 15:14:37.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3410, 958, 958, 1265323315, '19620817 14:20:20.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3411, 121, 121, 1126490196, '19840901 08:42:36.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3412, 109, 109, 980627209, '19700303 12:55:03.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3413, 570, 570, 286976527, '19730105 12:34:30.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3414, 833, 833, 1349093207, '19551121 06:06:19.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3415, 6, 6, 1171415787, '19601015 07:23:49.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3416, 231, 231, 1049821347, '19990912 10:29:07.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3417, 896, 896, 747941135, '19750204 09:28:18.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3418, 529, 529, 987746712, '19980407 13:48:55.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3419, 156, 156, 29739991, '19570119 14:32:30.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3420, 34, 34, 206375150, '19670522 21:39:54.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3421, 327, 327, 1903333007, '19781122 00:32:48.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3422, 90, 90, 2121310409, '20000509 19:25:34.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3423, 401, 401, 1751967265, '19910423 22:32:47.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3424, 355, 355, 1482839955, '19530501 05:59:59.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3425, 432, 432, 1548355700, '19781019 07:22:56.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3426, 122, 122, 2055503662, '20060319 03:50:38.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3427, 634, 634, 1794260886, '19810409 08:22:29.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3428, 135, 135, 75251659, '19880922 11:55:17.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3429, 705, 705, 1770106131, '19870317 11:25:25.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3430, 328, 328, 652652642, '19540714 08:48:53.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3431, 732, 732, 24457771, '19690707 11:20:22.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3432, 130, 130, 765086675, '20010127 02:18:26.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3433, 411, 411, 1335973179, '19850801 21:48:38.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3434, 576, 576, 509270785, '19871027 04:38:55.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3435, 7, 7, 963521295, '19710930 04:21:45.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3436, 213, 213, 1270672947, '20061222 22:13:38.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3437, 407, 407, 1432686485, '20051117 05:51:35.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3438, 400, 400, 215696463, '19880721 10:51:53.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3439, 725, 725, 536795566, '19530126 14:06:09.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3440, 88, 88, 301457840, '19750630 07:29:34.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3441, 430, 430, 684740059, '19930921 00:52:36.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3442, 817, 817, 35350733, '19961221 22:35:11.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3443, 141, 141, 544782106, '19690726 21:46:58.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3444, 939, 939, 1996891325, '20070808 20:10:43.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3445, 283, 283, 1596053845, '19720203 18:57:35.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3446, 345, 345, 1701161719, '20060531 12:32:59.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3447, 89, 89, 1238670821, '19570722 13:28:04.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3448, 233, 233, 979992796, '19770403 04:45:57.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3449, 164, 164, 150035889, '20011104 12:30:58.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3450, 682, 682, 1688212203, '20000508 13:08:06.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3451, 833, 833, 1689709341, '19870224 00:23:01.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3452, 121, 121, 1571422562, '19801225 06:08:39.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3453, 942, 942, 1114932194, '19920303 16:49:43.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3454, 881, 881, 756430860, '19820507 14:47:31.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3455, 32, 32, 504065870, '19820310 02:25:13.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3456, 440, 440, 1440391448, '19920719 08:35:10.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3457, 457, 457, 1167677889, '19841011 22:58:53.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3458, 446, 446, 910885596, '19680212 18:48:09.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3459, 971, 971, 1329691443, '19921125 06:09:07.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3460, 149, 149, 347750938, '19640504 08:18:14.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3461, 218, 218, 1159353994, '19890907 06:23:33.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3462, 949, 949, 2140027394, '19630725 01:50:04.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3463, 905, 905, 1218370447, '19630907 23:35:49.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3464, 352, 352, 1126501559, '19670904 06:25:44.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3465, 110, 110, 1045200615, '20010209 02:59:57.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3466, 742, 742, 1117906827, '19910805 06:24:09.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3467, 236, 236, 1823167971, '19920731 15:06:24.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3468, 325, 325, 1009640777, '19930311 07:55:35.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3469, 533, 533, 1317253026, '19920221 01:44:33.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3470, 377, 377, 181637900, '19611023 09:10:11.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3471, 269, 269, 969646112, '20011025 04:37:32.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3472, 333, 333, 1549628893, '19940715 22:36:25.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3473, 339, 339, 2036975077, '19980312 23:42:46.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3474, 188, 188, 989896105, '19890722 07:02:56.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3475, 41, 41, 921089369, '19810830 20:47:18.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3476, 995, 995, 1426784501, '19891130 01:57:36.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3477, 959, 959, 1214110746, '19831015 21:38:36.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3478, 749, 749, 416148400, '19910915 02:22:04.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3479, 758, 758, 98301383, '19890327 11:02:23.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3480, 430, 430, 442063266, '19800519 18:49:57.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3481, 203, 203, 362428454, '20010828 14:22:33.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3482, 923, 923, 504054204, '19570107 03:36:31.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3483, 743, 743, 13545777, '19941117 23:24:19.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3484, 903, 903, 343216542, '19941107 22:17:19.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3485, 471, 471, 1824403218, '19750520 08:25:51.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3486, 653, 653, 1138322731, '19960712 05:11:42.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3487, 482, 482, 443001243, '19611126 09:28:43.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3488, 850, 850, 263271030, '19560328 07:01:07.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3489, 238, 238, 1190504592, '19580820 02:13:41.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3490, 118, 118, 35411954, '19870312 19:46:35.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3491, 348, 348, 64696377, '19750313 23:14:44.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3492, 856, 856, 2038267172, '19901006 11:03:25.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3493, 422, 422, 103989614, '20030826 04:42:45.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3494, 138, 138, 1677701175, '19960923 05:47:55.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3495, 122, 122, 1518968614, '19931022 01:06:00.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3496, 813, 813, 1666802608, '19830228 23:02:32.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3497, 148, 148, 1777889000, '19860416 22:59:22.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3498, 167, 167, 946426756, '19541123 15:21:14.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3499, 316, 316, 1902716702, '19590630 17:10:45.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3500, 552, 552, 568503249, '19880630 12:33:26.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3501, 272, 272, 509314989, '19661031 21:26:35.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3502, 558, 558, 1820730634, '19720512 05:32:29.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3503, 490, 490, 1868559656, '19930210 03:01:24.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3504, 160, 160, 156137323, '19930113 03:20:46.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3505, 361, 361, 2110658553, '20060714 08:30:33.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3506, 666, 666, 2054038557, '20000810 01:46:35.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3507, 430, 430, 394728605, '19901014 06:25:53.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3508, 212, 212, 987004416, '19550814 09:46:46.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3509, 565, 565, 1456833283, '19530907 18:00:13.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3510, 835, 835, 1498354328, '20000409 00:27:36.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3511, 282, 282, 1165845646, '19611005 10:56:33.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3512, 500, 500, 1767114533, '20010126 20:36:49.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3513, 991, 991, 924963958, '19861117 07:45:46.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3514, 980, 980, 1437202181, '19650709 11:19:10.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3515, 438, 438, 239952618, '19700905 17:55:40.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3516, 926, 926, 1068283382, '19850901 02:47:02.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3517, 53, 53, 1004387622, '19760905 02:25:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3518, 997, 997, 641008499, '19761030 01:18:30.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3519, 651, 651, 44395632, '20000415 21:59:52.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3520, 44, 44, 1595550107, '19570731 21:48:15.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3521, 119, 119, 274300927, '19820909 20:55:25.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3522, 263, 263, 1061136280, '19890506 08:05:17.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3523, 544, 544, 939513366, '19870724 05:41:54.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3524, 898, 898, 5361453, '19571212 05:57:57.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3525, 986, 986, 290416338, '19940811 09:55:26.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3526, 460, 460, 552870731, '19640120 17:34:06.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3527, 467, 467, 269348324, '19981119 17:53:40.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3528, 326, 326, 1248808079, '19540620 17:54:51.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3529, 158, 158, 674384667, '20030930 05:56:55.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3530, 814, 814, 737964060, '20060701 21:44:45.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3531, 703, 703, 4283389, '19560816 02:58:14.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3532, 106, 106, 1566700916, '19811123 06:17:22.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3533, 545, 545, 182298338, '19850318 09:11:18.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3534, 252, 252, 742248359, '19530927 22:28:57.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3535, 972, 972, 798214924, '19660720 21:23:22.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3536, 892, 892, 1126857647, '19820419 08:50:04.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3537, 263, 263, 1952478692, '19711127 00:35:07.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3538, 236, 236, 1764935225, '19541105 20:03:32.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3539, 754, 754, 1815881112, '19960425 13:46:45.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3540, 894, 894, 1649391393, '19821009 06:39:16.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3541, 881, 881, 1543774246, '19580929 22:45:48.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3542, 378, 378, 1022647798, '19590415 23:41:57.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3543, 193, 193, 614362850, '19550722 13:00:54.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3544, 138, 138, 1304588738, '19660512 01:46:04.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3545, 652, 652, 1669835358, '19780608 08:50:02.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3546, 892, 892, 483041575, '19820214 02:37:55.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3547, 628, 628, 929549867, '19561120 03:17:39.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3548, 357, 357, 458883192, '19910217 17:23:35.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3549, 886, 886, 142006215, '19790119 11:52:14.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3550, 390, 390, 269376797, '19610220 22:18:58.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3551, 432, 432, 2008203302, '19590626 20:36:47.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3552, 528, 528, 279639945, '19620618 21:40:52.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3553, 643, 643, 1008262771, '19620808 17:21:21.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3554, 228, 228, 1770918330, '19541130 19:22:30.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3555, 112, 112, 602847300, '19581022 15:38:00.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3556, 434, 434, 1215026360, '19850626 13:21:18.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3557, 509, 509, 1608944862, '19921018 23:50:34.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3558, 60, 60, 2007598089, '19880301 21:03:27.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3559, 478, 478, 1041650860, '20060603 17:25:20.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3560, 249, 249, 402712308, '19950625 14:56:26.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3561, 190, 190, 1861693053, '19540922 07:52:55.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3562, 535, 535, 1459243541, '19890426 12:31:02.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3563, 605, 605, 420278420, '19591114 03:49:51.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3564, 253, 253, 241070217, '19550308 20:15:27.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3565, 315, 315, 1346662279, '19960824 21:29:22.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3566, 80, 80, 116842838, '19871112 04:39:11.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3567, 403, 403, 733059003, '19681111 11:25:31.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3568, 510, 510, 1169586915, '19860220 09:16:48.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3569, 908, 908, 426528473, '20061219 13:55:48.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3570, 579, 579, 1723457100, '19960518 09:05:36.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3571, 157, 157, 2027335855, '19661007 02:11:54.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3572, 748, 748, 489886421, '19741101 06:22:11.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3573, 351, 351, 1555508608, '19880705 11:31:44.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3574, 858, 858, 1530774192, '19700709 15:20:35.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3575, 230, 230, 742194846, '20061031 23:02:26.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3576, 244, 244, 389576879, '19760527 21:13:28.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3577, 487, 487, 1432393011, '19861016 19:04:22.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3578, 299, 299, 1548704555, '19740315 03:55:49.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3579, 43, 43, 298716598, '19870705 21:07:55.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3580, 811, 811, 40559744, '19650628 07:17:31.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3581, 391, 391, 1932561394, '19600302 14:16:27.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3582, 617, 617, 1791177604, '19601003 00:30:05.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3583, 723, 723, 1624499972, '19830601 06:02:36.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3584, 439, 439, 2134851960, '19950810 07:37:57.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3585, 299, 299, 790373897, '20000107 01:07:58.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3586, 887, 887, 1019070142, '20020227 05:17:22.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3587, 906, 906, 871950513, '19720417 12:56:00.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3588, 640, 640, 639179259, '19830419 13:48:48.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3589, 76, 76, 233370038, '20021206 06:50:56.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3590, 783, 783, 829291206, '19890123 08:02:03.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3591, 936, 936, 322410128, '19970630 08:59:29.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3592, 515, 515, 718213351, '19910926 03:31:40.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3593, 102, 102, 709227449, '19560603 02:38:12.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3594, 766, 766, 1501291047, '19531101 22:50:18.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3595, 632, 632, 2058998348, '19810117 22:46:21.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3596, 524, 524, 1755705726, '19770604 10:14:46.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3597, 571, 571, 521249887, '20070601 19:52:05.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3598, 136, 136, 1130293696, '19530516 16:45:26.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3599, 796, 796, 1744280370, '19770916 04:16:42.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3600, 295, 295, 388579924, '19980727 04:10:50.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3601, 151, 151, 798461022, '19660405 15:12:24.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3602, 951, 951, 1629094660, '19780929 18:00:50.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3603, 85, 85, 2005785567, '19920229 03:27:46.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3604, 498, 498, 2025921358, '19900902 02:46:38.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3605, 462, 462, 1018354132, '20020516 20:07:03.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3606, 295, 295, 256395670, '19920824 14:14:36.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3607, 617, 617, 176566140, '19811213 10:09:07.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3608, 117, 117, 820474882, '20000130 02:00:46.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3609, 410, 410, 345427414, '19560129 20:20:03.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3610, 300, 300, 942299169, '19900526 18:24:32.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3611, 72, 72, 1072465096, '20060909 18:16:55.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3612, 858, 858, 836530918, '19710807 19:54:44.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3613, 898, 898, 131354939, '19530828 23:55:31.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3614, 867, 867, 1032333163, '19931207 10:07:48.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3615, 470, 470, 1846589846, '19880425 00:39:59.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3616, 753, 753, 1950663828, '20011219 07:22:50.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3617, 16, 16, 461838405, '19581127 06:28:25.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3618, 506, 506, 958208337, '19720404 20:11:53.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3619, 145, 145, 561401992, '19630301 19:07:29.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3620, 824, 824, 1575386759, '20020627 16:12:00.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3621, 361, 361, 75509569, '19680727 15:43:10.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3622, 85, 85, 1942828522, '19930724 21:36:42.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3623, 884, 884, 231735352, '19910317 02:25:52.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3624, 513, 513, 317846808, '19701127 05:53:44.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3625, 256, 256, 1539333347, '20061118 00:06:06.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3626, 37, 37, 858563259, '19830110 22:40:14.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3627, 210, 210, 2103939366, '19710601 03:43:59.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3628, 496, 496, 1125405930, '19870904 12:41:25.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3629, 599, 599, 1002645967, '19970620 16:34:14.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3630, 609, 609, 87707044, '19820530 12:47:40.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3631, 527, 527, 1024886136, '19761226 01:21:22.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3632, 17, 17, 169854906, '19860603 02:18:56.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3633, 285, 285, 1271805646, '20040627 23:39:07.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3634, 182, 182, 2087281962, '19961208 16:57:04.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3635, 520, 520, 1600576128, '20070324 16:05:07.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3636, 453, 453, 721212927, '19890602 20:15:36.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3637, 617, 617, 2003327637, '19760803 21:02:18.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3638, 38, 38, 250402245, '20000928 03:15:58.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3639, 293, 293, 1415032695, '20010325 11:30:53.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3640, 219, 219, 2076976125, '19600515 10:53:21.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3641, 48, 48, 206810383, '19730317 19:08:14.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3642, 182, 182, 1164824970, '19800318 10:55:14.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3643, 759, 759, 1423374672, '19800321 17:28:45.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3644, 323, 323, 1869759516, '19650713 12:26:24.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3645, 726, 726, 1213587500, '19900515 13:45:08.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3646, 157, 157, 701812746, '19781124 13:04:44.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3647, 205, 205, 1736039326, '19910129 03:36:08.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3648, 311, 311, 1299136163, '19700626 16:30:23.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3649, 97, 97, 2141894661, '19730708 22:10:18.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3650, 279, 279, 1859690521, '19870129 15:23:31.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3651, 170, 170, 883277968, '19710710 03:46:20.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3652, 226, 226, 616710159, '19880227 23:40:11.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3653, 788, 788, 2060379720, '19980316 21:37:57.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3654, 621, 621, 616536191, '19830321 12:04:41.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3655, 426, 426, 886007687, '19821231 12:27:39.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3656, 642, 642, 1944837830, '19800910 17:35:41.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3657, 706, 706, 145638590, '19950715 15:34:58.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3658, 629, 629, 1663511319, '19740404 21:34:02.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3659, 711, 711, 1378163540, '19911015 02:40:32.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3660, 44, 44, 859056571, '19720506 21:26:49.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3661, 469, 469, 1416338696, '19740327 10:30:36.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3662, 903, 903, 1639562047, '20020409 21:27:42.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3663, 147, 147, 561687068, '19550812 09:26:31.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3664, 323, 323, 1638910097, '19810831 07:32:22.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3665, 230, 230, 1779281936, '19660601 17:03:09.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3666, 255, 255, 1256051429, '19730409 15:57:59.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3667, 99, 99, 459008488, '19750208 20:15:37.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3668, 807, 807, 452080965, '19640919 06:58:27.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3669, 933, 933, 1821736500, '19940914 18:02:41.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3670, 116, 116, 1668268803, '20061122 04:24:23.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3671, 954, 954, 1180408575, '19780518 10:20:32.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3672, 250, 250, 586904433, '19660228 03:12:27.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3673, 492, 492, 1253276073, '19790110 08:41:00.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3674, 407, 407, 199617289, '19551018 08:14:07.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3675, 342, 342, 1676399975, '19820412 21:03:45.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3676, 805, 805, 775436151, '19960510 04:47:56.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3677, 370, 370, 1163825169, '19660506 04:07:57.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3678, 700, 700, 1508278681, '19780904 13:59:28.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3679, 92, 92, 1631892539, '19880713 16:08:35.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3680, 561, 561, 1217036583, '19801225 11:01:21.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3681, 668, 668, 1643769986, '19991212 19:04:05.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3682, 62, 62, 1541206626, '19531206 11:13:36.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3683, 572, 572, 1840117528, '19670227 14:31:07.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3684, 596, 596, 870577686, '19630524 01:10:43.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3685, 974, 974, 447750792, '19631122 09:01:19.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3686, 623, 623, 830423505, '19961028 01:41:10.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3687, 511, 511, 198063114, '19960318 04:40:58.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3688, 467, 467, 1675008335, '19740410 11:34:25.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3689, 124, 124, 147868778, '19661210 04:29:24.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3690, 558, 558, 703330200, '19790714 22:29:25.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3691, 400, 400, 1971143891, '20011118 04:40:37.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3692, 909, 909, 1281983661, '19550502 23:28:15.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3693, 557, 557, 2069790163, '19611216 00:35:25.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3694, 751, 751, 53561070, '19811119 14:04:03.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3695, 842, 842, 208436284, '19940218 00:22:45.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3696, 35, 35, 815190214, '19781207 21:40:32.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3697, 100, 100, 707561127, '19770808 01:28:42.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3698, 977, 977, 281849925, '20060721 09:56:22.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3699, 455, 455, 844452024, '20070210 19:23:14.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3700, 667, 667, 1646010133, '19700205 21:47:09.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3701, 408, 408, 1498351115, '19561016 16:49:07.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3702, 957, 957, 137603593, '19790512 20:37:40.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3703, 211, 211, 1009236973, '19831011 22:27:41.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3704, 817, 817, 203989735, '19740816 17:45:54.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3705, 478, 478, 523251076, '19610914 05:02:59.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3706, 662, 662, 2146847670, '19580513 00:33:53.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3707, 794, 794, 1684157815, '19620218 14:59:11.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3708, 610, 610, 608548947, '19950531 13:23:50.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3709, 656, 656, 561936195, '19531209 15:00:55.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3710, 524, 524, 1118557255, '19940822 07:39:43.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3711, 634, 634, 1961876626, '19670509 13:27:43.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3712, 352, 352, 163302877, '19691111 01:35:30.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3713, 125, 125, 485484438, '19930921 05:25:27.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3714, 665, 665, 1576249213, '19631021 15:39:11.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3715, 461, 461, 865384882, '19800525 02:22:43.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3716, 405, 405, 583447527, '19730421 23:17:00.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3717, 364, 364, 271350276, '19880212 06:56:34.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3718, 109, 109, 1607155682, '20000320 08:15:48.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3719, 43, 43, 1288126253, '19701010 22:31:03.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3720, 735, 735, 1406953917, '19770803 15:21:58.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3721, 708, 708, 1862094016, '19850121 11:17:00.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3722, 579, 579, 2133710800, '19531101 08:41:11.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3723, 669, 669, 302940231, '20051010 02:29:03.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3724, 867, 867, 843248930, '19680303 19:33:15.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3725, 888, 888, 1987964631, '19580104 23:43:46.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3726, 136, 136, 51444517, '19760117 10:52:16.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3727, 567, 567, 598841813, '19570316 02:37:01.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3728, 465, 465, 1678051186, '20050219 18:36:56.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3729, 480, 480, 738317073, '19690829 07:51:21.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3730, 51, 51, 1315075420, '19560506 23:23:13.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3731, 470, 470, 1074593845, '19711004 03:19:14.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3732, 270, 270, 44780611, '19671015 18:11:34.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3733, 12, 12, 775341533, '19790724 18:36:13.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3734, 795, 795, 545158864, '19710608 18:21:25.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3735, 769, 769, 274400424, '19770311 18:12:13.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3736, 322, 322, 794878927, '19730802 22:26:25.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3737, 507, 507, 263464234, '19780224 12:45:54.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3738, 392, 392, 606338944, '20000712 20:45:12.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3739, 941, 941, 1827702731, '19540909 20:07:44.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3740, 903, 903, 1232507472, '19580713 08:27:26.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3741, 877, 877, 1319400187, '19870910 10:41:59.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3742, 115, 115, 1803265688, '19531019 15:17:08.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3743, 380, 380, 1266091742, '19730502 20:33:30.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3744, 492, 492, 68742406, '19800418 20:49:40.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3745, 736, 736, 1494883566, '19650308 09:01:41.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3746, 532, 532, 683886378, '19850108 03:05:06.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3747, 513, 513, 1284800907, '19690810 18:02:48.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3748, 780, 780, 1565578731, '20060224 08:56:14.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3749, 25, 25, 1975126858, '19540628 11:41:19.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3750, 554, 554, 1925694309, '19731030 01:05:44.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3751, 342, 342, 527264031, '19981025 14:43:57.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3752, 699, 699, 1455309511, '19850519 17:12:54.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3753, 791, 791, 1053960148, '19881011 11:25:19.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3754, 707, 707, 1496747998, '19820711 04:01:16.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3755, 551, 551, 2070335830, '19930114 10:30:09.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3756, 317, 317, 1069539676, '19551217 08:07:56.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3757, 185, 185, 1995288989, '19810801 18:08:36.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3758, 674, 674, 1490178877, '19680810 02:54:26.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3759, 414, 414, 1717707022, '19690812 18:02:08.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3760, 689, 689, 1290977251, '19930827 18:10:42.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3761, 448, 448, 338051173, '19540227 21:56:51.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3762, 197, 197, 1311306791, '19641228 20:22:15.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3763, 636, 636, 649942762, '19781003 05:32:28.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3764, 101, 101, 1995837510, '20050803 15:37:42.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3765, 428, 428, 939710403, '19751120 04:20:29.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3766, 453, 453, 869854654, '20070724 19:16:09.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3767, 879, 879, 18124491, '19980419 06:59:17.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3768, 174, 174, 1033281157, '19750416 11:04:01.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3769, 901, 901, 1059969383, '19940810 21:26:57.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3770, 418, 418, 110069305, '19591024 03:56:18.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3771, 520, 520, 92253818, '20030225 10:31:06.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3772, 376, 376, 1563026126, '19950801 10:11:22.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3773, 458, 458, 228951342, '19980712 12:08:04.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3774, 305, 305, 713733724, '19650331 14:03:37.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3775, 974, 974, 745320376, '19971123 04:39:59.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3776, 590, 590, 1992123401, '19840404 19:59:51.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3777, 946, 946, 1506890027, '19880630 12:07:41.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3778, 232, 232, 1515646346, '19780624 05:39:22.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3779, 402, 402, 665778783, '19551228 10:31:33.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3780, 208, 208, 323230757, '19801226 20:38:39.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3781, 539, 539, 1032953151, '19590609 16:49:28.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3782, 131, 131, 284999736, '19590119 17:40:46.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3783, 2, 2, 1442633637, '20030826 06:55:36.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3784, 309, 309, 480278751, '20031030 06:45:37.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3785, 519, 519, 1761418120, '19650712 08:39:16.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3786, 781, 781, 1739223101, '19940516 10:06:19.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3787, 851, 851, 918165499, '19870103 06:46:15.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3788, 442, 442, 1013840069, '20050112 14:34:56.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3789, 215, 215, 1774645819, '19860524 07:51:16.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3790, 133, 133, 894441709, '19740327 10:04:16.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3791, 415, 415, 1858525005, '20000101 04:17:48.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3792, 147, 147, 1884481288, '19620718 09:51:28.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3793, 983, 983, 1207899996, '19831202 02:43:04.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3794, 371, 371, 329570925, '19690112 01:57:01.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3795, 626, 626, 386992221, '19570516 10:30:34.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3796, 762, 762, 807832683, '19750913 14:19:44.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3797, 585, 585, 2025928144, '19830116 09:44:40.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3798, 87, 87, 1336160534, '19750929 04:55:47.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3799, 224, 224, 1440967471, '19570531 16:29:11.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3800, 328, 328, 1626954594, '19650814 13:45:32.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3801, 281, 281, 778946207, '19940920 20:05:49.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3802, 410, 410, 2016471296, '20020426 06:58:46.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3803, 741, 741, 852898087, '19640717 11:29:16.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3804, 978, 978, 1774043515, '20020904 07:45:01.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3805, 177, 177, 1898144620, '19780903 14:34:38.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3806, 100, 100, 270872677, '19560327 04:32:34.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3807, 606, 606, 1325660625, '19941107 05:04:50.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3808, 8, 8, 1311432685, '19760713 21:21:42.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3809, 698, 698, 453152378, '19920816 23:21:17.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3810, 399, 399, 190103915, '19611012 14:30:17.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3811, 730, 730, 843546027, '19750617 20:00:15.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3812, 340, 340, 1253627087, '19560209 12:29:14.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3813, 460, 460, 523344267, '19650814 16:22:52.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3814, 686, 686, 937176683, '19960815 21:23:28.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3815, 443, 443, 348390665, '19870322 01:21:13.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3816, 201, 201, 787502730, '20030208 04:16:04.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3817, 797, 797, 164010562, '19690504 13:26:38.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3818, 809, 809, 759255243, '19821203 22:46:50.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3819, 290, 290, 577297730, '19930123 06:58:25.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3820, 792, 792, 973137329, '19890705 18:14:09.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3821, 573, 573, 278550713, '19730722 12:29:53.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3822, 199, 199, 1494393632, '20010405 16:24:20.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3823, 902, 902, 2028785674, '19961122 03:12:45.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3824, 738, 738, 1478118370, '19730517 11:22:41.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3825, 956, 956, 1850991260, '19671022 23:38:30.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3826, 277, 277, 1077053297, '19930811 00:39:38.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3827, 218, 218, 338407861, '19640831 07:28:18.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3828, 911, 911, 1787711638, '19820701 10:11:02.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3829, 343, 343, 1460971617, '19601116 03:33:02.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3830, 90, 90, 1038282805, '19750313 14:20:15.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3831, 802, 802, 1840548625, '19540321 10:15:11.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3832, 27, 27, 1941252648, '19651003 07:11:53.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3833, 346, 346, 1649684336, '19740124 13:10:10.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3834, 223, 223, 1256513218, '19980514 20:46:00.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3835, 269, 269, 173257710, '19940407 00:32:50.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3836, 250, 250, 821087162, '19650212 09:50:42.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3837, 748, 748, 1440431185, '20020704 06:11:29.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3838, 550, 550, 848050274, '19531222 23:10:35.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3839, 241, 241, 1075240306, '19780226 16:10:59.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3840, 429, 429, 1195944921, '19620418 04:06:41.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3841, 766, 766, 395190616, '20070709 05:01:29.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3842, 371, 371, 562871587, '19630623 09:24:33.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3843, 171, 171, 567310637, '19650529 15:13:39.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3844, 635, 635, 1567823636, '19770812 17:20:58.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3845, 223, 223, 405093487, '20061009 14:04:01.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3846, 679, 679, 169561669, '19961122 15:48:33.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3847, 487, 487, 884317952, '20041202 17:28:35.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3848, 880, 880, 462278745, '19950417 05:19:35.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3849, 872, 872, 1505023562, '19891023 00:35:48.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3850, 96, 96, 1396109130, '19620407 06:14:29.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3851, 783, 783, 1009364910, '19590513 00:53:05.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3852, 588, 588, 2057956985, '19530213 10:57:49.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3853, 433, 433, 739334705, '19900904 21:57:22.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3854, 294, 294, 466757570, '19751125 22:15:02.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3855, 654, 654, 1454122209, '20000123 01:15:39.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3856, 159, 159, 128853157, '20010616 03:41:28.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3857, 644, 644, 1890763073, '19580605 03:46:01.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3858, 403, 403, 466893399, '19990302 00:06:35.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3859, 606, 606, 2122537254, '19871114 08:06:30.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3860, 987, 987, 1955712599, '19930122 13:54:59.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3861, 235, 235, 920628440, '19990726 21:04:16.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3862, 719, 719, 725605861, '19650509 18:53:48.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3863, 727, 727, 884157416, '19680828 17:48:39.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3864, 165, 165, 867459140, '19700607 09:01:01.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3865, 59, 59, 32353224, '19600725 04:15:06.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3866, 807, 807, 426136680, '19620915 12:48:22.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3867, 507, 507, 1040791863, '19900114 23:19:03.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3868, 65, 65, 1312614157, '19750401 19:36:51.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3869, 992, 992, 584865647, '19550512 20:50:38.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3870, 148, 148, 196117739, '19750207 15:30:31.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3871, 774, 774, 220676950, '19530807 22:04:34.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3872, 395, 395, 76721059, '19680513 14:16:03.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3873, 216, 216, 503047410, '19571008 06:35:51.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3874, 984, 984, 1961423967, '19831009 02:51:44.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3875, 434, 434, 1429581910, '19891227 13:12:40.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3876, 105, 105, 739528163, '19630131 03:05:20.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3877, 871, 871, 1371504542, '19881108 01:10:40.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3878, 505, 505, 919006552, '19720412 09:51:46.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3879, 459, 459, 960543715, '19740807 21:18:39.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3880, 373, 373, 1641384115, '19781129 07:49:56.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3881, 604, 604, 598677764, '19960907 07:11:02.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3882, 227, 227, 885164489, '19770516 02:08:42.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3883, 572, 572, 1012522208, '20000907 09:35:14.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3884, 840, 840, 1037434, '20060810 21:18:33.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3885, 463, 463, 1640821084, '19681101 13:27:09.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3886, 648, 648, 121418887, '19540205 23:17:22.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3887, 504, 504, 148402636, '19830128 09:14:31.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3888, 140, 140, 663429661, '20060228 14:55:08.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3889, 813, 813, 44166870, '20060421 19:30:21.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3890, 303, 303, 112051554, '20001020 20:51:21.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3891, 368, 368, 1955236420, '19590911 06:04:41.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3892, 790, 790, 951839236, '19560505 06:04:54.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3893, 863, 863, 417789377, '19740207 15:04:04.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3894, 796, 796, 1745227741, '19930203 02:15:59.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3895, 921, 921, 340380827, '19700922 07:02:24.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3896, 699, 699, 969722796, '19950302 10:07:41.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3897, 155, 155, 813998753, '20021024 15:35:53.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3898, 538, 538, 1497504032, '20021222 06:12:38.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3899, 247, 247, 1223022192, '19700119 13:05:52.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3900, 397, 397, 2046087611, '19970125 01:15:39.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3901, 438, 438, 1947046886, '19591109 16:29:30.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3902, 251, 251, 109865159, '19820903 21:51:44.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3903, 164, 164, 1439166501, '19921206 08:28:57.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3904, 398, 398, 149537071, '19670916 09:05:16.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3905, 317, 317, 1869903582, '19610831 08:09:54.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3906, 408, 408, 844310109, '19981229 10:37:01.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3907, 276, 276, 436117965, '20030508 04:21:57.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3908, 234, 234, 1978703076, '19591128 19:05:37.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3909, 550, 550, 1081080815, '19931128 09:02:22.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3910, 716, 716, 1955487305, '19891223 22:10:19.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3911, 812, 812, 1451637759, '19650809 02:30:47.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3912, 535, 535, 1343047633, '19940221 17:54:14.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3913, 564, 564, 1359354085, '19770726 02:47:55.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3914, 202, 202, 2109150646, '19611217 00:16:33.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3915, 488, 488, 1706865947, '20040516 06:43:57.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3916, 428, 428, 1813458959, '19750313 18:55:34.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3917, 660, 660, 584033402, '19720831 09:18:33.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3918, 627, 627, 1665237188, '19700117 20:30:06.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3919, 246, 246, 2006417390, '19540806 19:33:51.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3920, 921, 921, 641783691, '19590620 04:57:44.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3921, 243, 243, 1095081922, '19870818 03:33:50.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3922, 279, 279, 1251539752, '19911116 08:23:54.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3923, 631, 631, 1971659423, '19761210 00:06:30.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3924, 107, 107, 674319985, '19620721 23:59:16.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3925, 110, 110, 1407432452, '19680530 09:25:50.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3926, 128, 128, 357257521, '20050403 15:59:40.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3927, 657, 657, 1798802827, '20020404 23:11:58.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3928, 67, 67, 856583375, '19720903 04:19:52.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3929, 597, 597, 384290472, '19660117 19:49:19.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3930, 681, 681, 319403462, '20021027 03:04:58.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3931, 791, 791, 1224878416, '19680407 11:29:26.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3932, 972, 972, 318505913, '19931116 18:58:02.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3933, 419, 419, 1747246103, '19550325 20:45:54.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3934, 567, 567, 764822696, '19850712 20:03:00.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3935, 435, 435, 1732571971, '19720121 15:20:26.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3936, 7, 7, 206680090, '19670105 09:19:18.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3937, 710, 710, 1040763607, '19920608 17:39:45.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3938, 485, 485, 905665335, '19851224 00:29:57.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3939, 763, 763, 79053227, '19971211 13:08:38.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3940, 420, 420, 1763037819, '19771103 02:50:07.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3941, 521, 521, 1663339049, '19580930 18:55:24.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3942, 620, 620, 1500480323, '19760302 14:08:53.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3943, 722, 722, 1117488213, '19650403 05:52:46.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3944, 856, 856, 1918932937, '19690429 21:20:02.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3945, 361, 361, 1223219260, '19880314 18:20:34.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3946, 459, 459, 1380769732, '19730719 12:10:26.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3947, 844, 844, 1548839075, '19861010 03:16:58.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3948, 190, 190, 681729561, '19650222 14:47:31.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3949, 694, 694, 398368202, '19960918 19:32:01.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3950, 816, 816, 1867860774, '20030711 12:06:50.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3951, 867, 867, 1003527648, '19750703 00:49:07.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3952, 176, 176, 575670765, '19851007 19:05:46.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3953, 914, 914, 95553175, '20010518 10:38:46.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3954, 406, 406, 348634195, '19630803 08:08:08.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3955, 445, 445, 458889112, '19620610 21:41:48.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3956, 244, 244, 220128234, '19751224 08:05:35.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3957, 583, 583, 1908592545, '19580925 21:45:14.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3958, 237, 237, 2008033859, '19830519 08:29:41.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3959, 53, 53, 262962875, '20070417 23:39:25.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3960, 919, 919, 7815322, '19640528 16:10:13.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3961, 292, 292, 2108955780, '20040924 11:25:03.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3962, 502, 502, 1195252359, '19830905 00:02:05.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3963, 55, 55, 398545915, '20011109 23:16:17.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3964, 530, 530, 1524031599, '19990201 05:57:23.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3965, 786, 786, 53415634, '19740918 10:40:52.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3966, 698, 698, 654345392, '19790922 07:32:44.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3967, 339, 339, 1228789101, '19911130 21:08:19.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3968, 289, 289, 1983748431, '20000113 06:44:55.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3969, 890, 890, 1527920206, '19971125 08:56:07.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3970, 713, 713, 2104652390, '19900512 21:24:39.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3971, 782, 782, 743476336, '19901004 01:15:48.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3972, 607, 607, 1239410157, '19940907 08:48:36.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3973, 822, 822, 1110805061, '19800206 07:21:28.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3974, 245, 245, 1794903599, '19841002 16:43:44.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3975, 794, 794, 952481340, '19530920 10:02:19.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3976, 967, 967, 1194269175, '19640617 13:24:57.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3977, 441, 441, 924059638, '19790816 02:31:08.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3978, 538, 538, 943959602, '19600813 02:46:28.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3979, 569, 569, 1066818494, '19820508 05:38:41.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3980, 785, 785, 1897496376, '20021110 21:15:23.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3981, 679, 679, 659192500, '19710626 12:42:42.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3982, 717, 717, 633861812, '19900211 08:24:27.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3983, 964, 964, 1924217437, '19831215 08:47:51.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3984, 670, 670, 414762155, '19700709 07:42:29.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3985, 482, 482, 2085336441, '19800427 02:15:51.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3986, 206, 206, 181549406, '19900630 16:23:40.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3987, 997, 997, 137290223, '20000630 08:19:16.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3988, 700, 700, 712134322, '19990822 12:37:46.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3989, 386, 386, 637270250, '19760202 22:21:12.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3990, 853, 853, 268909916, '20040128 07:14:50.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3991, 886, 886, 1811112931, '19610412 11:34:03.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3992, 337, 337, 70092961, '19620122 09:10:03.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3993, 472, 472, 1263726485, '19860907 00:50:32.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3994, 944, 944, 370602274, '19860716 20:58:24.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3995, 773, 773, 1542001244, '19810207 15:25:04.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3996, 450, 450, 619501791, '19830125 18:53:19.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3997, 647, 647, 822224688, '19820422 14:52:36.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3998, 666, 666, 1985583440, '19740301 23:55:23.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (3999, 896, 896, 1054895097, '20020811 10:39:09.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4000, 839, 839, 1250355251, '19610624 10:47:49.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4001, 273, 273, 2055246766, '19890818 15:02:07.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4002, 847, 847, 1836868703, '19940926 20:32:02.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4003, 701, 701, 772809130, '19750330 05:51:24.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4004, 709, 709, 2066527149, '19590510 22:07:21.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4005, 612, 612, 1888122108, '19651008 10:51:01.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4006, 218, 218, 1179086342, '19881024 16:00:31.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4007, 115, 115, 74914169, '19580902 11:44:18.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4008, 260, 260, 2003510129, '19690816 17:55:01.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4009, 428, 428, 329708755, '19621113 22:05:48.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4010, 965, 965, 1188097860, '20051223 08:16:50.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4011, 353, 353, 702347694, '20040509 05:34:27.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4012, 336, 336, 231450340, '20060211 18:58:46.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4013, 877, 877, 1092630466, '19540112 02:50:59.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4014, 691, 691, 516009874, '19570608 02:24:01.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4015, 673, 673, 98564231, '20001202 03:27:31.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4016, 553, 553, 1631382316, '19670815 03:00:36.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4017, 473, 473, 8450108, '20070920 15:14:14.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4018, 156, 156, 1274431826, '19840504 15:33:48.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4019, 978, 978, 1013816777, '19711008 03:41:31.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4020, 666, 666, 473731613, '19920319 18:17:12.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4021, 484, 484, 1792341475, '19870323 23:13:28.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4022, 633, 633, 1385904678, '20000409 08:30:33.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4023, 489, 489, 1336587820, '19761211 08:23:42.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4024, 972, 972, 1946658462, '20011028 01:41:16.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4025, 749, 749, 492255326, '19820131 09:50:35.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4026, 417, 417, 13965863, '19810912 16:05:45.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4027, 759, 759, 1894455246, '19610101 07:58:04.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4028, 542, 542, 335122745, '20010722 10:23:03.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4029, 861, 861, 1789544860, '19560826 01:18:39.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4030, 367, 367, 1776726813, '19780825 15:09:00.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4031, 720, 720, 172711857, '19900224 22:32:21.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4032, 696, 696, 1582452679, '19580617 02:35:45.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4033, 564, 564, 1004345490, '19660101 16:07:18.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4034, 869, 869, 152789881, '19731114 18:50:52.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4035, 888, 888, 1588353252, '19660326 06:13:16.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4036, 329, 329, 468427648, '19840928 16:10:39.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4037, 189, 189, 1061073102, '19671116 02:33:02.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4038, 984, 984, 1641716457, '19770807 10:40:29.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4039, 744, 744, 779726326, '19571001 20:51:27.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4040, 293, 293, 693183717, '19990703 10:15:20.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4041, 629, 629, 778726691, '19841029 04:39:21.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4042, 935, 935, 82547874, '19831115 14:24:14.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4043, 646, 646, 1792989807, '19891009 14:31:57.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4044, 229, 229, 1901914162, '19780209 14:04:21.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4045, 940, 940, 1043800378, '20070920 01:40:23.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4046, 170, 170, 462784624, '19630228 16:35:17.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4047, 808, 808, 1726273887, '19610111 06:19:03.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4048, 384, 384, 22323571, '19820401 22:26:30.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4049, 926, 926, 1931765718, '19930813 17:30:53.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4050, 30, 30, 228173737, '19660627 12:24:28.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4051, 306, 306, 1842011727, '19830507 03:39:05.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4052, 907, 907, 1169087922, '20051217 23:18:47.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4053, 887, 887, 358711778, '19550526 20:13:51.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4054, 439, 439, 1540982722, '19630525 16:21:57.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4055, 457, 457, 1543289981, '19820402 11:34:21.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4056, 838, 838, 1155052916, '19970510 06:31:33.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4057, 262, 262, 1886648319, '19701017 12:08:20.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4058, 603, 603, 88272386, '19810601 04:10:07.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4059, 757, 757, 1650136401, '19850408 12:16:46.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4060, 142, 142, 610149818, '19920125 18:45:16.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4061, 408, 408, 1923253714, '19801024 08:02:27.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4062, 388, 388, 873416137, '19650210 01:21:15.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4063, 970, 970, 1007949129, '19651222 16:36:22.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4064, 756, 756, 190274223, '19920320 06:56:47.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4065, 35, 35, 742026786, '19681029 09:44:29.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4066, 146, 146, 1367860956, '19981124 02:58:42.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4067, 493, 493, 918241021, '19930211 02:51:27.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4068, 570, 570, 1856364189, '19880227 08:00:07.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4069, 809, 809, 1999911970, '19990314 20:10:45.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4070, 29, 29, 1087286974, '19690306 11:16:51.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4071, 925, 925, 389510024, '20070929 00:27:33.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4072, 335, 335, 441779232, '19830214 04:33:44.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4073, 16, 16, 255947680, '19790804 18:42:20.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4074, 588, 588, 1274779420, '19800405 17:26:11.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4075, 557, 557, 592201971, '19600522 13:37:50.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4076, 988, 988, 1795842018, '19560509 08:49:13.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4077, 615, 615, 260527313, '19630702 17:58:35.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4078, 444, 444, 116866474, '20061031 18:19:20.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4079, 377, 377, 1516065645, '20020207 00:00:53.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4080, 988, 988, 161056091, '19711204 17:15:17.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4081, 664, 664, 1861534169, '19730902 09:46:41.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4082, 41, 41, 1253348742, '19861001 09:31:34.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4083, 77, 77, 707643081, '19601209 16:52:10.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4084, 330, 330, 1706716993, '19980228 12:54:11.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4085, 213, 213, 556787820, '20030420 11:29:55.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4086, 421, 421, 1046656425, '19920308 23:13:33.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4087, 732, 732, 233409491, '19560123 06:21:54.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4088, 452, 452, 692566339, '19550810 23:45:20.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4089, 192, 192, 1334962250, '19990814 07:16:31.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4090, 41, 41, 128546126, '19761114 23:41:43.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4091, 370, 370, 609667513, '19661214 04:02:19.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4092, 434, 434, 1285345052, '19940615 22:22:55.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4093, 495, 495, 1768605138, '20000429 22:23:42.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4094, 225, 225, 722263703, '19730906 02:06:11.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4095, 393, 393, 235220834, '19710910 02:12:53.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4096, 422, 422, 1357525450, '19720919 03:18:06.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4097, 824, 824, 1247513437, '19701124 21:47:52.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4098, 336, 336, 666062482, '20050721 07:35:10.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4099, 664, 664, 936498360, '19620414 04:19:52.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4100, 714, 714, 1151108476, '19611027 22:41:41.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4101, 671, 671, 810493941, '19780116 13:43:50.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4102, 780, 780, 2057227163, '19801113 22:18:56.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4103, 937, 937, 1002664513, '19910118 02:15:44.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4104, 154, 154, 1311297727, '19770610 06:14:01.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4105, 12, 12, 1546468966, '19660928 11:56:54.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4106, 837, 837, 1761350106, '19530323 23:05:20.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4107, 831, 831, 465355789, '19790517 04:36:27.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4108, 105, 105, 414156170, '19830219 02:47:39.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4109, 375, 375, 437505410, '19560104 02:44:07.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4110, 986, 986, 568536841, '19781125 02:45:07.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4111, 115, 115, 361117833, '19861109 12:32:57.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4112, 499, 499, 747269007, '19711216 17:48:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4113, 58, 58, 24421638, '19870424 04:09:14.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4114, 587, 587, 1463649265, '19660506 19:01:28.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4115, 943, 943, 2081666448, '19710526 08:58:34.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4116, 453, 453, 1469159869, '20020122 22:30:52.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4117, 64, 64, 320506360, '19570304 08:29:05.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4118, 889, 889, 2127547232, '19751024 03:42:11.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4119, 726, 726, 1620768389, '19961129 19:26:51.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4120, 350, 350, 1236216843, '19840821 10:30:56.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4121, 973, 973, 1126158641, '19951102 20:36:47.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4122, 94, 94, 2050096505, '19900705 03:06:07.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4123, 518, 518, 853814625, '19960715 00:43:35.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4124, 31, 31, 1985427853, '19750430 20:29:01.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4125, 418, 418, 695958640, '19550325 07:14:32.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4126, 139, 139, 1347639391, '19660416 02:04:38.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4127, 169, 169, 1074172960, '19901016 06:10:02.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4128, 307, 307, 958931956, '19581129 16:36:08.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4129, 46, 46, 1930796446, '19610728 15:13:18.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4130, 297, 297, 1320341326, '19950902 10:19:44.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4131, 991, 991, 1915997345, '19930614 11:01:20.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4132, 456, 456, 536572256, '19651211 10:23:25.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4133, 130, 130, 1044946028, '19970720 13:59:28.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4134, 779, 779, 2079711714, '19930413 01:19:12.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4135, 825, 825, 1763052504, '20011117 03:31:27.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4136, 518, 518, 216004300, '20001018 11:27:44.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4137, 918, 918, 1931063217, '20030913 07:15:49.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4138, 570, 570, 1097451025, '19910701 10:38:08.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4139, 547, 547, 718719512, '19840603 00:57:16.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4140, 861, 861, 1536974972, '20030128 12:24:35.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4141, 894, 894, 257189047, '19651024 18:37:06.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4142, 881, 881, 1572663961, '19801203 03:34:14.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4143, 312, 312, 1344627582, '20070806 21:01:12.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4144, 427, 427, 1902636820, '19730920 04:31:23.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4145, 405, 405, 1076693796, '19980105 11:08:45.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4146, 614, 614, 1644717719, '20021228 10:13:20.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4147, 298, 298, 1890309510, '19600223 18:13:40.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4148, 595, 595, 305813345, '19861226 03:22:14.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4149, 760, 760, 657043813, '19550414 17:07:36.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4150, 727, 727, 1459253177, '19770817 03:42:00.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4151, 624, 624, 333907968, '19680718 18:49:01.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4152, 622, 622, 111171507, '20030131 18:05:41.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4153, 128, 128, 1366403201, '19610822 12:08:18.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4154, 923, 923, 1481351127, '19850822 17:48:56.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4155, 878, 878, 22468569, '19731225 02:04:53.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4156, 866, 866, 349409597, '19950714 10:37:42.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4157, 473, 473, 1984575493, '19920430 21:35:21.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4158, 904, 904, 326230624, '19680920 05:46:43.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4159, 723, 723, 2083153918, '19750319 22:59:29.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4160, 961, 961, 1804541704, '19530615 09:52:16.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4161, 312, 312, 1176870800, '19700607 16:55:18.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4162, 393, 393, 586069602, '19730618 12:00:18.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4163, 823, 823, 265678936, '19740725 11:34:20.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4164, 4, 4, 169756637, '19680502 16:24:23.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4165, 396, 396, 1348317975, '19930611 15:43:46.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4166, 163, 163, 1489542050, '19731130 02:09:32.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4167, 141, 141, 1674137962, '19820528 03:49:30.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4168, 777, 777, 362575590, '20020110 02:50:02.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4169, 902, 902, 77060223, '19720618 15:30:00.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4170, 574, 574, 1712347993, '19780806 21:30:50.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4171, 221, 221, 1944103515, '20060511 15:15:03.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4172, 268, 268, 1848205224, '19730902 21:50:56.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4173, 358, 358, 749881353, '19990522 02:44:54.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4174, 881, 881, 664132097, '20011101 07:02:16.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4175, 341, 341, 1616507, '19711029 15:53:49.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4176, 939, 939, 317845859, '19671201 17:02:33.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4177, 27, 27, 756130838, '19901128 06:04:54.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4178, 653, 653, 815225752, '19751026 20:12:11.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4179, 127, 127, 66610384, '19850424 09:20:16.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4180, 840, 840, 1411906984, '19600326 21:01:12.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4181, 959, 959, 1944326805, '19590222 07:50:57.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4182, 768, 768, 1904575826, '19561022 02:47:47.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4183, 409, 409, 59746772, '19560817 23:28:32.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4184, 549, 549, 1418234640, '19911211 11:31:18.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4185, 940, 940, 841824993, '19800215 15:30:42.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4186, 821, 821, 1039936170, '19980513 16:55:39.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4187, 519, 519, 1032704150, '19570421 22:15:07.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4188, 296, 296, 1045795067, '19641128 20:10:32.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4189, 119, 119, 1626995307, '19720419 23:14:18.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4190, 996, 996, 1464741162, '19590507 16:53:44.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4191, 849, 849, 872315598, '19610620 13:52:23.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4192, 297, 297, 1186785033, '19871225 01:29:06.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4193, 697, 697, 350011171, '19690414 11:38:39.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4194, 43, 43, 1010999540, '19831220 15:04:59.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4195, 280, 280, 1990555675, '19850823 19:29:17.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4196, 81, 81, 1535843148, '20000507 06:36:48.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4197, 8, 8, 1162012575, '19590512 15:59:54.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4198, 39, 39, 226785742, '19920406 04:36:49.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4199, 569, 569, 2113876311, '19880409 12:47:36.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4200, 60, 60, 1854128939, '19770206 08:59:13.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4201, 937, 937, 718405876, '19730803 06:23:49.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4202, 540, 540, 358432206, '19660212 15:23:37.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4203, 992, 992, 1851119218, '19670710 11:52:13.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4204, 851, 851, 1843500142, '19840906 19:36:46.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4205, 956, 956, 540743850, '19790407 12:26:56.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4206, 614, 614, 2080088959, '20021114 20:58:04.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4207, 21, 21, 2004194817, '19560912 15:20:46.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4208, 394, 394, 519800797, '19671021 05:06:02.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4209, 913, 913, 1256881215, '19661026 01:55:06.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4210, 672, 672, 78342188, '19590125 09:02:20.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4211, 508, 508, 792505149, '19570817 04:32:48.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4212, 687, 687, 2039915365, '19690707 01:23:09.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4213, 660, 660, 2037402582, '19910528 20:26:27.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4214, 670, 670, 579036394, '19671225 01:58:17.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4215, 846, 846, 1387528208, '20020423 02:01:18.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4216, 930, 930, 1486993295, '19660817 14:07:30.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4217, 450, 450, 1013355237, '19691101 12:31:46.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4218, 989, 989, 1900381823, '19900814 00:03:02.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4219, 899, 899, 758550878, '19960317 00:53:40.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4220, 413, 413, 240242466, '20030128 22:48:07.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4221, 575, 575, 91012258, '19690811 03:54:25.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4222, 142, 142, 493546563, '19700630 07:38:57.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4223, 594, 594, 781039698, '19820923 03:35:43.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4224, 68, 68, 181481901, '19660212 22:36:16.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4225, 684, 684, 670702106, '19700217 07:38:27.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4226, 56, 56, 1464781522, '19710519 13:45:57.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4227, 503, 503, 866324697, '19570522 10:12:17.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4228, 372, 372, 301355160, '19680602 11:39:54.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4229, 523, 523, 1855416086, '19690311 11:32:58.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4230, 390, 390, 243555700, '19790623 09:17:01.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4231, 923, 923, 321785646, '19610723 01:02:38.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4232, 152, 152, 2064307606, '20060823 01:28:04.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4233, 32, 32, 534379534, '19950718 07:24:34.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4234, 26, 26, 1179712974, '19610319 00:21:02.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4235, 891, 891, 1550774687, '19940823 14:37:22.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4236, 171, 171, 1672377122, '20010108 16:27:20.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4237, 952, 952, 1607683365, '19970413 14:55:34.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4238, 865, 865, 1500003075, '19791211 03:51:45.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4239, 576, 576, 275153851, '19650905 23:04:22.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4240, 526, 526, 286092732, '19850401 18:32:37.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4241, 170, 170, 1613800111, '19940901 01:34:53.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4242, 914, 914, 1902242684, '19970701 17:09:04.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4243, 852, 852, 1210211615, '20060202 18:15:26.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4244, 628, 628, 856980404, '19660327 14:11:58.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4245, 449, 449, 361268837, '19540921 12:20:56.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4246, 733, 733, 180612193, '19991214 12:29:13.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4247, 910, 910, 1783977501, '20040728 05:02:39.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4248, 818, 818, 1946338339, '19540422 09:40:22.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4249, 58, 58, 84692018, '19890828 13:03:41.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4250, 548, 548, 575162196, '19720108 05:21:47.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4251, 364, 364, 883969387, '19830707 18:05:02.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4252, 621, 621, 618693149, '19760927 15:56:52.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4253, 166, 166, 665505460, '20040120 03:43:09.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4254, 40, 40, 1286235694, '19930311 13:59:29.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4255, 492, 492, 1054864951, '19600629 05:04:48.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4256, 381, 381, 1399190214, '19560204 22:44:51.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4257, 456, 456, 2068534781, '19910522 11:47:54.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4258, 739, 739, 2026226369, '19540528 13:15:57.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4259, 147, 147, 1930942351, '19670722 11:58:19.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4260, 674, 674, 2016502339, '19601119 22:40:58.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4261, 11, 11, 1439148995, '19980626 10:45:47.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4262, 443, 443, 1954345830, '19960411 03:40:51.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4263, 14, 14, 1207232974, '20060810 17:33:03.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4264, 102, 102, 993776261, '19950504 16:38:05.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4265, 212, 212, 1719475897, '20050704 07:59:41.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4266, 166, 166, 539945069, '19581225 03:04:43.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4267, 174, 174, 1577135647, '19811219 17:58:34.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4268, 65, 65, 1551886559, '19830312 20:05:24.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4269, 250, 250, 1255058865, '19810502 11:20:54.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4270, 924, 924, 27579815, '19540415 09:33:58.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4271, 645, 645, 545677811, '19770504 23:11:55.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4272, 464, 464, 403628408, '19971121 08:40:00.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4273, 563, 563, 1524638893, '19771209 00:58:39.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4274, 735, 735, 1832923234, '19631217 06:21:02.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4275, 599, 599, 537786197, '19610530 21:13:14.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4276, 581, 581, 486430526, '19800209 10:45:20.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4277, 351, 351, 1779429908, '19720526 13:23:31.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4278, 987, 987, 654873635, '19690629 13:23:44.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4279, 753, 753, 507679252, '19640525 10:15:19.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4280, 216, 216, 757072214, '19780305 19:09:13.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4281, 172, 172, 479127103, '19741021 08:43:18.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4282, 862, 862, 882905712, '19560201 00:31:54.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4283, 819, 819, 1936968822, '19861004 22:36:13.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4284, 761, 761, 723693011, '20050303 06:11:11.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4285, 767, 767, 1831607864, '20031216 15:11:59.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4286, 638, 638, 2081757067, '19921024 09:05:46.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4287, 979, 979, 1711230874, '19550804 21:44:55.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4288, 353, 353, 1418224041, '19550509 17:25:04.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4289, 53, 53, 179390727, '19530920 19:16:13.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4290, 724, 724, 750209666, '19910720 15:52:30.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4291, 408, 408, 1306704991, '19620821 04:39:26.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4292, 143, 143, 416712431, '19951118 01:39:37.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4293, 713, 713, 1615663601, '19650522 15:53:25.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4294, 788, 788, 1665676759, '19571018 00:23:23.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4295, 514, 514, 1731989191, '19941006 07:46:50.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4296, 775, 775, 398929035, '20060521 21:54:01.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4297, 959, 959, 1012071332, '19981120 23:36:01.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4298, 568, 568, 2088500261, '19631003 01:37:21.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4299, 553, 553, 1336838770, '19680922 06:12:17.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4300, 401, 401, 474035939, '20030927 09:16:13.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4301, 417, 417, 1837121830, '19701226 18:30:38.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4302, 798, 798, 209899314, '19740519 08:57:14.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4303, 132, 132, 1896474458, '19801219 22:19:27.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4304, 798, 798, 1889135167, '19880516 03:29:43.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4305, 776, 776, 718448821, '20020904 06:09:51.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4306, 899, 899, 2025924123, '19930815 09:25:01.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4307, 4, 4, 1504570938, '20061019 14:58:12.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4308, 37, 37, 1330612265, '19930203 21:22:07.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4309, 296, 296, 1583704933, '19841012 16:46:15.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4310, 294, 294, 1568947492, '19880519 18:19:28.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4311, 267, 267, 490367740, '19910910 09:21:20.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4312, 754, 754, 740569964, '19741123 22:24:09.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4313, 806, 806, 1184280046, '19980102 03:00:38.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4314, 743, 743, 656551887, '19970517 16:49:06.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4315, 627, 627, 607629254, '19940129 13:57:40.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4316, 671, 671, 1795860813, '19950527 10:13:53.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4317, 93, 93, 2039190712, '19620709 05:04:38.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4318, 205, 205, 1477907797, '19540610 11:21:52.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4319, 586, 586, 258101848, '19990519 01:26:05.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4320, 3, 3, 1213479032, '19650910 22:53:54.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4321, 497, 497, 660737159, '19560523 05:19:48.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4322, 946, 946, 551674399, '19790814 00:33:30.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4323, 82, 82, 173087312, '19820623 00:49:10.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4324, 455, 455, 1003873343, '19971012 19:28:23.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4325, 192, 192, 774283866, '19990825 04:54:32.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4326, 97, 97, 75149774, '19890616 21:32:18.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4327, 751, 751, 1285263949, '19850702 16:46:35.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4328, 57, 57, 234044207, '19730222 00:35:16.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4329, 858, 858, 1582073972, '19770311 22:34:12.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4330, 909, 909, 1787550040, '19630108 23:19:12.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4331, 967, 967, 1838744582, '19890320 11:09:18.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4332, 639, 639, 167553667, '19610825 11:46:10.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4333, 95, 95, 214724914, '19531008 07:11:27.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4334, 543, 543, 890799265, '19680828 00:59:05.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4335, 799, 799, 1966183127, '19600311 00:38:35.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4336, 611, 611, 1320717731, '19530604 23:46:03.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4337, 404, 404, 1236112706, '19830211 02:12:26.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4338, 86, 86, 1369307974, '20060520 19:06:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4339, 209, 209, 1049379038, '19550701 00:01:19.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4340, 273, 273, 1225600190, '19630504 05:46:58.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4341, 803, 803, 636292653, '19940104 18:07:34.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4342, 613, 613, 1929186341, '19700630 00:22:47.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4343, 150, 150, 15181341, '19780727 00:38:49.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4344, 261, 261, 2123584824, '19730503 00:56:45.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4345, 976, 976, 1281988313, '20071109 06:31:10.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4346, 573, 573, 43131164, '19950927 06:15:16.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4347, 44, 44, 881125263, '20051114 22:38:58.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4348, 181, 181, 1815951795, '19760104 23:04:19.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4349, 34, 34, 612812121, '19710918 10:25:42.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4350, 673, 673, 1439320030, '20070511 21:32:57.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4351, 114, 114, 1490962431, '19961113 16:49:22.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4352, 207, 207, 1093847381, '19970613 12:14:08.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4353, 869, 869, 1852650294, '19720516 00:11:15.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4354, 140, 140, 201382026, '19560113 07:18:23.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4355, 167, 167, 1743052565, '20000507 03:56:25.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4356, 929, 929, 1266328835, '19990514 17:57:08.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4357, 374, 374, 1806954913, '19991126 08:08:04.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4358, 625, 625, 2025449462, '19910309 02:51:04.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4359, 578, 578, 2026271823, '19960920 22:35:10.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4360, 302, 302, 612936362, '19660321 08:37:32.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4361, 33, 33, 1862199644, '19610213 16:38:25.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4362, 248, 248, 825361481, '19860828 14:22:55.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4363, 404, 404, 165199376, '19681125 22:47:55.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4364, 907, 907, 589078226, '19741005 17:27:03.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4365, 118, 118, 680475209, '20070301 07:10:09.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4366, 313, 313, 1976551994, '19830116 21:35:10.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4367, 111, 111, 183259639, '19740216 15:12:42.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4368, 685, 685, 1240638098, '19820508 02:01:32.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4369, 594, 594, 1632021164, '19900309 16:10:30.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4370, 843, 843, 264938455, '19930827 14:11:36.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4371, 940, 940, 2036253985, '19650416 08:01:26.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4372, 65, 65, 1478744901, '19640218 09:58:08.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4373, 626, 626, 5370835, '20061210 11:20:32.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4374, 186, 186, 1344839077, '19890115 19:39:07.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4375, 315, 315, 53643590, '19790906 04:46:20.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4376, 669, 669, 200016204, '19931123 04:57:00.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4377, 79, 79, 1763121491, '19540118 23:54:40.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4378, 755, 755, 1495121341, '19620220 23:52:24.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4379, 868, 868, 635138694, '19971203 12:57:13.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4380, 672, 672, 1746729700, '19561128 22:39:15.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4381, 279, 279, 1984131943, '19910801 22:53:19.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4382, 235, 235, 1904004398, '19620629 17:42:16.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4383, 311, 311, 353110602, '19540607 14:09:33.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4384, 246, 246, 564468602, '19771031 01:01:15.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4385, 54, 54, 37204939, '19740224 06:29:49.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4386, 384, 384, 1775822167, '19991006 22:55:10.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4387, 559, 559, 1730898421, '19970410 11:34:54.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4388, 183, 183, 864625386, '20050924 23:53:04.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4389, 566, 566, 1924851132, '19760421 21:02:40.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4390, 834, 834, 1784337786, '19681027 06:41:26.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4391, 37, 37, 1028431689, '19610709 15:37:59.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4392, 20, 20, 682049574, '19991204 23:21:22.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4393, 706, 706, 1890552208, '19620830 20:31:19.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4394, 827, 827, 1406023459, '19970410 15:23:47.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4395, 780, 780, 1741866983, '19550321 13:08:32.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4396, 728, 728, 1459346216, '19600510 03:44:38.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4397, 682, 682, 325331858, '19540805 01:34:52.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4398, 388, 388, 1974342117, '19561022 07:11:46.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4399, 98, 98, 1003948761, '19740303 17:46:36.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4400, 331, 331, 1062084298, '19771024 08:56:00.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4401, 445, 445, 1269065679, '19740811 15:02:34.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4402, 64, 64, 973611658, '19760710 20:37:26.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4403, 33, 33, 2000995415, '19931027 06:53:48.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4404, 962, 962, 1559038610, '19860120 20:14:06.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4405, 193, 193, 1132316817, '19950126 13:31:30.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4406, 429, 429, 616259263, '19850927 06:51:14.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4407, 561, 561, 321973708, '19980704 00:53:36.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4408, 142, 142, 1490932772, '19910430 04:32:08.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4409, 233, 233, 1216360713, '19840508 02:32:02.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4410, 994, 994, 1383189824, '19590615 22:59:24.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4411, 848, 848, 87614103, '19980426 18:02:27.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4412, 413, 413, 1024416747, '19901006 08:15:39.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4413, 300, 300, 258104275, '20010404 13:53:51.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4414, 337, 337, 2003947664, '19921023 23:55:55.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4415, 257, 257, 2044182050, '19820819 09:44:12.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4416, 54, 54, 1966092768, '20060816 22:56:09.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4417, 188, 188, 495565927, '19850324 00:13:22.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4418, 483, 483, 1065120310, '19990125 21:46:40.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4419, 149, 149, 752133618, '19530812 10:57:13.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4420, 203, 203, 15837264, '19600526 08:14:59.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4421, 952, 952, 724939999, '19931007 10:00:15.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4422, 808, 808, 509526064, '19760523 15:19:37.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4423, 752, 752, 525436329, '19590117 04:58:52.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4424, 161, 161, 1565372619, '19740514 09:29:04.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4425, 232, 232, 2076594984, '19850218 22:33:37.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4426, 199, 199, 1359323956, '19730512 08:40:03.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4427, 356, 356, 1030703308, '19540621 13:26:48.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4428, 761, 761, 1985059384, '19620901 19:56:45.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4429, 908, 908, 1651805589, '19861029 06:30:34.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4430, 464, 464, 996781255, '19720429 01:01:41.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4431, 734, 734, 1486129444, '19920421 03:22:07.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4432, 269, 269, 4617430, '20050328 16:42:54.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4433, 146, 146, 1788110036, '19951220 06:05:48.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4434, 632, 632, 1418077460, '19730210 04:01:12.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4435, 73, 73, 613320829, '19900419 07:36:40.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4436, 720, 720, 1240785366, '19680123 02:15:53.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4437, 323, 323, 65316486, '19760901 10:48:27.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4438, 857, 857, 1685214433, '19760516 17:55:27.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4439, 882, 882, 677733289, '19901004 11:29:44.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4440, 273, 273, 517556663, '19960528 23:38:35.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4441, 898, 898, 1032949361, '19540405 22:01:34.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4442, 107, 107, 1455484800, '19581213 07:02:46.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4443, 559, 559, 504666717, '19740520 21:21:01.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4444, 309, 309, 1529326101, '19691107 22:03:15.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4445, 450, 450, 1123058593, '19780702 12:38:59.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4446, 955, 955, 86400648, '19781003 07:22:20.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4447, 249, 249, 1569617212, '20060831 09:27:31.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4448, 582, 582, 938170697, '19771105 20:35:24.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4449, 271, 271, 1518559918, '19670824 05:39:34.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4450, 641, 641, 579220386, '19560803 14:12:23.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4451, 390, 390, 881242776, '19830216 03:31:16.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4452, 134, 134, 181053535, '19630710 03:48:12.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4453, 731, 731, 1318646825, '19560312 20:14:32.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4454, 392, 392, 857478522, '19661009 09:31:36.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4455, 875, 875, 1661135953, '19920116 22:55:44.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4456, 406, 406, 2060316501, '20060319 23:42:56.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4457, 365, 365, 1660679823, '19700625 15:38:33.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4458, 259, 259, 719703951, '19720615 21:24:44.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4459, 717, 717, 1940183145, '19531202 21:40:29.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4460, 267, 267, 291415526, '19740917 04:51:26.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4461, 704, 704, 1650759137, '19840408 17:24:26.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4462, 391, 391, 1889300662, '19881102 04:56:50.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4463, 823, 823, 1085670305, '19570702 22:01:33.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4464, 906, 906, 926092454, '19650110 01:30:21.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4465, 347, 347, 374081872, '19750223 19:37:17.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4466, 441, 441, 972632862, '20010128 01:19:32.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4467, 362, 362, 580344186, '20021017 02:09:50.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4468, 254, 254, 1160045640, '19810223 09:52:38.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4469, 618, 618, 241849904, '19550708 16:19:14.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4470, 247, 247, 760155143, '19670729 07:28:19.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4471, 56, 56, 548600994, '19821216 12:07:41.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4472, 832, 832, 134343148, '19611108 06:17:54.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4473, 318, 318, 213052727, '19610424 10:16:55.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4474, 380, 380, 334711129, '19650315 11:18:37.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4475, 268, 268, 764705471, '19590221 10:13:25.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4476, 31, 31, 1455401677, '19871027 02:57:28.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4477, 52, 52, 1651713530, '19550104 17:58:36.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4478, 236, 236, 1086170237, '19970310 06:55:37.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4479, 173, 173, 115061828, '20031111 20:50:04.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4480, 89, 89, 2139138670, '19590520 15:11:16.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4481, 433, 433, 505591994, '19740910 23:12:32.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4482, 862, 862, 1729372959, '19840814 16:51:24.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4483, 74, 74, 1338023367, '20030108 14:17:11.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4484, 437, 437, 903252946, '19830328 16:18:10.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4485, 619, 619, 288146112, '19970312 21:30:25.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4486, 328, 328, 1070342681, '19811013 23:33:55.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4487, 326, 326, 876695792, '20020116 20:28:21.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4488, 214, 214, 1595306901, '19820313 20:34:11.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4489, 299, 299, 1099369628, '19890124 05:05:28.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4490, 99, 99, 100048227, '19920130 07:53:44.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4491, 235, 235, 824214021, '20050729 10:37:19.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4492, 408, 408, 1110056583, '19570319 13:23:43.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4493, 300, 300, 1018288419, '19750615 20:14:58.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4494, 568, 568, 468519352, '19690118 06:38:17.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4495, 845, 845, 2101794666, '19650220 06:14:09.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4496, 710, 710, 544642719, '19730602 17:04:43.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4497, 60, 60, 687237373, '19540613 09:01:12.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4498, 477, 477, 714873921, '19620511 19:50:39.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4499, 240, 240, 481596119, '20020914 02:25:57.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4500, 183, 183, 1567128369, '19850603 11:19:26.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4501, 746, 746, 920776041, '19831217 05:12:29.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4502, 997, 997, 87878026, '19780708 23:34:53.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4503, 920, 920, 703119076, '19750502 04:16:10.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4504, 437, 437, 262860648, '19530126 22:11:14.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4505, 396, 396, 2147475924, '19810818 02:04:42.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4506, 474, 474, 1349328410, '19740410 21:13:21.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4507, 870, 870, 841646175, '19550318 17:31:16.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4508, 195, 195, 993181297, '19981228 08:55:55.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4509, 119, 119, 1982081077, '19600818 23:18:11.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4510, 705, 705, 1168569290, '19570323 19:58:15.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4511, 874, 874, 801841971, '20040316 05:44:20.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4512, 480, 480, 407854275, '19810416 08:42:56.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4513, 57, 57, 1558385140, '19760804 00:34:39.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4514, 135, 135, 971964424, '20020715 06:29:12.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4515, 771, 771, 503559336, '19530107 05:38:54.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4516, 651, 651, 1958320242, '20070825 00:33:01.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4517, 752, 752, 1711987851, '19931024 14:39:38.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4518, 677, 677, 1682991686, '19820406 05:43:23.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4519, 975, 975, 462190287, '19751030 03:59:56.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4520, 129, 129, 44984685, '20010511 20:03:21.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4521, 45, 45, 1503526050, '20070111 04:51:11.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4522, 854, 854, 28151994, '19730806 05:35:38.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4523, 305, 305, 490497161, '20000130 04:47:10.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4524, 200, 200, 1541490675, '19710608 08:25:30.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4525, 374, 374, 97062174, '19691229 20:50:59.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4526, 50, 50, 254110614, '19780929 22:43:57.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4527, 164, 164, 379451487, '19940525 10:02:56.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4528, 481, 481, 288333080, '20000405 03:38:38.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4529, 380, 380, 547055420, '19530124 05:04:28.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4530, 916, 916, 115607559, '19930919 17:08:41.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4531, 525, 525, 208399394, '19860516 17:56:16.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4532, 23, 23, 1226222477, '20000824 22:07:56.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4533, 427, 427, 1946093167, '20020626 04:29:39.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4534, 845, 845, 811198605, '19710611 09:30:38.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4535, 130, 130, 306068661, '19830603 09:58:46.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4536, 477, 477, 1580408431, '20040303 23:37:39.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4537, 527, 527, 1448719583, '19620416 12:35:13.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4538, 592, 592, 1815057386, '20021213 16:05:56.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4539, 365, 365, 846941234, '19540811 14:13:28.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4540, 853, 853, 1491003448, '19751204 00:17:03.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4541, 1000, 1000, 90452384, '19790730 06:02:39.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4542, 108, 108, 763131969, '19560121 11:32:26.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4543, 488, 488, 1195188876, '19740726 21:16:00.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4544, 458, 458, 1763445137, '19841103 09:07:12.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4545, 366, 366, 869023743, '19951115 02:09:23.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4546, 72, 72, 1751492920, '19770415 01:54:23.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4547, 122, 122, 1930370211, '19880814 12:49:04.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4548, 971, 971, 703267028, '19801130 13:45:45.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4549, 302, 302, 1877562323, '19690112 00:59:22.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4550, 324, 324, 527486439, '19650628 05:41:07.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4551, 464, 464, 634233359, '19870809 02:25:04.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4552, 342, 342, 1463133734, '19800308 03:17:49.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4553, 816, 816, 1955208622, '19940712 15:50:43.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4554, 284, 284, 112825128, '19540506 06:22:36.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4555, 475, 475, 1802196665, '19860523 06:28:15.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4556, 320, 320, 1875000245, '19630513 23:36:51.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4557, 712, 712, 431904113, '19860608 18:47:42.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4558, 312, 312, 1157520488, '19561124 19:50:40.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4559, 401, 401, 1360465464, '19910128 01:20:15.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4560, 152, 152, 1062533038, '19551120 03:20:44.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4561, 454, 454, 1423115095, '19871116 11:10:24.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4562, 318, 318, 962749256, '19621212 13:52:37.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4563, 979, 979, 1419597401, '19981205 03:51:26.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4564, 276, 276, 689309960, '19741129 06:09:30.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4565, 255, 255, 437722727, '19781107 02:01:59.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4566, 238, 238, 1733003670, '19560723 07:36:23.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4567, 688, 688, 1922250907, '19861021 04:13:16.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4568, 770, 770, 656916829, '19580224 15:04:01.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4569, 884, 884, 1776545888, '19720212 20:30:25.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4570, 676, 676, 625675892, '19561104 06:01:14.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4571, 180, 180, 1808992741, '19980511 11:57:48.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4572, 181, 181, 1661246254, '19981110 22:33:42.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4573, 970, 970, 146325746, '19800826 15:29:54.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4574, 414, 414, 15643684, '20071126 03:42:53.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4575, 904, 904, 122089630, '19741014 14:00:42.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4576, 466, 466, 1693210738, '20031222 17:18:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4577, 449, 449, 172016618, '20070110 08:19:38.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4578, 652, 652, 1342278453, '19680328 19:39:58.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4579, 847, 847, 1032696188, '19830724 06:16:06.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4580, 511, 511, 392782602, '20000914 18:56:36.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4581, 61, 61, 1341622743, '19980214 09:54:53.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4582, 778, 778, 1199260997, '19660625 20:17:10.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4583, 524, 524, 263193625, '19840325 02:39:15.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4584, 915, 915, 642845324, '19950728 23:23:21.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4585, 138, 138, 1832459460, '19590212 14:43:37.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4586, 508, 508, 1217826036, '19590311 14:38:27.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4587, 452, 452, 551376181, '19590214 06:17:13.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4588, 136, 136, 1662158285, '20010220 22:07:03.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4589, 600, 600, 1580304988, '19930118 03:02:22.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4590, 184, 184, 814675590, '19730121 10:21:55.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4591, 485, 485, 2043599914, '19700927 04:49:56.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4592, 515, 515, 1609120180, '19580523 16:44:33.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4593, 576, 576, 1363137740, '19641116 14:45:41.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4594, 333, 333, 1633366865, '20060921 10:52:43.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4595, 488, 488, 1678298741, '19960117 13:06:38.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4596, 490, 490, 1863947424, '19690818 16:10:01.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4597, 474, 474, 323422341, '19650215 07:41:00.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4598, 223, 223, 798746649, '20070827 15:06:30.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4599, 110, 110, 197354129, '19581229 07:05:12.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4600, 456, 456, 1001898337, '19920424 18:32:59.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4601, 84, 84, 1032550984, '19980624 21:41:06.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4602, 529, 529, 3890054, '19830621 21:45:02.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4603, 551, 551, 1526985952, '19611019 17:15:20.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4604, 826, 826, 688976714, '19650310 18:58:08.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4605, 536, 536, 1855397015, '19750216 17:43:19.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4606, 893, 893, 2060103177, '19960927 03:51:22.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4607, 758, 758, 1091327013, '20070712 11:47:54.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4608, 398, 398, 1104877949, '19940816 12:07:50.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4609, 47, 47, 1061492210, '19870722 16:21:53.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4610, 288, 288, 1677677741, '19900531 13:09:31.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4611, 25, 25, 2001508605, '19640502 15:17:13.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4612, 419, 419, 835812753, '19710313 23:07:44.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4613, 96, 96, 463936347, '19810503 13:34:33.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4614, 693, 693, 821132721, '19980514 06:23:38.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4615, 222, 222, 806662483, '19651004 17:25:50.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4616, 314, 314, 1692304181, '19740524 14:53:13.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4617, 917, 917, 843242602, '19860918 11:13:21.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4618, 427, 427, 167422599, '19560510 04:28:05.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4619, 977, 977, 1780977984, '19681017 15:25:52.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4620, 552, 552, 1732930274, '19720829 11:23:32.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4621, 996, 996, 632634012, '20050608 01:19:10.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4622, 443, 443, 229475936, '19930829 06:06:13.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4623, 610, 610, 1158764594, '19730205 12:01:39.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4624, 969, 969, 665174371, '20070121 10:08:30.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4625, 533, 533, 1563419236, '19940207 01:11:18.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4626, 290, 290, 898368489, '19921219 19:13:14.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4627, 115, 115, 909763320, '19861226 07:48:00.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4628, 876, 876, 1733482944, '19811205 04:37:10.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4629, 409, 409, 2125166729, '19641109 14:36:14.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4630, 535, 535, 1089319288, '19580226 21:50:41.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4631, 332, 332, 1511414875, '19911107 09:37:44.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4632, 599, 599, 1166313173, '20070515 17:13:08.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4633, 388, 388, 1376249692, '19620401 12:34:45.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4634, 476, 476, 491796293, '19990330 11:43:07.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4635, 479, 479, 2048355405, '19550324 21:15:29.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4636, 436, 436, 2078524124, '19670827 12:09:51.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4637, 38, 38, 572360030, '19570907 03:01:50.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4638, 327, 327, 1691051114, '19720116 07:41:07.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4639, 6, 6, 1181935004, '19730612 05:40:01.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4640, 133, 133, 1749794389, '19700517 10:52:14.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4641, 338, 338, 690635880, '19590830 02:50:32.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4642, 495, 495, 1659358339, '19720629 18:09:22.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4643, 527, 527, 1299162002, '19660803 05:45:09.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4644, 356, 356, 1722153888, '19550821 13:52:51.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4645, 607, 607, 914440964, '19610921 19:04:42.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4646, 303, 303, 297178967, '20070716 05:42:11.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4647, 457, 457, 105582048, '19850119 03:09:59.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4648, 224, 224, 1308924359, '19740705 08:22:02.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4649, 422, 422, 966338822, '19931218 17:26:53.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4650, 634, 634, 1528462470, '19740826 22:13:24.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4651, 800, 800, 604184680, '19901130 04:56:39.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4652, 570, 570, 1115998566, '19611009 03:12:54.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4653, 537, 537, 294321939, '19911110 23:40:37.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4654, 581, 581, 1361919306, '19940430 19:41:40.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4655, 339, 339, 530141082, '19941116 17:13:48.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4656, 258, 258, 1019461409, '19571027 15:34:53.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4657, 385, 385, 1780207514, '19630517 09:43:23.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4658, 580, 580, 721535210, '19620928 07:06:49.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4659, 488, 488, 1945080136, '19790131 17:46:49.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4660, 206, 206, 1229183323, '19900226 22:30:04.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4661, 644, 644, 1673042368, '19621003 20:03:22.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4662, 506, 506, 1186725998, '19780808 07:10:43.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4663, 108, 108, 1721552803, '19821007 21:31:35.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4664, 805, 805, 865905577, '19820526 18:31:12.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4665, 771, 771, 304102204, '20060724 03:31:46.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4666, 641, 641, 1694131253, '19641217 22:04:05.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4667, 571, 571, 1173463555, '19611213 10:32:59.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4668, 175, 175, 911254339, '19900203 01:51:26.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4669, 429, 429, 806517964, '19960221 09:08:09.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4670, 447, 447, 1998424567, '20060207 05:15:59.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4671, 530, 530, 459174400, '19690917 11:51:23.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4672, 375, 375, 1747213920, '19670904 03:32:14.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4673, 993, 993, 384176609, '19901127 22:48:03.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4674, 543, 543, 1839477685, '20060602 04:33:38.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4675, 481, 481, 1719152927, '19660101 08:32:59.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4676, 271, 271, 1334040141, '19851210 07:09:47.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4677, 121, 121, 1381704531, '19800128 00:21:03.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4678, 920, 920, 688248559, '19700618 22:08:47.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4679, 689, 689, 1097040066, '19980502 15:03:48.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4680, 444, 444, 508765578, '19940724 19:29:06.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4681, 762, 762, 134717108, '19601201 16:03:14.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4682, 84, 84, 1610742201, '19650623 23:25:57.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4683, 389, 389, 1459042908, '19951217 11:10:17.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4684, 368, 368, 1471002619, '19980315 16:22:49.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4685, 612, 612, 1368529001, '19750329 16:54:01.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4686, 759, 759, 2008452237, '19830130 06:24:49.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4687, 591, 591, 1758753273, '19680705 17:32:30.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4688, 297, 297, 2006421011, '19751201 16:53:04.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4689, 245, 245, 234857579, '19570514 18:29:18.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4690, 592, 592, 458797259, '20050527 05:40:36.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4691, 428, 428, 159413288, '19570413 02:26:27.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4692, 761, 761, 67905204, '20021209 19:55:00.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4693, 231, 231, 774625657, '20001214 13:54:17.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4694, 627, 627, 663173734, '19910415 07:09:57.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4695, 420, 420, 694234506, '19600814 14:48:52.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4696, 189, 189, 1297267105, '19890120 19:39:48.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4697, 81, 81, 2029078517, '19970921 20:37:46.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4698, 773, 773, 1360759974, '19920308 11:13:57.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4699, 879, 879, 121486485, '19570127 10:21:04.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4700, 646, 646, 157948914, '20041003 21:00:36.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4701, 647, 647, 117680465, '19980803 19:09:12.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4702, 165, 165, 857643887, '20021216 01:18:33.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4703, 521, 521, 1319763320, '19860513 23:13:53.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4704, 387, 387, 2107664653, '19951110 12:10:53.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4705, 489, 489, 1196020320, '19571210 10:22:01.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4706, 298, 298, 1339542029, '19760329 01:24:25.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4707, 751, 751, 803455774, '19781111 04:24:51.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4708, 201, 201, 1406252423, '19930609 19:06:58.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4709, 983, 983, 221423211, '19810430 11:08:40.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4710, 698, 698, 1832553738, '19611208 10:04:00.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4711, 319, 319, 1076646, '19850929 15:13:49.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4712, 420, 420, 1709441441, '20001127 11:34:35.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4713, 270, 270, 2029341454, '19720527 16:03:01.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4714, 169, 169, 1616914151, '19920708 22:17:43.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4715, 185, 185, 1863605955, '19820329 06:26:49.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4716, 306, 306, 768468685, '20050410 20:37:25.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4717, 471, 471, 1117322743, '19900821 20:00:26.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4718, 626, 626, 587830707, '19920724 05:08:46.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4719, 589, 589, 519146803, '19600229 01:37:10.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4720, 666, 666, 1490369859, '19760624 21:06:56.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4721, 151, 151, 1058725856, '20040613 04:31:34.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4722, 628, 628, 3772624, '19940111 17:39:54.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4723, 40, 40, 2013228300, '19850922 07:22:08.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4724, 981, 981, 1259190845, '19980926 03:27:33.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4725, 207, 207, 654666990, '20011028 02:49:32.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4726, 274, 274, 1022164595, '19741008 15:56:22.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4727, 515, 515, 2077957572, '19740919 13:37:56.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4728, 446, 446, 1354892242, '20070814 15:38:05.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4729, 676, 676, 1262312957, '19981019 13:44:46.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4730, 226, 226, 415571614, '19841210 12:53:10.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4731, 13, 13, 707641207, '19960319 10:32:00.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4732, 176, 176, 485041881, '19951120 13:07:06.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4733, 624, 624, 2043917121, '19660523 11:47:43.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4734, 207, 207, 979656506, '20010728 18:03:11.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4735, 344, 344, 1858318126, '20031221 00:19:54.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4736, 830, 830, 1761220386, '19651216 14:44:41.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4737, 415, 415, 1605206458, '19870210 00:12:03.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4738, 678, 678, 1468829967, '19530206 22:59:24.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4739, 494, 494, 791214276, '19930406 06:00:48.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4740, 44, 44, 1371239490, '20061230 15:29:36.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4741, 309, 309, 1367268144, '19570322 01:59:58.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4742, 890, 890, 1661834831, '19830125 22:25:31.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4743, 577, 577, 22793880, '20020802 05:44:24.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4744, 677, 677, 854617343, '20030606 08:25:17.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4745, 648, 648, 1020021451, '19720828 14:26:46.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4746, 117, 117, 2048970297, '19640515 14:51:51.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4747, 907, 907, 1955122043, '19830716 03:51:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4748, 704, 704, 976257393, '19610608 15:36:34.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4749, 578, 578, 482954102, '19620118 00:43:07.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4750, 479, 479, 1355109055, '19630505 18:11:27.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4751, 960, 960, 8226272, '20060531 23:39:22.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4752, 468, 468, 2121498443, '19580301 15:29:00.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4753, 499, 499, 568792169, '19850109 09:36:47.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4754, 367, 367, 527301964, '19880803 13:14:07.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4755, 435, 435, 963141168, '19530423 16:29:02.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4756, 702, 702, 1468520711, '19570724 01:29:17.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4757, 367, 367, 1902841597, '19700326 17:56:24.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4758, 844, 844, 526480166, '19950817 19:46:19.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4759, 611, 611, 1586145629, '20020112 09:21:21.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4760, 582, 582, 957452794, '19910303 18:25:39.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4761, 545, 545, 102922679, '19540711 11:46:28.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4762, 754, 754, 118305114, '19790329 12:46:45.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4763, 568, 568, 1414707989, '20020821 05:22:12.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4764, 363, 363, 494870211, '20040519 22:15:29.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4765, 372, 372, 1271484260, '19730919 23:31:59.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4766, 838, 838, 697610206, '19971109 02:06:43.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4767, 654, 654, 897210143, '19870708 23:46:52.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4768, 626, 626, 195774964, '19781029 21:59:50.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4769, 47, 47, 189433566, '19960718 21:57:49.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4770, 681, 681, 907513737, '19690413 15:42:07.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4771, 749, 749, 1902154258, '19710302 20:25:21.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4772, 924, 924, 572708533, '19900715 21:01:01.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4773, 187, 187, 586174205, '20070418 23:07:57.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4774, 668, 668, 1820123278, '19610228 10:07:33.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4775, 981, 981, 971421600, '19720405 19:06:58.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4776, 491, 491, 524142826, '19740520 06:06:02.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4777, 706, 706, 213556802, '19990611 11:55:30.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4778, 471, 471, 1999949179, '19900417 22:56:50.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4779, 750, 750, 1442080703, '19790129 13:00:47.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4780, 288, 288, 164344147, '19900615 11:57:40.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4781, 18, 18, 913804391, '19990325 12:04:22.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4782, 67, 67, 1584988268, '19660413 22:01:21.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4783, 586, 586, 1061900535, '19980728 14:54:57.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4784, 493, 493, 445859458, '19880616 19:33:19.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4785, 540, 540, 15795411, '19860711 13:13:48.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4786, 23, 23, 302788869, '19910119 19:03:00.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4787, 466, 466, 248705883, '19631112 03:30:19.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4788, 172, 172, 1988136041, '19851018 22:33:35.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4789, 660, 660, 1128901730, '20010407 01:34:08.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4790, 795, 795, 1562145827, '19990530 22:50:36.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4791, 430, 430, 1586841352, '20030921 20:48:16.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4792, 442, 442, 1927046177, '19990625 04:25:44.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4793, 694, 694, 142771832, '19590125 13:38:03.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4794, 776, 776, 480389699, '19550204 11:35:08.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4795, 272, 272, 378252539, '20050622 03:43:07.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4796, 11, 11, 324846321, '19851223 13:13:13.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4797, 221, 221, 1649556962, '19880605 17:03:18.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4798, 234, 234, 921540356, '20060314 07:28:54.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4799, 504, 504, 1146372802, '19820917 08:53:17.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4800, 578, 578, 346093820, '19821019 12:20:03.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4801, 604, 604, 457067936, '19841106 15:04:59.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4802, 676, 676, 1038737639, '19570917 05:52:08.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4803, 670, 670, 181246086, '19721119 17:38:45.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4804, 601, 601, 151770420, '20001005 09:01:00.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4805, 678, 678, 1919133657, '20000303 21:46:05.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4806, 710, 710, 1527904272, '19681117 02:38:20.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4807, 615, 615, 416619004, '19581112 16:21:03.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4808, 345, 345, 193889032, '19761111 23:29:13.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4809, 345, 345, 188747248, '19690430 18:07:09.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4810, 919, 919, 153454676, '19861204 10:22:59.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4811, 728, 728, 578368557, '19660211 13:33:47.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4812, 528, 528, 1733139322, '19871208 18:59:34.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4813, 131, 131, 1902357246, '19690720 06:45:32.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4814, 201, 201, 1672601324, '19640730 21:23:40.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4815, 22, 22, 1799432311, '19991209 06:21:16.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4816, 176, 176, 1388052566, '19960328 13:45:07.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4817, 571, 571, 511700584, '19880830 21:51:47.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4818, 631, 631, 446522148, '19670307 09:48:53.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4819, 729, 729, 1598400479, '19701110 09:01:41.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4820, 793, 793, 210718342, '19900831 04:28:59.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4821, 358, 358, 2002120165, '19861229 22:36:23.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4822, 413, 413, 281162555, '19540921 01:13:17.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4823, 440, 440, 85517943, '19850723 20:25:41.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4824, 510, 510, 1229142262, '20050217 23:07:12.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4825, 953, 953, 1114648447, '19730722 18:53:50.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4826, 630, 630, 721139246, '19790907 15:59:36.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4827, 720, 720, 2024456299, '19840621 07:22:58.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4828, 62, 62, 2124694434, '20050315 11:32:48.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4829, 724, 724, 1953322645, '19630908 06:24:25.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4830, 764, 764, 1991161172, '19940413 05:53:45.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4831, 682, 682, 150193318, '19931213 13:02:43.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4832, 125, 125, 1404086226, '20010329 04:26:36.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4833, 683, 683, 1298696714, '19600801 14:03:33.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4834, 804, 804, 409649776, '20040411 00:40:43.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4835, 755, 755, 1185768087, '19580809 20:52:41.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4836, 612, 612, 1223099584, '19940709 06:12:13.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4837, 623, 623, 1033554430, '20010524 04:22:35.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4838, 832, 832, 1528080497, '20051022 05:53:57.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4839, 460, 460, 728926179, '19960413 21:47:13.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4840, 40, 40, 298921971, '19700825 10:35:27.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4841, 551, 551, 1950512904, '19850310 02:41:56.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4842, 774, 774, 2010853548, '19941230 04:01:05.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4843, 38, 38, 675805512, '19690621 04:26:26.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4844, 160, 160, 619072543, '19670505 15:11:09.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4845, 459, 459, 1035132659, '19860419 09:16:49.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4846, 434, 434, 1315765857, '19681015 01:48:42.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4847, 374, 374, 1411075705, '19821206 21:40:11.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4848, 3, 3, 441244695, '20020626 16:14:22.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4849, 589, 589, 793027169, '19630226 05:13:52.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4850, 404, 404, 68731240, '19620327 13:58:00.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4851, 948, 948, 2079895641, '20050423 15:21:25.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4852, 559, 559, 275457063, '19740402 07:14:25.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4853, 274, 274, 1476310937, '19880504 22:27:12.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4854, 723, 723, 870767289, '20000117 04:24:17.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4855, 438, 438, 2111817786, '20031020 13:43:39.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4856, 750, 750, 1281946978, '19830217 13:51:41.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4857, 596, 596, 1792107896, '19800224 09:26:27.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4858, 24, 24, 1131036783, '19751001 18:31:33.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4859, 873, 873, 1154324977, '19800316 14:07:09.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4860, 564, 564, 1844400281, '19730628 05:46:29.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4861, 542, 542, 1815334884, '19920527 19:15:21.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4862, 338, 338, 1676435527, '19610829 04:48:14.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4863, 764, 764, 1663707548, '19660307 17:04:47.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4864, 708, 708, 1440391111, '19830117 12:13:23.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4865, 210, 210, 1952969858, '20001220 21:20:15.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4866, 123, 123, 1751083201, '19721114 09:07:11.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4867, 28, 28, 501836734, '19800509 04:56:00.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4868, 369, 369, 774042435, '19730409 06:04:49.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4869, 810, 810, 2111489188, '19581223 00:30:59.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4870, 714, 714, 228805500, '19580603 00:09:02.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4871, 79, 79, 2015657928, '20021104 09:22:31.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4872, 144, 144, 1252776112, '19901108 15:57:49.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4873, 356, 356, 134006278, '19781122 12:01:40.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4874, 763, 763, 1534332129, '19530318 22:26:13.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4875, 187, 187, 128337519, '19580624 01:47:03.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4876, 693, 693, 169899689, '19991230 18:35:18.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4877, 73, 73, 2074551513, '19930331 20:46:50.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4878, 746, 746, 1197852126, '19710322 05:14:31.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4879, 669, 669, 2132629338, '19711102 13:50:23.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4880, 865, 865, 307307035, '19571008 17:05:08.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4881, 346, 346, 718029878, '20040930 18:19:25.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4882, 179, 179, 1780986750, '19891225 15:08:36.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4883, 740, 740, 1563038849, '19950119 06:18:56.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4884, 451, 451, 195498114, '19540614 16:26:25.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4885, 811, 811, 1459350402, '19961219 14:32:19.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4886, 737, 737, 1270994487, '19720913 05:48:18.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4887, 88, 88, 688355531, '19651125 05:59:23.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4888, 956, 956, 788110028, '19680715 09:39:15.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4889, 983, 983, 1415431154, '19530623 10:57:03.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4890, 423, 423, 901908157, '19830622 07:00:59.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4891, 48, 48, 416447098, '19670516 20:45:46.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4892, 733, 733, 834174350, '19780824 09:51:02.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4893, 875, 875, 360186923, '19780808 15:46:48.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4894, 591, 591, 1867505102, '19751018 16:00:44.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4895, 968, 968, 1932489833, '19860329 15:20:05.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4896, 584, 584, 1377527818, '19760712 21:53:42.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4897, 993, 993, 2035501895, '19811025 10:56:17.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4898, 717, 717, 680136866, '19940603 16:13:02.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4899, 297, 297, 1382176863, '19740514 17:50:53.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4900, 651, 651, 1603899208, '19660606 00:09:37.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4901, 709, 709, 1012626648, '19960606 20:52:41.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4902, 835, 835, 733582663, '19620830 15:35:22.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4903, 587, 587, 799755564, '19960705 15:43:23.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4904, 619, 619, 1886231220, '19570926 05:04:49.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4905, 320, 320, 1639314536, '19670523 04:35:28.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4906, 828, 828, 930000957, '19670616 23:23:36.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4907, 308, 308, 485118116, '20030510 19:12:44.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4908, 31, 31, 298868817, '19880218 00:00:59.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4909, 695, 695, 1999660755, '19940727 02:37:14.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4910, 860, 860, 1928349500, '19561021 19:08:21.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4911, 28, 28, 232647444, '19971117 17:04:51.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4912, 800, 800, 49498424, '19611206 04:11:55.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4913, 389, 389, 535422720, '19561130 04:41:10.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4914, 198, 198, 1685595518, '19750609 21:02:00.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4915, 746, 746, 1775220246, '19760926 11:27:03.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4916, 505, 505, 582521946, '19550604 04:06:45.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4917, 286, 286, 946918090, '19740809 22:29:17.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4918, 897, 897, 410508334, '19640924 00:38:22.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4919, 974, 974, 1448206035, '19940127 21:41:04.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4920, 996, 996, 1207744675, '19810409 15:31:56.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4921, 801, 801, 1888201764, '19591222 03:07:48.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4922, 569, 569, 1302611515, '19641025 19:16:45.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4923, 661, 661, 1818011972, '19721017 19:07:45.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4924, 905, 905, 311333714, '19830702 17:29:59.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4925, 780, 780, 288362590, '19990117 03:23:16.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4926, 707, 707, 2071736239, '19770313 23:31:29.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4927, 736, 736, 2051231330, '19650403 00:11:00.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4928, 624, 624, 462474666, '19560205 20:00:56.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4929, 362, 362, 299160853, '19741220 07:06:07.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4930, 637, 637, 1712463751, '19891211 03:53:20.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4931, 843, 843, 207883501, '19710308 07:39:00.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4932, 657, 657, 795752111, '20061028 04:33:48.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4933, 782, 782, 93022667, '20041106 11:23:38.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4934, 101, 101, 798659159, '19580530 13:40:45.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4935, 885, 885, 1526336283, '19690502 20:12:26.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4936, 905, 905, 1414108009, '19950202 02:44:02.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4937, 2, 2, 908488860, '20010620 23:25:12.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4938, 677, 677, 1269113218, '19900426 01:14:06.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4939, 900, 900, 145068387, '19950122 11:50:56.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4940, 893, 893, 1974552886, '19820706 15:08:42.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4941, 668, 668, 1305038211, '19770505 10:35:34.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4942, 112, 112, 1260980290, '19851007 05:58:23.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4943, 877, 877, 1658691327, '19811219 07:02:01.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4944, 507, 507, 320413711, '20040831 15:48:42.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4945, 786, 786, 818053202, '19930803 13:56:08.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4946, 265, 265, 842368790, '19580611 16:33:50.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4947, 364, 364, 502453659, '19740925 05:09:52.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4948, 647, 647, 556273613, '19560301 18:44:47.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4949, 939, 939, 463694568, '20070121 04:33:40.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4950, 752, 752, 520168356, '19831027 11:13:20.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4951, 463, 463, 373203065, '19541204 23:24:25.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4952, 268, 268, 606662335, '19700201 10:17:54.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4953, 531, 531, 661591625, '19530507 18:31:57.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4954, 962, 962, 1991409908, '20010203 02:18:56.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4955, 883, 883, 1472438301, '19590616 21:01:49.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4956, 107, 107, 951359564, '19840813 01:35:56.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4957, 607, 607, 2089234182, '19971111 20:27:37.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4958, 503, 503, 951014624, '19660104 22:13:23.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4959, 289, 289, 1800116877, '19660909 01:41:33.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4960, 519, 519, 1618904761, '19980310 05:03:59.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4961, 616, 616, 1954562430, '19550316 23:12:35.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4962, 472, 472, 1613462954, '20000404 23:11:48.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4963, 966, 966, 462454029, '19660301 16:54:51.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4964, 423, 423, 855379324, '19570816 22:43:54.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4965, 823, 823, 1101296583, '19930815 11:29:20.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4966, 580, 580, 1490493803, '19990120 12:31:03.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4967, 802, 802, 1083860955, '19650128 16:48:17.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4968, 706, 706, 64099252, '20060702 15:00:24.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4969, 260, 260, 774899741, '19590208 00:49:34.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4970, 785, 785, 1684295489, '19890824 08:43:01.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4971, 10, 10, 1711856215, '19611213 04:41:32.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4972, 562, 562, 1281543030, '19920414 21:15:10.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4973, 959, 959, 510204465, '19770901 12:47:25.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4974, 400, 400, 1737635727, '19640725 06:32:21.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4975, 520, 520, 1804735714, '19561206 04:56:22.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4976, 625, 625, 1321955014, '19820315 21:09:24.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4977, 826, 826, 1774819785, '19901106 12:14:44.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4978, 145, 145, 1450526294, '19760216 03:19:02.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4979, 879, 879, 572722773, '19971128 03:33:51.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4980, 593, 593, 668569386, '19930808 10:49:25.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4981, 78, 78, 2137464900, '19550620 18:21:36.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4982, 915, 915, 668830045, '19620202 05:26:13.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4983, 938, 938, 1315199589, '19570114 15:27:16.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4984, 395, 395, 1274511062, '19990222 19:52:47.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4985, 182, 182, 1140123499, '19880108 04:28:55.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4986, 855, 855, 1948578316, '19540204 21:21:06.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4987, 917, 917, 1865957656, '20060623 10:01:50.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4988, 24, 24, 139435887, '19561004 09:04:41.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4989, 412, 412, 1608267421, '20061213 16:38:55.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4990, 617, 617, 1350621131, '19920918 18:36:29.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4991, 944, 944, 1218311491, '20050322 06:16:25.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4992, 555, 555, 1801763455, '19880617 01:11:49.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4993, 990, 990, 1027430039, '19760817 23:32:33.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4994, 506, 506, 1018645424, '20041113 06:46:56.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4995, 447, 447, 1413905163, '19800422 15:56:07.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4996, 512, 512, 236721018, '19850129 11:23:46.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4997, 757, 757, 1986831271, '19720808 13:03:32.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4998, 500, 500, 687117324, '19770505 08:18:06.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (4999, 872, 872, 2063433444, '19640118 04:19:22.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5000, 680, 680, 643978889, '20020714 01:25:04.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5001, 727, 727, 180284576, '20010512 23:45:33.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5002, 322, 322, 571117757, '19760324 14:09:27.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5003, 716, 716, 1690178614, '20050121 17:55:13.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5004, 985, 985, 511196250, '19700530 19:50:38.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5005, 440, 440, 1268072675, '19741115 06:31:47.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5006, 788, 788, 1571170004, '19700821 02:09:14.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5007, 873, 873, 1497267162, '20000531 21:30:28.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5008, 642, 642, 1606600606, '19961012 11:59:36.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5009, 655, 655, 195627244, '19970228 21:22:34.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5010, 916, 916, 1015882104, '19850401 23:52:25.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5011, 782, 782, 1458595147, '20011006 13:21:12.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5012, 889, 889, 2105864856, '19740927 17:08:34.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5013, 376, 376, 978591637, '19760206 18:39:32.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5014, 32, 32, 431234453, '19810130 14:52:07.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5015, 711, 711, 1929416334, '19950921 10:42:23.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5016, 876, 876, 1717751059, '20010210 17:46:22.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5017, 252, 252, 561262502, '19960322 07:44:31.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5018, 382, 382, 302654114, '19750106 21:02:03.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5019, 896, 896, 853267308, '19770808 18:14:58.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5020, 842, 842, 1006413489, '19920711 14:08:14.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5021, 934, 934, 1720670472, '20000729 02:29:12.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5022, 982, 982, 1908376964, '19610427 07:43:36.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5023, 116, 116, 1627744260, '20070719 22:21:29.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5024, 467, 467, 1322395745, '19740521 06:13:04.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5025, 179, 179, 1547750080, '19920603 02:26:36.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5026, 177, 177, 772394573, '19810627 03:29:43.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5027, 412, 412, 1119511808, '19680828 21:42:37.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5028, 192, 192, 261265688, '19801018 06:00:28.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5029, 383, 383, 1480709970, '19920402 14:36:14.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5030, 996, 996, 296327287, '19791106 17:32:36.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5031, 912, 912, 1271334474, '19620808 08:05:51.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5032, 930, 930, 1824329913, '19660705 03:56:37.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5033, 173, 173, 827183797, '19650129 22:59:40.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5034, 330, 330, 953702526, '20030517 02:08:46.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5035, 609, 609, 956718403, '20000327 11:03:52.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5036, 229, 229, 953591844, '19870328 04:12:08.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5037, 830, 830, 1801301966, '19650110 11:30:59.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5038, 566, 566, 1606280367, '19940816 19:36:37.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5039, 742, 742, 589090069, '19770410 13:21:00.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5040, 663, 663, 1682174094, '19700521 02:19:41.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5041, 696, 696, 2101486957, '19610904 23:50:37.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5042, 636, 636, 1458824185, '19620911 22:02:14.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5043, 280, 280, 1600980504, '19670805 11:42:06.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5044, 889, 889, 1458867733, '19740913 16:46:30.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5045, 24, 24, 1488299185, '19981213 05:15:17.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5046, 690, 690, 1519735814, '19830626 13:07:51.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5047, 5, 5, 921957626, '19650512 06:32:16.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5048, 145, 145, 1156102760, '20030718 08:40:25.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5049, 725, 725, 154270642, '19620223 20:04:33.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5050, 684, 684, 796961948, '19870311 22:09:45.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5051, 166, 166, 1158204712, '19961108 03:39:14.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5052, 967, 967, 2055438546, '20050801 16:01:28.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5053, 282, 282, 2073333673, '20070926 14:03:07.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5054, 927, 927, 1614726648, '19790708 14:11:07.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5055, 526, 526, 208500977, '19541216 22:55:52.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5056, 17, 17, 1237002246, '19930116 16:01:56.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5057, 861, 861, 1952982561, '19760904 15:47:58.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5058, 563, 563, 1533518072, '19830904 11:42:08.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5059, 57, 57, 1465856242, '19851226 17:24:02.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5060, 22, 22, 87751935, '20010520 03:02:03.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5061, 394, 394, 1828890025, '19541224 04:26:36.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5062, 813, 813, 144323696, '19720814 15:29:59.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5063, 93, 93, 766097343, '19570712 21:23:21.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5064, 151, 151, 1944615295, '19700426 03:49:57.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5065, 975, 975, 304454804, '19750826 15:46:33.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5066, 938, 938, 2090290167, '19880404 09:24:35.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5067, 630, 630, 841736568, '19620829 18:08:53.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5068, 690, 690, 1953668758, '19800921 16:30:45.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5069, 897, 897, 364167670, '19881104 03:48:15.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5070, 951, 951, 189917343, '19610627 06:30:14.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5071, 341, 341, 2055704916, '19890201 06:15:22.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5072, 71, 71, 676799090, '19540807 12:07:54.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5073, 104, 104, 40084889, '20050927 07:41:03.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5074, 834, 834, 1074786242, '19600321 15:55:17.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5075, 883, 883, 1341256175, '19831107 14:17:37.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5076, 562, 562, 1625101098, '19901118 04:26:58.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5077, 45, 45, 590841491, '20010921 20:01:29.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5078, 525, 525, 267678052, '19851106 05:34:58.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5079, 9, 9, 1196485777, '19830608 00:57:47.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5080, 109, 109, 587274693, '19611209 13:18:45.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5081, 965, 965, 1629041580, '19690215 20:57:27.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5082, 765, 765, 501423166, '19730210 13:02:12.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5083, 502, 502, 1066349909, '19710826 09:55:55.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5084, 498, 498, 333822240, '19580123 16:26:29.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5085, 860, 860, 1066902222, '19901228 13:53:22.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5086, 518, 518, 55768730, '19650105 16:04:22.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5087, 824, 824, 1204572751, '19661008 13:53:29.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5088, 49, 49, 110869334, '19930724 08:48:33.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5089, 853, 853, 465345190, '20010601 03:12:54.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5090, 750, 750, 1107674601, '19600311 19:01:55.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5091, 153, 153, 761179595, '19630724 12:24:10.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5092, 963, 963, 1880238838, '19890507 23:48:51.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5093, 985, 985, 962294423, '19610822 02:12:34.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5094, 575, 575, 393396974, '19831121 10:18:56.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5095, 718, 718, 1462381997, '19680528 21:53:04.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5096, 633, 633, 147160900, '19970120 08:20:37.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5097, 650, 650, 895496378, '19580303 00:38:52.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5098, 386, 386, 1766991167, '20050410 07:52:09.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5099, 949, 949, 455056600, '20070118 00:59:56.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5100, 859, 859, 1530708836, '19630910 19:50:42.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5101, 76, 76, 192108607, '19731028 18:58:57.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5102, 721, 721, 1405661708, '19920819 14:01:31.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5103, 43, 43, 1631807892, '19670914 04:52:10.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5104, 629, 629, 226015154, '19530830 13:34:19.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5105, 54, 54, 483444546, '20060206 15:54:22.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5106, 536, 536, 455531285, '19950404 15:31:19.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5107, 399, 399, 2027657881, '20071104 08:20:25.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5108, 577, 577, 1447164797, '20000707 22:07:50.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5109, 858, 858, 1881701915, '20030830 23:53:30.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5110, 941, 941, 982755051, '19720128 18:28:53.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5111, 870, 870, 1540895863, '19990426 20:00:27.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5112, 756, 756, 704103380, '19981029 10:12:59.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5113, 58, 58, 442387727, '19530330 10:44:21.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5114, 406, 406, 450843601, '19770118 04:05:17.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5115, 869, 869, 955808555, '19850404 06:04:36.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5116, 482, 482, 1092942745, '19891111 15:24:24.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5117, 98, 98, 1081834011, '19531221 05:34:04.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5118, 866, 866, 48057886, '20070619 04:56:52.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5119, 857, 857, 873547722, '19870428 13:56:34.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5120, 743, 743, 291076386, '19630821 23:42:10.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5121, 63, 63, 1337747725, '19740628 19:31:05.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5122, 674, 674, 1309763683, '19770204 09:20:20.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5123, 220, 220, 964890774, '19870422 13:17:51.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5124, 106, 106, 1989034732, '19810826 08:46:19.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5125, 924, 924, 1971899952, '20051203 18:06:04.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5126, 905, 905, 1780771367, '20070926 06:26:31.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5127, 256, 256, 970220034, '20001215 09:55:19.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5128, 17, 17, 1566487653, '19741107 21:22:06.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5129, 529, 529, 53130816, '19991023 18:02:13.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5130, 185, 185, 848032938, '19941017 05:57:00.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5131, 922, 922, 1274801032, '19850918 03:48:06.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5132, 872, 872, 446215547, '20040612 12:09:20.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5133, 604, 604, 1222718527, '19861019 04:35:02.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5134, 359, 359, 1797076213, '19720928 05:07:05.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5135, 360, 360, 299050311, '19960210 18:19:47.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5136, 99, 99, 1970475316, '19840628 06:55:56.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5137, 979, 979, 1670147186, '19580531 08:10:01.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5138, 490, 490, 1852878913, '19701227 20:21:35.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5139, 545, 545, 322421413, '19591217 00:32:06.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5140, 7, 7, 1318410611, '20030924 22:22:02.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5141, 395, 395, 692786957, '19650303 07:43:57.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5142, 302, 302, 1182317168, '19740401 15:45:39.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5143, 765, 765, 185404864, '19971123 08:55:02.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5144, 675, 675, 763375541, '19820505 08:44:00.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5145, 548, 548, 344971096, '19681114 23:01:28.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5146, 122, 122, 1024556404, '19720924 02:11:10.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5147, 606, 606, 276274851, '19890208 13:04:29.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5148, 951, 951, 1233914407, '19920802 22:07:17.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5149, 139, 139, 692640970, '20060818 04:14:20.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5150, 713, 713, 1145227667, '19860717 06:28:39.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5151, 353, 353, 528180043, '19960201 02:46:32.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5152, 715, 715, 249337866, '19580913 19:41:59.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5153, 206, 206, 226669919, '19701214 17:55:34.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5154, 47, 47, 283049114, '19960530 01:17:45.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5155, 309, 309, 291571244, '19970314 00:19:36.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5156, 191, 191, 857029463, '20040922 09:38:37.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5157, 385, 385, 266268739, '19580501 00:43:39.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5158, 78, 78, 45394748, '19940117 20:05:51.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5159, 78, 78, 267582148, '19550926 19:28:14.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5160, 827, 827, 1327819287, '20060513 09:27:51.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5161, 67, 67, 2041439831, '20020419 05:36:00.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5162, 42, 42, 1151112421, '19851228 10:58:18.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5163, 684, 684, 1401924775, '19530916 04:05:36.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5164, 72, 72, 1012682738, '19611114 17:56:30.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5165, 27, 27, 430417604, '19940511 14:40:47.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5166, 957, 957, 1224909705, '20021113 07:51:07.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5167, 147, 147, 1151431875, '19650111 05:37:57.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5168, 880, 880, 109362030, '19880701 05:37:16.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5169, 560, 560, 1420624267, '19881207 09:45:29.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5170, 471, 471, 325460383, '19531007 23:08:40.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5171, 789, 789, 11106210, '19840614 07:14:23.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5172, 881, 881, 1959581234, '19901224 09:12:28.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5173, 809, 809, 1988422454, '20000704 04:24:46.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5174, 684, 684, 1561804618, '19910803 15:34:32.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5175, 295, 295, 581241754, '20060619 15:58:13.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5176, 887, 887, 383146968, '19530330 03:45:26.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5177, 359, 359, 2060755444, '19870314 00:25:17.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5178, 956, 956, 1084314222, '19571219 04:33:50.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5179, 157, 157, 1041960837, '19651012 09:44:50.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5180, 348, 348, 881135018, '19860312 15:54:54.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5181, 640, 640, 212555599, '19710818 17:22:01.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5182, 819, 819, 269557408, '19610515 11:48:01.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5183, 719, 719, 190690786, '19760321 17:07:46.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5184, 747, 747, 279996762, '19660409 11:33:33.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5185, 478, 478, 2046354813, '20060915 03:10:27.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5186, 737, 737, 243367811, '19800106 08:06:06.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5187, 585, 585, 1505116964, '19860630 18:13:45.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5188, 791, 791, 619503582, '19980521 03:17:16.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5189, 36, 36, 1390710460, '19830717 04:47:28.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5190, 818, 818, 1949843361, '19990520 08:41:10.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5191, 237, 237, 1260618468, '19790301 06:12:16.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5192, 155, 155, 1804283620, '19720512 12:04:10.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5193, 654, 654, 334470470, '19680403 00:53:21.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5194, 650, 650, 1744441175, '19610804 15:04:14.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5195, 414, 414, 1403309629, '19540608 16:46:02.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5196, 871, 871, 833719528, '19870305 20:45:39.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5197, 349, 349, 1542662430, '19730717 11:40:02.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5198, 882, 882, 1252389073, '19890109 14:58:32.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5199, 560, 560, 1279898704, '19951224 18:03:13.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5200, 523, 523, 2032726243, '19740101 15:10:21.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5201, 132, 132, 542252929, '19600913 20:33:12.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5202, 325, 325, 1038396976, '19530811 07:27:13.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5203, 217, 217, 1373767587, '19560827 12:21:47.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5204, 885, 885, 156189042, '20051111 05:05:40.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5205, 836, 836, 1321599220, '19550202 23:14:16.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5206, 165, 165, 1173268743, '19580208 17:34:03.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5207, 981, 981, 1378745721, '19660311 15:17:13.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5208, 245, 245, 14744179, '19870513 02:21:02.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5209, 113, 113, 2058334518, '19971210 09:19:32.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5210, 777, 777, 313225157, '19961215 20:34:09.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5211, 747, 747, 1037958140, '19700713 09:13:20.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5212, 532, 532, 935976750, '19530513 20:09:49.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5213, 995, 995, 1590263321, '19810408 10:21:00.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5214, 445, 445, 324288343, '19770716 03:33:20.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5215, 75, 75, 964403258, '19870926 16:05:49.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5216, 125, 125, 1898683478, '19931205 17:47:58.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5217, 186, 186, 576165630, '19621009 17:50:31.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5218, 558, 558, 1692685575, '19950608 16:32:03.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5219, 486, 486, 982243598, '19630301 14:46:02.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5220, 176, 176, 1360382326, '19670507 06:34:41.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5221, 779, 779, 1041570996, '19690516 13:37:21.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5222, 287, 287, 872262983, '19740823 02:20:41.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5223, 931, 931, 1522099140, '19571217 00:49:47.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5224, 579, 579, 65485065, '19970720 01:04:19.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5225, 561, 561, 482848669, '19820808 16:56:24.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5226, 172, 172, 563349748, '19650202 19:10:12.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5227, 573, 573, 300723449, '19750923 08:19:07.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5228, 935, 935, 1246155735, '19911201 13:20:31.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5229, 806, 806, 2030448489, '19900225 22:48:29.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5230, 103, 103, 780565220, '19720416 19:12:33.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5231, 875, 875, 203490888, '19870912 16:05:24.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5232, 102, 102, 484052582, '20060303 09:26:45.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5233, 94, 94, 771725966, '19691225 10:30:36.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5234, 363, 363, 1298869652, '19991010 18:34:29.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5235, 346, 346, 1644455879, '19780629 19:21:42.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5236, 605, 605, 1716586749, '19710108 09:54:48.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5237, 807, 807, 69122353, '19570918 23:26:14.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5238, 903, 903, 98843398, '19780510 12:02:06.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5239, 950, 950, 930811841, '19640307 12:19:16.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5240, 841, 841, 23628563, '20010807 09:36:23.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5241, 483, 483, 417401577, '19661028 16:48:52.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5242, 986, 986, 1411816658, '20070217 15:52:43.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5243, 31, 31, 1670086867, '19530611 17:57:43.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5244, 690, 690, 915842952, '19940731 08:13:19.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5245, 317, 317, 578251793, '19811107 23:27:50.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5246, 34, 34, 1356923711, '19781019 10:02:26.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5247, 264, 264, 372332906, '19990203 01:43:09.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5248, 209, 209, 676942173, '19980917 21:20:00.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5249, 468, 468, 618017695, '19811107 22:58:24.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5250, 783, 783, 2121819908, '19680702 22:58:04.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5251, 929, 929, 707809554, '19770528 02:55:07.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5252, 907, 907, 217709432, '19860207 19:07:58.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5253, 820, 820, 124799003, '19781112 00:12:29.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5254, 335, 335, 1131502910, '19810820 11:28:32.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5255, 321, 321, 1022983257, '19570819 01:32:59.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5256, 798, 798, 1029591735, '19940121 18:12:30.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5257, 811, 811, 1781728305, '20030825 06:37:25.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5258, 910, 910, 947169661, '19670207 11:17:27.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5259, 705, 705, 349490043, '19760405 12:09:16.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5260, 855, 855, 381482941, '19971230 04:04:04.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5261, 194, 194, 1158194451, '19900518 09:14:55.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5262, 344, 344, 1289093573, '19820410 01:56:42.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5263, 787, 787, 464072796, '20050316 03:32:32.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5264, 864, 864, 1436081904, '19780826 14:06:59.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5265, 126, 126, 1244150077, '19620406 04:28:44.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5266, 717, 717, 391611989, '19720510 23:46:34.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5267, 215, 215, 2034560342, '19910519 09:39:12.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5268, 942, 942, 1707012778, '19890628 15:46:31.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5269, 139, 139, 1687900113, '20070116 08:11:38.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5270, 912, 912, 1863575294, '19690919 06:11:01.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5271, 211, 211, 295199988, '19890319 18:21:44.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5272, 822, 822, 578839364, '19920531 05:48:25.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5273, 931, 931, 686028230, '19840403 04:12:46.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5274, 185, 185, 1920379298, '19690724 05:09:38.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5275, 75, 75, 1524639579, '19530710 13:45:48.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5276, 843, 843, 1466817903, '19700328 21:44:38.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5277, 109, 109, 1441841504, '19740314 08:22:57.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5278, 49, 49, 1195295013, '19710518 16:36:28.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5279, 653, 653, 1155805052, '19680912 01:36:29.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5280, 702, 702, 911626787, '19561021 06:53:57.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5281, 192, 192, 1231786644, '19731230 17:27:02.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5282, 644, 644, 1109144039, '19850104 10:59:07.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5283, 613, 613, 1573655588, '19630124 14:22:06.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5284, 254, 254, 939266632, '19740826 23:50:25.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5285, 101, 101, 1881927896, '20021119 16:17:25.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5286, 13, 13, 2007127436, '19540805 20:57:25.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5287, 219, 219, 1282304951, '19800422 09:14:15.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5288, 936, 936, 1908396571, '19960505 23:02:04.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5289, 59, 59, 815435654, '19950222 17:01:29.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5290, 274, 274, 1898620429, '19920607 01:09:11.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5291, 929, 929, 684674679, '19750516 03:17:22.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5292, 72, 72, 23748203, '19980811 12:08:46.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5293, 116, 116, 535505514, '19550204 23:52:50.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5294, 79, 79, 1935697211, '19740308 08:15:12.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5295, 292, 292, 1796296077, '19640323 00:21:28.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5296, 436, 436, 1350603862, '19920719 14:52:10.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5297, 901, 901, 595824264, '19541205 12:20:10.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5298, 150, 150, 2065398969, '19821016 03:50:44.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5299, 146, 146, 83313813, '19850427 03:44:34.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5300, 100, 100, 112725507, '19620630 23:41:15.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5301, 744, 744, 1200896634, '19950602 00:23:14.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5302, 41, 41, 1672991288, '19620808 09:56:38.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5303, 295, 295, 1010388040, '19990901 13:08:22.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5304, 272, 272, 751235598, '19650219 16:47:23.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5305, 39, 39, 1821452225, '19870415 04:36:19.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5306, 245, 245, 645582527, '19921226 21:06:42.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5307, 835, 835, 2039787059, '19541107 14:55:11.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5308, 729, 729, 1471654706, '19620421 19:02:50.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5309, 640, 640, 345640260, '19810210 21:42:44.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5310, 487, 487, 345048307, '19950523 03:48:21.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5311, 476, 476, 1173365103, '19721110 09:49:33.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5312, 534, 534, 1288808432, '19850409 14:00:57.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5313, 260, 260, 509215143, '20060527 09:40:58.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5314, 229, 229, 393698374, '19720616 05:15:19.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5315, 383, 383, 375358405, '19761231 10:37:01.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5316, 580, 580, 1289419864, '19580514 22:15:48.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5317, 928, 928, 59110998, '19720317 11:34:35.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5318, 80, 80, 1294626037, '19830722 03:42:07.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5319, 733, 733, 570603097, '19831006 21:49:33.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5320, 300, 300, 889684925, '19600901 07:31:18.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5321, 554, 554, 1976037902, '20000118 14:32:18.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5322, 484, 484, 295515208, '20030112 10:37:08.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5323, 275, 275, 481353805, '20020505 22:45:01.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5324, 187, 187, 1069068962, '19670812 07:02:27.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5325, 243, 243, 395059438, '20020506 02:53:38.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5326, 492, 492, 1493570561, '19980808 06:12:58.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5327, 246, 246, 1720504588, '19900427 05:55:34.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5328, 919, 919, 1473995293, '19630127 19:57:34.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5329, 195, 195, 1841433650, '19580504 04:48:10.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5330, 83, 83, 955017642, '19681220 22:53:37.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5331, 814, 814, 1776297093, '19680424 09:24:27.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5332, 153, 153, 946582477, '19990529 04:32:12.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5333, 486, 486, 1920994570, '19940120 12:51:53.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5334, 933, 933, 760731090, '19590316 01:55:14.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5335, 501, 501, 1740675445, '19690521 06:30:42.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5336, 262, 262, 1773743375, '19640525 07:30:23.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5337, 454, 454, 396065364, '19930504 21:50:45.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5338, 775, 775, 1594040853, '20051203 21:34:42.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5339, 403, 403, 2125278855, '19950513 19:14:05.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5340, 25, 25, 1342734388, '19621124 19:10:42.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5341, 953, 953, 42005345, '20070928 06:02:13.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5342, 98, 98, 472972700, '19710102 14:11:25.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5343, 86, 86, 733104290, '19680326 01:19:20.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5344, 242, 242, 1457312595, '20071002 13:13:08.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5345, 731, 731, 75251612, '19720728 15:19:37.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5346, 231, 231, 287641294, '19980204 13:16:25.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5347, 5, 5, 1321760401, '20000316 02:27:48.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5348, 332, 332, 1674307792, '19900820 18:37:31.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5349, 248, 248, 90746722, '20050306 21:38:11.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5350, 972, 972, 2033596191, '19581110 02:05:39.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5351, 549, 549, 290048429, '19730505 03:17:34.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5352, 547, 547, 3092074, '19790516 08:38:02.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5353, 656, 656, 911484682, '20070109 06:01:11.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5354, 405, 405, 1598501841, '19770826 20:13:15.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5355, 788, 788, 1165605825, '19700612 09:08:56.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5356, 980, 980, 606662219, '20000519 13:46:06.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5357, 314, 314, 531123682, '19680403 11:11:37.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5358, 465, 465, 1216433437, '19850121 06:05:54.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5359, 348, 348, 483972580, '19701016 13:53:45.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5360, 711, 711, 353393402, '19960905 22:23:21.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5361, 284, 284, 1161738636, '19550902 15:11:07.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5362, 980, 980, 357446831, '19991010 18:57:37.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5363, 63, 63, 1279245469, '19561219 14:14:39.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5364, 275, 275, 960401598, '19650221 22:49:06.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5365, 911, 911, 2046231781, '19800129 18:23:53.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5366, 893, 893, 1227699203, '19810614 05:17:20.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5367, 932, 932, 404411224, '19990320 01:09:03.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5368, 732, 732, 1905354629, '20000314 07:45:43.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5369, 998, 998, 1054447847, '19560127 22:44:36.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5370, 126, 126, 1821351232, '19650809 03:06:37.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5371, 512, 512, 1709707018, '19730109 00:25:03.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5372, 177, 177, 763138032, '19740413 13:59:52.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5373, 850, 850, 1456413446, '19960309 08:28:02.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5374, 546, 546, 1296393526, '19731113 02:38:50.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5375, 960, 960, 59320983, '19601204 01:29:04.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5376, 789, 789, 1269472117, '19820117 00:20:53.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5377, 336, 336, 1144375887, '19871020 09:17:48.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5378, 374, 374, 1040918201, '20020803 09:31:53.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5379, 968, 968, 278923685, '20030113 15:42:49.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5380, 973, 973, 518425435, '19570401 13:37:12.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5381, 131, 131, 54870291, '20060524 03:45:09.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5382, 393, 393, 825041295, '20070906 11:18:03.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5383, 236, 236, 890656390, '19651121 22:19:23.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5384, 377, 377, 457081914, '20070624 02:42:31.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5385, 80, 80, 180711175, '20030817 19:36:03.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5386, 160, 160, 406682432, '19961209 00:46:25.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5387, 301, 301, 456574786, '20000518 22:31:01.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5388, 822, 822, 1198045987, '19690527 16:38:37.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5389, 250, 250, 400060851, '19961002 16:46:18.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5390, 249, 249, 565489226, '19761231 16:44:36.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5391, 902, 902, 1859563651, '20040220 20:18:46.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5392, 976, 976, 420115011, '19610414 15:44:50.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5393, 635, 635, 1686552846, '19880219 07:40:57.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5394, 29, 29, 883821339, '20060905 20:50:43.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5395, 926, 926, 1189090331, '19600325 03:59:34.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5396, 239, 239, 200452965, '19601219 11:04:36.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5397, 642, 642, 715367299, '19670114 23:56:45.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5398, 196, 196, 1410779714, '19560203 02:30:13.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5399, 299, 299, 12578419, '19800903 18:49:14.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5400, 15, 15, 1552103789, '19990213 10:02:16.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5401, 466, 466, 2104675583, '20061117 12:07:21.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5402, 953, 953, 222171874, '19530102 18:42:04.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5403, 945, 945, 1394636856, '19870725 19:52:54.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5404, 171, 171, 998948380, '19920729 18:31:33.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5405, 1, 1, 998460978, '19931031 01:40:35.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5406, 679, 679, 1273211305, '20070121 13:17:41.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5407, 371, 371, 1818956581, '19910308 00:09:59.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5408, 878, 878, 1589187254, '19860227 03:22:20.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5409, 302, 302, 723172622, '19690923 18:44:11.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5410, 255, 255, 255746513, '19960525 08:48:03.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5411, 932, 932, 1057137251, '19650801 04:28:18.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5412, 839, 839, 1352839326, '19730831 01:39:44.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5413, 134, 134, 1896263056, '19900107 14:23:05.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5414, 202, 202, 1465187454, '19660718 00:16:32.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5415, 18, 18, 972598893, '19980414 18:38:11.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5416, 253, 253, 2011936771, '19551227 03:53:04.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5417, 158, 158, 423514370, '19861119 20:38:14.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5418, 847, 847, 1821943513, '19570627 11:32:08.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5419, 993, 993, 1812136975, '19690707 03:13:03.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5420, 373, 373, 336359499, '19910219 17:37:28.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5421, 514, 514, 1534687104, '19890125 06:46:19.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5422, 336, 336, 1468206567, '19821024 08:30:26.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5423, 908, 908, 241007130, '19560612 14:59:25.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5424, 943, 943, 905646310, '19870126 06:00:00.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5425, 546, 546, 1069103279, '19690618 06:47:50.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5426, 330, 330, 1428683626, '19640927 08:40:13.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5427, 659, 659, 1283027619, '19940222 06:18:55.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5428, 286, 286, 914199829, '19970703 11:37:19.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5429, 697, 697, 1696591836, '19660821 22:39:16.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5430, 658, 658, 430168657, '20071215 14:24:28.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5431, 736, 736, 945535490, '19680104 00:24:07.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5432, 622, 622, 468689687, '19840917 06:47:34.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5433, 230, 230, 757246029, '19741130 14:42:38.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5434, 247, 247, 983998732, '19561201 05:40:32.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5435, 836, 836, 898278926, '19580515 01:29:51.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5436, 491, 491, 235746895, '20060522 09:03:05.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5437, 527, 527, 473217328, '19730212 15:25:08.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5438, 32, 32, 813598571, '19810424 03:47:49.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5439, 266, 266, 247550423, '19660825 01:01:55.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5440, 636, 636, 971908219, '20040726 06:18:21.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5441, 245, 245, 1733137255, '19581005 00:36:25.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5442, 164, 164, 1166186342, '19670323 19:08:40.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5443, 618, 618, 936148643, '20070902 21:54:26.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5444, 228, 228, 1584597717, '19530511 07:58:14.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5445, 241, 241, 901177788, '19640602 12:16:18.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5446, 90, 90, 1408507946, '19830623 18:39:02.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5447, 104, 104, 382298927, '19790407 01:21:44.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5448, 686, 686, 35047274, '19740805 07:24:25.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5449, 670, 670, 47584894, '19610525 02:12:31.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5450, 389, 389, 1765387443, '19570331 00:06:30.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5451, 416, 416, 1117317046, '19820129 14:26:21.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5452, 382, 382, 1406730687, '19620721 12:24:37.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5453, 333, 333, 1003605292, '19940728 23:17:09.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5454, 327, 327, 1270215737, '19970715 01:11:46.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5455, 334, 334, 502869820, '19630120 03:15:57.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5456, 13, 13, 1131569862, '19770125 03:36:55.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5457, 487, 487, 184831951, '20040722 03:42:39.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5458, 123, 123, 2055234128, '19530630 13:52:54.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5459, 28, 28, 2032749509, '19760212 11:43:42.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5460, 321, 321, 554847317, '19820203 17:00:22.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5461, 514, 514, 1031528101, '19651130 06:58:46.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5462, 158, 158, 1314286515, '20010903 12:32:40.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5463, 425, 425, 776936828, '19720709 04:43:04.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5464, 683, 683, 1545335716, '19691009 04:19:42.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5465, 419, 419, 313858297, '19810523 08:23:55.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5466, 455, 455, 493277774, '19881112 21:40:43.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5467, 248, 248, 346947706, '20060930 10:57:05.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5468, 886, 886, 1719369519, '19860207 08:42:33.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5469, 455, 455, 1697488652, '19610305 22:46:41.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5470, 737, 737, 1270358586, '19991123 09:35:06.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5471, 16, 16, 2092206272, '19901112 12:27:55.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5472, 55, 55, 751659806, '19580729 16:50:25.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5473, 65, 65, 1953837859, '19981102 10:30:12.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5474, 202, 202, 67516371, '19721210 20:54:42.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5475, 70, 70, 1336227755, '19850518 17:01:02.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5476, 87, 87, 1671710862, '19741105 11:37:38.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5477, 343, 343, 1366566198, '19830220 10:35:59.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5478, 597, 597, 1311193177, '19560203 07:01:10.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5479, 668, 668, 1661925742, '19750826 17:43:42.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5480, 331, 331, 1223746078, '19931225 12:08:48.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5481, 223, 223, 205947052, '19930622 07:18:29.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5482, 508, 508, 1945791490, '19720720 22:54:29.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5483, 291, 291, 953801208, '19890209 09:24:48.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5484, 661, 661, 448642017, '19620524 22:32:45.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5485, 874, 874, 1456226200, '19781116 23:58:07.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5486, 787, 787, 378534363, '19580616 11:59:30.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5487, 53, 53, 932059745, '19980219 07:30:24.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5488, 948, 948, 1304318453, '19850517 13:30:52.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5489, 753, 753, 51551123, '20011111 02:24:34.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5490, 8, 8, 1578668259, '19890418 21:52:55.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5491, 318, 318, 1157859623, '19541101 05:20:25.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5492, 39, 39, 1241543974, '19720816 01:32:14.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5493, 491, 491, 1326422961, '19580313 16:04:06.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5494, 609, 609, 1869131791, '19920720 08:01:33.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5495, 978, 978, 415282826, '19910827 23:19:34.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5496, 608, 608, 757517096, '19650130 12:03:45.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5497, 333, 333, 1643114662, '20020913 14:25:36.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5498, 160, 160, 1151635217, '19901125 17:34:44.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5499, 844, 844, 1228098505, '19791218 23:34:19.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5500, 61, 61, 647781243, '19830720 14:35:34.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5501, 194, 194, 568485782, '19840922 07:41:56.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5502, 718, 718, 719153551, '20010226 16:39:11.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5503, 343, 343, 975467198, '19660602 08:37:44.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5504, 11, 11, 1619917833, '19690701 16:37:24.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5505, 324, 324, 1359511558, '19740518 11:38:34.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5506, 692, 692, 1972079100, '19760703 21:35:56.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5507, 393, 393, 1664265430, '19710918 01:54:25.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5508, 524, 524, 30582696, '19740819 02:22:27.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5509, 777, 777, 1745867328, '19650227 08:10:44.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5510, 135, 135, 1615024336, '19960315 15:38:18.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5511, 966, 966, 1395449199, '20011204 17:00:55.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5512, 179, 179, 1482024171, '20010619 20:41:28.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5513, 201, 201, 1908342578, '19851103 20:09:12.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5514, 739, 739, 1817741402, '19900216 23:34:54.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5515, 784, 784, 1642796595, '19960813 09:41:53.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5516, 425, 425, 2082676163, '20010511 08:04:17.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5517, 350, 350, 1416166564, '19650726 03:07:51.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5518, 30, 30, 1882418578, '19630216 06:10:18.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5519, 365, 365, 1527416619, '19981122 04:21:34.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5520, 156, 156, 2015610463, '19751208 20:24:25.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5521, 208, 208, 653570047, '19980923 14:10:18.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5522, 301, 301, 2072161665, '19740516 21:26:13.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5523, 227, 227, 2121729737, '19920328 06:17:58.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5524, 509, 509, 1062226818, '19791117 00:53:45.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5525, 524, 524, 900077467, '19980123 04:14:40.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5526, 952, 952, 493588710, '19710330 10:55:41.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5527, 508, 508, 1013164145, '19530519 00:46:18.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5528, 921, 921, 1588774963, '19590416 02:28:38.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5529, 470, 470, 468798239, '19890414 21:35:07.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5530, 772, 772, 1498285166, '19730419 04:57:16.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5531, 256, 256, 385661133, '19800221 21:12:01.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5532, 492, 492, 923178561, '20000326 17:01:15.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5533, 304, 304, 1286451464, '19840215 07:26:50.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5534, 793, 793, 522626142, '20000206 03:08:07.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5535, 455, 455, 1248620176, '19620405 04:26:51.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5536, 799, 799, 394143092, '20000424 14:39:17.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5537, 486, 486, 2111263233, '19590219 14:16:44.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5538, 565, 565, 192837120, '19720811 16:47:23.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5539, 269, 269, 583767787, '19960106 10:54:10.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5540, 891, 891, 464345079, '19550517 02:22:11.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5541, 565, 565, 1155395667, '19940928 10:05:05.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5542, 515, 515, 175063936, '19760704 05:07:57.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5543, 426, 426, 528113276, '19730321 05:20:07.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5544, 240, 240, 1433494655, '19580829 10:46:15.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5545, 426, 426, 1300636428, '19950515 04:52:00.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5546, 378, 378, 460309891, '19610513 08:38:57.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5547, 431, 431, 442143463, '19941012 05:23:02.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5548, 721, 721, 793737997, '19760124 16:29:12.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5549, 340, 340, 2072692319, '20031206 22:19:40.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5550, 422, 422, 775894624, '19980417 15:15:17.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5551, 312, 312, 2092794096, '20070707 08:55:54.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5552, 76, 76, 589060620, '19920729 08:15:17.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5553, 476, 476, 1486246607, '20000103 13:13:09.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5554, 77, 77, 715071129, '19570111 03:09:54.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5555, 341, 341, 682278777, '19921026 00:25:15.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5556, 346, 346, 1748113599, '19630510 10:15:43.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5557, 455, 455, 838451305, '19611202 10:21:12.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5558, 193, 193, 2070935580, '19940716 07:43:59.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5559, 556, 556, 351102296, '19790608 12:22:43.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5560, 411, 411, 1835584998, '19560220 00:42:53.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5561, 406, 406, 1191995758, '19760216 20:49:37.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5562, 42, 42, 1769184905, '19650604 23:25:46.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5563, 605, 605, 1358835326, '19930505 04:47:19.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5564, 14, 14, 1131869090, '19991110 03:13:27.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5565, 390, 390, 706805830, '19690123 18:26:17.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5566, 265, 265, 1394468594, '19540910 23:59:39.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5567, 405, 405, 1747361954, '19700506 13:14:37.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5568, 534, 534, 1466606000, '19930927 17:01:04.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5569, 145, 145, 1747242468, '19801114 19:08:02.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5570, 574, 574, 935553527, '20040419 19:02:35.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5571, 388, 388, 745928458, '19950323 17:47:32.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5572, 831, 831, 247664230, '20001213 10:20:27.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5573, 300, 300, 1030040832, '19750209 19:16:08.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5574, 541, 541, 1521180662, '19960709 01:59:23.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5575, 505, 505, 2087052754, '19890311 10:19:19.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5576, 138, 138, 850043341, '19750323 09:02:21.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5577, 534, 534, 1076588592, '19540226 16:06:05.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5578, 104, 104, 1525469036, '19860731 19:31:42.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5579, 397, 397, 1614959708, '19920703 20:01:45.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5580, 213, 213, 1137529066, '19890912 19:35:43.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5581, 602, 602, 1979783868, '19840617 05:32:38.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5582, 331, 331, 1194185512, '19850424 08:17:06.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5583, 856, 856, 2095854574, '19630510 04:08:58.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5584, 616, 616, 1040143876, '19981228 06:19:50.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5585, 969, 969, 306495459, '19731012 20:01:22.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5586, 701, 701, 2077623669, '19950725 12:56:44.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5587, 111, 111, 1609165533, '19530325 03:48:06.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5588, 230, 230, 1345895864, '19800205 04:16:56.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5589, 4, 4, 418760727, '19600413 02:42:51.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5590, 232, 232, 586895404, '20061125 18:11:08.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5591, 406, 406, 1696914183, '19910525 04:18:05.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5592, 153, 153, 912552383, '19720805 06:32:45.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5593, 567, 567, 289131788, '20010305 04:24:40.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5594, 638, 638, 1869920970, '19921117 10:11:16.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5595, 313, 313, 1735172032, '19870330 05:32:34.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5596, 465, 465, 1525000464, '19820426 10:39:18.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5597, 484, 484, 1559751723, '19910301 00:20:37.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5598, 116, 116, 1896642663, '19810511 02:06:40.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5599, 756, 756, 1899413108, '19970804 16:19:58.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5600, 867, 867, 1064315842, '19930904 04:52:20.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5601, 921, 921, 2020099555, '19990106 19:24:19.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5602, 255, 255, 222016721, '19540115 12:21:58.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5603, 127, 127, 1893706785, '20030310 21:21:09.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5604, 432, 432, 210085220, '20070818 03:17:05.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5605, 85, 85, 1573209401, '19560126 21:27:45.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5606, 713, 713, 43245689, '19590725 22:35:26.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5607, 829, 829, 532712625, '19700620 12:59:09.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5608, 264, 264, 1939946698, '19560627 11:13:45.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5609, 310, 310, 1170464531, '19751101 16:50:34.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5610, 134, 134, 1819769965, '19700806 15:22:54.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5611, 922, 922, 920833286, '19620314 18:09:37.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5612, 216, 216, 465555285, '19830503 14:49:30.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5613, 297, 297, 446541407, '19550113 11:42:13.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5614, 194, 194, 1834316134, '19970924 16:46:59.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5615, 214, 214, 716258725, '19790903 19:10:15.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5616, 614, 614, 1567693149, '19981024 12:32:30.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5617, 522, 522, 1801962017, '19550127 19:16:47.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5618, 289, 289, 1580005564, '20020507 22:27:29.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5619, 352, 352, 1882397993, '19790129 07:12:04.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5620, 462, 462, 2018091519, '19810701 05:29:32.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5621, 625, 625, 868268321, '19540619 20:11:33.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5622, 951, 951, 1550014583, '19980331 08:57:41.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5623, 974, 974, 1971858107, '19860617 14:18:12.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5624, 516, 516, 1252079477, '19811220 00:56:54.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5625, 540, 540, 69910163, '19651127 14:44:29.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5626, 594, 594, 2120662157, '19750819 11:14:46.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5627, 558, 558, 376846900, '20071010 05:55:47.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5628, 206, 206, 1058295229, '19900324 09:04:51.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5629, 379, 379, 1790861026, '19620412 20:26:49.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5630, 402, 402, 1455135106, '19591116 23:40:01.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5631, 263, 263, 1437765434, '19920122 08:41:44.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5632, 565, 565, 461278443, '19801017 13:59:24.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5633, 934, 934, 1859378749, '19961226 03:11:43.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5634, 436, 436, 1222835164, '20061029 15:09:25.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5635, 603, 603, 1326127501, '19980906 00:11:24.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5636, 613, 613, 1658230075, '19830603 17:10:40.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5637, 953, 953, 1706746632, '19900213 10:55:57.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5638, 33, 33, 1894992790, '19630922 00:51:53.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5639, 391, 391, 344673325, '19951203 08:52:04.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5640, 10, 10, 1161743134, '19670321 21:25:56.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5641, 411, 411, 967564483, '19780204 23:57:34.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5642, 913, 913, 190960837, '20040926 16:34:21.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5643, 786, 786, 1302816136, '19570406 11:26:21.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5644, 264, 264, 411095067, '19970906 11:19:57.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5645, 168, 168, 770697130, '19970914 00:01:31.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5646, 450, 450, 345510759, '19610122 13:28:35.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5647, 203, 203, 1992427930, '19700724 18:50:31.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5648, 21, 21, 1443472152, '19560611 11:37:40.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5649, 489, 489, 1361252936, '19660317 15:01:01.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5650, 93, 93, 1817430504, '19960604 17:00:03.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5651, 770, 770, 1572335412, '19800330 15:22:31.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5652, 444, 444, 1622458451, '19961023 01:53:07.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5653, 113, 113, 80990007, '19550413 18:54:36.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5654, 81, 81, 1836537444, '19690204 10:50:25.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5655, 551, 551, 764807630, '19920318 08:40:46.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5656, 329, 329, 1159431622, '19531009 10:26:37.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5657, 375, 375, 1925089844, '19750730 22:03:45.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5658, 851, 851, 1503014451, '19740322 20:24:15.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5659, 516, 516, 831995205, '19940921 12:32:35.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5660, 392, 392, 1141882264, '19880608 10:12:58.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5661, 704, 704, 1155791346, '19591015 16:39:39.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5662, 962, 962, 80506782, '19880328 03:54:18.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5663, 377, 377, 1913849015, '20020316 14:46:56.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5664, 355, 355, 1099182682, '19681216 17:10:33.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5665, 354, 354, 357456125, '19860715 06:41:10.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5666, 673, 673, 1112906579, '19890527 04:10:12.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5667, 217, 217, 591085923, '19940507 11:37:46.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5668, 915, 915, 1628403545, '19560315 20:32:48.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5669, 285, 285, 1558239026, '20070118 16:35:34.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5670, 518, 518, 65025136, '20040401 01:59:34.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5671, 945, 945, 590902899, '19610911 01:36:33.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5672, 344, 344, 710899159, '19990507 18:24:54.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5673, 530, 530, 394184355, '19590606 13:35:24.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5674, 227, 227, 982041183, '19641110 09:46:08.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5675, 442, 442, 2127110748, '19560527 05:31:58.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5676, 876, 876, 87058896, '19570922 03:37:12.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5677, 837, 837, 1184659388, '19931225 21:31:19.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5678, 105, 105, 1145112305, '19961010 02:58:14.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5679, 382, 382, 1110048349, '19920406 00:55:22.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5680, 601, 601, 330871962, '19571106 01:15:54.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5681, 265, 265, 1151094332, '19580126 16:24:14.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5682, 525, 525, 2115818670, '20040430 18:18:06.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5683, 57, 57, 350394030, '19770107 18:03:49.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5684, 105, 105, 991881276, '19731107 03:26:46.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5685, 142, 142, 1994701723, '19870818 08:17:30.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5686, 137, 137, 79945597, '20030331 06:48:36.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5687, 182, 182, 1897245484, '19780707 19:04:48.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5688, 520, 520, 942836543, '19801121 16:21:17.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5689, 190, 190, 1261957467, '19670814 06:28:38.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5690, 671, 671, 2028806413, '19971128 13:44:47.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5691, 241, 241, 1697222597, '19601104 19:06:54.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5692, 207, 207, 1318219803, '19681124 14:31:41.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5693, 503, 503, 1827363965, '19761230 12:19:17.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5694, 767, 767, 634648895, '19600626 22:39:06.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5695, 378, 378, 136789175, '19600607 04:46:17.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5696, 136, 136, 959670752, '19971108 20:03:16.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5697, 325, 325, 369324102, '19550715 01:47:25.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5698, 169, 169, 1024813387, '19960419 18:15:48.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5699, 3, 3, 1064050811, '19640224 04:38:47.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5700, 634, 634, 109073824, '19610420 19:51:18.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5701, 381, 381, 991887753, '19740917 01:50:48.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5702, 494, 494, 1743330058, '19670511 22:17:43.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5703, 941, 941, 1028617502, '19570523 19:02:06.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5704, 504, 504, 1926754399, '19691215 13:01:26.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5705, 468, 468, 2146522842, '19870925 15:23:29.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5706, 83, 83, 240680362, '19881122 20:57:36.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5707, 848, 848, 967228196, '19900520 12:17:42.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5708, 631, 631, 345910370, '19980602 09:08:28.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5709, 607, 607, 779529264, '19650911 05:18:27.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5710, 120, 120, 2109974277, '19870628 05:03:34.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5711, 301, 301, 1244070025, '19671014 12:55:17.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5712, 49, 49, 20185933, '19861019 19:05:31.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5713, 219, 219, 908917865, '19891215 19:28:52.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5714, 682, 682, 1311266787, '19891116 11:16:40.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5715, 590, 590, 107068939, '19830514 17:48:44.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5716, 119, 119, 979409349, '19630615 22:21:32.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5717, 238, 238, 1561776721, '19640321 09:50:28.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5718, 659, 659, 479203318, '19810510 11:20:09.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5719, 77, 77, 87211861, '19890430 08:53:03.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5720, 64, 64, 1779724476, '19910415 23:52:33.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5721, 815, 815, 1804603240, '19631121 09:05:23.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5722, 856, 856, 720069594, '19660616 19:16:28.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5723, 884, 884, 225935020, '19960801 14:04:10.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5724, 257, 257, 1463889006, '19620221 02:50:47.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5725, 407, 407, 1077056656, '19960528 06:52:40.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5726, 586, 586, 350064174, '20001016 11:04:51.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5727, 513, 513, 315760737, '19750509 06:05:36.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5728, 864, 864, 100191863, '20061209 14:56:18.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5729, 721, 721, 1342603507, '19570606 04:59:51.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5730, 702, 702, 464091886, '19660718 09:28:41.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5731, 368, 368, 1954698927, '19550312 01:49:47.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5732, 315, 315, 1349868832, '20050906 03:15:31.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5733, 662, 662, 126985479, '19850817 22:19:26.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5734, 335, 335, 830689444, '19831218 05:04:03.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5735, 713, 713, 1619422429, '19910218 23:25:06.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5736, 883, 883, 417812001, '19980916 18:06:31.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5737, 666, 666, 415012778, '19991209 23:16:00.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5738, 544, 544, 517645790, '19600125 05:02:22.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5739, 970, 970, 1522960541, '19940211 12:03:16.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5740, 196, 196, 1613510304, '20060924 11:19:53.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5741, 237, 237, 1180730058, '19651111 18:30:54.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5742, 244, 244, 984699280, '19880204 09:56:20.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5743, 781, 781, 536920752, '19680313 11:02:50.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5744, 814, 814, 216495334, '19880216 01:25:03.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5745, 718, 718, 2058861939, '19830216 00:49:29.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5746, 664, 664, 967798484, '19820116 00:01:23.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5747, 445, 445, 65904027, '19861209 19:02:49.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5748, 627, 627, 172075204, '19950213 01:02:37.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5749, 805, 805, 6092148, '19850212 04:50:22.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5750, 243, 243, 1096591132, '20041222 06:24:44.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5751, 915, 915, 860068585, '19860821 10:12:48.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5752, 552, 552, 1166001653, '19820305 14:27:15.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5753, 453, 453, 101977522, '19591222 09:22:45.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5754, 251, 251, 940007905, '19801109 04:46:14.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5755, 846, 846, 878385483, '20050530 10:45:55.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5756, 690, 690, 2050969877, '19610403 06:34:20.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5757, 489, 489, 1963469943, '19781009 08:13:33.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5758, 118, 118, 1588484317, '20030401 16:11:18.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5759, 159, 159, 402129166, '19810719 06:08:46.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5760, 115, 115, 724538772, '19941209 04:18:38.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5761, 838, 838, 565670778, '19660718 14:51:59.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5762, 472, 472, 1457690731, '19910610 21:21:23.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5763, 369, 369, 826658733, '19931228 04:08:37.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5764, 401, 401, 1963854325, '20070223 19:49:45.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5765, 677, 677, 1573167583, '19850419 03:13:47.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5766, 253, 253, 93614508, '19700206 09:39:46.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5767, 361, 361, 1012640896, '19540304 20:46:04.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5768, 321, 321, 505574898, '19581230 14:24:48.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5769, 398, 398, 492675300, '20060927 11:51:34.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5770, 364, 364, 1588439930, '19920826 23:42:12.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5771, 84, 84, 1315619240, '19710706 23:05:32.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5772, 603, 603, 458773815, '19570226 04:48:05.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5773, 87, 87, 735394495, '19950328 23:16:53.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5774, 69, 69, 1888530714, '19900805 21:33:09.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5775, 703, 703, 1716993078, '19780605 05:21:38.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5776, 204, 204, 1446077207, '19831016 23:09:03.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5777, 741, 741, 1265928702, '20060404 08:13:38.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5778, 643, 643, 1025581776, '19610617 12:39:06.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5779, 776, 776, 1858354723, '19870105 02:01:17.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5780, 473, 473, 596794687, '19670514 06:51:16.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5781, 293, 293, 1156840320, '19661108 16:02:02.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5782, 68, 68, 68795228, '19880325 05:02:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5783, 191, 191, 1166871125, '19741028 10:05:56.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5784, 305, 305, 1197552343, '19600613 22:35:06.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5785, 479, 479, 740357152, '19871126 23:15:53.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5786, 152, 152, 1774047875, '19810105 11:22:32.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5787, 599, 599, 1496191965, '19980916 17:52:46.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5788, 123, 123, 1403009416, '19571009 17:33:12.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5789, 215, 215, 2038985494, '19860719 18:18:08.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5790, 329, 329, 34500280, '19821118 16:50:46.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5791, 9, 9, 789634254, '19721209 09:52:58.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5792, 267, 267, 2078709513, '20040908 07:04:41.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5793, 62, 62, 1287296256, '19860707 22:53:36.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5794, 665, 665, 1863125556, '20070305 07:44:37.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5795, 615, 615, 2082860987, '19930309 20:27:54.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5796, 780, 780, 772444647, '19820602 21:09:30.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5797, 363, 363, 38192995, '20020206 05:47:43.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5798, 941, 941, 1627995224, '19690117 15:13:04.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5799, 500, 500, 1569153010, '19551030 22:11:16.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5800, 52, 52, 181265803, '19660110 15:09:43.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5801, 798, 798, 1944705019, '19801114 03:15:18.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5802, 246, 246, 1920385175, '19801211 04:38:00.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5803, 175, 175, 6270182, '19960517 13:11:03.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5804, 529, 529, 1855534932, '20000617 05:08:09.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5805, 756, 756, 1269920354, '19860619 07:19:11.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5806, 287, 287, 184298463, '19820626 05:24:42.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5807, 282, 282, 177086937, '19941207 15:10:21.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5808, 239, 239, 411239727, '19770517 11:49:35.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5809, 833, 833, 1775514579, '19550607 23:24:35.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5810, 906, 906, 604087909, '19740815 11:36:52.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5811, 698, 698, 793726290, '19621229 22:20:41.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5812, 319, 319, 2044555215, '19700424 19:34:26.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5813, 694, 694, 1275018888, '19690328 14:10:00.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5814, 280, 280, 802690311, '19670308 23:17:30.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5815, 762, 762, 1305081402, '19810131 12:16:35.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5816, 699, 699, 1733527549, '19860424 09:49:00.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5817, 460, 460, 420390611, '19690814 11:15:27.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5818, 168, 168, 544911995, '19860717 05:33:30.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5819, 501, 501, 62817049, '19720330 20:33:51.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5820, 526, 526, 188339926, '19570414 15:51:15.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5821, 80, 80, 1005909043, '19790522 15:47:00.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5822, 50, 50, 1486959920, '20040525 03:12:51.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5823, 719, 719, 2119003829, '19800611 20:06:39.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5824, 602, 602, 2043482502, '19761109 19:00:48.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5825, 779, 779, 20671830, '19720918 13:49:14.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5826, 513, 513, 2128331510, '19741027 16:00:50.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5827, 474, 474, 537894252, '19780821 05:54:29.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5828, 914, 914, 1011067470, '19960123 15:32:15.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5829, 205, 205, 1062215232, '20050528 01:05:15.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5830, 283, 283, 2086883596, '20040102 08:12:08.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5831, 775, 775, 131574645, '19890908 17:21:19.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5832, 225, 225, 824745678, '19900318 17:00:33.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5833, 548, 548, 693080764, '19580819 14:27:49.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5834, 74, 74, 340823720, '19731226 10:51:34.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5835, 680, 680, 377933890, '19940629 03:35:57.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5836, 709, 709, 637910755, '19931127 11:24:01.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5837, 387, 387, 759587016, '19991107 15:51:56.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5838, 636, 636, 1323154280, '19820512 04:57:47.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5839, 745, 745, 1248679443, '19811225 15:15:54.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5840, 923, 923, 499099051, '19580603 17:51:11.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5841, 771, 771, 1085395278, '19940128 20:12:10.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5842, 868, 868, 1948775355, '19740503 06:03:10.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5843, 607, 607, 1813282758, '19550505 18:08:36.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5844, 56, 56, 576105384, '19641205 06:41:16.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5845, 705, 705, 1707619128, '19721120 18:30:04.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5846, 252, 252, 709759004, '19550818 14:18:31.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5847, 209, 209, 1998632551, '19880613 16:54:40.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5848, 817, 817, 1279394035, '19720501 23:36:06.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5849, 564, 564, 1013689959, '19790202 19:28:02.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5850, 899, 899, 411988678, '19591117 10:38:53.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5851, 837, 837, 1403335229, '19651020 09:54:03.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5852, 148, 148, 1262586781, '19680724 00:14:13.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5853, 621, 621, 1892891962, '20041018 18:39:12.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5854, 439, 439, 1129183322, '20060717 06:20:01.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5855, 278, 278, 747923497, '19940820 23:22:42.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5856, 356, 356, 965871964, '19840621 00:02:26.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5857, 620, 620, 274162373, '19530702 08:31:20.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5858, 617, 617, 1148165876, '19720708 18:10:15.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5859, 764, 764, 1199881524, '19800929 15:18:54.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5860, 451, 451, 190009377, '19640823 15:18:20.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5861, 839, 839, 1505806559, '19561105 23:30:13.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5862, 712, 712, 1713709572, '20061113 23:38:05.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5863, 282, 282, 76990132, '19791220 10:44:19.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5864, 163, 163, 13666824, '19590605 15:12:27.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5865, 768, 768, 1263665960, '19921205 18:15:32.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5866, 212, 212, 89084339, '19801012 05:20:07.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5867, 860, 860, 733353957, '19640906 05:06:37.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5868, 692, 692, 117086025, '20030402 03:18:26.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5869, 672, 672, 1345037311, '19800908 19:41:32.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5870, 810, 810, 769049941, '19950306 00:52:34.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5871, 869, 869, 1308489744, '19940616 17:57:04.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5872, 347, 347, 247697896, '19950404 06:17:40.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5873, 585, 585, 935589334, '19570723 14:17:35.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5874, 638, 638, 2058348574, '19661029 02:42:40.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5875, 349, 349, 1798642161, '19710316 19:39:04.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5876, 679, 679, 1059480153, '19580120 09:43:49.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5877, 918, 918, 1020045906, '20020121 09:04:15.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5878, 701, 701, 316503903, '19680708 13:25:23.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5879, 658, 658, 1203411524, '19561221 00:30:43.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5880, 920, 920, 230674383, '19700201 23:30:43.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5881, 404, 404, 916709215, '19940514 23:06:10.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5882, 692, 692, 2097830733, '19590422 06:18:23.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5883, 760, 760, 892950654, '19691221 20:04:13.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5884, 506, 506, 1890223184, '19980712 14:26:21.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5885, 108, 108, 156021405, '19910315 22:18:04.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5886, 844, 844, 1284473068, '19740216 17:07:06.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5887, 813, 813, 105795933, '19930529 22:21:21.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5888, 154, 154, 2003207365, '19600202 08:07:48.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5889, 91, 91, 1976237394, '19870507 11:28:51.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5890, 703, 703, 1385374919, '19630109 03:33:30.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5891, 259, 259, 1962005412, '19930529 02:52:40.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5892, 596, 596, 2011155551, '19800501 21:41:40.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5893, 937, 937, 1168491238, '19540813 13:38:52.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5894, 722, 722, 816186660, '19700504 23:57:34.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5895, 552, 552, 1849327669, '19540729 18:20:57.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5896, 447, 447, 887653695, '19950316 20:34:05.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5897, 539, 539, 734111215, '20020513 19:18:50.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5898, 770, 770, 1512884367, '20031130 02:56:08.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5899, 161, 161, 666723692, '19791230 12:25:44.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5900, 976, 976, 1435121130, '20000208 13:09:57.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5901, 842, 842, 1611857703, '19981211 09:11:53.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5902, 579, 579, 1354019564, '19930313 13:36:14.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5903, 550, 550, 1660377882, '19990823 03:54:33.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5904, 952, 952, 2114435496, '19911129 18:35:28.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5905, 727, 727, 358184035, '19730601 16:41:49.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5906, 867, 867, 1993525633, '19780718 03:36:23.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5907, 480, 480, 572573722, '19640102 09:56:37.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5908, 641, 641, 868241965, '19901222 15:56:32.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5909, 899, 899, 396802646, '19880503 10:40:56.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5910, 821, 821, 1195799649, '19890801 13:38:53.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5911, 490, 490, 1686547264, '19900530 14:58:11.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5912, 784, 784, 1712395311, '19921223 19:05:57.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5913, 786, 786, 427421904, '19680719 17:39:31.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5914, 15, 15, 322447131, '19630829 15:48:11.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5915, 855, 855, 151585247, '19780410 16:12:10.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5916, 569, 569, 1815165035, '20050716 17:11:50.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5917, 175, 175, 1883901615, '19891124 03:33:51.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5918, 334, 334, 895961428, '19890608 20:17:58.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5919, 200, 200, 1729282746, '19760321 16:54:23.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5920, 882, 882, 120337400, '19711021 01:08:25.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5921, 191, 191, 990933928, '19950515 06:58:46.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5922, 618, 618, 946451427, '19570805 20:58:49.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5923, 739, 739, 1435818080, '19681126 15:49:34.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5924, 358, 358, 1104809837, '19700901 16:08:01.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5925, 536, 536, 994571422, '19541008 21:59:53.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5926, 417, 417, 982802337, '19670215 20:15:23.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5927, 340, 340, 1560920684, '19930822 16:38:48.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5928, 218, 218, 1876211917, '19950320 14:20:01.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5929, 945, 945, 1068286069, '19650402 08:21:43.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5930, 633, 633, 1393656018, '19831230 23:04:59.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5931, 592, 592, 1393267149, '19630908 14:24:58.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5932, 567, 567, 82276129, '20060221 06:08:06.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5933, 650, 650, 2088598837, '19960709 00:59:38.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5934, 631, 631, 778287363, '19641111 11:20:46.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5935, 885, 885, 2096874417, '19790221 14:18:49.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5936, 697, 697, 360585154, '19540304 09:29:55.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5937, 554, 554, 1751869476, '19670830 02:23:49.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5938, 18, 18, 1025518294, '19860122 01:28:45.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5939, 637, 637, 153657219, '19780210 21:44:31.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5940, 733, 733, 1421267744, '19650829 18:41:41.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5941, 827, 827, 352572884, '19630216 07:10:28.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5942, 326, 326, 1677847054, '19550609 06:24:48.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5943, 645, 645, 1003094432, '19790930 21:26:51.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5944, 643, 643, 1953563671, '20051004 21:49:58.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5945, 870, 870, 832370707, '19800810 12:35:19.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5946, 424, 424, 1312440416, '19530606 07:46:42.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5947, 741, 741, 1798749067, '19641014 04:02:08.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5948, 773, 773, 1925629542, '19981214 21:50:40.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5949, 680, 680, 1647434495, '20000124 07:45:23.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5950, 759, 759, 921809569, '19570112 01:09:06.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5951, 801, 801, 2006244888, '19580423 17:00:13.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5952, 556, 556, 810673150, '19651205 23:00:52.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5953, 812, 812, 568969893, '19800910 10:01:45.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5954, 725, 725, 828899774, '19610312 11:17:18.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5955, 790, 790, 891617471, '19570927 06:11:10.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5956, 750, 750, 355601899, '19940508 12:13:04.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5957, 531, 531, 1370633291, '19770417 21:46:40.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5958, 561, 561, 1260713813, '19811222 11:46:32.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5959, 946, 946, 246687188, '19900221 20:35:34.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5960, 683, 683, 590418304, '19590417 20:26:26.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5961, 565, 565, 253296922, '19921124 10:57:35.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5962, 851, 851, 1363439422, '19761013 19:36:36.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5963, 995, 995, 1870130566, '19780922 07:34:49.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5964, 809, 809, 19247679, '19570505 11:35:57.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5965, 66, 66, 827501114, '19781124 23:13:54.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5966, 225, 225, 452303771, '19920407 08:50:04.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5967, 675, 675, 680068034, '20040615 18:06:19.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5968, 351, 351, 1707084802, '19560908 06:18:45.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5969, 10, 10, 797975249, '19920707 01:29:22.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5970, 580, 580, 800543952, '19770207 06:42:15.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5971, 808, 808, 1721095107, '19901118 14:48:01.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5972, 632, 632, 676054266, '19561103 02:05:06.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5973, 819, 819, 31100408, '19640428 22:33:26.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5974, 334, 334, 285767932, '19630724 22:12:41.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5975, 606, 606, 2005775152, '19610904 17:57:56.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5976, 533, 533, 27922957, '19921207 00:33:57.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5977, 276, 276, 1237479614, '19851104 23:31:58.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5978, 957, 957, 11151745, '19710221 17:59:36.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5979, 794, 794, 1357785347, '19980121 03:32:42.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5980, 463, 463, 1468919795, '19540505 14:13:10.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5981, 761, 761, 595572396, '19550505 16:13:14.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5982, 916, 916, 857190277, '20020907 18:48:07.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5983, 922, 922, 1133384966, '20030224 06:34:37.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5984, 542, 542, 933824077, '19610322 07:12:36.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5985, 846, 846, 805533002, '19780909 06:04:46.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5986, 811, 811, 495076496, '20051002 15:24:06.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5987, 527, 527, 59701977, '19780613 20:06:21.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5988, 620, 620, 524691603, '19880429 13:42:20.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5989, 602, 602, 682045006, '19600216 05:09:35.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5990, 259, 259, 706548336, '19921015 02:05:45.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5991, 641, 641, 2061542891, '19841115 11:43:15.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5992, 359, 359, 1848314477, '19930906 14:37:16.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5993, 979, 979, 1424498456, '20031130 04:53:10.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5994, 810, 810, 964456320, '19711028 01:18:05.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5995, 860, 860, 450615488, '19801004 07:44:06.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5996, 946, 946, 2010867940, '19940504 11:33:52.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5997, 211, 211, 982639455, '19840915 22:49:59.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5998, 779, 779, 866443782, '19750529 09:50:53.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (5999, 662, 662, 1274219868, '19791111 22:36:03.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6000, 143, 143, 1648130427, '19960504 03:45:14.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6001, 600, 600, 1712672958, '19561220 13:40:23.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6002, 995, 995, 1702730269, '19610205 21:43:23.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6003, 859, 859, 581209509, '19590610 20:21:18.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6004, 584, 584, 1493640580, '19751218 01:03:08.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6005, 876, 876, 519779645, '19740223 10:21:05.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6006, 44, 44, 1898905520, '19540621 14:55:06.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6007, 183, 183, 1707265755, '19540810 00:27:25.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6008, 446, 446, 2036163198, '19700218 11:49:03.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6009, 948, 948, 2058630709, '20070707 17:19:21.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6010, 676, 676, 1656017692, '19721020 05:37:13.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6011, 430, 430, 737910697, '19610704 12:41:06.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6012, 913, 913, 938209456, '19590226 03:47:04.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6013, 91, 91, 1249901704, '19911201 08:13:50.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6014, 642, 642, 1323413750, '19881020 06:22:24.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6015, 954, 954, 1738041657, '19561214 04:13:11.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6016, 373, 373, 870804559, '19980318 16:09:27.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6017, 1, 1, 918722751, '19810819 13:01:58.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6018, 10, 10, 621391273, '19700704 00:22:12.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6019, 303, 303, 471691568, '19860826 05:31:11.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6020, 616, 616, 243747420, '19810221 07:49:48.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6021, 897, 897, 158657482, '19661026 12:43:43.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6022, 891, 891, 189558963, '19690217 04:23:59.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6023, 429, 429, 1812544239, '20040723 01:09:10.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6024, 630, 630, 1862750695, '20070921 23:23:36.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6025, 156, 156, 491962298, '20000323 18:59:00.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6026, 790, 790, 1023255317, '20050313 00:10:44.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6027, 674, 674, 2103604658, '19601203 21:11:55.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6028, 313, 313, 812048164, '20000701 21:15:21.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6029, 991, 991, 106217178, '19901019 14:28:35.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6030, 928, 928, 357412548, '19750503 06:24:04.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6031, 222, 222, 618369178, '19610324 01:43:57.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6032, 973, 973, 298418092, '19630610 13:41:04.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6033, 722, 722, 617504318, '19990411 19:23:32.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6034, 665, 665, 600383224, '19540919 23:47:27.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6035, 933, 933, 2072771208, '20050515 00:32:46.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6036, 719, 719, 1866305943, '20020328 18:29:51.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6037, 166, 166, 862227748, '19960330 22:26:49.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6038, 574, 574, 42940648, '19800310 05:31:29.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6039, 668, 668, 34926503, '19950126 20:51:31.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6040, 785, 785, 1186665765, '19770321 15:09:39.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6041, 582, 582, 1588349123, '20040224 14:56:40.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6042, 654, 654, 2109284892, '19610426 08:17:17.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6043, 2, 2, 1610671969, '19881022 20:22:58.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6044, 531, 531, 2044662574, '19950427 23:32:22.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6045, 504, 504, 1655458690, '19840413 13:24:38.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6046, 981, 981, 9038765, '19780921 07:56:10.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6047, 625, 625, 1313237869, '19541008 06:23:26.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6048, 892, 892, 1337823064, '19680210 22:30:46.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6049, 404, 404, 1031919968, '19671114 21:04:53.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6050, 819, 819, 1655197708, '19900718 15:34:38.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6051, 213, 213, 164610154, '19650915 22:31:53.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6052, 834, 834, 1935660345, '19670316 22:27:47.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6053, 339, 339, 906138629, '19961002 04:19:42.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6054, 1, 1, 1094939139, '20060919 14:46:14.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6055, 581, 581, 1892669207, '19820710 15:01:30.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6056, 971, 971, 1972084929, '19951102 09:16:24.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6057, 849, 849, 335661853, '19640715 20:34:12.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6058, 884, 884, 1895534314, '19590918 20:57:41.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6059, 862, 862, 85483257, '19830925 06:04:08.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6060, 117, 117, 570384618, '19761212 10:10:20.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6061, 598, 598, 1074684832, '20010717 17:43:11.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6062, 583, 583, 1355880187, '19620207 03:12:04.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6063, 738, 738, 1060767562, '19870501 21:20:27.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6064, 855, 855, 210432481, '19850307 10:55:16.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6065, 427, 427, 401259256, '19640730 03:53:15.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6066, 604, 604, 523136314, '20060123 23:00:02.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6067, 204, 204, 409875081, '19671116 08:23:31.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6068, 865, 865, 1041720233, '19900314 08:26:22.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6069, 366, 366, 1821772795, '19910607 05:49:37.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6070, 421, 421, 463470675, '19620917 09:43:20.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6071, 712, 712, 2096512204, '19541219 17:42:38.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6072, 954, 954, 366153221, '19540612 07:30:29.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6073, 450, 450, 1911816313, '19830607 03:30:41.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6074, 590, 590, 1160450019, '19620607 14:21:32.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6075, 473, 473, 1893027594, '19841227 16:53:08.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6076, 424, 424, 1124249612, '19580703 04:26:25.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6077, 239, 239, 659461695, '19880426 08:01:00.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6078, 281, 281, 1913479294, '19620329 01:36:47.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6079, 310, 310, 1790498925, '19760610 09:58:58.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6080, 743, 743, 508872054, '19740704 11:02:49.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6081, 983, 983, 1305597247, '20030606 17:47:18.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6082, 407, 407, 1568300108, '20001022 22:41:09.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6083, 118, 118, 1266144433, '19850819 00:10:27.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6084, 791, 791, 941664722, '19530403 22:53:57.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6085, 909, 909, 256845574, '19620818 07:52:11.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6086, 559, 559, 828865995, '20020107 03:16:10.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6087, 133, 133, 1534661888, '19740907 09:21:21.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6088, 649, 649, 536758748, '20000722 04:37:18.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6089, 969, 969, 1460479483, '19800311 08:45:56.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6090, 468, 468, 501720876, '19620714 15:16:22.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6091, 857, 857, 2028551405, '19900913 21:55:38.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6092, 89, 89, 101366307, '19890714 01:29:07.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6093, 684, 684, 2089010679, '20040614 23:27:21.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6094, 999, 999, 1854381506, '19710215 10:41:10.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6095, 467, 467, 1653731463, '19830903 21:26:27.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6096, 958, 958, 2020180204, '19950118 11:44:36.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6097, 626, 626, 107170729, '19811225 10:56:49.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6098, 29, 29, 790090044, '19560818 09:27:41.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6099, 790, 790, 690678385, '19830929 19:39:07.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6100, 473, 473, 353693861, '19860320 14:24:36.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6101, 968, 968, 1210423938, '19631106 23:32:39.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6102, 157, 157, 352576217, '19720725 21:57:03.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6103, 202, 202, 1173782259, '19840904 16:41:08.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6104, 41, 41, 416547597, '19580228 11:21:33.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6105, 80, 80, 1705944824, '19880730 21:51:59.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6106, 917, 917, 1575551396, '19640406 15:01:23.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6107, 3, 3, 46456037, '19911009 18:57:05.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6108, 92, 92, 1392501307, '19870428 13:58:10.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6109, 920, 920, 1552127809, '19740923 21:53:06.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6110, 991, 991, 1959107355, '19770108 10:35:05.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6111, 421, 421, 3474563, '19600709 01:15:23.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6112, 538, 538, 187894954, '19550420 18:57:25.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6113, 694, 694, 714099548, '19910409 10:58:43.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6114, 541, 541, 436011566, '19620324 19:01:18.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6115, 693, 693, 1851681870, '19810707 16:23:02.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6116, 470, 470, 1961483319, '19530925 19:02:02.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6117, 485, 485, 1870865302, '19840620 03:01:36.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6118, 907, 907, 1755016578, '19870128 22:26:30.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6119, 275, 275, 1825307353, '19750722 03:03:05.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6120, 190, 190, 1301267689, '19700723 00:37:04.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6121, 62, 62, 1996522464, '19840519 13:38:40.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6122, 46, 46, 63780510, '19750426 03:46:12.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6123, 308, 308, 284593389, '19630103 23:40:26.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6124, 745, 745, 1342663924, '19811124 14:33:15.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6125, 478, 478, 1165426247, '19800103 11:47:41.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6126, 871, 871, 1241106541, '19730606 16:13:31.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6127, 704, 704, 1142814155, '19571227 08:03:08.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6128, 944, 944, 1009357888, '19650422 16:49:31.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6129, 922, 922, 2029992114, '19861003 16:55:04.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6130, 350, 350, 846437160, '19971209 05:08:31.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6131, 233, 233, 44040171, '19840707 17:29:35.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6132, 655, 655, 1795973727, '19840908 22:33:19.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6133, 546, 546, 1902416512, '19860629 05:57:40.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6134, 972, 972, 523795993, '19980322 19:34:21.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6135, 911, 911, 934361855, '19630829 11:30:09.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6136, 854, 854, 918699417, '19831112 19:50:14.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6137, 878, 878, 2086949764, '19690218 06:00:00.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6138, 824, 824, 162869249, '19800621 12:48:54.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6139, 974, 974, 372564877, '19720903 01:01:58.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6140, 475, 475, 1958015280, '20060512 16:50:47.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6141, 925, 925, 1900971313, '19630401 08:19:05.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6142, 91, 91, 2106275381, '19950511 19:23:11.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6143, 840, 840, 276132345, '19781030 06:44:12.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6144, 180, 180, 1510711111, '19560303 22:10:51.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6145, 965, 965, 1520469464, '19550106 14:00:58.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6146, 983, 983, 769796045, '19880527 02:58:13.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6147, 645, 645, 915617909, '20060405 14:30:24.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6148, 108, 108, 608122882, '19950325 04:26:02.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6149, 742, 742, 368022346, '19970809 18:18:07.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6150, 886, 886, 1256124237, '19821210 02:24:25.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6151, 911, 911, 950576757, '19630801 08:42:59.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6152, 765, 765, 1067205421, '20021126 12:30:19.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6153, 49, 49, 1369485085, '19890127 06:24:35.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6154, 148, 148, 797136484, '19660310 19:02:02.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6155, 977, 977, 1196747644, '19541102 00:45:56.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6156, 106, 106, 2019334204, '19960712 19:46:26.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6157, 271, 271, 762787916, '19620723 22:16:37.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6158, 821, 821, 1397884828, '19551013 02:07:53.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6159, 909, 909, 425815487, '19860224 23:33:51.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6160, 97, 97, 2090844798, '19680225 05:38:28.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6161, 697, 697, 1706282313, '19590412 06:58:15.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6162, 135, 135, 1004295254, '19790620 02:07:34.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6163, 349, 349, 1975843107, '19530726 21:03:06.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6164, 78, 78, 65166506, '19841014 16:44:34.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6165, 799, 799, 1832179, '20000702 17:05:29.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6166, 185, 185, 1063403597, '19831030 02:42:04.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6167, 218, 218, 1839891000, '19761021 12:59:44.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6168, 79, 79, 436633687, '20010116 15:24:21.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6169, 797, 797, 1559019308, '20060726 07:31:09.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6170, 772, 772, 1105152940, '20050824 20:32:48.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6171, 117, 117, 678677122, '19920806 13:02:01.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6172, 150, 150, 359760736, '19561230 14:12:42.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6173, 475, 475, 837345129, '19670528 21:24:31.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6174, 590, 590, 754615740, '19770310 10:12:18.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6175, 645, 645, 174247808, '19600423 16:17:59.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6176, 609, 609, 1815619883, '19970107 18:15:28.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6177, 188, 188, 748112290, '20040625 21:02:00.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6178, 8, 8, 1234581815, '19591103 01:29:34.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6179, 864, 864, 1830784968, '19791120 00:32:16.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6180, 186, 186, 1204761549, '19990808 08:49:28.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6181, 66, 66, 1213315639, '19750303 01:43:06.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6182, 173, 173, 392882249, '19701004 03:37:06.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6183, 462, 462, 1485471653, '19750912 22:31:23.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6184, 157, 157, 1389053171, '19561025 14:30:39.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6185, 263, 263, 2051148795, '19870511 20:25:31.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6186, 728, 728, 1126352104, '19890811 04:59:15.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6187, 326, 326, 1437408232, '20030812 16:08:43.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6188, 817, 817, 1909423595, '19730421 10:55:37.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6189, 172, 172, 1146461482, '19960521 18:48:25.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6190, 962, 962, 205263977, '19750215 15:43:42.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6191, 64, 64, 794163667, '19740422 21:33:36.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6192, 932, 932, 36181270, '19660510 03:52:07.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6193, 217, 217, 2000415591, '20020426 13:15:02.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6194, 124, 124, 1180298552, '19570710 19:23:30.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6195, 845, 845, 2077536631, '20000201 09:52:31.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6196, 985, 985, 1391180277, '19911012 06:11:30.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6197, 164, 164, 131607177, '19941016 22:20:05.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6198, 593, 593, 490113470, '20020114 13:59:37.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6199, 699, 699, 922917680, '19630901 05:05:22.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6200, 65, 65, 1760413403, '19790309 11:18:54.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6201, 156, 156, 2128399378, '19640806 13:58:28.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6202, 758, 758, 1861169130, '19580320 23:06:03.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6203, 19, 19, 2022604542, '19960828 20:54:53.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6204, 543, 543, 53024456, '19991215 21:45:19.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6205, 904, 904, 1337179641, '19980504 13:22:23.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6206, 960, 960, 736255337, '19590802 18:30:17.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6207, 849, 849, 1254409486, '19880701 15:05:48.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6208, 162, 162, 1564944513, '19641119 20:12:17.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6209, 488, 488, 1596957513, '19581118 02:44:03.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6210, 120, 120, 324736126, '19651024 06:30:27.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6211, 544, 544, 1480591214, '20000116 22:44:26.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6212, 681, 681, 783733876, '19550921 20:47:03.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6213, 266, 266, 1067777048, '19831123 01:35:36.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6214, 649, 649, 293620857, '19940719 14:44:23.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6215, 769, 769, 1857756100, '20001224 03:55:21.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6216, 888, 888, 349967371, '19960708 03:21:09.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6217, 803, 803, 1505599052, '19561008 03:36:10.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6218, 149, 149, 448783348, '20040930 06:32:27.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6219, 19, 19, 469944442, '20050604 20:19:02.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6220, 171, 171, 878426073, '19631123 12:06:13.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6221, 73, 73, 1900204379, '19880318 10:33:21.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6222, 895, 895, 596408229, '19560703 02:04:07.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6223, 211, 211, 2036899153, '19570828 20:35:55.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6224, 637, 637, 744535686, '19840610 15:47:26.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6225, 514, 514, 1990653053, '19840504 22:59:11.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6226, 517, 517, 1532298240, '19790331 09:09:54.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6227, 26, 26, 636962250, '19620905 00:57:39.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6228, 365, 365, 1463624833, '19621119 02:01:01.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6229, 16, 16, 190942795, '19850206 00:19:46.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6230, 66, 66, 503615813, '19760712 10:06:28.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6231, 852, 852, 686756693, '19550325 19:55:23.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6232, 132, 132, 1395198098, '19550613 07:02:23.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6233, 35, 35, 1783692415, '20040302 20:24:11.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6234, 189, 189, 793342892, '19530913 13:13:22.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6235, 562, 562, 26071129, '19880102 18:50:59.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6236, 140, 140, 1365777219, '19691214 02:37:03.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6237, 38, 38, 1488849002, '19820204 06:42:13.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6238, 731, 731, 2033054646, '19830928 00:46:04.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6239, 937, 937, 902793658, '19660623 01:08:15.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6240, 19, 19, 1934831298, '19801010 01:55:13.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6241, 677, 677, 1361013991, '19540210 13:53:27.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6242, 511, 511, 887616916, '19910923 19:56:26.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6243, 423, 423, 1542836896, '19670605 08:11:34.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6244, 908, 908, 476103359, '19830730 12:17:58.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6245, 372, 372, 318137562, '19830129 16:59:16.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6246, 707, 707, 1445197625, '19710803 00:46:33.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6247, 68, 68, 108656242, '19900617 02:16:31.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6248, 210, 210, 1520066190, '19601008 22:30:39.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6249, 72, 72, 1900855373, '19640716 15:28:08.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6250, 902, 902, 2081344644, '19560727 06:31:22.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6251, 927, 927, 675216637, '19880105 02:35:20.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6252, 947, 947, 1443060255, '19980115 15:47:38.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6253, 53, 53, 604239819, '20040810 17:40:34.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6254, 25, 25, 1272777684, '20071008 16:59:09.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6255, 499, 499, 984674797, '19981221 00:45:33.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6256, 936, 936, 1309169452, '19610204 11:54:21.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6257, 95, 95, 1323316498, '19530724 02:30:07.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6258, 216, 216, 1118854068, '19650321 05:07:26.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6259, 598, 598, 268443676, '19680812 22:46:07.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6260, 385, 385, 2087916728, '19720205 04:12:28.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6261, 820, 820, 220828189, '20041126 17:32:38.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6262, 714, 714, 1500801603, '19780814 13:04:46.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6263, 994, 994, 60766116, '19871013 19:52:30.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6264, 766, 766, 1244143989, '19900507 16:37:34.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6265, 323, 323, 530628165, '19630802 10:35:04.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6266, 51, 51, 495092159, '19970806 15:42:02.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6267, 220, 220, 287664637, '19590721 00:22:51.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6268, 123, 123, 1071484528, '19830312 12:22:13.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6269, 184, 184, 1006186590, '19590718 19:53:23.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6270, 203, 203, 447025126, '19840112 01:18:17.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6271, 443, 443, 89404446, '19670604 20:38:56.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6272, 26, 26, 1441894000, '19810110 02:50:05.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6273, 170, 170, 2130638698, '19910411 05:24:11.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6274, 522, 522, 756441444, '19770826 18:23:48.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6275, 43, 43, 1965302740, '19621013 22:12:45.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6276, 259, 259, 970258646, '20040624 14:36:54.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6277, 6, 6, 946369462, '19970127 17:52:32.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6278, 147, 147, 1880917069, '19820129 08:17:57.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6279, 73, 73, 206775857, '19540513 22:48:09.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6280, 199, 199, 406754160, '19651003 22:12:38.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6281, 163, 163, 1300150152, '19961013 06:53:23.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6282, 23, 23, 2125453361, '19541128 02:27:00.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6283, 456, 456, 520788043, '20060504 10:32:52.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6284, 548, 548, 1524267306, '19810713 17:48:24.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6285, 649, 649, 1240828056, '19960707 07:31:08.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6286, 592, 592, 117443768, '19650309 04:07:44.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6287, 667, 667, 327737298, '19581101 13:21:48.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6288, 48, 48, 1763465448, '20040525 03:25:02.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6289, 126, 126, 544972447, '19620923 12:27:49.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6290, 873, 873, 227808860, '19791129 06:56:38.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6291, 107, 107, 592140037, '19690524 00:06:56.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6292, 153, 153, 1808098572, '19691117 01:34:46.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6293, 828, 828, 52951679, '19680216 01:59:57.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6294, 320, 320, 24229636, '20020517 20:55:46.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6295, 400, 400, 1615050821, '19831113 08:22:35.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6296, 89, 89, 799977718, '19830629 00:48:41.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6297, 818, 818, 2134141170, '19561212 00:03:56.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6298, 729, 729, 154123750, '19850127 15:33:59.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6299, 796, 796, 975156445, '19721229 01:42:53.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6300, 453, 453, 1218530684, '19930624 01:17:13.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6301, 895, 895, 1905553122, '19650114 00:23:42.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6302, 298, 298, 445086004, '19600407 13:54:17.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6303, 814, 814, 460085153, '19540325 02:37:15.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6304, 865, 865, 1929613274, '19880704 14:09:51.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6305, 915, 915, 282439981, '19970221 09:52:26.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6306, 201, 201, 1120317905, '19591226 23:45:14.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6307, 261, 261, 231960721, '19591008 10:23:27.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6308, 103, 103, 811001947, '19791216 23:16:45.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6309, 729, 729, 2133728743, '19971226 18:46:24.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6310, 130, 130, 1165229031, '20020627 10:08:38.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6311, 560, 560, 917227281, '19720107 18:01:48.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6312, 36, 36, 566476740, '19790624 18:12:10.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6313, 408, 408, 295640106, '19631109 06:19:17.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6314, 693, 693, 1238737263, '19551111 00:33:28.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6315, 254, 254, 1096014076, '19830423 21:19:05.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6316, 997, 997, 1751514987, '20021231 15:05:37.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6317, 941, 941, 1732011925, '19800411 02:31:53.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6318, 120, 120, 27430547, '19590403 02:04:06.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6319, 1000, 1000, 1967229765, '20011030 09:06:26.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6320, 166, 166, 1580727734, '20060525 06:27:20.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6321, 804, 804, 805355937, '19911007 02:20:14.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6322, 993, 993, 412508298, '19630224 20:57:48.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6323, 7, 7, 1527819403, '19730619 23:54:23.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6324, 351, 351, 314456285, '19870819 12:56:44.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6325, 212, 212, 380364841, '19670822 01:11:21.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6326, 496, 496, 1351232261, '20050717 19:04:09.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6327, 284, 284, 835612383, '19651126 00:50:08.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6328, 833, 833, 305644395, '19961123 08:28:24.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6329, 440, 440, 1982686688, '20011013 10:01:12.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6330, 726, 726, 938406622, '19870416 21:24:04.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6331, 342, 342, 213165223, '20000714 14:32:58.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6332, 605, 605, 1310693429, '19650101 02:18:33.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6333, 827, 827, 646956458, '19620201 06:35:03.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6334, 121, 121, 594363097, '19681118 21:30:55.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6335, 482, 482, 2037521108, '19530920 21:48:56.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6336, 896, 896, 2105743921, '19890708 16:59:05.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6337, 818, 818, 575916952, '19530905 23:49:44.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6338, 42, 42, 1307794203, '19701031 20:23:01.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6339, 661, 661, 589501585, '19920521 07:55:57.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6340, 197, 197, 787645008, '19890713 07:45:53.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6341, 357, 357, 1817171349, '19580602 17:44:17.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6342, 857, 857, 1243994331, '19861115 14:05:02.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6343, 40, 40, 1222239477, '19590531 08:38:38.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6344, 363, 363, 2108506935, '19680328 02:19:11.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6345, 607, 607, 1300707212, '19601122 12:54:49.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6346, 953, 953, 1377672444, '19971129 05:54:45.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6347, 673, 673, 314415371, '19590109 19:15:29.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6348, 842, 842, 865358494, '19650408 01:26:28.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6349, 670, 670, 520405100, '19720125 23:36:41.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6350, 594, 594, 990034895, '19881111 17:16:58.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6351, 675, 675, 269521624, '19560319 22:16:47.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6352, 161, 161, 1307754973, '20050909 21:59:49.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6353, 11, 11, 964132239, '19910330 06:27:32.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6354, 730, 730, 330308448, '19740805 19:15:33.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6355, 461, 461, 591701785, '19540918 22:56:59.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6356, 437, 437, 797902702, '19541121 03:25:53.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6357, 778, 778, 397615391, '19941018 13:59:54.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6358, 710, 710, 558746093, '19740805 13:40:31.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6359, 982, 982, 1969473100, '19731114 12:58:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6360, 653, 653, 1290557920, '19990806 14:48:16.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6361, 129, 129, 113599932, '20020129 22:55:06.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6362, 361, 361, 1767236544, '19701114 01:55:02.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6363, 353, 353, 1967159241, '19860304 13:15:33.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6364, 85, 85, 1553482461, '19630912 21:22:19.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6365, 268, 268, 2035369421, '19541214 19:35:39.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6366, 330, 330, 113368684, '19600107 15:43:15.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6367, 703, 703, 86934138, '19700524 11:37:06.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6368, 568, 568, 1911508063, '20021220 08:48:21.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6369, 187, 187, 1954415327, '19550221 02:44:32.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6370, 307, 307, 155207189, '20011015 04:20:00.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6371, 987, 987, 233176632, '20020427 15:15:53.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6372, 956, 956, 1607621292, '19620612 06:08:52.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6373, 162, 162, 1084134619, '19741111 18:08:08.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6374, 346, 346, 744861738, '19650420 01:20:33.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6375, 184, 184, 1954790397, '20001223 12:43:02.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6376, 777, 777, 619587350, '19571122 12:15:11.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6377, 546, 546, 1152845501, '19560927 12:19:10.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6378, 892, 892, 353487665, '19580325 21:35:12.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6379, 825, 825, 1292814355, '19790928 00:01:55.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6380, 945, 945, 323646649, '19770921 19:16:35.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6381, 248, 248, 367575464, '19990709 23:48:40.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6382, 643, 643, 819767260, '19530820 23:23:39.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6383, 497, 497, 632243748, '19771030 08:51:43.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6384, 200, 200, 2072848038, '19651202 16:44:14.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6385, 276, 276, 1950549818, '19840128 23:07:17.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6386, 856, 856, 61041733, '20021103 16:33:08.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6387, 773, 773, 1511498224, '19811003 19:51:00.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6388, 224, 224, 1223447079, '19950628 11:19:30.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6389, 375, 375, 1070420249, '19670303 22:33:56.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6390, 517, 517, 1740896396, '20061030 18:23:02.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6391, 796, 796, 301884505, '20020920 02:59:11.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6392, 183, 183, 705526181, '19870131 10:09:13.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6393, 364, 364, 1651237628, '20041216 07:24:30.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6394, 940, 940, 964525037, '20001014 17:07:41.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6395, 572, 572, 1106672079, '19950613 08:50:47.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6396, 186, 186, 388210258, '19950719 15:49:14.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6397, 310, 310, 75871610, '19530914 00:49:29.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6398, 947, 947, 1477649771, '20030917 11:16:18.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6399, 464, 464, 1712672035, '19660415 06:43:31.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6400, 198, 198, 542662075, '19531116 21:11:34.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6401, 42, 42, 1613877803, '19800708 18:17:38.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6402, 180, 180, 1389169695, '19640120 10:27:07.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6403, 296, 296, 550206707, '19630216 22:41:55.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6404, 976, 976, 1980514759, '19780411 19:16:40.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6405, 698, 698, 1091214873, '19940717 02:01:04.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6406, 357, 357, 1337439405, '20011007 16:07:54.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6407, 516, 516, 2110536681, '19831031 03:51:41.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6408, 169, 169, 1458240335, '19781211 05:06:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6409, 184, 184, 1273437164, '19810812 06:32:30.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6410, 64, 64, 53364220, '20041026 10:41:47.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6411, 949, 949, 21698129, '20060222 15:06:43.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6412, 471, 471, 2037495409, '19890726 16:24:41.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6413, 882, 882, 1398107, '20021107 13:38:36.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6414, 124, 124, 1371121147, '20040222 17:41:54.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6415, 142, 142, 223424332, '19530128 14:59:36.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6416, 845, 845, 1575790553, '20010611 23:31:26.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6417, 900, 900, 1337320797, '20010113 17:03:19.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6418, 566, 566, 112218931, '19730402 20:31:18.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6419, 751, 751, 1119560547, '19870814 22:15:02.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6420, 337, 337, 2123082746, '19600210 03:02:31.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6421, 921, 921, 1930296338, '19860404 19:52:15.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6422, 675, 675, 271585493, '19821126 00:17:36.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6423, 419, 419, 766692321, '19881019 10:14:25.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6424, 748, 748, 1361154688, '19560422 08:21:30.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6425, 608, 608, 1525163942, '20070125 01:20:49.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6426, 132, 132, 1000782957, '19680328 05:06:39.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6427, 381, 381, 1010629718, '19650625 22:44:22.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6428, 414, 414, 55427731, '19820127 01:00:26.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6429, 40, 40, 557815517, '19771106 16:29:45.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6430, 318, 318, 1272055048, '19580607 20:53:47.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6431, 59, 59, 64732563, '19570311 11:25:42.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6432, 605, 605, 951697428, '19610110 01:02:51.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6433, 152, 152, 1224710136, '19991210 14:51:40.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6434, 181, 181, 1308318484, '19781112 02:50:21.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6435, 355, 355, 639344765, '20050315 00:58:56.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6436, 715, 715, 1372842863, '19880620 13:21:32.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6437, 243, 243, 1926724896, '19980704 00:41:44.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6438, 308, 308, 2116583405, '20070720 13:35:02.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6439, 333, 333, 1524804153, '19790518 14:43:09.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6440, 101, 101, 2146495056, '19900422 06:59:22.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6441, 502, 502, 1548563151, '19720105 12:41:26.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6442, 182, 182, 1297897300, '19551025 14:44:01.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6443, 224, 224, 1951575829, '19661117 04:46:59.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6444, 221, 221, 69538414, '19700507 11:52:08.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6445, 990, 990, 274033657, '19530908 03:16:19.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6446, 992, 992, 1997476592, '19660225 10:34:29.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6447, 106, 106, 27012698, '19920324 20:30:37.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6448, 207, 207, 1483323432, '19531025 13:42:36.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6449, 51, 51, 1162294151, '20000917 02:08:05.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6450, 365, 365, 1376428549, '20011231 09:19:21.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6451, 271, 271, 2100282916, '20020704 22:45:55.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6452, 418, 418, 832446795, '19880612 04:18:11.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6453, 454, 454, 240112856, '19690203 13:01:15.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6454, 83, 83, 1042635033, '19590307 03:41:00.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6455, 370, 370, 263674281, '19750814 01:19:18.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6456, 408, 408, 90403184, '20050812 18:00:02.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6457, 190, 190, 1084097376, '19830402 00:12:42.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6458, 256, 256, 1433963244, '19591028 14:20:25.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6459, 586, 586, 1080781688, '19790317 17:55:51.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6460, 205, 205, 1410071642, '19790421 20:54:24.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6461, 226, 226, 1413404130, '19890414 17:23:31.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6462, 270, 270, 1640289905, '19541005 02:51:15.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6463, 387, 387, 1242328140, '19540204 12:37:13.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6464, 933, 933, 1129149338, '19760307 09:38:43.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6465, 231, 231, 1339688774, '20000817 23:16:04.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6466, 749, 749, 320847180, '19980213 14:03:51.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6467, 999, 999, 1281153645, '19970816 01:33:00.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6468, 362, 362, 2005910340, '19761227 10:48:15.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6469, 914, 914, 619607791, '20061209 16:42:57.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6470, 393, 393, 2105183988, '19720809 01:38:03.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6471, 959, 959, 1734335723, '19551210 22:49:42.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6472, 70, 70, 1895569604, '20010626 03:28:16.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6473, 918, 918, 382474890, '20011115 05:48:08.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6474, 206, 206, 1047645613, '20050423 15:15:40.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6475, 100, 100, 2001560402, '19960204 14:21:04.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6476, 894, 894, 455422514, '19830612 05:08:13.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6477, 655, 655, 1399146333, '19690109 19:30:36.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6478, 71, 71, 302736695, '19710615 22:22:16.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6479, 760, 760, 1575834007, '19550816 05:05:11.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6480, 755, 755, 1337998364, '19931130 14:46:20.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6481, 409, 409, 2136640184, '19840103 08:36:02.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6482, 739, 739, 878363404, '19640901 09:01:46.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6483, 96, 96, 1571456126, '19890510 22:52:21.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6484, 453, 453, 1945899583, '19831106 07:10:24.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6485, 419, 419, 545081253, '19631203 22:07:51.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6486, 434, 434, 140553832, '19760928 07:07:31.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6487, 961, 961, 626985252, '19991206 12:01:35.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6488, 918, 918, 2019899195, '19931006 11:10:40.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6489, 94, 94, 2075820854, '19560401 01:31:02.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6490, 285, 285, 857733623, '20070802 06:58:54.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6491, 24, 24, 837188756, '19580322 13:08:49.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6492, 455, 455, 289133680, '19910907 10:21:18.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6493, 388, 388, 200836901, '19810505 19:39:11.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6494, 213, 213, 536076508, '19530127 17:48:45.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6495, 401, 401, 1338564317, '19540108 13:35:50.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6496, 99, 99, 2125314351, '19700403 09:50:11.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6497, 903, 903, 1674539726, '19540921 02:06:47.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6498, 94, 94, 1917855242, '19980910 19:08:16.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6499, 751, 751, 726242204, '19770918 12:36:03.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6500, 360, 360, 1198868566, '19630725 13:12:27.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6501, 260, 260, 1027992573, '19760711 09:01:28.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6502, 448, 448, 942760531, '19680329 09:42:22.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6503, 4, 4, 837186582, '19541115 20:59:38.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6504, 658, 658, 762661154, '19810209 00:30:01.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6505, 639, 639, 2044745379, '19990122 10:29:39.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6506, 901, 901, 725702924, '19540109 19:17:39.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6507, 555, 555, 881988305, '19940727 22:30:03.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6508, 82, 82, 739315201, '19711012 21:45:35.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6509, 394, 394, 1829039221, '19710130 13:19:56.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6510, 438, 438, 1667163859, '20000302 20:11:05.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6511, 481, 481, 1730087147, '19890803 22:29:26.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6512, 436, 436, 1930930908, '19641018 01:50:26.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6513, 442, 442, 2049951477, '19570315 09:15:14.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6514, 49, 49, 385070657, '19930416 03:09:31.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6515, 1000, 1000, 1880171917, '20030418 12:18:22.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6516, 672, 672, 550715350, '19770814 08:21:45.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6517, 413, 413, 1966063942, '19730526 03:58:53.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6518, 648, 648, 795069944, '19780331 05:26:48.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6519, 695, 695, 735215640, '19650405 11:30:51.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6520, 480, 480, 66609698, '19761120 16:08:33.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6521, 85, 85, 316023266, '20060310 02:02:16.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6522, 663, 663, 533269011, '19561110 14:22:20.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6523, 808, 808, 765988312, '19730928 09:17:12.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6524, 114, 114, 145381204, '20070509 09:44:03.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6525, 377, 377, 947705971, '19670521 12:29:13.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6526, 410, 410, 1586616432, '19720403 12:28:23.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6527, 522, 522, 1983304258, '19730221 07:49:05.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6528, 126, 126, 2062136442, '20011019 11:59:23.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6529, 151, 151, 1268121018, '20040416 01:39:49.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6530, 872, 872, 1002431432, '19781104 04:30:52.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6531, 151, 151, 1861404545, '19810922 03:01:25.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6532, 999, 999, 1089656375, '19780501 00:22:20.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6533, 520, 520, 622547436, '20010926 09:46:12.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6534, 415, 415, 333936971, '20000121 15:52:44.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6535, 474, 474, 338836468, '19700523 05:44:51.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6536, 57, 57, 818877123, '19681006 22:53:40.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6537, 659, 659, 1165784400, '19621018 12:02:07.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6538, 181, 181, 255061657, '19610401 22:22:19.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6539, 936, 936, 106049594, '19921013 20:40:45.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6540, 503, 503, 1712598295, '19621125 02:50:12.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6541, 688, 688, 41544941, '19900303 08:37:28.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6542, 531, 531, 2007741891, '19810224 14:16:00.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6543, 298, 298, 834889243, '19750907 21:50:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6544, 801, 801, 1810757597, '19640129 05:19:56.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6545, 493, 493, 1877840934, '19701230 08:29:28.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6546, 410, 410, 1009394406, '20010603 11:18:23.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6547, 550, 550, 1813261353, '19870626 01:06:04.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6548, 208, 208, 1830133784, '19960119 16:29:39.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6549, 136, 136, 300593881, '19571011 05:30:22.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6550, 891, 891, 2060632718, '19840526 05:14:04.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6551, 498, 498, 246617470, '20041107 05:51:18.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6552, 989, 989, 2073540785, '19840621 20:39:59.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6553, 102, 102, 4142166, '19860608 07:37:25.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6554, 696, 696, 1303765988, '19531029 20:27:29.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6555, 835, 835, 2025015148, '19650516 11:10:11.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6556, 512, 512, 1864859276, '19720831 18:39:08.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6557, 771, 771, 1766740309, '20020701 00:25:09.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6558, 439, 439, 1538768152, '19550709 11:15:34.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6559, 577, 577, 1538270152, '19660922 12:00:48.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6560, 112, 112, 19576213, '19791021 22:01:15.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6561, 290, 290, 961813258, '19881117 11:28:34.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6562, 912, 912, 257355674, '20001007 10:30:39.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6563, 544, 544, 1085094416, '19750627 20:05:46.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6564, 910, 910, 1352431594, '20000327 08:49:03.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6565, 465, 465, 1514200865, '19710612 17:09:39.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6566, 186, 186, 1241618536, '19640405 22:07:05.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6567, 293, 293, 494045268, '19710121 16:04:13.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6568, 493, 493, 344102080, '19650221 17:22:30.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6569, 525, 525, 972090287, '19751125 21:24:39.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6570, 10, 10, 1633591550, '19870713 13:24:41.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6571, 540, 540, 2118419213, '19671028 20:19:38.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6572, 523, 523, 755162872, '19650224 05:36:34.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6573, 191, 191, 123771606, '19930616 08:46:02.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6574, 610, 610, 612585435, '19550513 08:40:38.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6575, 220, 220, 955117414, '19940919 07:31:05.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6576, 425, 425, 31596343, '19780114 11:46:15.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6577, 943, 943, 1414514482, '19890304 16:31:35.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6578, 383, 383, 653119393, '19620901 03:57:16.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6579, 178, 178, 434768965, '19890511 01:14:35.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6580, 415, 415, 1112725073, '19731217 01:10:50.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6581, 375, 375, 784024462, '19921008 11:22:19.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6582, 336, 336, 1419736039, '19850201 15:19:25.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6583, 202, 202, 120506747, '19970109 06:29:00.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6584, 448, 448, 1269320661, '19721121 20:25:44.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6585, 285, 285, 67609800, '19811226 22:39:34.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6586, 919, 919, 415439312, '20050401 06:21:26.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6587, 382, 382, 2121582737, '19991122 16:44:54.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6588, 341, 341, 1353040913, '20001128 13:18:42.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6589, 240, 240, 848626374, '19870908 04:42:32.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6590, 573, 573, 1265072939, '20050920 11:05:43.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6591, 379, 379, 1860778910, '19740407 22:28:31.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6592, 747, 747, 1537251077, '19600412 00:46:32.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6593, 413, 413, 656397297, '20020710 10:21:31.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6594, 776, 776, 768018698, '19651224 22:39:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6595, 680, 680, 314316953, '19820106 15:21:38.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6596, 160, 160, 1062134982, '19970525 22:06:48.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6597, 862, 862, 229060847, '19580831 18:10:13.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6598, 870, 870, 1528211734, '19830612 13:01:40.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6599, 980, 980, 395431698, '20000817 12:10:17.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6600, 523, 523, 1950977718, '19590926 10:22:22.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6601, 214, 214, 906506813, '19830514 19:14:10.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6602, 700, 700, 1218841211, '19750505 07:43:34.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6603, 128, 128, 1485619049, '19730224 19:05:00.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6604, 664, 664, 808641787, '19780330 16:05:41.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6605, 395, 395, 1531098972, '19690730 08:54:26.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6606, 390, 390, 1132056327, '19920914 00:14:44.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6607, 538, 538, 1814650160, '19990105 11:53:56.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6608, 583, 583, 965630903, '19840127 22:56:46.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6609, 337, 337, 1331864896, '19670209 12:56:24.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6610, 590, 590, 925561206, '19950501 23:23:55.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6611, 222, 222, 161101455, '19910629 02:07:32.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6612, 966, 966, 172731859, '19921030 20:27:53.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6613, 963, 963, 197160211, '19740227 10:00:59.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6614, 553, 553, 190075048, '20001005 10:49:58.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6615, 127, 127, 144491571, '19950112 10:38:56.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6616, 334, 334, 1630539571, '19561016 20:09:09.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6617, 892, 892, 817578891, '19560929 04:01:39.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6618, 614, 614, 1628787382, '19550806 23:40:02.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6619, 294, 294, 1017641027, '19710402 10:09:29.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6620, 466, 466, 1041597920, '19740312 10:48:12.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6621, 652, 652, 336422449, '19720514 05:22:11.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6622, 602, 602, 614316831, '19780222 02:45:30.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6623, 116, 116, 1461637110, '19850615 12:39:57.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6624, 110, 110, 497754427, '19780306 10:18:56.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6625, 290, 290, 443409200, '19660407 14:56:10.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6626, 159, 159, 1008819832, '19600718 19:33:05.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6627, 574, 574, 1752356839, '19700816 19:15:02.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6628, 409, 409, 107291819, '19800623 10:06:32.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6629, 154, 154, 1026546585, '19810505 17:19:00.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6630, 837, 837, 1575079815, '20050425 09:24:17.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6631, 900, 900, 496127222, '19720516 17:36:02.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6632, 580, 580, 898455932, '19580924 03:29:55.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6633, 994, 994, 297541365, '19700113 15:46:58.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6634, 958, 958, 750588901, '19820816 14:52:13.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6635, 486, 486, 1982882594, '19980804 05:56:39.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6636, 441, 441, 1106158205, '19700607 03:38:44.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6637, 35, 35, 130170816, '19641208 20:14:25.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6638, 12, 12, 1360979939, '19711013 14:23:18.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6639, 529, 529, 2040597177, '19560425 11:31:17.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6640, 810, 810, 901714876, '19970412 22:24:50.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6641, 352, 352, 160838792, '19590328 18:27:29.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6642, 850, 850, 320935252, '19681026 17:48:07.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6643, 12, 12, 2038815683, '19861021 00:22:18.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6644, 897, 897, 1940930273, '20000508 05:18:37.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6645, 578, 578, 1640758036, '19670326 08:58:11.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6646, 399, 399, 1939120293, '19890607 02:00:38.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6647, 589, 589, 1805477130, '19940212 14:45:33.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6648, 479, 479, 547809627, '19541005 23:31:33.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6649, 293, 293, 974098910, '19781212 12:00:33.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6650, 245, 245, 1871451677, '19780323 19:12:28.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6651, 23, 23, 1256621712, '19930827 18:05:08.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6652, 17, 17, 454904298, '19560126 18:30:11.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6653, 514, 514, 740205473, '19650313 02:52:11.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6654, 473, 473, 167233968, '19790609 01:22:04.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6655, 894, 894, 434559710, '19950514 05:00:10.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6656, 597, 597, 2069066607, '19580323 16:28:39.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6657, 357, 357, 1560633855, '19971118 19:03:36.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6658, 505, 505, 916174409, '20021221 08:46:03.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6659, 596, 596, 797103904, '19641224 01:09:31.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6660, 133, 133, 463970736, '19620112 13:21:20.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6661, 90, 90, 207193761, '19750130 04:59:41.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6662, 315, 315, 1861651379, '19710716 01:47:24.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6663, 52, 52, 596099432, '19550925 05:37:46.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6664, 620, 620, 1514539940, '19691016 03:32:07.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6665, 686, 686, 407556551, '19751217 05:47:52.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6666, 504, 504, 1611810473, '19851005 22:37:36.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6667, 782, 782, 396868377, '19751019 04:40:54.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6668, 409, 409, 1261410736, '19990713 19:08:45.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6669, 141, 141, 189407220, '19550305 04:53:18.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6670, 397, 397, 1303502993, '19770808 07:00:11.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6671, 107, 107, 52305321, '19991107 23:54:43.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6672, 252, 252, 235697585, '19921216 13:38:20.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6673, 601, 601, 1892507932, '20070412 12:08:44.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6674, 288, 288, 2131515258, '19811219 11:44:39.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6675, 576, 576, 2025331514, '19671216 16:20:42.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6676, 412, 412, 568763035, '19560719 11:34:03.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6677, 74, 74, 651449216, '19990504 02:23:12.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6678, 706, 706, 64905638, '19930206 07:21:19.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6679, 506, 506, 493106103, '19631213 01:20:44.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6680, 600, 600, 2030894842, '19841030 12:55:31.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6681, 831, 831, 1672815924, '19740605 04:47:32.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6682, 287, 287, 105945900, '19681111 19:43:29.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6683, 280, 280, 1938153439, '19540713 22:05:58.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6684, 161, 161, 1030519682, '19560213 22:06:31.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6685, 174, 174, 2090603925, '19640829 15:19:08.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6686, 671, 671, 1411333341, '19690421 23:05:51.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6687, 738, 738, 1529710849, '20010713 00:37:44.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6688, 836, 836, 2121786895, '19980806 14:24:54.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6689, 547, 547, 1229107803, '19950404 09:52:02.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6690, 301, 301, 1497509240, '19930514 13:28:00.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6691, 338, 338, 422538869, '19800718 08:35:08.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6692, 234, 234, 2104049682, '19691218 11:28:21.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6693, 848, 848, 999357379, '19591021 13:13:47.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6694, 283, 283, 487500946, '20020412 22:09:56.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6695, 580, 580, 564709836, '19750315 17:25:08.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6696, 463, 463, 1036277963, '19950912 16:40:05.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6697, 725, 725, 565400033, '19660202 12:10:21.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6698, 462, 462, 1473611190, '19700105 20:50:11.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6699, 148, 148, 660846844, '19770523 23:30:44.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6700, 682, 682, 1521349803, '19890619 10:20:34.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6701, 765, 765, 971681082, '19660820 21:19:43.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6702, 420, 420, 415915985, '20020801 19:36:47.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6703, 617, 617, 1033585442, '20070802 18:38:14.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6704, 251, 251, 1105427340, '19540507 05:00:22.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6705, 967, 967, 172333142, '19860515 19:17:45.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6706, 593, 593, 965896309, '19530912 04:26:47.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6707, 512, 512, 1047463417, '19561015 06:21:26.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6708, 197, 197, 1194582479, '19910324 15:07:31.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6709, 236, 236, 1031955729, '19640624 09:01:21.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6710, 984, 984, 2132918084, '19911026 17:26:07.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6711, 141, 141, 1548034070, '19661119 14:05:27.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6712, 548, 548, 1681217343, '19571013 11:42:16.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6713, 563, 563, 497996024, '19920110 07:25:19.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6714, 289, 289, 1524026791, '19880223 12:13:59.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6715, 371, 371, 1358441900, '19950809 08:33:48.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6716, 521, 521, 1220445828, '19590321 09:16:11.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6717, 649, 649, 1175765629, '19700103 03:41:25.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6718, 14, 14, 843443056, '20070811 07:31:14.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6719, 385, 385, 1329509810, '19580217 12:12:58.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6720, 47, 47, 1322022816, '19590828 06:42:01.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6721, 502, 502, 1055013233, '19920324 21:59:52.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6722, 363, 363, 26505339, '19850313 14:16:00.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6723, 215, 215, 183388095, '19570411 09:16:42.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6724, 924, 924, 1481752197, '19691020 15:25:18.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6725, 210, 210, 1484360552, '20050119 22:25:03.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6726, 854, 854, 2055040102, '19821121 12:26:22.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6727, 925, 925, 280950903, '19860227 00:24:33.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6728, 988, 988, 1431424385, '19571231 13:58:47.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6729, 167, 167, 666209591, '19591007 18:19:30.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6730, 291, 291, 1910117881, '19800403 23:40:37.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6731, 536, 536, 903932937, '19980615 23:23:41.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6732, 653, 653, 546976913, '19820429 05:33:00.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6733, 889, 889, 771208627, '19680916 07:50:35.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6734, 619, 619, 2076463777, '19820624 15:00:10.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6735, 228, 228, 1468236567, '19710313 15:35:47.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6736, 124, 124, 570287710, '19791103 09:10:44.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6737, 644, 644, 642664266, '19690411 01:05:14.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6738, 667, 667, 278472904, '19530309 17:05:36.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6739, 571, 571, 1342307786, '19770930 02:48:46.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6740, 23, 23, 1275792778, '19631219 10:52:20.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6741, 721, 721, 1642776011, '19650708 16:44:24.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6742, 606, 606, 486109086, '19630422 09:30:12.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6743, 20, 20, 1191146067, '19870213 05:23:32.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6744, 431, 431, 341908473, '19560609 16:25:55.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6745, 518, 518, 765889954, '19790626 23:22:32.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6746, 268, 268, 771337029, '19751006 20:52:51.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6747, 658, 658, 144448230, '19851208 04:03:02.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6748, 997, 997, 1431965351, '19790828 00:59:47.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6749, 458, 458, 751989019, '19590905 13:36:08.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6750, 92, 92, 242738410, '19681226 08:08:56.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6751, 629, 629, 1151221619, '19780910 12:58:40.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6752, 431, 431, 510946462, '19660624 04:39:07.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6753, 434, 434, 1284676801, '19641119 08:37:12.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6754, 368, 368, 697541126, '19700927 16:48:42.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6755, 304, 304, 2059567023, '20050326 12:20:41.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6756, 732, 732, 1738917802, '19890608 07:03:43.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6757, 469, 469, 1510853847, '19980423 10:20:05.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6758, 69, 69, 1715303103, '19901014 03:12:56.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6759, 557, 557, 480456860, '19570416 06:35:18.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6760, 509, 509, 1832785562, '19560625 06:51:22.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6761, 428, 428, 1128458, '19750716 04:02:14.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6762, 958, 958, 947107894, '20061014 16:22:39.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6763, 226, 226, 1255982482, '19840617 20:48:00.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6764, 874, 874, 1641790145, '19920321 09:20:44.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6765, 754, 754, 1246662143, '20010511 18:02:25.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6766, 180, 180, 695978271, '19920722 08:32:26.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6767, 563, 563, 283322679, '19970126 03:51:41.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6768, 745, 745, 1565547902, '19620719 16:25:09.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6769, 691, 691, 1069800496, '19691213 20:38:11.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6770, 349, 349, 1506859656, '19681006 23:23:04.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6771, 449, 449, 1036774946, '19971209 08:10:56.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6772, 714, 714, 1630071783, '19691027 10:35:49.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6773, 219, 219, 1852606318, '19821112 04:42:27.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6774, 795, 795, 1326976632, '20020301 01:20:37.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6775, 218, 218, 530133930, '20020218 13:57:37.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6776, 379, 379, 681732248, '19811204 12:29:40.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6777, 868, 868, 588255236, '20060128 08:52:27.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6778, 56, 56, 912964527, '19531102 16:50:47.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6779, 96, 96, 1994374293, '19980425 16:02:45.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6780, 588, 588, 1891216263, '19820417 01:32:12.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6781, 262, 262, 2137133513, '20041213 08:23:20.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6782, 145, 145, 1162468576, '19590703 23:24:45.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6783, 304, 304, 1427886301, '20060427 00:22:39.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6784, 345, 345, 930567646, '19981011 10:10:33.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6785, 621, 621, 224715894, '19540726 10:41:57.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6786, 174, 174, 1746614519, '19841223 18:44:34.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6787, 644, 644, 550405732, '19700610 20:55:48.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6788, 417, 417, 1999882392, '20051219 15:01:52.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6789, 699, 699, 22021341, '19850330 02:39:28.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6790, 779, 779, 754668400, '19891004 08:32:04.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6791, 270, 270, 1707356119, '19890711 22:50:38.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6792, 412, 412, 1923525962, '19860627 21:52:18.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6793, 817, 817, 582035193, '20031123 10:30:17.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6794, 416, 416, 1759774521, '19740407 19:57:23.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6795, 67, 67, 2029108469, '19960604 06:50:06.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6796, 833, 833, 325122856, '19660924 00:21:45.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6797, 406, 406, 155814976, '19861103 12:42:11.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6798, 635, 635, 1717539837, '20021123 20:02:48.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6799, 858, 858, 692282299, '19630128 22:23:30.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6800, 661, 661, 1369283009, '19941204 14:50:06.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6801, 487, 487, 1373120942, '19860909 17:01:09.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6802, 763, 763, 186822519, '19760522 11:37:53.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6803, 610, 610, 1156020456, '19620915 04:21:35.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6804, 942, 942, 84295385, '19981128 14:13:03.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6805, 324, 324, 1058244600, '19790116 23:58:00.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6806, 424, 424, 1712676166, '19611115 02:22:50.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6807, 770, 770, 854215980, '19910811 23:56:39.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6808, 470, 470, 235786598, '19700920 07:16:34.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6809, 829, 829, 793233648, '19760808 02:51:05.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6810, 460, 460, 1757886012, '19760422 23:51:01.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6811, 803, 803, 2028849467, '19910510 22:11:15.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6812, 766, 766, 560656740, '19970621 17:29:18.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6813, 414, 414, 1444916348, '20000619 11:10:11.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6814, 557, 557, 1161182301, '19821230 05:03:06.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6815, 95, 95, 518202154, '19590712 22:28:02.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6816, 305, 305, 852774260, '19690113 04:37:28.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6817, 448, 448, 818592920, '19530620 15:59:59.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6818, 475, 475, 1587791891, '19930906 10:37:26.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6819, 625, 625, 981074392, '19900827 22:38:47.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6820, 698, 698, 1482974007, '19690519 23:17:50.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6821, 272, 272, 1096760055, '19750212 11:36:38.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6822, 901, 901, 1980767777, '19990206 12:49:48.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6823, 905, 905, 764041265, '19850420 13:45:40.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6824, 319, 319, 1393299864, '19880310 12:06:07.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6825, 167, 167, 1344818558, '19870328 00:32:26.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6826, 706, 706, 1847391860, '19640614 10:18:38.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6827, 46, 46, 1917560424, '19731204 00:05:31.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6828, 235, 235, 1403841980, '19610807 08:45:03.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6829, 827, 827, 1492923468, '19580927 18:30:31.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6830, 214, 214, 1200123525, '19880528 13:35:52.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6831, 742, 742, 1446685419, '20030130 23:47:28.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6832, 291, 291, 1951834706, '19560307 12:49:39.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6833, 802, 802, 1484866292, '19981004 18:27:16.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6834, 940, 940, 958948346, '19560523 01:12:39.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6835, 81, 81, 510004813, '20030807 08:31:02.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6836, 307, 307, 2138621170, '19810724 20:45:27.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6837, 823, 823, 1335467401, '20041017 19:03:09.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6838, 262, 262, 681338423, '19600530 10:09:36.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6839, 317, 317, 1534850518, '19720925 10:12:33.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6840, 322, 322, 1738174118, '20000909 08:19:06.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6841, 726, 726, 185769229, '20010513 18:47:54.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6842, 411, 411, 856772604, '20070920 13:39:13.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6843, 814, 814, 1245815967, '19820514 12:10:47.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6844, 601, 601, 1901164564, '19611208 02:48:27.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6845, 570, 570, 1924990938, '20060526 10:20:48.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6846, 594, 594, 1429649570, '20000120 05:21:20.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6847, 618, 618, 847895613, '19940107 10:42:07.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6848, 277, 277, 341621038, '19731125 05:27:10.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6849, 591, 591, 551950912, '19670927 21:29:21.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6850, 561, 561, 2059633792, '19800523 02:12:38.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6851, 130, 130, 809858714, '19660406 08:21:45.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6852, 51, 51, 723597239, '20010226 02:04:45.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6853, 665, 665, 1642597361, '19650329 21:24:00.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6854, 76, 76, 551223814, '20010909 23:05:39.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6855, 843, 843, 512283845, '19721023 03:13:28.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6856, 769, 769, 456122399, '19950803 04:11:20.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6857, 747, 747, 751645842, '19990131 21:52:12.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6858, 519, 519, 1208000874, '19820706 16:15:28.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6859, 403, 403, 789310559, '19640903 13:40:37.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6860, 952, 952, 2118924370, '19670805 13:39:21.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6861, 203, 203, 785987914, '19951212 02:17:19.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6862, 633, 633, 739330718, '19830105 15:11:35.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6863, 661, 661, 1854441733, '19641224 12:46:03.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6864, 840, 840, 1849917700, '19960311 13:15:12.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6865, 343, 343, 79299152, '19810323 00:03:32.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6866, 885, 885, 966567060, '19880305 09:21:35.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6867, 757, 757, 1351402148, '20060916 23:02:01.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6868, 347, 347, 271385170, '19970128 09:57:31.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6869, 88, 88, 2044639771, '19870524 20:32:04.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6870, 616, 616, 1574174720, '19851218 01:42:35.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6871, 788, 788, 12052207, '19720327 09:34:18.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6872, 104, 104, 186428628, '20010120 05:50:23.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6873, 139, 139, 108555650, '19731213 00:24:52.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6874, 496, 496, 142215001, '19971217 14:19:40.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6875, 384, 384, 747258512, '19760106 04:29:55.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6876, 192, 192, 762813515, '19750524 21:57:24.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6877, 753, 753, 281671459, '19690926 00:39:01.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6878, 832, 832, 2127995471, '19760514 10:57:13.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6879, 788, 788, 126771460, '19891015 01:45:19.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6880, 183, 183, 572489007, '19950307 19:11:05.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6881, 22, 22, 1142935720, '19780606 23:36:30.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6882, 39, 39, 1344037945, '19530109 18:38:20.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6883, 696, 696, 406040280, '20011109 11:15:42.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6884, 586, 586, 2099194498, '19860506 16:17:53.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6885, 734, 734, 668472888, '19750223 05:14:06.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6886, 244, 244, 1302228587, '19541205 21:42:43.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6887, 751, 751, 691628446, '19981208 15:25:38.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6888, 646, 646, 2060194523, '20050123 19:21:36.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6889, 768, 768, 2073676403, '19910730 21:59:11.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6890, 724, 724, 1236970711, '19610105 04:19:41.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6891, 939, 939, 2100955192, '19900622 22:53:14.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6892, 503, 503, 1621983552, '19750415 02:47:41.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6893, 457, 457, 513884480, '20030925 20:28:59.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6894, 615, 615, 2126752714, '19580221 20:33:11.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6895, 526, 526, 658087085, '19570930 06:01:47.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6896, 639, 639, 1880846233, '19710510 03:36:19.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6897, 392, 392, 1654989149, '19950928 00:53:09.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6898, 772, 772, 742489527, '19940303 22:55:35.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6899, 662, 662, 1439871473, '19880916 02:44:54.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6900, 647, 647, 1556154913, '19710324 00:59:13.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6901, 950, 950, 1248567545, '20010504 06:19:19.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6902, 30, 30, 313306488, '20041209 00:44:35.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6903, 1000, 1000, 1451732957, '19940703 08:55:06.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6904, 652, 652, 742868739, '19891009 19:46:46.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6905, 509, 509, 1172522424, '19610226 16:38:19.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6906, 222, 222, 1671668049, '19730316 02:31:22.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6907, 868, 868, 941499364, '19800317 01:39:52.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6908, 331, 331, 269614374, '19750412 07:04:19.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6909, 235, 235, 900448759, '19780905 18:35:44.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6910, 278, 278, 1278298631, '20050531 05:16:03.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6911, 665, 665, 2014494573, '19781108 03:32:19.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6912, 988, 988, 489725317, '19750920 10:54:59.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6913, 896, 896, 1868047163, '20000920 14:30:09.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6914, 689, 689, 305574378, '19770628 18:29:31.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6915, 736, 736, 1387225880, '19970227 14:02:51.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6916, 723, 723, 714801510, '19951203 07:38:58.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6917, 739, 739, 1923108856, '19890225 03:55:53.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6918, 539, 539, 594404207, '19860819 20:28:10.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6919, 781, 781, 2071261743, '19740118 08:01:05.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6920, 607, 607, 1083715463, '19790419 02:20:49.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6921, 859, 859, 1210798865, '19970327 17:55:56.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6922, 552, 552, 1241130070, '19540825 03:40:25.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6923, 162, 162, 319688554, '19580702 11:58:20.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6924, 923, 923, 1616925965, '19790520 16:12:22.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6925, 659, 659, 1061293547, '20030627 02:49:20.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6926, 450, 450, 1240603582, '20041212 06:46:36.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6927, 383, 383, 2065825884, '20050426 09:21:24.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6928, 348, 348, 1251895254, '19681022 03:51:41.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6929, 476, 476, 1495322720, '19930320 08:17:53.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6930, 825, 825, 1151993694, '19570830 00:53:36.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6931, 689, 689, 1414571563, '19870826 21:04:15.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6932, 562, 562, 166150045, '19830725 01:43:26.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6933, 407, 407, 29756626, '19950828 08:12:18.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6934, 895, 895, 1987048681, '19710726 00:46:05.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6935, 738, 738, 632248137, '20011102 12:51:46.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6936, 801, 801, 1608686018, '19810627 22:51:55.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6937, 820, 820, 2019841356, '19660709 09:43:14.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6938, 258, 258, 1136456198, '19650131 15:28:56.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6939, 420, 420, 481846243, '19780310 23:39:33.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6940, 877, 877, 627022004, '19541211 02:42:44.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6941, 97, 97, 1157359668, '19820919 20:02:51.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6942, 18, 18, 446910624, '19760829 08:21:19.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6943, 873, 873, 1553177942, '19790522 00:45:52.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6944, 477, 477, 2032435122, '19940228 16:43:08.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6945, 571, 571, 60088899, '19900227 00:47:22.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6946, 841, 841, 399964928, '19671004 11:58:15.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6947, 332, 332, 402567352, '19820724 12:17:32.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6948, 13, 13, 889969582, '19790331 01:59:27.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6949, 784, 784, 1914898, '19681224 06:30:20.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6950, 435, 435, 1189969503, '19691027 22:22:48.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6951, 850, 850, 795134513, '19900313 23:40:26.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6952, 389, 389, 781077663, '19620209 04:24:58.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6953, 447, 447, 1485069460, '19730214 14:54:29.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6954, 65, 65, 462301231, '19620601 00:24:05.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6955, 832, 832, 1293669121, '19811226 07:03:16.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6956, 538, 538, 2044871786, '19990911 02:38:54.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6957, 193, 193, 1260426075, '19990610 12:46:14.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6958, 707, 707, 1815054949, '19680214 16:42:44.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6959, 274, 274, 935237589, '19940414 16:57:25.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6960, 247, 247, 1026953393, '19640316 09:51:42.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6961, 61, 61, 1983422863, '19751119 17:09:58.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6962, 912, 912, 1148008650, '19640526 21:48:10.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6963, 632, 632, 965007236, '19900121 22:46:26.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6964, 978, 978, 173565923, '19740107 17:42:07.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6965, 793, 793, 503817415, '19701006 08:11:47.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6966, 467, 467, 1110732591, '20030416 01:48:53.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6967, 360, 360, 1455059597, '19880123 02:42:40.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6968, 306, 306, 1471267508, '19820227 13:44:03.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6969, 628, 628, 1888829085, '19830826 05:37:45.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6970, 748, 748, 1044115409, '19680902 15:10:55.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6971, 430, 430, 736298580, '19820527 21:55:44.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6972, 883, 883, 1420539885, '19770125 12:26:56.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6973, 708, 708, 1166407582, '19610611 20:48:37.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6974, 805, 805, 896310226, '19720209 05:18:21.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6975, 312, 312, 1664955015, '20040729 06:17:58.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6976, 612, 612, 766782624, '19730730 09:34:37.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6977, 975, 975, 1741858916, '19830405 02:54:32.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6978, 111, 111, 1197223362, '19720502 19:15:12.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6979, 329, 329, 1528481211, '19970322 15:25:00.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6980, 761, 761, 878797793, '19880923 14:51:05.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6981, 351, 351, 1990302783, '19750522 18:29:04.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6982, 269, 269, 1815946650, '19790128 07:21:56.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6983, 188, 188, 1179828617, '20071028 21:21:20.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6984, 439, 439, 457282330, '19760524 09:55:04.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6985, 98, 98, 995406808, '19750618 01:13:09.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6986, 716, 716, 508267119, '19780718 16:39:16.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6987, 688, 688, 1510753138, '19630610 10:48:56.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6988, 131, 131, 1905141765, '19860330 07:48:13.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6989, 32, 32, 216563, '19970729 17:42:49.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6990, 988, 988, 1432515853, '19550223 10:12:52.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6991, 260, 260, 2140880395, '19900117 10:05:40.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6992, 946, 946, 977639975, '20060524 17:00:29.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6993, 774, 774, 1457775390, '19781018 22:31:30.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6994, 28, 28, 791127034, '19661226 13:47:51.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6995, 726, 726, 1437559348, '19870121 09:32:46.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6996, 484, 484, 1247355891, '19710426 22:14:40.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6997, 964, 964, 1201950373, '19940807 09:34:52.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6998, 154, 154, 2041105594, '19580515 07:03:44.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (6999, 398, 398, 2125584708, '19760526 08:31:20.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7000, 359, 359, 1838230393, '19941112 22:58:28.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7001, 595, 595, 1207023782, '19870911 09:15:34.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7002, 417, 417, 956763067, '19530527 22:33:29.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7003, 685, 685, 1458961044, '20030804 20:21:41.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7004, 489, 489, 1241919706, '19530423 15:19:24.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7005, 863, 863, 810273622, '19950602 00:27:04.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7006, 507, 507, 67757622, '19660217 11:13:29.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7007, 454, 454, 1629933990, '19530831 07:36:21.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7008, 653, 653, 826639280, '19540107 09:36:08.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7009, 423, 423, 1975507352, '19651101 18:06:06.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7010, 817, 817, 1292384614, '19610529 21:28:38.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7011, 775, 775, 262216708, '19690608 23:44:21.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7012, 780, 780, 664432335, '19800208 17:31:02.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7013, 936, 936, 2086829251, '19781124 01:17:44.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7014, 687, 687, 312856896, '19580723 02:06:20.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7015, 302, 302, 1490467225, '19961024 15:22:37.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7016, 380, 380, 862467731, '20041021 09:48:01.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7017, 208, 208, 487055280, '19640729 00:26:50.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7018, 947, 947, 774288822, '19660831 12:51:21.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7019, 100, 100, 1661625196, '20060722 16:28:57.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7020, 21, 21, 765988735, '20000319 15:32:30.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7021, 885, 885, 1411107335, '19921106 14:59:56.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7022, 517, 517, 977018569, '19541026 18:54:27.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7023, 309, 309, 540484328, '19920730 20:01:13.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7024, 339, 339, 656071099, '19810703 19:24:53.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7025, 712, 712, 457975419, '19860816 05:05:15.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7026, 461, 461, 1659035809, '19840103 04:55:14.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7027, 559, 559, 1672962562, '20060408 13:55:26.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7028, 664, 664, 739257436, '20020616 07:00:45.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7029, 511, 511, 1142478744, '19930117 19:45:34.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7030, 24, 24, 534833906, '19860405 08:03:18.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7031, 958, 958, 1878206266, '19861222 23:59:44.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7032, 565, 565, 58955656, '19771121 19:50:47.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7033, 652, 652, 1870315942, '20031207 10:43:52.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7034, 32, 32, 409380443, '19550511 16:26:31.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7035, 863, 863, 1610239717, '19540114 05:35:30.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7036, 178, 178, 376184541, '19741226 19:55:35.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7037, 401, 401, 258520569, '19830625 11:00:15.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7038, 597, 597, 1410661534, '20070708 06:01:55.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7039, 429, 429, 914197039, '19881222 09:28:00.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7040, 280, 280, 1227061296, '19620501 13:59:40.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7041, 20, 20, 826058909, '19771118 09:02:55.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7042, 396, 396, 1105859611, '19620604 01:12:48.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7043, 373, 373, 167211734, '19730529 13:42:06.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7044, 425, 425, 343559006, '19890302 20:14:11.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7045, 785, 785, 2099505611, '19930915 10:28:30.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7046, 752, 752, 553386581, '19621210 16:34:38.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7047, 465, 465, 1226424058, '19800701 15:42:45.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7048, 891, 891, 1235445225, '19730329 20:25:10.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7049, 360, 360, 2131284349, '19780302 22:25:13.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7050, 251, 251, 1780620667, '19900331 23:44:45.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7051, 303, 303, 1479430236, '19590929 21:47:49.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7052, 689, 689, 1976985233, '19801207 20:43:31.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7053, 113, 113, 1131849480, '19591023 14:34:46.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7054, 968, 968, 886352712, '19840307 16:58:49.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7055, 539, 539, 1900767283, '19550107 07:58:31.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7056, 787, 787, 1864952784, '19851116 14:21:07.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7057, 507, 507, 1424565963, '19681027 02:32:15.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7058, 266, 266, 527817331, '19750202 00:56:48.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7059, 622, 622, 1635659552, '19740907 10:14:09.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7060, 90, 90, 1061144138, '19640530 19:31:50.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7061, 634, 634, 1053602170, '19671110 21:18:03.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7062, 140, 140, 756077972, '19590317 00:35:35.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7063, 881, 881, 862344862, '19681219 13:50:33.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7064, 75, 75, 1131234965, '19870729 10:02:48.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7065, 731, 731, 909228455, '19820606 21:28:53.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7066, 283, 283, 170862455, '19990718 03:53:33.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7067, 121, 121, 1420203153, '19840303 06:47:09.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7068, 241, 241, 1684832870, '19760716 08:51:12.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7069, 837, 837, 1639125686, '19570709 20:30:50.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7070, 768, 768, 397047194, '19741030 19:27:02.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7071, 950, 950, 1168758262, '19740429 22:47:45.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7072, 321, 321, 1816941384, '19650120 18:24:54.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7073, 113, 113, 1778711768, '19850908 03:23:20.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7074, 840, 840, 267084379, '19970324 02:29:17.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7075, 294, 294, 1115704837, '19750503 06:29:35.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7076, 275, 275, 1328895915, '19830606 13:47:08.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7077, 803, 803, 1312481441, '19890529 05:12:20.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7078, 705, 705, 237204627, '19560530 23:47:02.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7079, 964, 964, 169678942, '19951018 08:56:22.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7080, 790, 790, 1820100886, '19760906 12:30:36.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7081, 60, 60, 1391367290, '19560704 13:12:29.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7082, 628, 628, 965251223, '19860110 17:30:15.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7083, 264, 264, 1766150421, '19770416 08:35:32.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7084, 895, 895, 257322757, '19551020 20:00:48.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7085, 651, 651, 2132262882, '19790708 10:15:28.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7086, 181, 181, 1991678779, '19590116 03:16:12.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7087, 45, 45, 396710746, '19710131 05:16:01.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7088, 824, 824, 654589693, '19721001 17:45:02.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7089, 604, 604, 1572162271, '19530505 08:28:00.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7090, 458, 458, 1556572516, '19760227 15:14:22.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7091, 556, 556, 1954643726, '19590302 17:23:19.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7092, 933, 933, 888371088, '19610524 10:03:27.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7093, 924, 924, 1412177447, '19851031 19:47:45.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7094, 320, 320, 330532137, '19770725 13:56:09.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7095, 740, 740, 1681330899, '20020620 16:41:36.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7096, 601, 601, 804364875, '19710905 08:27:19.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7097, 657, 657, 598255474, '20010614 11:22:15.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7098, 273, 273, 1287527837, '19931031 03:39:18.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7099, 831, 831, 1422430072, '19590927 22:45:17.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7100, 730, 730, 188190951, '20020228 06:34:57.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7101, 110, 110, 385202394, '19861008 09:47:28.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7102, 669, 669, 1719516366, '19561217 06:51:32.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7103, 611, 611, 2055412924, '19680920 23:54:19.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7104, 823, 823, 1174269390, '19560504 02:58:10.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7105, 736, 736, 1921041793, '19681203 00:56:59.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7106, 692, 692, 260610218, '20020908 03:22:55.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7107, 80, 80, 767387920, '20030331 17:20:10.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7108, 962, 962, 1784613704, '19700731 12:05:28.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7109, 54, 54, 1922168275, '19611106 10:29:14.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7110, 219, 219, 2042483768, '19790802 18:11:23.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7111, 753, 753, 1213535617, '20040619 09:08:47.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7112, 624, 624, 627149382, '19650530 02:45:13.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7113, 48, 48, 896892285, '19870416 16:00:25.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7114, 354, 354, 2085811439, '20051230 21:43:33.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7115, 88, 88, 732599808, '19601127 06:19:20.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7116, 811, 811, 785727444, '19891030 03:47:47.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7117, 752, 752, 1072104666, '19891129 16:00:03.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7118, 32, 32, 908359137, '19660302 17:49:44.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7119, 195, 195, 145093353, '19610121 23:47:20.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7120, 894, 894, 1578201507, '19760522 18:12:41.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7121, 874, 874, 1048948666, '19810617 22:37:32.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7122, 366, 366, 693548915, '19640602 13:02:07.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7123, 541, 541, 1707632861, '19760314 00:23:12.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7124, 394, 394, 1617329010, '19890512 05:16:27.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7125, 120, 120, 1595274437, '19680831 02:03:43.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7126, 551, 551, 1434018271, '19651207 12:44:18.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7127, 728, 728, 1688505520, '19870321 22:37:08.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7128, 398, 398, 317827328, '19610214 13:27:11.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7129, 792, 792, 1453367134, '20021003 09:47:41.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7130, 530, 530, 581113010, '19560828 22:02:15.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7131, 713, 713, 96987840, '19891222 02:24:52.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7132, 816, 816, 1490153221, '20030729 01:33:02.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7133, 155, 155, 88179811, '20040902 01:01:45.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7134, 938, 938, 1058504130, '20010819 02:21:25.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7135, 57, 57, 272945738, '19971129 02:43:07.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7136, 794, 794, 2109521279, '20070718 06:20:57.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7137, 380, 380, 1339125603, '19700422 17:35:56.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7138, 353, 353, 531447108, '19731214 05:37:21.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7139, 991, 991, 1356216927, '19941116 19:03:49.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7140, 884, 884, 1022392621, '19841029 06:52:32.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7141, 112, 112, 1671892326, '19631017 09:56:01.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7142, 165, 165, 2144319440, '19530703 17:10:32.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7143, 422, 422, 2071398254, '19631127 07:15:47.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7144, 28, 28, 1521376582, '19811002 14:16:36.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7145, 746, 746, 95050266, '19790909 06:05:35.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7146, 748, 748, 324941159, '20011002 14:38:05.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7147, 504, 504, 976736428, '19820206 18:03:01.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7148, 456, 456, 479658310, '19871031 22:04:12.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7149, 935, 935, 430755123, '19690829 07:36:49.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7150, 381, 381, 577807503, '19650822 12:53:48.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7151, 432, 432, 836477419, '19891005 16:27:16.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7152, 866, 866, 217746683, '19880414 17:32:30.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7153, 237, 237, 638937719, '19851010 03:51:57.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7154, 125, 125, 1460944652, '19910507 04:32:35.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7155, 81, 81, 1923651932, '19730913 07:57:24.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7156, 630, 630, 256603686, '19750508 20:45:21.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7157, 576, 576, 90814900, '19881004 06:28:19.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7158, 707, 707, 1389266556, '19870511 18:37:52.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7159, 598, 598, 414676632, '19950903 00:54:27.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7160, 750, 750, 434547514, '19551228 12:12:41.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7161, 31, 31, 1450582040, '19680620 04:45:46.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7162, 13, 13, 259460057, '19950215 03:52:59.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7163, 428, 428, 92612934, '19751028 02:17:47.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7164, 508, 508, 1235528671, '19580311 12:26:59.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7165, 7, 7, 122437241, '19970810 15:46:30.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7166, 950, 950, 488849179, '19531123 07:35:45.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7167, 711, 711, 457096387, '19680926 01:43:27.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7168, 47, 47, 1834566684, '19930827 13:39:00.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7169, 757, 757, 1720508721, '19610201 19:00:25.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7170, 352, 352, 490503823, '19610511 23:58:23.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7171, 102, 102, 1395708589, '19720710 10:11:51.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7172, 637, 637, 2054489521, '19681217 10:22:42.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7173, 545, 545, 677626867, '19790415 22:45:55.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7174, 14, 14, 14812754, '19840324 16:54:47.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7175, 168, 168, 1422743362, '19650807 08:16:39.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7176, 350, 350, 216431193, '19801216 05:27:00.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7177, 856, 856, 822110316, '19530708 05:46:19.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7178, 729, 729, 567424152, '20020611 10:06:36.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7179, 562, 562, 989926078, '19620903 23:10:52.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7180, 20, 20, 42042180, '19741128 11:25:37.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7181, 545, 545, 2087181003, '19911030 18:41:17.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7182, 148, 148, 808187157, '20070521 00:32:56.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7183, 804, 804, 1056857492, '19990618 05:50:21.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7184, 241, 241, 275858057, '19900211 20:53:52.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7185, 55, 55, 192543487, '19741123 05:34:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7186, 925, 925, 2048450520, '19540909 08:52:22.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7187, 833, 833, 2036356765, '19701019 21:41:04.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7188, 445, 445, 2101909298, '19660429 20:29:09.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7189, 584, 584, 648684161, '19801206 18:24:01.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7190, 815, 815, 1338911684, '19750725 05:57:46.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7191, 254, 254, 1156219865, '19711014 23:52:38.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7192, 638, 638, 1399339616, '19901211 22:58:04.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7193, 128, 128, 1389833735, '19860412 04:42:54.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7194, 703, 703, 859982219, '19911121 06:51:07.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7195, 648, 648, 1883099115, '19690512 02:06:45.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7196, 830, 830, 383762754, '19760831 06:03:02.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7197, 410, 410, 1247274912, '19850905 14:52:44.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7198, 103, 103, 117266953, '19580918 18:48:48.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7199, 179, 179, 1034779156, '19920221 22:30:05.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7200, 949, 949, 1433772075, '19781017 22:29:49.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7201, 955, 955, 1757613168, '19860107 12:54:38.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7202, 961, 961, 1868751159, '19960611 04:24:01.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7203, 733, 733, 928358952, '19790930 03:03:46.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7204, 140, 140, 791739358, '19610420 07:38:25.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7205, 570, 570, 1338858075, '20010211 02:41:57.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7206, 876, 876, 519856793, '19731020 06:04:33.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7207, 143, 143, 2077024111, '19611231 18:46:35.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7208, 433, 433, 789081048, '19540719 10:57:09.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7209, 250, 250, 612994295, '19780930 20:15:56.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7210, 875, 875, 639584345, '20000927 02:30:24.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7211, 951, 951, 1211729245, '19741101 14:59:01.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7212, 549, 549, 68363507, '19940424 20:21:42.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7213, 544, 544, 74507326, '19770907 19:26:59.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7214, 847, 847, 1427783017, '19731006 13:28:49.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7215, 117, 117, 2007772684, '19720227 17:31:24.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7216, 643, 643, 1040381608, '19690130 04:12:50.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7217, 611, 611, 996404892, '20030829 22:02:37.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7218, 244, 244, 110755741, '19930914 05:23:55.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7219, 852, 852, 1985537137, '19910418 06:51:59.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7220, 685, 685, 1833313791, '19951203 06:54:08.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7221, 547, 547, 1205756418, '19910203 09:54:40.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7222, 173, 173, 941754542, '19550531 05:14:18.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7223, 760, 760, 1375480785, '19650921 19:14:59.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7224, 850, 850, 551225713, '19930710 13:02:39.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7225, 6, 6, 68561592, '19970728 00:05:45.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7226, 956, 956, 925126082, '19890729 11:13:47.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7227, 723, 723, 203659170, '19900906 05:39:47.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7228, 541, 541, 2053864155, '19950525 15:54:47.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7229, 27, 27, 687443031, '19671114 14:48:02.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7230, 585, 585, 1955520771, '19961206 02:13:36.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7231, 928, 928, 2123178801, '20030411 14:34:15.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7232, 145, 145, 425421435, '20021019 10:57:30.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7233, 68, 68, 673107249, '19630421 11:36:30.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7234, 605, 605, 430888368, '19911117 00:41:02.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7235, 78, 78, 1643081606, '19961119 22:30:59.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7236, 931, 931, 531418957, '20030520 14:17:15.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7237, 169, 169, 325230316, '19800821 21:29:09.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7238, 844, 844, 529147324, '19910228 22:11:56.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7239, 534, 534, 804948872, '19961230 18:11:55.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7240, 537, 537, 1871319030, '19540203 23:30:25.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7241, 987, 987, 1073605810, '20000907 14:05:46.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7242, 911, 911, 258664472, '19690403 10:43:55.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7243, 225, 225, 1343930288, '19950730 00:13:12.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7244, 880, 880, 675523297, '19880310 15:53:36.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7245, 93, 93, 1388163247, '19530923 14:58:44.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7246, 913, 913, 621756034, '19850621 03:30:55.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7247, 870, 870, 289710951, '19660406 03:31:04.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7248, 640, 640, 1063081412, '19650707 15:14:04.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7249, 826, 826, 1204313439, '19720924 13:19:43.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7250, 567, 567, 25272070, '19530412 21:53:55.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7251, 168, 168, 673064771, '19810101 08:00:24.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7252, 172, 172, 1293092913, '19991222 09:28:49.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7253, 635, 635, 1107517473, '19750602 11:56:48.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7254, 60, 60, 2077334980, '20040320 15:35:57.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7255, 82, 82, 861173341, '19950913 12:35:09.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7256, 300, 300, 1088039983, '19830811 07:40:19.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7257, 189, 189, 2119341432, '19830921 09:09:02.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7258, 176, 176, 377405265, '19940321 14:01:07.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7259, 110, 110, 1941634827, '19710923 07:32:40.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7260, 17, 17, 1723607229, '19640716 15:28:09.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7261, 927, 927, 2108590796, '19910213 00:24:46.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7262, 244, 244, 605682890, '19740808 02:51:47.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7263, 298, 298, 1141874052, '19940903 20:09:07.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7264, 950, 950, 915351339, '19891024 18:02:19.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7265, 810, 810, 1510883325, '20050617 11:56:08.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7266, 468, 468, 1376007449, '19800113 04:01:30.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7267, 105, 105, 1729029707, '19840105 08:45:12.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7268, 265, 265, 444973061, '19931021 18:45:56.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7269, 205, 205, 1493482879, '19841115 14:57:49.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7270, 443, 443, 1397898519, '19761009 03:14:08.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7271, 198, 198, 574853957, '19960609 06:43:40.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7272, 949, 949, 892394531, '19650701 23:50:41.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7273, 101, 101, 408145012, '20040913 11:11:59.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7274, 480, 480, 1244244360, '19900315 07:21:34.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7275, 384, 384, 633237161, '20030326 16:48:21.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7276, 963, 963, 271365114, '19741102 23:10:45.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7277, 939, 939, 1783293476, '19671101 05:01:05.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7278, 905, 905, 2128789785, '19850713 03:21:22.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7279, 156, 156, 1419460468, '19921017 22:03:54.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7280, 183, 183, 1490671809, '19650206 20:34:49.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7281, 793, 793, 1195287547, '19760425 07:42:42.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7282, 2, 2, 14860761, '19780302 14:25:42.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7283, 144, 144, 828722360, '19750901 02:35:04.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7284, 98, 98, 899004904, '19670804 16:54:06.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7285, 578, 578, 776435627, '19681205 18:13:12.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7286, 218, 218, 2138087424, '19560421 05:05:25.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7287, 656, 656, 2113714994, '19800519 23:00:42.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7288, 233, 233, 526992490, '19670130 20:00:33.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7289, 25, 25, 1006151435, '20040304 12:05:53.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7290, 683, 683, 776151999, '19660412 14:50:39.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7291, 157, 157, 1303201830, '19720830 05:08:13.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7292, 137, 137, 828834374, '19940602 07:28:02.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7293, 395, 395, 1293173549, '19720608 14:39:15.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7294, 34, 34, 726433935, '19850617 02:43:45.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7295, 23, 23, 199884041, '19701221 23:05:39.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7296, 224, 224, 2061865263, '19790201 11:13:59.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7297, 984, 984, 2060568800, '19820730 14:34:47.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7298, 174, 174, 131293467, '19581006 06:10:53.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7299, 672, 672, 2138282196, '19900923 03:57:28.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7300, 159, 159, 1669600519, '19810910 10:57:13.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7301, 297, 297, 1594707715, '19540617 18:45:43.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7302, 623, 623, 1708133365, '19800614 08:45:08.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7303, 535, 535, 390288197, '19880821 00:16:15.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7304, 271, 271, 126047773, '20031216 10:05:34.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7305, 166, 166, 1603566869, '19641102 15:10:15.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7306, 783, 783, 989592840, '19680703 08:09:42.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7307, 773, 773, 1106058277, '20030408 22:16:49.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7308, 651, 651, 595646900, '19930319 04:35:14.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7309, 358, 358, 1147015419, '19531226 22:47:35.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7310, 228, 228, 1028671678, '19731111 13:24:23.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7311, 826, 826, 1831356202, '19681011 02:39:14.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7312, 668, 668, 1464061554, '20060310 05:47:39.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7313, 845, 845, 551887140, '19540604 15:57:13.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7314, 998, 998, 448949110, '19590817 10:57:50.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7315, 246, 246, 282835205, '19960322 07:45:26.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7316, 257, 257, 2141854461, '19651214 09:59:03.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7317, 251, 251, 1178805726, '20061207 00:16:42.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7318, 971, 971, 2072688998, '19800201 03:15:00.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7319, 256, 256, 333956970, '19731119 23:49:07.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7320, 211, 211, 1227114119, '20020227 06:50:42.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7321, 831, 831, 485611997, '20070825 05:00:48.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7322, 327, 327, 951111539, '19691206 12:44:38.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7323, 82, 82, 680861416, '19970818 06:40:02.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7324, 423, 423, 1016223501, '19710807 00:07:09.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7325, 762, 762, 1285990490, '19570209 22:05:55.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7326, 223, 223, 1181175047, '19550108 23:15:38.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7327, 478, 478, 2019258923, '20010122 09:11:26.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7328, 766, 766, 123198010, '19720330 08:28:14.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7329, 374, 374, 1737230765, '19720325 08:15:55.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7330, 655, 655, 1629897328, '19770223 05:34:22.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7331, 279, 279, 360293869, '20000404 08:35:57.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7332, 902, 902, 740968114, '19620126 22:50:12.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7333, 134, 134, 1071975862, '20021019 23:23:54.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7334, 631, 631, 460267856, '19640208 11:06:40.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7335, 236, 236, 1119295215, '19630824 01:49:06.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7336, 888, 888, 1228809948, '20031110 22:57:34.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7337, 199, 199, 1005058980, '19970711 14:09:27.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7338, 735, 735, 1342291231, '19790915 16:29:30.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7339, 322, 322, 600348693, '19551003 01:43:51.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7340, 763, 763, 1574880046, '19530605 10:03:29.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7341, 313, 313, 1990734325, '19610112 06:48:36.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7342, 542, 542, 936360909, '19950302 18:25:27.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7343, 94, 94, 815132534, '19660204 21:12:58.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7344, 525, 525, 794772132, '19830424 22:41:29.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7345, 957, 957, 1933855115, '20050701 12:11:25.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7346, 842, 842, 1733387865, '19740621 23:20:33.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7347, 522, 522, 315516418, '19921229 15:30:48.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7348, 928, 928, 1388968576, '19651024 03:41:24.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7349, 776, 776, 2112079323, '19970326 18:58:18.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7350, 174, 174, 1949696870, '19580108 13:06:35.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7351, 376, 376, 343888397, '19800225 10:57:16.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7352, 399, 399, 2044256665, '19550630 11:19:47.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7353, 313, 313, 107472745, '19921115 06:21:45.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7354, 817, 817, 1847544213, '19960725 21:06:45.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7355, 131, 131, 529369946, '19820116 05:56:24.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7356, 794, 794, 548450651, '19920711 06:01:04.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7357, 444, 444, 760765310, '19901027 02:05:05.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7358, 869, 869, 134307875, '19700116 00:09:05.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7359, 554, 554, 1534425378, '19991106 11:59:39.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7360, 136, 136, 2052656890, '19621026 15:54:36.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7361, 998, 998, 345565675, '19750610 22:58:16.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7362, 516, 516, 1656513353, '19840110 13:48:35.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7363, 307, 307, 1716539371, '19731225 20:19:18.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7364, 734, 734, 925590895, '19841101 17:13:13.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7365, 957, 957, 225089490, '19810808 04:48:26.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7366, 555, 555, 1569391141, '19590916 03:49:02.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7367, 189, 189, 289614498, '19560522 06:23:45.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7368, 247, 247, 1669899335, '19980426 04:50:33.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7369, 215, 215, 1859435110, '20031224 09:08:44.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7370, 619, 619, 1924525809, '20000511 08:47:52.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7371, 330, 330, 1599308663, '19760603 19:49:35.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7372, 84, 84, 1968996213, '19800324 07:47:12.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7373, 347, 347, 525028297, '19770502 01:31:09.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7374, 143, 143, 387306395, '19730617 13:45:37.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7375, 125, 125, 495004453, '19940216 00:02:06.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7376, 104, 104, 470618197, '19650625 10:35:21.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7377, 204, 204, 1127416332, '19561101 15:31:40.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7378, 113, 113, 1878783891, '19670427 07:58:32.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7379, 253, 253, 1253087443, '19740204 11:55:44.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7380, 510, 510, 659250644, '19900820 22:45:21.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7381, 716, 716, 947094038, '19700109 07:44:50.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7382, 568, 568, 1080469328, '19880401 05:30:02.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7383, 21, 21, 5350498, '19830103 13:29:55.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7384, 999, 999, 706978930, '19670318 19:09:19.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7385, 317, 317, 292271120, '20041228 18:37:05.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7386, 310, 310, 754711636, '19971006 21:16:10.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7387, 261, 261, 1959903956, '19770312 16:28:26.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7388, 350, 350, 749428283, '19590328 02:17:09.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7389, 781, 781, 1499818783, '19900122 05:10:15.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7390, 325, 325, 503423006, '19790211 19:48:02.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7391, 373, 373, 457037419, '19660115 20:52:29.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7392, 59, 59, 1192999626, '19800626 14:00:22.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7393, 37, 37, 1822844072, '19871109 04:29:50.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7394, 360, 360, 612870954, '20001206 09:49:15.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7395, 495, 495, 763380236, '19851225 11:05:13.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7396, 124, 124, 24470267, '19850101 17:00:00.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7397, 83, 83, 260657165, '19740308 22:06:08.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7398, 709, 709, 2004646437, '19890404 03:59:45.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7399, 908, 908, 1564345996, '19540917 17:53:03.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7400, 641, 641, 1448906795, '19981017 08:22:22.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7401, 186, 186, 1289476577, '19710131 16:56:48.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7402, 249, 249, 1820351958, '20020904 10:40:15.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7403, 158, 158, 34785084, '19691030 18:32:40.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7404, 418, 418, 511028925, '20041114 10:10:26.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7405, 501, 501, 1849244774, '19890805 17:17:00.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7406, 915, 915, 1017554882, '20071203 03:10:04.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7407, 119, 119, 1428647329, '19860227 09:48:37.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7408, 622, 622, 953575157, '19720531 16:36:08.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7409, 387, 387, 43450586, '19550610 21:04:38.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7410, 743, 743, 1664269224, '19690724 19:21:03.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7411, 283, 283, 1697358573, '19880909 14:29:24.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7412, 338, 338, 39046904, '19760702 18:06:32.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7413, 20, 20, 264712960, '20031211 12:13:20.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7414, 71, 71, 506235115, '19620319 13:14:17.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7415, 372, 372, 1009541945, '20001016 08:09:46.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7416, 497, 497, 1887723945, '19950310 17:28:13.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7417, 889, 889, 162207914, '19540108 00:18:39.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7418, 149, 149, 155434853, '19591011 01:09:59.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7419, 73, 73, 1052798220, '19871103 22:36:08.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7420, 857, 857, 1672934138, '19911101 07:32:16.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7421, 87, 87, 1705286861, '19900705 11:20:35.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7422, 986, 986, 17450291, '20050224 04:06:36.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7423, 898, 898, 1016035153, '19610404 23:40:17.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7424, 345, 345, 494436542, '19771112 13:20:41.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7425, 304, 304, 6253751, '19870427 11:55:22.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7426, 521, 521, 580531612, '20031208 05:49:12.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7427, 193, 193, 1034902660, '20000515 03:17:21.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7428, 386, 386, 85649017, '19840525 15:41:54.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7429, 579, 579, 1527909474, '19950623 02:40:23.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7430, 242, 242, 1014850087, '19620215 07:02:05.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7431, 89, 89, 271232623, '19990418 12:29:12.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7432, 972, 972, 1141853786, '19750730 11:31:54.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7433, 62, 62, 239784523, '19650810 14:05:29.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7434, 934, 934, 945145345, '19830420 03:33:21.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7435, 376, 376, 564115690, '19720721 05:48:33.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7436, 388, 388, 1485634091, '19750506 21:04:34.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7437, 854, 854, 83130214, '19710603 10:57:21.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7438, 931, 931, 1390071999, '19860218 03:19:29.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7439, 860, 860, 1773644174, '19850812 01:52:18.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7440, 494, 494, 1683612246, '20050125 15:27:00.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7441, 812, 812, 120744056, '19640810 11:27:32.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7442, 316, 316, 295238854, '19571012 23:52:17.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7443, 442, 442, 1528827353, '19561018 05:12:30.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7444, 4, 4, 242128766, '19730701 09:49:11.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7445, 364, 364, 1191741926, '19980603 05:18:37.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7446, 199, 199, 1894899655, '19970715 02:45:57.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7447, 46, 46, 807554531, '19840716 01:47:02.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7448, 239, 239, 349681034, '19780823 15:15:32.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7449, 416, 416, 813294758, '19530221 01:39:28.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7450, 977, 977, 2123432160, '19981016 17:37:00.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7451, 266, 266, 119455308, '19831226 16:41:20.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7452, 799, 799, 50732753, '19670530 20:56:09.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7453, 76, 76, 1159918298, '19540602 05:23:36.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7454, 67, 67, 1131414112, '19701116 10:20:47.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7455, 440, 440, 1258443228, '19610414 21:01:46.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7456, 315, 315, 92877831, '19731206 12:50:11.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7457, 74, 74, 530837379, '19940602 10:59:57.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7458, 608, 608, 1774881396, '19991218 05:11:58.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7459, 816, 816, 988039288, '19700721 22:15:04.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7460, 880, 880, 627878319, '19930828 11:27:47.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7461, 92, 92, 1575347136, '19600720 23:52:42.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7462, 438, 438, 869187505, '19541005 18:06:43.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7463, 433, 433, 1244908824, '19841208 13:55:45.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7464, 331, 331, 1512766740, '20010424 14:02:32.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7465, 327, 327, 1561122095, '19780407 06:51:51.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7466, 927, 927, 1108352480, '19660212 02:57:29.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7467, 425, 425, 824901231, '19631124 04:01:02.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7468, 992, 992, 1993084826, '19730824 08:39:58.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7469, 481, 481, 1233087906, '19970828 07:25:44.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7470, 802, 802, 1627438309, '19780613 11:05:11.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7471, 549, 549, 1855509583, '19761008 06:30:52.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7472, 267, 267, 1299542746, '19751119 20:59:10.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7473, 68, 68, 669581541, '19820227 23:17:40.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7474, 371, 371, 1546489764, '19901008 07:09:07.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7475, 848, 848, 682342772, '19800323 20:04:43.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7476, 297, 297, 1829120645, '19850923 11:28:18.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7477, 532, 532, 290521005, '20010509 22:54:06.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7478, 627, 627, 703375186, '19951004 13:51:05.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7479, 118, 118, 1021545282, '19870611 08:02:04.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7480, 679, 679, 2051479886, '19971012 09:09:25.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7481, 335, 335, 1496456733, '19720525 04:02:10.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7482, 322, 322, 586240343, '19740923 12:01:48.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7483, 745, 745, 1732166252, '19840404 14:02:25.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7484, 238, 238, 2128021600, '20040905 09:03:23.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7485, 954, 954, 1560886511, '19860219 14:20:45.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7486, 475, 475, 1608291612, '19841119 15:33:02.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7487, 986, 986, 51715659, '19740228 06:08:18.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7488, 670, 670, 787250506, '20020924 03:44:42.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7489, 484, 484, 1639776874, '19750106 06:31:34.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7490, 336, 336, 1941667047, '20060814 16:58:21.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7491, 984, 984, 2144717413, '19881203 10:04:37.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7492, 798, 798, 493326064, '19790616 05:45:23.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7493, 782, 782, 1941095468, '19680731 05:04:25.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7494, 312, 312, 238660964, '19991214 14:24:31.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7495, 479, 479, 1515015484, '19970708 15:34:17.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7496, 531, 531, 429691458, '19621106 17:20:49.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7497, 828, 828, 1873085313, '19801103 09:56:32.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7498, 480, 480, 1318114442, '19630625 15:09:57.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7499, 194, 194, 1516713474, '20030326 02:57:20.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7500, 228, 228, 159717586, '19850422 02:21:07.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7501, 319, 319, 1684983157, '19860822 22:44:54.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7502, 223, 223, 553703260, '19631123 17:07:03.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7503, 483, 483, 504384869, '19881225 07:49:48.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7504, 982, 982, 189818101, '19820909 14:34:17.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7505, 813, 813, 305060056, '19750109 11:06:07.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7506, 701, 701, 1743899138, '19610206 19:42:10.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7507, 572, 572, 43259835, '19930331 21:38:29.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7508, 449, 449, 381038997, '19710824 22:14:29.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7509, 191, 191, 1726667191, '19980824 14:16:03.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7510, 75, 75, 1989582589, '19830722 09:33:28.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7511, 160, 160, 398776718, '19800728 13:56:05.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7512, 346, 346, 710009513, '20060828 21:08:51.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7513, 904, 904, 639157699, '19650709 21:09:53.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7514, 114, 114, 1804244788, '19801008 13:05:38.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7515, 866, 866, 465596641, '19740406 07:25:37.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7516, 221, 221, 1275644179, '19931027 11:50:53.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7517, 836, 836, 134434936, '19780702 04:04:17.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7518, 955, 955, 1031853618, '19880403 04:52:21.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7519, 86, 86, 14455071, '19680305 23:41:47.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7520, 996, 996, 2035342047, '20010517 15:18:48.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7521, 805, 805, 607779304, '19991214 20:49:11.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7522, 774, 774, 501272434, '19690301 00:16:20.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7523, 151, 151, 1160476971, '20060818 02:08:24.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7524, 918, 918, 1525424177, '19990113 14:27:23.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7525, 33, 33, 828942931, '19970710 01:00:34.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7526, 973, 973, 2116726737, '20050424 00:45:28.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7527, 252, 252, 1821733043, '19600421 15:54:45.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7528, 112, 112, 707108961, '19900316 08:53:09.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7529, 437, 437, 695990339, '20010331 15:34:49.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7530, 737, 737, 2099405908, '19700519 02:43:54.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7531, 130, 130, 1716254874, '19571121 01:31:45.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7532, 435, 435, 677442163, '19901116 07:44:08.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7533, 623, 623, 476409725, '20000710 10:53:45.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7534, 326, 326, 592957822, '19550220 05:40:56.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7535, 695, 695, 1356462142, '19640221 10:24:31.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7536, 583, 583, 68413509, '19610703 10:55:07.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7537, 103, 103, 1216169550, '19930928 04:12:00.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7538, 573, 573, 83682508, '19540727 23:28:08.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7539, 725, 725, 1288142879, '19820828 21:57:15.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7540, 650, 650, 1791000725, '19780113 18:38:34.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7541, 38, 38, 325555083, '19990820 17:54:33.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7542, 353, 353, 1987209202, '19550708 07:53:48.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7543, 337, 337, 304920228, '19570131 13:28:38.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7544, 68, 68, 1375437318, '19990616 20:58:05.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7545, 898, 898, 1607178043, '19790118 03:02:15.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7546, 377, 377, 2099223960, '19900406 12:55:45.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7547, 115, 115, 1315728936, '19661208 08:35:29.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7548, 524, 524, 74433596, '19951022 15:58:46.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7549, 412, 412, 1581820005, '19780910 06:58:54.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7550, 613, 613, 955292891, '19560912 03:43:23.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7551, 37, 37, 1964400947, '19920506 13:16:31.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7552, 12, 12, 881727177, '20000801 05:04:11.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7553, 304, 304, 678087113, '20030420 15:28:09.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7554, 634, 634, 1343999620, '19541109 11:38:31.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7555, 886, 886, 458315088, '19930508 05:31:56.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7556, 463, 463, 1757453481, '19700624 22:28:33.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7557, 69, 69, 1001214388, '19650406 14:58:39.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7558, 375, 375, 304957056, '19971211 17:22:25.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7559, 397, 397, 114068102, '19930301 13:33:42.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7560, 629, 629, 1409279135, '19770916 10:20:39.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7561, 541, 541, 1466258753, '19531019 03:47:24.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7562, 966, 966, 1744460335, '19560117 12:45:19.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7563, 233, 233, 1485212541, '19780525 06:39:39.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7564, 495, 495, 856712943, '19810408 11:32:09.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7565, 96, 96, 651546915, '19780901 08:01:43.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7566, 852, 852, 1409992729, '19970911 06:11:56.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7567, 265, 265, 1676985995, '19590219 10:15:06.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7568, 77, 77, 1393707347, '19630521 15:28:57.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7569, 782, 782, 1137495176, '19690819 02:41:07.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7570, 479, 479, 2032732919, '19651004 20:30:29.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7571, 163, 163, 1666442683, '19530130 07:38:53.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7572, 281, 281, 207864074, '19761206 04:36:09.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7573, 63, 63, 1681289305, '19580807 06:55:06.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7574, 557, 557, 1270714802, '19980220 05:03:13.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7575, 519, 519, 250414227, '19540928 21:24:15.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7576, 264, 264, 1928114993, '19970609 12:55:22.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7577, 718, 718, 867426830, '19650129 10:47:42.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7578, 575, 575, 1147489441, '19600304 05:10:18.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7579, 258, 258, 349449064, '19721227 11:25:07.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7580, 306, 306, 1617600793, '19600405 12:04:49.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7581, 938, 938, 706801783, '19910518 16:09:59.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7582, 90, 90, 21965000, '19720629 23:55:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7583, 333, 333, 1333536516, '19640707 01:54:15.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7584, 232, 232, 1200297855, '19970720 11:51:25.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7585, 969, 969, 1642557561, '19860111 13:27:23.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7586, 975, 975, 750750125, '19650420 20:27:34.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7587, 669, 669, 1303888229, '19950728 16:15:58.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7588, 477, 477, 1289156229, '19980901 23:15:13.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7589, 910, 910, 1354827238, '19691014 00:09:00.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7590, 791, 791, 1605139169, '20010828 11:55:57.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7591, 556, 556, 534886902, '20040327 19:56:27.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7592, 826, 826, 2021955062, '20031017 10:49:35.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7593, 939, 939, 1542864726, '19690527 09:54:26.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7594, 498, 498, 1613717404, '19571213 11:36:36.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7595, 248, 248, 896495086, '19770328 14:51:09.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7596, 471, 471, 297375465, '19960804 05:09:13.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7597, 35, 35, 585721276, '19850212 01:14:08.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7598, 289, 289, 182087365, '19831025 01:56:29.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7599, 398, 398, 76569412, '19731016 12:56:22.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7600, 359, 359, 1825079576, '19890507 20:50:18.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7601, 332, 332, 904615923, '19840217 02:40:38.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7602, 334, 334, 2005615586, '19560720 17:06:32.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7603, 554, 554, 784170346, '19790306 13:17:38.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7604, 469, 469, 911205726, '19651207 10:28:24.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7605, 258, 258, 1242402907, '19560813 20:04:30.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7606, 41, 41, 537389455, '19680601 08:40:22.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7607, 487, 487, 1742897945, '19941226 22:09:05.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7608, 369, 369, 1268455536, '19580301 10:24:56.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7609, 724, 724, 1845471018, '19530211 14:14:16.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7610, 7, 7, 2044033232, '20031128 16:36:33.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7611, 286, 286, 602554495, '19580527 11:40:51.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7612, 394, 394, 1791769991, '19580203 09:48:21.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7613, 680, 680, 434498884, '19771216 05:57:17.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7614, 941, 941, 560226340, '19851127 01:28:53.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7615, 277, 277, 1406513826, '19940430 18:10:40.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7616, 961, 961, 1641362579, '19890420 03:51:25.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7617, 841, 841, 2129473784, '19990712 10:51:04.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7618, 328, 328, 2094146531, '19881103 18:48:14.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7619, 53, 53, 68050733, '20030327 22:04:46.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7620, 596, 596, 1667543902, '19660514 11:34:09.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7621, 225, 225, 218967695, '19550215 13:55:57.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7622, 434, 434, 1023234226, '19680618 10:59:52.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7623, 283, 283, 1113659021, '20010806 15:19:57.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7624, 976, 976, 732214411, '19751221 14:45:10.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7625, 522, 522, 1104450572, '19690709 00:34:02.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7626, 612, 612, 1322254703, '19570414 20:49:18.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7627, 992, 992, 470086146, '19600712 18:41:43.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7628, 839, 839, 874802753, '19530825 19:18:29.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7629, 610, 610, 342588269, '19731125 14:12:03.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7630, 472, 472, 157782816, '19660223 16:15:01.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7631, 362, 362, 1486805305, '19650428 11:41:14.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7632, 424, 424, 277141646, '19890406 08:51:12.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7633, 806, 806, 1162559167, '19940518 16:13:55.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7634, 173, 173, 1064767091, '19910821 14:34:48.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7635, 133, 133, 1555149124, '19840217 09:24:10.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7636, 469, 469, 428832253, '19871029 23:03:26.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7637, 459, 459, 425398810, '20010425 10:38:20.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7638, 143, 143, 538547837, '20060730 15:25:50.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7639, 324, 324, 2004363400, '19931207 15:46:55.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7640, 998, 998, 47371025, '19700813 04:47:01.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7641, 640, 640, 1118264400, '19780424 22:18:28.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7642, 389, 389, 2121498842, '19900530 05:51:02.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7643, 735, 735, 1053340548, '19980722 09:00:56.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7644, 286, 286, 1324708123, '19731115 07:32:26.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7645, 880, 880, 1809815525, '19960404 00:15:06.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7646, 893, 893, 1579745116, '19990223 10:08:06.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7647, 26, 26, 1698277756, '19781102 04:52:17.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7648, 527, 527, 1276964103, '19910630 08:25:33.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7649, 422, 422, 312254952, '19710814 17:25:55.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7650, 862, 862, 1579474942, '19951208 10:59:44.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7651, 452, 452, 1850168580, '20050122 18:18:08.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7652, 302, 302, 421456938, '20040409 06:25:54.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7653, 159, 159, 1817369700, '19880729 03:51:43.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7654, 532, 532, 893315693, '19600605 01:22:12.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7655, 594, 594, 2047401123, '19870324 06:54:21.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7656, 343, 343, 1478600621, '19680831 15:40:45.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7657, 754, 754, 2060440439, '19621214 01:46:35.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7658, 100, 100, 2006482559, '19560316 22:32:28.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7659, 694, 694, 889944865, '20040530 09:54:21.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7660, 293, 293, 120654346, '20070430 23:15:12.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7661, 829, 829, 1578247739, '19601120 13:58:38.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7662, 777, 777, 1145553295, '19940504 02:50:35.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7663, 595, 595, 474704601, '19920404 20:12:53.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7664, 84, 84, 319559854, '19941218 21:59:15.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7665, 446, 446, 922958456, '19910803 04:55:19.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7666, 647, 647, 1372838879, '19770219 02:49:38.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7667, 213, 213, 1765228234, '19710917 17:34:25.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7668, 201, 201, 922585305, '19940426 15:22:29.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7669, 198, 198, 198719764, '19541011 16:04:42.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7670, 720, 720, 1421832151, '19590703 19:07:13.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7671, 667, 667, 1705909128, '19951225 17:13:04.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7672, 461, 461, 353038800, '20001211 19:25:14.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7673, 711, 711, 719934463, '20021127 03:01:19.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7674, 547, 547, 1227148531, '19850815 17:17:45.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7675, 408, 408, 242460145, '19960119 13:15:40.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7676, 305, 305, 1586906840, '19720917 08:04:55.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7677, 155, 155, 1345156839, '19771127 01:58:56.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7678, 587, 587, 2001164539, '19800922 07:47:30.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7679, 536, 536, 538169232, '19870917 14:30:03.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7680, 472, 472, 1547856484, '19780515 14:25:55.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7681, 881, 881, 810645167, '19860612 15:57:00.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7682, 483, 483, 1172608486, '19770112 10:16:09.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7683, 370, 370, 87000089, '19900111 01:52:34.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7684, 138, 138, 83810561, '19851218 03:12:19.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7685, 668, 668, 162780784, '19690130 21:56:52.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7686, 603, 603, 915523698, '19690118 05:15:19.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7687, 951, 951, 643208354, '19530909 04:59:29.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7688, 581, 581, 1865282075, '19861214 14:51:42.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7689, 477, 477, 258606964, '19570531 07:40:27.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7690, 688, 688, 1516417430, '19680618 17:43:24.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7691, 209, 209, 1498881186, '19771116 21:16:51.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7692, 981, 981, 1274264392, '19980209 12:05:52.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7693, 614, 614, 674933004, '19550303 05:31:28.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7694, 690, 690, 565295931, '19940808 16:31:43.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7695, 565, 565, 259238723, '19620923 14:48:22.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7696, 335, 335, 2122791394, '19911221 19:27:53.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7697, 170, 170, 499509523, '20060223 09:38:09.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7698, 242, 242, 1961251464, '19560804 11:01:41.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7699, 39, 39, 97550393, '19900414 02:37:07.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7700, 466, 466, 462764998, '19720214 21:25:27.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7701, 385, 385, 17861994, '19800608 16:33:41.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7702, 529, 529, 274800838, '19920707 13:29:48.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7703, 535, 535, 988236030, '19890919 16:20:50.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7704, 521, 521, 1699813254, '19650211 22:18:41.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7705, 415, 415, 1511710783, '20071213 17:46:39.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7706, 676, 676, 1646679487, '19570211 22:52:54.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7707, 373, 373, 586873687, '19540515 03:24:35.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7708, 749, 749, 260057555, '19551215 10:33:58.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7709, 269, 269, 1051900685, '19720516 12:06:32.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7710, 273, 273, 337123354, '19670707 22:49:25.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7711, 132, 132, 1264035645, '19981004 13:41:49.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7712, 501, 501, 1651495171, '19900322 17:59:05.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7713, 124, 124, 1520294474, '19760629 08:02:24.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7714, 383, 383, 1782443169, '19790116 19:28:26.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7715, 853, 853, 2136197010, '19731119 07:18:11.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7716, 809, 809, 1674481863, '19911108 03:42:29.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7717, 771, 771, 1084341559, '19570424 00:58:00.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7718, 300, 300, 355170050, '19590419 17:00:34.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7719, 980, 980, 1466564432, '19781119 00:02:22.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7720, 108, 108, 178446345, '19750716 23:39:59.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7721, 974, 974, 309837448, '19760612 21:50:09.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7722, 197, 197, 1171491888, '19921003 02:42:42.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7723, 804, 804, 736111844, '19891127 07:42:01.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7724, 122, 122, 1606093828, '19940423 22:21:17.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7725, 369, 369, 1516807901, '19890815 21:50:21.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7726, 498, 498, 685779363, '20051114 05:07:11.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7727, 34, 34, 77600910, '19981012 13:53:45.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7728, 234, 234, 1612257473, '19610421 10:29:35.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7729, 299, 299, 742861264, '19751124 02:29:23.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7730, 456, 456, 2110352278, '19570129 17:47:47.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7731, 200, 200, 1742755496, '19740724 22:26:46.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7732, 617, 617, 1553962298, '19740425 14:57:14.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7733, 679, 679, 176506969, '19980610 05:10:22.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7734, 592, 592, 96603704, '19680803 17:04:36.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7735, 977, 977, 1202316755, '20051205 21:52:14.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7736, 178, 178, 247747721, '20011205 02:27:12.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7737, 897, 897, 703488262, '19950424 17:55:19.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7738, 391, 391, 2054087026, '19771130 03:33:52.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7739, 563, 563, 717279651, '19860104 09:25:39.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7740, 128, 128, 337236583, '19641219 23:03:58.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7741, 883, 883, 1827871451, '19670907 01:50:44.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7742, 971, 971, 1951942518, '20050924 18:25:31.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7743, 158, 158, 2141257841, '19670801 02:45:10.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7744, 581, 581, 840023612, '19971123 08:51:01.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7745, 813, 813, 1322834132, '19770914 04:01:34.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7746, 1, 1, 814375104, '19950827 03:17:46.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7747, 418, 418, 171767342, '19740814 04:03:28.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7748, 361, 361, 574178046, '19840214 10:03:02.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7749, 596, 596, 1706775228, '19730920 09:13:31.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7750, 243, 243, 506964823, '19781115 11:05:53.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7751, 150, 150, 915826403, '19870830 18:29:52.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7752, 51, 51, 1745994310, '19991107 16:37:34.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7753, 70, 70, 2063924299, '19850915 10:59:19.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7754, 482, 482, 1487872478, '19670930 02:57:08.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7755, 452, 452, 1039876130, '20030902 23:35:18.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7756, 704, 704, 1318947242, '19950905 13:50:59.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7757, 528, 528, 1135784173, '19550812 05:47:46.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7758, 742, 742, 881363642, '20030528 17:59:32.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7759, 493, 493, 1647829436, '19830629 00:28:20.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7760, 255, 255, 1136199935, '19550130 12:39:27.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7761, 759, 759, 890639672, '19660502 08:59:09.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7762, 636, 636, 1597651516, '20010122 16:54:59.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7763, 830, 830, 35929730, '19880121 08:04:35.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7764, 93, 93, 671234630, '19680417 18:18:45.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7765, 922, 922, 931522756, '20001213 00:22:38.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7766, 656, 656, 482251637, '19770611 22:44:35.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7767, 591, 591, 379293682, '19991012 12:48:42.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7768, 629, 629, 846416310, '19601126 14:57:47.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7769, 33, 33, 1383105087, '19810209 21:36:12.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7770, 657, 657, 670162042, '19791215 04:50:59.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7771, 592, 592, 1549708817, '20040715 09:47:09.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7772, 161, 161, 1110006794, '19870524 21:24:08.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7773, 635, 635, 128301237, '19810414 07:34:54.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7774, 137, 137, 467767329, '19661201 00:58:24.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7775, 653, 653, 243590511, '19601109 21:49:14.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7776, 675, 675, 188729719, '19780918 03:24:37.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7777, 970, 970, 2126543295, '19780305 23:57:31.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7778, 257, 257, 262292534, '20000103 22:51:00.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7779, 460, 460, 764559688, '19690810 18:19:43.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7780, 510, 510, 818183004, '20011219 18:32:35.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7781, 396, 396, 245284583, '19840403 01:03:43.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7782, 906, 906, 1285880824, '19670830 03:50:42.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7783, 778, 778, 1943727345, '19950801 01:16:04.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7784, 514, 514, 551050897, '20050107 15:23:29.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7785, 208, 208, 1497785831, '19770531 23:17:54.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7786, 863, 863, 621965342, '19820916 05:49:11.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7787, 485, 485, 243379739, '19960809 03:57:55.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7788, 495, 495, 1678653861, '19830912 02:13:13.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7789, 990, 990, 202999339, '19721130 17:29:18.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7790, 475, 475, 1213713811, '19630403 08:01:14.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7791, 755, 755, 447997234, '19990426 20:39:26.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7792, 481, 481, 408685420, '19991126 23:55:46.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7793, 209, 209, 1857458854, '20020603 03:05:32.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7794, 916, 916, 737857058, '19831206 20:46:12.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7795, 241, 241, 1069495489, '20060819 14:04:48.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7796, 786, 786, 1759339313, '19740814 08:55:44.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7797, 767, 767, 1476754610, '19700905 10:20:56.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7798, 50, 50, 1155577991, '20070414 08:26:25.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7799, 719, 719, 332778857, '20041210 08:28:22.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7800, 431, 431, 1008627095, '19530405 05:16:59.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7801, 480, 480, 1740762199, '20031114 14:29:03.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7802, 10, 10, 1974044600, '19660918 13:05:40.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7803, 859, 859, 713352449, '19560105 12:26:49.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7804, 473, 473, 1789954926, '20011006 04:22:32.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7805, 853, 853, 45272243, '19820502 01:18:43.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7806, 86, 86, 1601390952, '19530408 21:05:44.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7807, 985, 985, 1594351572, '19710727 09:02:40.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7808, 580, 580, 867281196, '19711017 10:00:54.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7809, 634, 634, 1475092008, '19591121 05:07:53.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7810, 737, 737, 1876386824, '19771216 20:10:40.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7811, 744, 744, 1741859888, '19700703 13:53:27.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7812, 661, 661, 1389584039, '19630808 06:56:46.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7813, 638, 638, 414257108, '19861018 23:39:49.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7814, 605, 605, 1659986868, '19890709 05:55:44.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7815, 740, 740, 1217812182, '19781028 11:35:44.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7816, 325, 325, 437220956, '20060902 05:08:26.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7817, 746, 746, 1671178361, '19580625 15:38:55.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7818, 942, 942, 1328564064, '19910112 16:41:05.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7819, 649, 649, 748400543, '19981116 19:00:50.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7820, 405, 405, 1117147485, '19710330 18:33:27.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7821, 815, 815, 597556279, '19881101 18:46:39.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7822, 930, 930, 893661233, '19690131 10:35:29.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7823, 351, 351, 624973403, '19951226 21:28:29.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7824, 392, 392, 871828220, '19701110 13:34:57.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7825, 5, 5, 1512397585, '19880819 08:11:32.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7826, 263, 263, 1155693868, '19570820 09:51:23.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7827, 835, 835, 1942225235, '19921220 18:18:35.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7828, 119, 119, 1420803339, '20050508 10:48:42.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7829, 288, 288, 29956271, '19680412 10:53:35.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7830, 801, 801, 2046809864, '19940328 12:53:30.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7831, 662, 662, 690490770, '19610113 17:03:41.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7832, 663, 663, 1756566409, '19781121 15:31:05.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7833, 682, 682, 2018670061, '20030123 14:22:37.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7834, 52, 52, 1196677978, '19690508 13:02:43.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7835, 598, 598, 921106036, '20060204 04:03:31.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7836, 198, 198, 1424955550, '19700717 11:58:02.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7837, 693, 693, 235061696, '19640825 15:23:52.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7838, 700, 700, 1348884978, '20011024 20:53:31.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7839, 439, 439, 865939835, '19750909 14:04:46.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7840, 287, 287, 1146341686, '19770223 02:12:09.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7841, 960, 960, 1674823173, '19640221 20:46:31.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7842, 990, 990, 253410065, '19771024 17:57:00.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7843, 873, 873, 1113399586, '19761022 21:05:19.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7844, 697, 697, 595127515, '20021214 21:18:37.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7845, 828, 828, 1998956363, '20000929 18:07:47.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7846, 546, 546, 1443556741, '19880919 13:42:40.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7847, 687, 687, 866822569, '19660209 00:15:56.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7848, 531, 531, 569030119, '19651125 21:09:47.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7849, 528, 528, 2047390706, '19580209 09:10:27.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7850, 955, 955, 374647358, '20071217 08:56:21.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7851, 640, 640, 1989772289, '19690219 17:16:49.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7852, 989, 989, 157081028, '19870823 17:39:51.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7853, 338, 338, 945932566, '19610529 13:25:34.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7854, 623, 623, 979802949, '19860913 13:54:55.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7855, 533, 533, 961357125, '19720602 10:30:19.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7856, 501, 501, 1145077632, '19871015 03:53:34.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7857, 301, 301, 1380076454, '19780922 15:37:10.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7858, 899, 899, 1043686928, '19930223 22:51:52.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7859, 368, 368, 289364350, '19660217 20:10:59.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7860, 162, 162, 1940943834, '19770912 15:27:20.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7861, 539, 539, 1923162572, '19680418 02:47:08.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7862, 171, 171, 603139022, '19740319 22:13:58.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7863, 838, 838, 503212935, '19560706 23:07:18.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7864, 506, 506, 1161927282, '19730824 16:14:23.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7865, 840, 840, 1557454187, '19691204 03:06:58.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7866, 865, 865, 888262461, '19990811 22:22:21.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7867, 716, 716, 1193950520, '19680714 16:34:08.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7868, 690, 690, 1925713101, '19700613 10:37:05.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7869, 277, 277, 1821513138, '19910604 01:52:13.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7870, 622, 622, 462761882, '19610413 23:37:41.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7871, 579, 579, 1757446639, '19950108 13:44:34.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7872, 533, 533, 1171265809, '19640830 18:45:23.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7873, 591, 591, 95651364, '19680506 02:36:18.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7874, 650, 650, 1787555802, '19740924 16:48:40.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7875, 758, 758, 617785477, '19600207 21:46:56.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7876, 553, 553, 1308324310, '19640110 00:49:38.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7877, 903, 903, 520239368, '20000409 13:30:09.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7878, 940, 940, 654945560, '20010111 00:09:52.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7879, 537, 537, 590764713, '19780209 19:27:10.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7880, 855, 855, 1072216931, '20071129 18:28:52.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7881, 328, 328, 301338127, '19990711 09:35:26.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7882, 621, 621, 1543830628, '19800126 21:08:48.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7883, 502, 502, 997546090, '20000330 01:38:14.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7884, 366, 366, 969468129, '19680426 01:57:14.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7885, 246, 246, 573163155, '19780206 19:36:40.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7886, 684, 684, 1192273100, '19810522 23:23:50.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7887, 537, 537, 666944552, '19700626 02:05:31.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7888, 717, 717, 1631063379, '19690513 00:27:41.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7889, 822, 822, 1962987133, '20041205 02:32:24.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7890, 780, 780, 1346344531, '19710424 00:09:56.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7891, 362, 362, 309007037, '19991024 20:20:51.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7892, 217, 217, 103287460, '19790701 16:32:00.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7893, 69, 69, 978399292, '19880907 00:42:32.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7894, 335, 335, 692663108, '20051226 22:37:26.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7895, 736, 736, 1606812061, '19611107 23:25:01.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7896, 253, 253, 835475671, '19971203 22:32:33.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7897, 887, 887, 826988738, '19740420 18:49:42.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7898, 691, 691, 519593264, '19560301 04:50:55.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7899, 816, 816, 1202207982, '19860111 18:11:38.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7900, 744, 744, 486398111, '19540219 19:45:25.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7901, 627, 627, 1114800656, '19730308 21:08:31.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7902, 88, 88, 572467534, '20030828 13:38:51.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7903, 109, 109, 480122164, '19820623 19:17:34.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7904, 260, 260, 1696558048, '20041027 09:32:46.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7905, 851, 851, 25511638, '19651209 19:11:47.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7906, 294, 294, 959984138, '19570622 22:31:26.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7907, 533, 533, 1540848568, '19720419 15:03:32.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7908, 749, 749, 1041732469, '19940903 20:36:53.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7909, 600, 600, 155442550, '19790807 16:07:59.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7910, 906, 906, 1834600796, '19610525 09:40:40.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7911, 789, 789, 1976901394, '19950706 14:23:25.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7912, 642, 642, 104860281, '19850910 15:27:18.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7913, 584, 584, 929454520, '19680116 03:24:42.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7914, 499, 499, 784395413, '19660321 01:42:06.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7915, 374, 374, 1013498803, '19860304 05:51:53.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7916, 584, 584, 1196141449, '19960322 05:38:20.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7917, 997, 997, 780280355, '19811219 20:35:43.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7918, 821, 821, 1457879351, '19960311 21:10:04.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7919, 881, 881, 341199962, '20030717 20:37:43.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7920, 282, 282, 822058231, '19960713 03:43:08.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7921, 557, 557, 393208532, '19820216 20:16:50.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7922, 504, 504, 386089581, '20070303 16:06:27.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7923, 770, 770, 264789401, '19730708 08:04:41.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7924, 585, 585, 459044882, '19730211 01:42:16.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7925, 542, 542, 217773553, '19690620 03:16:49.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7926, 342, 342, 1097877378, '19680710 21:12:33.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7927, 849, 849, 1701688981, '19831223 18:02:50.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7928, 807, 807, 135565573, '19700510 03:58:51.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7929, 517, 517, 683277517, '19651117 17:23:39.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7930, 710, 710, 70884413, '19700305 23:14:22.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7931, 793, 793, 590863418, '19970920 05:59:56.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7932, 789, 789, 121132144, '19970209 08:39:13.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7933, 711, 711, 1028210971, '19680101 05:58:13.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7934, 526, 526, 691289537, '19761221 23:41:44.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7935, 835, 835, 807808995, '19870923 21:20:21.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7936, 288, 288, 2027271156, '20031112 19:56:35.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7937, 770, 770, 626898038, '20050804 01:51:13.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7938, 548, 548, 1313538398, '20030602 16:05:27.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7939, 376, 376, 1975706320, '19550519 06:45:26.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7940, 411, 411, 157342036, '19730817 21:05:13.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7941, 686, 686, 504227525, '19620203 08:20:17.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7942, 755, 755, 263406799, '19831022 05:28:37.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7943, 82, 82, 1738234821, '19971005 08:19:42.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7944, 380, 380, 53766153, '19960713 16:51:44.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7945, 22, 22, 359425715, '19831018 09:46:31.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7946, 435, 435, 1162408280, '19670214 04:53:32.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7947, 290, 290, 2041677710, '19640616 13:07:17.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7948, 593, 593, 2061464783, '19750620 23:00:25.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7949, 775, 775, 1636841807, '19721025 16:45:32.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7950, 748, 748, 1393350730, '19730817 17:46:41.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7951, 356, 356, 1327383759, '19681216 01:56:49.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7952, 861, 861, 669250098, '19860207 21:39:37.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7953, 296, 296, 1011710144, '19600814 08:13:12.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7954, 929, 929, 1016905003, '19970630 14:28:30.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7955, 551, 551, 1574700918, '19800102 23:28:35.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7956, 719, 719, 1844444893, '19740105 05:02:04.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7957, 730, 730, 1066735883, '19830221 05:34:09.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7958, 347, 347, 86364053, '19620513 17:35:17.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7959, 858, 858, 780080269, '19880510 06:15:56.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7960, 150, 150, 1842592283, '20050530 21:59:13.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7961, 64, 64, 1740826482, '19810630 04:28:36.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7962, 404, 404, 1321523750, '19541212 11:04:40.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7963, 692, 692, 535787865, '19811018 03:13:14.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7964, 645, 645, 946688087, '19620604 16:53:36.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7965, 468, 468, 623723526, '19710904 03:40:43.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7966, 402, 402, 101908251, '20060526 05:44:11.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7967, 196, 196, 1357874699, '19700911 09:29:05.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7968, 997, 997, 1402034792, '19990124 03:42:57.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7969, 177, 177, 1361913779, '19860627 04:21:44.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7970, 61, 61, 1130133976, '19900421 09:55:18.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7971, 751, 751, 438117800, '19980818 03:47:06.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7972, 589, 589, 948253296, '19860719 04:30:15.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7973, 608, 608, 985491283, '19931025 14:24:37.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7974, 772, 772, 1949613841, '19821130 23:32:30.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7975, 918, 918, 145102598, '19870610 19:22:50.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7976, 507, 507, 717124841, '20060427 14:48:12.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7977, 9, 9, 1706683797, '19620530 07:46:44.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7978, 948, 948, 471450741, '19841224 15:12:57.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7979, 879, 879, 595183019, '19970425 15:55:44.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7980, 961, 961, 308606102, '19550507 22:23:17.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7981, 520, 520, 124165981, '19570125 08:05:16.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7982, 323, 323, 186070571, '19610705 19:02:24.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7983, 857, 857, 494849052, '20050714 11:13:19.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7984, 339, 339, 497410352, '20000331 23:36:58.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7985, 270, 270, 1166371359, '19540321 21:17:33.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7986, 681, 681, 497957232, '19640813 08:20:18.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7987, 589, 589, 1851535518, '19890629 00:26:01.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7988, 491, 491, 208812190, '19780702 15:29:43.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7989, 770, 770, 1777999784, '20041219 00:13:09.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7990, 58, 58, 681271320, '19660919 16:18:17.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7991, 921, 921, 1342263004, '19730922 14:22:25.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7992, 714, 714, 1897154859, '19960324 08:15:56.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7993, 492, 492, 1380384703, '19680124 09:49:31.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7994, 83, 83, 59028084, '19571218 08:46:13.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7995, 601, 601, 1192216612, '20000217 16:36:37.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7996, 122, 122, 1512098410, '19600223 21:15:37.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7997, 778, 778, 184148551, '19550105 02:15:23.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7998, 459, 459, 287970599, '19880503 15:26:05.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (7999, 699, 699, 131643306, '19771110 13:11:00.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8000, 470, 470, 1805922466, '19850524 04:02:20.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8001, 311, 311, 1873096111, '20040603 19:24:27.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8002, 656, 656, 1528125669, '19730523 09:24:19.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8003, 530, 530, 452018178, '19961223 18:38:40.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8004, 696, 696, 1098604089, '19900706 06:50:14.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8005, 572, 572, 972132559, '19821230 13:59:34.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8006, 227, 227, 1940862959, '19900529 21:26:33.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8007, 941, 941, 1678783827, '20000414 07:15:00.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8008, 203, 203, 1870342771, '19850913 08:40:42.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8009, 764, 764, 1776338819, '19630121 19:05:39.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8010, 726, 726, 468943646, '19810906 08:40:22.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8011, 660, 660, 1329559089, '19640808 21:15:19.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8012, 406, 406, 832234719, '20060226 14:21:12.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8013, 558, 558, 1779589440, '19730117 01:39:32.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8014, 947, 947, 2044306585, '19860104 07:52:39.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8015, 483, 483, 678526007, '20010506 13:53:57.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8016, 831, 831, 1031041803, '19721226 09:26:11.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8017, 901, 901, 1425753342, '19570529 23:51:21.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8018, 214, 214, 563344262, '19890718 03:36:16.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8019, 71, 71, 1273140156, '19610316 19:36:02.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8020, 517, 517, 295471759, '19600122 19:20:24.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8021, 983, 983, 929426477, '19691128 05:18:10.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8022, 758, 758, 1363468361, '20000310 17:59:21.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8023, 925, 925, 1111018208, '20020204 03:29:48.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8024, 853, 853, 240640729, '19721008 12:03:27.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8025, 804, 804, 1415894548, '19690730 19:32:52.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8026, 588, 588, 519241178, '19550527 19:31:09.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8027, 879, 879, 317008881, '19710626 18:40:44.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8028, 822, 822, 1783127740, '19881108 05:38:24.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8029, 306, 306, 1348124844, '19901013 06:55:52.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8030, 935, 935, 921412594, '19800417 22:07:13.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8031, 825, 825, 1820861483, '20040423 12:32:49.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8032, 916, 916, 943260883, '19820125 16:20:39.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8033, 779, 779, 1722276171, '19600215 02:01:57.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8034, 642, 642, 957644876, '19641203 11:53:24.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8035, 179, 179, 1092469959, '19581203 02:58:49.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8036, 295, 295, 977346493, '19910903 22:40:56.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8037, 694, 694, 216850695, '19720712 00:23:43.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8038, 738, 738, 1339141636, '19680110 04:23:04.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8039, 615, 615, 104730868, '19700403 09:37:23.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8040, 202, 202, 1298439277, '19711022 23:51:00.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8041, 928, 928, 621717775, '19720430 01:05:18.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8042, 765, 765, 703112701, '19561016 15:45:18.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8043, 194, 194, 946668651, '19680611 20:24:04.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8044, 401, 401, 340264276, '19760414 15:32:46.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8045, 772, 772, 1939106874, '19550211 19:02:57.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8046, 600, 600, 1625731683, '19750727 00:01:13.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8047, 892, 892, 1960390608, '19760308 06:36:23.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8048, 985, 985, 10677553, '19900119 01:56:52.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8049, 989, 989, 715205781, '19640812 18:52:15.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8050, 847, 847, 147552102, '19800223 07:10:26.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8051, 678, 678, 2001634820, '19550927 21:24:15.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8052, 632, 632, 1415481262, '19730618 22:39:06.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8053, 231, 231, 680159593, '19800218 19:50:02.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8054, 788, 788, 1671271669, '19701020 17:50:35.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8055, 238, 238, 1130807493, '19680626 22:44:10.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8056, 249, 249, 354652335, '19570327 01:25:06.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8057, 832, 832, 32287447, '19790418 05:54:31.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8058, 768, 768, 2105799927, '19770318 06:35:13.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8059, 955, 955, 1058548703, '19731207 11:17:21.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8060, 655, 655, 707037724, '19800805 18:28:25.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8061, 774, 774, 1864658571, '19711205 02:45:49.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8062, 637, 637, 761332190, '19640607 01:36:35.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8063, 316, 316, 581761792, '20021201 01:44:49.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8064, 314, 314, 149287979, '19901004 20:58:25.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8065, 13, 13, 2026517271, '19850514 20:07:32.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8066, 5, 5, 1555747890, '19900714 04:54:40.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8067, 943, 943, 2035560610, '19990113 12:19:15.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8068, 217, 217, 1955190148, '19610214 13:46:08.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8069, 175, 175, 805384076, '19800203 04:53:50.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8070, 771, 771, 488343974, '19620903 15:13:01.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8071, 494, 494, 1211974673, '19530617 09:02:27.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8072, 797, 797, 2119964013, '19970519 19:28:16.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8073, 912, 912, 1324433101, '19720416 17:58:52.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8074, 618, 618, 1113017597, '19970524 19:45:02.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8075, 4, 4, 1483355281, '19950923 18:15:06.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8076, 717, 717, 1890783630, '19660212 13:32:51.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8077, 451, 451, 570754831, '19840929 21:35:17.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8078, 909, 909, 2016932638, '19781023 11:57:02.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8079, 626, 626, 657083222, '19700828 17:00:29.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8080, 833, 833, 120055373, '20020103 19:31:39.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8081, 963, 963, 33572180, '19870320 12:54:46.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8082, 735, 735, 64398575, '19890606 16:43:52.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8083, 252, 252, 613162256, '19770329 10:46:09.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8084, 754, 754, 1719523022, '19630821 23:45:26.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8085, 996, 996, 1756506879, '19770723 00:42:58.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8086, 787, 787, 131335273, '19831106 13:53:42.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8087, 30, 30, 1787328483, '19541208 20:30:36.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8088, 903, 903, 1922022450, '19970426 08:11:21.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8089, 742, 742, 959671278, '20040607 13:09:14.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8090, 109, 109, 2019481563, '19540909 01:33:42.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8091, 970, 970, 1123873053, '19650519 16:46:25.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8092, 887, 887, 994393889, '20030425 17:48:30.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8093, 476, 476, 1837348550, '20020201 17:05:42.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8094, 875, 875, 1743816520, '19970827 15:08:58.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8095, 964, 964, 1842366373, '20071117 21:05:11.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8096, 320, 320, 693702786, '19601123 23:28:43.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8097, 806, 806, 954981985, '19611115 14:00:28.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8098, 92, 92, 730236309, '19850906 23:25:38.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8099, 721, 721, 2056982898, '19981130 19:25:13.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8100, 935, 935, 1609683499, '19720730 14:08:17.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8101, 11, 11, 627649499, '19840712 11:41:58.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8102, 732, 732, 1860895820, '19680123 16:50:15.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8103, 686, 686, 2131118894, '19621217 21:03:02.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8104, 882, 882, 580293302, '19541211 03:39:14.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8105, 716, 716, 908604381, '19790908 22:07:58.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8106, 19, 19, 1801013363, '19660510 01:55:58.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8107, 445, 445, 1420794152, '19540305 04:40:14.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8108, 907, 907, 819697827, '19900926 00:05:00.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8109, 170, 170, 1102962343, '19830126 23:35:29.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8110, 926, 926, 139705942, '19550516 09:11:18.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8111, 237, 237, 2097178574, '19800626 03:49:49.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8112, 36, 36, 365243712, '19530626 17:57:28.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8113, 815, 815, 144678571, '19590722 13:34:43.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8114, 998, 998, 2045212465, '19791203 15:45:42.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8115, 855, 855, 286734748, '20010518 05:33:38.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8116, 349, 349, 2110058645, '19900629 10:01:56.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8117, 296, 296, 146311459, '19950310 14:50:26.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8118, 838, 838, 1101497832, '19920412 01:59:23.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8119, 426, 426, 258877380, '19660315 20:15:27.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8120, 111, 111, 1116840228, '19540710 06:13:49.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8121, 730, 730, 2002747836, '19880806 08:24:04.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8122, 189, 189, 1930746923, '19540920 04:07:53.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8123, 639, 639, 1460318348, '19640908 00:36:53.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8124, 146, 146, 297744826, '19780528 03:20:07.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8125, 910, 910, 1100920450, '20050415 22:26:36.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8126, 846, 846, 1073297947, '19580221 15:13:57.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8127, 878, 878, 511714475, '20030417 02:22:33.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8128, 43, 43, 1828710376, '19820820 02:49:54.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8129, 121, 121, 1074880473, '19970706 22:39:51.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8130, 867, 867, 754810260, '19871031 18:46:22.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8131, 534, 534, 170917490, '19570330 23:32:22.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8132, 598, 598, 1806692798, '20070122 22:09:38.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8133, 309, 309, 1038942080, '19871122 16:31:48.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8134, 372, 372, 900401978, '20060128 02:52:12.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8135, 790, 790, 621659714, '19700625 07:49:40.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8136, 175, 175, 1844512232, '19720226 20:04:30.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8137, 946, 946, 134230631, '19790621 19:40:50.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8138, 454, 454, 1330344491, '19750420 07:06:55.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8139, 972, 972, 159624618, '19911213 01:37:28.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8140, 767, 767, 1329824115, '19640315 22:46:59.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8141, 284, 284, 1190843875, '19820904 09:13:28.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8142, 567, 567, 668556413, '19720313 20:25:35.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8143, 129, 129, 1891914178, '19670401 08:35:52.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8144, 552, 552, 1277205439, '20020124 03:57:55.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8145, 16, 16, 513892290, '19820314 21:43:53.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8146, 492, 492, 660644180, '19641123 22:48:56.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8147, 80, 80, 413836620, '19961005 04:13:46.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8148, 67, 67, 1207712890, '19750303 01:20:00.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8149, 611, 611, 509092503, '20040410 09:35:19.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8150, 616, 616, 1810191888, '19700522 11:03:14.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8151, 230, 230, 1253412629, '19730915 08:38:17.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8152, 62, 62, 1334323259, '19770804 12:01:05.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8153, 750, 750, 939157921, '19720625 03:10:11.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8154, 953, 953, 1208603933, '19970524 13:11:23.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8155, 789, 789, 906269673, '19911224 05:44:13.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8156, 887, 887, 2055820109, '19821024 07:34:04.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8157, 286, 286, 111587111, '19560517 16:13:21.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8158, 384, 384, 1026565853, '19920721 17:42:54.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8159, 135, 135, 344100925, '19570826 05:12:37.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8160, 438, 438, 1749574329, '19740719 06:54:00.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8161, 848, 848, 1905800607, '19710122 23:33:24.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8162, 983, 983, 1693610183, '19790716 01:50:19.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8163, 488, 488, 573289938, '20010321 01:25:09.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8164, 743, 743, 1611704022, '20030328 04:49:06.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8165, 21, 21, 236730585, '20060214 09:38:56.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8166, 45, 45, 1308670783, '19810603 05:40:11.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8167, 920, 920, 799201760, '19730804 01:25:40.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8168, 603, 603, 1470123618, '19610623 10:42:31.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8169, 450, 450, 1774337, '19620611 07:56:02.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8170, 892, 892, 1894552509, '19820323 09:29:07.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8171, 146, 146, 1974565058, '19640109 14:21:06.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8172, 427, 427, 737858428, '19721121 07:43:31.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8173, 890, 890, 703034119, '19530502 00:21:55.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8174, 539, 539, 1855698197, '19541231 21:28:28.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8175, 745, 745, 1979128059, '19791104 21:00:21.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8176, 316, 316, 935461193, '19690526 11:58:28.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8177, 326, 326, 1908393206, '19950620 19:32:00.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8178, 494, 494, 407262834, '19700814 20:38:57.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8179, 800, 800, 1023589254, '20040314 05:36:14.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8180, 221, 221, 1834402686, '19930523 23:37:39.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8181, 797, 797, 1485839748, '19690518 11:00:10.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8182, 564, 564, 1975259276, '19810215 01:02:33.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8183, 95, 95, 279538687, '19860510 17:14:35.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8184, 723, 723, 210649716, '19800216 11:36:36.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8185, 464, 464, 1493980815, '19670216 10:08:05.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8186, 674, 674, 427950250, '19870826 11:31:17.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8187, 87, 87, 1043254299, '19870731 18:59:27.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8188, 295, 295, 827967010, '19980630 03:20:24.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8189, 973, 973, 665899095, '19670205 21:07:58.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8190, 177, 177, 1949813886, '19950901 00:15:35.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8191, 769, 769, 681555227, '19681012 03:51:09.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8192, 71, 71, 1229046397, '19941130 01:57:55.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8193, 920, 920, 1397884156, '19700826 01:54:17.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8194, 185, 185, 357276241, '19700527 18:43:27.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8195, 553, 553, 835192645, '20010220 23:13:34.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8196, 2, 2, 1544412201, '19560221 07:23:09.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8197, 273, 273, 542846376, '19781223 19:00:26.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8198, 695, 695, 1603497955, '19720104 03:46:46.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8199, 344, 344, 1858384768, '20031209 18:18:59.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8200, 613, 613, 1957739771, '19531012 16:03:42.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8201, 431, 431, 1633598836, '19990421 21:23:16.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8202, 630, 630, 576954574, '19880414 17:31:14.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8203, 969, 969, 82095917, '19650922 17:23:58.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8204, 814, 814, 1830347232, '19750120 00:06:11.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8205, 342, 342, 1491560851, '19590514 20:42:07.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8206, 85, 85, 1175585601, '19531026 00:54:45.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8207, 906, 906, 788528569, '19770405 11:39:10.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8208, 138, 138, 1789479808, '19700626 05:41:43.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8209, 987, 987, 2140092197, '19700721 16:11:02.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8210, 610, 610, 1528390502, '19750731 17:45:44.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8211, 169, 169, 211762436, '19950505 12:02:04.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8212, 965, 965, 181422633, '19931002 19:34:23.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8213, 568, 568, 944317353, '19960508 12:06:40.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8214, 913, 913, 19069034, '19720403 05:34:57.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8215, 423, 423, 795777682, '19580303 19:53:50.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8216, 966, 966, 565002732, '19971206 22:30:50.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8217, 40, 40, 868588848, '20010306 08:35:44.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8218, 970, 970, 408612423, '19740203 13:48:32.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8219, 608, 608, 1271632355, '19890209 18:41:00.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8220, 691, 691, 718330323, '19710622 22:07:38.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8221, 482, 482, 235027557, '20011102 10:40:43.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8222, 705, 705, 1340610842, '19830203 22:05:15.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8223, 266, 266, 1932335158, '20020517 13:34:33.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8224, 907, 907, 1772952562, '19741010 07:40:26.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8225, 904, 904, 1144631768, '19660612 05:37:58.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8226, 402, 402, 1834043385, '19770208 12:23:11.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8227, 15, 15, 897556293, '19550330 05:49:13.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8228, 15, 15, 1151508819, '19901205 05:38:28.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8229, 60, 60, 1979994286, '19611108 22:14:53.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8230, 9, 9, 897874906, '19760914 13:37:12.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8231, 455, 455, 1793006901, '19980602 16:58:02.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8232, 822, 822, 251033031, '19760617 15:45:14.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8233, 803, 803, 2085583334, '19740905 02:19:57.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8234, 878, 878, 170409504, '20030603 13:32:31.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8235, 677, 677, 2091770522, '20020201 02:14:23.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8236, 176, 176, 1667013258, '19890201 17:28:56.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8237, 837, 837, 1346532193, '19680526 07:38:35.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8238, 217, 217, 566088979, '19640421 17:08:23.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8239, 180, 180, 2059018690, '19731005 14:54:29.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8240, 144, 144, 1822434016, '19660424 09:13:19.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8241, 648, 648, 1037843157, '19630523 23:52:06.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8242, 930, 930, 1320453297, '19700205 13:17:44.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8243, 604, 604, 193695034, '19801210 11:09:21.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8244, 874, 874, 1748306636, '19990708 03:22:14.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8245, 16, 16, 141441315, '19530429 12:13:31.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8246, 818, 818, 165153870, '19830109 08:16:45.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8247, 108, 108, 325853731, '20060723 13:51:15.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8248, 309, 309, 21576962, '20060523 20:19:19.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8249, 818, 818, 133174319, '19650326 22:49:37.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8250, 700, 700, 2001094361, '19560318 00:42:43.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8251, 947, 947, 467405412, '19621216 22:47:24.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8252, 70, 70, 22807701, '19571120 05:11:54.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8253, 239, 239, 1292806377, '19901123 09:05:45.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8254, 639, 639, 2031134352, '19850619 20:11:20.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8255, 528, 528, 970022005, '19591210 05:22:58.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8256, 220, 220, 1543659733, '19690318 23:18:01.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8257, 544, 544, 1362035000, '19931128 03:56:41.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8258, 211, 211, 853103987, '19981213 09:43:31.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8259, 86, 86, 881449308, '19610810 18:28:13.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8260, 5, 5, 1118649360, '19900404 08:18:56.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8261, 874, 874, 1797754093, '20060728 19:05:32.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8262, 82, 82, 583412932, '19940501 06:00:41.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8263, 782, 782, 1213535087, '19610818 07:26:49.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8264, 893, 893, 1542190919, '20011105 02:33:50.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8265, 612, 612, 1926021174, '19850227 00:47:42.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8266, 175, 175, 768122589, '19711118 20:16:49.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8267, 657, 657, 900485572, '19720129 17:14:25.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8268, 939, 939, 1234644800, '20001204 22:34:08.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8269, 626, 626, 991482330, '19780302 03:20:34.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8270, 844, 844, 753436401, '19770129 02:24:53.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8271, 219, 219, 605094512, '19820714 14:52:15.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8272, 510, 510, 509939505, '19891115 15:27:21.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8273, 432, 432, 186241460, '19530501 22:54:02.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8274, 128, 128, 2093837456, '19751020 09:27:40.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8275, 11, 11, 2025720809, '19610130 22:15:32.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8276, 847, 847, 866324063, '19840927 21:22:59.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8277, 756, 756, 702992842, '19550225 10:55:53.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8278, 382, 382, 1957229453, '19551111 10:12:18.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8279, 354, 354, 140005727, '19740613 19:26:54.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8280, 625, 625, 43542711, '19910602 21:21:13.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8281, 944, 944, 924484721, '19780718 22:31:56.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8282, 397, 397, 1714429263, '19561105 09:29:54.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8283, 484, 484, 1747186493, '19780911 06:48:51.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8284, 490, 490, 367612578, '19580824 21:32:09.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8285, 97, 97, 752275446, '19660930 14:49:47.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8286, 496, 496, 1651295615, '19930714 11:46:08.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8287, 63, 63, 36966607, '19930725 06:39:29.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8288, 216, 216, 1264638007, '19970317 06:08:37.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8289, 827, 827, 1819491512, '19960625 08:09:33.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8290, 378, 378, 153148112, '19851116 02:56:22.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8291, 283, 283, 1486012463, '20030306 13:32:14.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8292, 834, 834, 778063614, '19770612 21:55:03.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8293, 257, 257, 99475192, '19550912 22:40:10.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8294, 936, 936, 874870455, '19910703 06:35:33.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8295, 279, 279, 1478930135, '19670926 14:07:47.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8296, 316, 316, 174418768, '19770121 17:51:24.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8297, 746, 746, 879738189, '19610621 05:50:54.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8298, 660, 660, 1186825312, '19870204 08:35:31.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8299, 465, 465, 385347593, '19670512 02:34:31.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8300, 758, 758, 1427275581, '19890610 15:56:42.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8301, 922, 922, 1470556858, '19631212 15:02:19.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8302, 429, 429, 2091536009, '19580819 15:17:06.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8303, 55, 55, 185202215, '19810324 16:58:45.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8304, 897, 897, 379546526, '19960225 20:24:43.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8305, 932, 932, 996828964, '19810903 09:50:28.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8306, 50, 50, 375223459, '19810130 07:20:03.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8307, 76, 76, 1543471808, '19570722 06:17:51.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8308, 233, 233, 1595653113, '19680204 23:38:04.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8309, 932, 932, 1561540771, '19770520 21:55:48.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8310, 299, 299, 978489379, '19830313 07:59:59.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8311, 862, 862, 1848909839, '19670123 12:22:07.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8312, 863, 863, 331829250, '19910118 17:44:22.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8313, 960, 960, 364260721, '19770702 14:16:37.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8314, 554, 554, 152172040, '19780309 21:06:59.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8315, 538, 538, 1816967104, '19640917 09:46:59.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8316, 645, 645, 219721806, '20020923 09:35:38.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8317, 164, 164, 890183293, '19680820 23:11:50.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8318, 471, 471, 1861259421, '19551226 09:54:40.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8319, 898, 898, 1332619818, '19880206 11:44:03.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8320, 852, 852, 1718871815, '19990815 13:01:34.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8321, 708, 708, 1045804751, '19860425 13:37:20.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8322, 766, 766, 1976017996, '19821113 11:05:48.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8323, 284, 284, 1239342031, '19570612 14:24:35.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8324, 787, 787, 704052362, '20000415 00:24:11.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8325, 387, 387, 2085437675, '19811124 12:52:39.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8326, 708, 708, 1344854853, '19580201 16:57:11.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8327, 917, 917, 600969200, '19870306 16:47:10.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8328, 729, 729, 1285776611, '19691029 16:18:29.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8329, 819, 819, 868917823, '19610124 19:19:53.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8330, 367, 367, 1854919198, '19920109 04:24:07.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8331, 50, 50, 1391500031, '19760409 15:32:05.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8332, 931, 931, 1992440402, '19760121 02:20:22.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8333, 858, 858, 439204810, '19960701 07:37:47.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8334, 41, 41, 860656545, '19930102 03:30:12.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8335, 137, 137, 1205832530, '19800622 06:18:53.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8336, 87, 87, 1834932398, '19721130 07:14:49.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8337, 663, 663, 464370326, '19830813 16:31:08.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8338, 139, 139, 76624771, '19900716 10:24:07.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8339, 235, 235, 2045889511, '19841220 11:41:40.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8340, 248, 248, 1439231245, '19930830 07:29:44.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8341, 288, 288, 631555113, '19881224 05:28:17.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8342, 641, 641, 434785951, '19780621 07:01:24.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8343, 171, 171, 1308354823, '19721028 08:12:48.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8344, 829, 829, 1852435645, '19660416 00:09:34.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8345, 29, 29, 1868839768, '19711025 14:34:14.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8346, 807, 807, 1408193714, '19650216 19:47:51.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8347, 255, 255, 1245067830, '20071211 07:38:26.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8348, 77, 77, 1564664080, '19850705 01:33:10.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8349, 590, 590, 234267420, '19791016 20:48:33.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8350, 715, 715, 644163907, '19730102 04:32:09.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8351, 499, 499, 784147855, '19610603 18:39:34.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8352, 963, 963, 857695376, '19580627 19:56:14.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8353, 513, 513, 594433831, '20061228 20:51:28.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8354, 195, 195, 101736931, '19750926 13:32:57.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8355, 962, 962, 1552588053, '19560217 02:19:22.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8356, 569, 569, 302272865, '19890128 03:56:30.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8357, 687, 687, 1194956958, '19540310 00:52:31.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8358, 624, 624, 365523271, '19881208 16:34:33.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8359, 859, 859, 521670473, '19670404 07:32:03.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8360, 901, 901, 663426169, '19760802 16:53:16.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8361, 307, 307, 257900078, '20011125 14:32:52.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8362, 556, 556, 258758868, '19950922 13:59:21.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8363, 926, 926, 1795667156, '19600112 04:18:11.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8364, 393, 393, 1511453208, '19930511 17:31:41.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8365, 989, 989, 130608451, '19591204 16:27:54.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8366, 275, 275, 1398746102, '19990102 10:01:45.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8367, 103, 103, 751780272, '20020719 10:06:35.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8368, 321, 321, 82792112, '19920629 10:46:25.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8369, 66, 66, 716355642, '20050916 14:48:06.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8370, 316, 316, 976581277, '19991018 02:32:10.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8371, 829, 829, 1521864207, '19720211 17:04:29.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8372, 296, 296, 697509155, '19860204 12:47:42.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8373, 839, 839, 653826181, '19790104 22:12:59.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8374, 178, 178, 1798441952, '20020608 04:14:19.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8375, 778, 778, 2007731649, '19630823 07:33:17.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8376, 769, 769, 1506063973, '19601105 06:35:55.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8377, 680, 680, 152968134, '19630117 02:53:00.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8378, 720, 720, 97293456, '19990226 14:15:00.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8379, 181, 181, 1165851076, '19810622 09:49:57.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8380, 27, 27, 1380876943, '19691008 17:04:47.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8381, 911, 911, 1371847902, '19580221 09:18:45.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8382, 264, 264, 830674552, '19540902 15:14:00.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8383, 36, 36, 203706349, '19980112 19:29:55.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8384, 225, 225, 868677019, '19960122 14:47:44.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8385, 201, 201, 1730909455, '19830808 09:44:33.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8386, 178, 178, 613060602, '19701014 19:35:50.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8387, 997, 997, 1318811317, '19770122 05:28:54.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8388, 292, 292, 1987512825, '19731006 18:04:49.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8389, 574, 574, 1018226286, '19891117 01:10:49.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8390, 977, 977, 1742015937, '19990525 02:22:23.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8391, 165, 165, 809424816, '19710924 06:22:18.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8392, 515, 515, 893084882, '20020903 23:56:34.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8393, 256, 256, 419598595, '19760414 02:52:03.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8394, 787, 787, 1941510127, '19610520 18:48:24.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8395, 116, 116, 1891718238, '19991004 16:56:51.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8396, 292, 292, 557014751, '20010402 15:28:55.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8397, 269, 269, 1450243947, '19710611 02:43:13.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8398, 74, 74, 799310542, '19870618 14:41:06.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8399, 469, 469, 650574091, '19590514 07:41:40.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8400, 957, 957, 1951248188, '19801022 04:32:28.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8401, 97, 97, 397639269, '19700731 09:41:15.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8402, 698, 698, 1060217143, '19680613 20:52:01.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8403, 99, 99, 1359657280, '19871018 10:45:04.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8404, 149, 149, 821566954, '19871229 18:16:23.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8405, 259, 259, 772344646, '19531122 11:27:39.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8406, 216, 216, 1949813873, '19830429 05:51:51.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8407, 913, 913, 466675596, '19870622 21:43:14.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8408, 426, 426, 1373457495, '19570723 16:37:08.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8409, 124, 124, 323298816, '19650204 05:59:40.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8410, 206, 206, 1959463096, '20030412 19:43:26.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8411, 379, 379, 811099514, '19790113 01:03:29.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8412, 672, 672, 343702412, '19630111 10:37:30.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8413, 550, 550, 1730138802, '19600619 06:44:36.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8414, 919, 919, 1639415631, '20050625 14:27:16.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8415, 466, 466, 1710830660, '19710613 07:34:31.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8416, 259, 259, 273560988, '20000325 23:18:51.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8417, 387, 387, 1411160222, '20050908 18:29:25.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8418, 714, 714, 1312139067, '19711220 13:30:27.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8419, 546, 546, 246505607, '19621005 07:47:08.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8420, 356, 356, 89378762, '19970218 20:52:03.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8421, 636, 636, 1723068695, '19741212 04:32:51.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8422, 111, 111, 2026621667, '19811013 16:01:46.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8423, 86, 86, 816709926, '19550814 09:35:35.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8424, 350, 350, 1696205085, '19590426 12:25:42.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8425, 882, 882, 766587805, '19810124 20:09:52.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8426, 136, 136, 1987804898, '19770608 17:07:54.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8427, 410, 410, 1171872366, '19621024 09:55:39.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8428, 373, 373, 1174346367, '19700818 03:24:35.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8429, 212, 212, 1667870364, '19550905 11:17:27.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8430, 150, 150, 895531202, '19700521 16:04:21.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8431, 185, 185, 1785764, '19970527 03:52:41.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8432, 26, 26, 627835265, '19830731 12:11:54.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8433, 107, 107, 1100402613, '19921016 06:33:20.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8434, 890, 890, 1760991561, '19530901 05:17:28.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8435, 258, 258, 1301721725, '20070310 07:23:31.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8436, 659, 659, 296149209, '19970908 12:26:44.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8437, 849, 849, 937832483, '19741115 04:28:56.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8438, 357, 357, 680793136, '19630116 01:13:32.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8439, 199, 199, 1219133412, '19950303 03:20:04.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8440, 375, 375, 732968213, '19530412 03:52:41.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8441, 864, 864, 317569705, '19910423 21:52:36.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8442, 974, 974, 668954215, '19720702 12:51:45.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8443, 550, 550, 591056330, '19610902 12:05:08.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8444, 796, 796, 1026466820, '19940806 05:27:23.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8445, 354, 354, 869319106, '19730513 01:18:53.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8446, 933, 933, 1567245988, '19610913 19:44:47.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8447, 691, 691, 832161278, '19950318 17:11:58.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8448, 660, 660, 981700874, '19781019 12:24:47.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8449, 44, 44, 1486349998, '19971207 11:13:52.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8450, 519, 519, 2111957648, '20070710 17:37:59.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8451, 838, 838, 121448978, '20030724 20:59:29.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8452, 107, 107, 317854924, '20070621 13:12:46.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8453, 984, 984, 1113235307, '19770914 06:53:57.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8454, 292, 292, 1314469638, '19700324 10:49:36.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8455, 586, 586, 557717879, '19581111 23:59:37.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8456, 684, 684, 567560466, '19961016 17:39:29.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8457, 929, 929, 925054438, '19651101 11:16:25.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8458, 278, 278, 775150216, '19810624 22:19:21.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8459, 319, 319, 1282929281, '19591205 22:06:31.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8460, 254, 254, 504295058, '19840614 18:19:44.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8461, 117, 117, 1757133324, '19730706 19:56:11.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8462, 929, 929, 1171358073, '19691105 18:18:39.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8463, 904, 904, 1939804275, '19541119 05:19:41.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8464, 429, 429, 1162813119, '20020915 13:55:18.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8465, 796, 796, 1777166961, '19581117 15:50:45.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8466, 697, 697, 1139003049, '20030614 12:51:35.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8467, 129, 129, 1848172444, '19780327 04:04:10.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8468, 540, 540, 1285787021, '19591020 01:27:08.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8469, 464, 464, 100338068, '20060417 07:03:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8470, 981, 981, 1927992902, '19811003 19:07:47.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8471, 166, 166, 890925958, '19780512 18:49:55.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8472, 826, 826, 2133647295, '19950825 17:15:53.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8473, 707, 707, 1228126588, '19841018 10:10:22.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8474, 646, 646, 884653988, '19620626 03:54:26.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8475, 331, 331, 368992271, '19581029 22:59:26.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8476, 425, 425, 1973647994, '19550612 15:41:06.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8477, 378, 378, 668699178, '19730211 03:56:37.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8478, 192, 192, 2064199364, '19690107 04:08:12.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8479, 142, 142, 850933778, '19931213 11:06:49.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8480, 718, 718, 844733224, '19720514 00:25:04.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8481, 363, 363, 579401885, '19900322 23:55:56.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8482, 902, 902, 1820464789, '19920104 21:30:51.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8483, 79, 79, 1704957565, '19800909 16:10:42.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8484, 328, 328, 919206700, '19560225 17:39:27.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8485, 783, 783, 2023052852, '19741027 19:04:52.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8486, 159, 159, 1590487121, '19971205 14:39:55.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8487, 609, 609, 1919136389, '19581117 05:17:56.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8488, 92, 92, 579698755, '19750726 19:43:43.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8489, 577, 577, 1272640321, '20021020 05:17:51.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8490, 362, 362, 1560512662, '19630525 13:44:02.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8491, 578, 578, 1324471381, '19841108 01:10:18.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8492, 843, 843, 498439546, '20010522 06:09:34.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8493, 574, 574, 1605174358, '19560212 03:07:01.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8494, 220, 220, 160398642, '19630919 09:00:19.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8495, 459, 459, 1971434323, '19871006 07:56:29.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8496, 706, 706, 860802046, '19740619 03:33:57.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8497, 112, 112, 919292042, '19700815 07:32:04.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8498, 412, 412, 1552417969, '19661217 22:09:50.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8499, 294, 294, 2112418, '19880919 13:36:38.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8500, 586, 586, 49528167, '19771128 12:27:17.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8501, 51, 51, 589996701, '19910619 15:40:37.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8502, 366, 366, 1352138762, '19880530 15:44:50.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8503, 453, 453, 841111577, '19800703 05:21:03.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8504, 866, 866, 1058690210, '19690306 16:06:05.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8505, 266, 266, 132686176, '19820228 22:48:03.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8506, 374, 374, 1642074026, '19601130 03:43:36.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8507, 512, 512, 372039543, '19750904 03:02:24.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8508, 122, 122, 1496430180, '19680322 02:59:30.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8509, 999, 999, 540884466, '19640526 11:50:10.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8510, 969, 969, 373005471, '19560602 08:18:30.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8511, 942, 942, 1495677821, '19760905 13:42:52.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8512, 583, 583, 1657812140, '20041024 07:08:13.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8513, 234, 234, 350095991, '19950712 11:12:31.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8514, 479, 479, 328720704, '19950723 21:41:26.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8515, 473, 473, 882245463, '20020324 18:23:48.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8516, 109, 109, 59106627, '19890701 22:25:20.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8517, 232, 232, 434543155, '19970225 02:07:56.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8518, 823, 823, 1891520732, '20060923 11:40:13.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8519, 861, 861, 1804191862, '19801119 18:50:26.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8520, 358, 358, 1868902994, '19681212 01:10:49.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8521, 464, 464, 810236566, '19970729 07:33:38.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8522, 681, 681, 527981917, '19550902 08:20:26.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8523, 602, 602, 49775880, '19641229 20:09:16.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8524, 188, 188, 1230712003, '19951124 17:19:42.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8525, 515, 515, 241855191, '20041124 17:57:28.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8526, 48, 48, 384459301, '19841220 12:40:21.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8527, 145, 145, 1274483881, '19920714 14:08:52.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8528, 471, 471, 1618395061, '19740131 01:10:02.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8529, 757, 757, 1344480174, '19820919 19:57:56.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8530, 433, 433, 955481888, '19920511 19:25:29.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8531, 587, 587, 744585849, '19921027 08:09:02.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8532, 237, 237, 456626220, '19590226 05:46:47.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8533, 573, 573, 1036700432, '19880418 14:31:34.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8534, 393, 393, 1006061121, '19690115 22:39:32.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8535, 308, 308, 816552515, '19881124 08:44:27.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8536, 497, 497, 171215253, '19541024 08:11:06.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8537, 240, 240, 2487798, '19640704 16:09:48.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8538, 493, 493, 984588209, '19640707 00:04:37.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8539, 830, 830, 1343185135, '19811226 18:51:23.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8540, 817, 817, 1554272411, '19661128 15:21:15.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8541, 783, 783, 1648586305, '19750404 11:37:31.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8542, 252, 252, 167325230, '19980827 02:18:26.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8543, 463, 463, 1803846137, '19640302 07:53:33.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8544, 987, 987, 1244161678, '19990520 20:59:20.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8545, 408, 408, 1024471632, '19940918 00:01:09.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8546, 314, 314, 874817460, '19880114 18:02:05.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8547, 527, 527, 1762620222, '19581111 18:57:02.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8548, 414, 414, 1275583113, '19680721 05:25:34.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8549, 340, 340, 512384384, '19690626 14:36:31.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8550, 503, 503, 191025478, '20060405 09:31:09.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8551, 424, 424, 561620294, '19850423 01:26:00.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8552, 995, 995, 103965919, '19711121 19:51:50.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8553, 944, 944, 1164398437, '19940127 03:19:23.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8554, 250, 250, 1355185655, '19721009 12:25:48.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8555, 298, 298, 491340652, '19880501 04:53:38.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8556, 535, 535, 600901407, '19881018 07:20:10.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8557, 683, 683, 632888354, '19760601 19:35:33.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8558, 447, 447, 2147196575, '19920808 12:01:20.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8559, 333, 333, 1665299090, '19720410 22:08:36.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8560, 278, 278, 2012789969, '20050310 10:07:42.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8561, 38, 38, 1046991388, '19760517 13:34:44.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8562, 483, 483, 1015233162, '19540805 01:52:21.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8563, 416, 416, 245898721, '19930702 07:01:33.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8564, 10, 10, 1184324272, '19800114 16:24:40.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8565, 182, 182, 32102202, '19710806 00:09:28.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8566, 37, 37, 1903330987, '19700712 07:56:05.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8567, 420, 420, 2002821706, '19690708 16:36:39.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8568, 102, 102, 368277928, '19790627 12:32:59.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8569, 143, 143, 377978173, '19590830 12:56:58.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8570, 418, 418, 1255140209, '20000601 10:12:42.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8571, 979, 979, 619506492, '19771229 06:15:32.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8572, 152, 152, 1532726360, '19850822 02:03:19.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8573, 576, 576, 397345234, '19770928 16:48:49.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8574, 538, 538, 631787485, '19661224 03:29:11.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8575, 706, 706, 852850990, '20010908 13:33:18.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8576, 286, 286, 1395199601, '20061202 05:15:12.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8577, 68, 68, 988698874, '19990702 00:26:52.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8578, 569, 569, 273696605, '19730810 23:09:56.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8579, 808, 808, 9317332, '19540309 17:18:32.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8580, 566, 566, 1022723935, '19691111 23:55:23.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8581, 280, 280, 1926722874, '19790209 17:43:18.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8582, 332, 332, 1136846997, '19761225 08:43:17.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8583, 79, 79, 602140705, '19570807 10:33:30.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8584, 773, 773, 1824829536, '19840615 10:26:47.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8585, 522, 522, 1416436086, '19600120 17:59:28.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8586, 558, 558, 1189078241, '19731207 12:17:11.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8587, 631, 631, 1237404040, '19730129 02:27:23.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8588, 444, 444, 79131278, '19680711 02:05:46.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8589, 467, 467, 358947716, '19880915 17:45:53.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8590, 542, 542, 1652781185, '19530207 01:24:16.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8591, 56, 56, 422235573, '19860523 12:35:32.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8592, 460, 460, 1703501254, '19791126 04:08:27.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8593, 88, 88, 71577384, '20000327 01:56:00.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8594, 101, 101, 1307792916, '19841017 08:43:41.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8595, 208, 208, 72568459, '19980713 01:46:31.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8596, 834, 834, 1608712310, '19730831 09:45:09.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8597, 43, 43, 23462285, '19580225 19:16:52.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8598, 42, 42, 12698411, '19920216 15:28:53.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8599, 455, 455, 1150053534, '19801015 20:49:52.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8600, 70, 70, 649779384, '19770309 16:05:04.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8601, 525, 525, 2062636351, '19710710 01:25:25.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8602, 7, 7, 1445509625, '19870517 06:24:02.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8603, 696, 696, 724595195, '19611122 16:28:36.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8604, 487, 487, 1021107203, '19770124 04:23:49.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8605, 994, 994, 486998252, '19810408 03:15:37.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8606, 713, 713, 1844774251, '20070831 23:22:41.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8607, 695, 695, 1282784239, '20020222 03:03:01.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8608, 899, 899, 580568399, '19800204 23:50:12.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8609, 360, 360, 2022259609, '19790130 22:52:29.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8610, 980, 980, 1477839986, '19890529 23:38:25.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8611, 314, 314, 2067178929, '19970418 06:53:18.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8612, 917, 917, 214054828, '19551023 20:25:37.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8613, 916, 916, 1086572936, '19910602 18:42:48.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8614, 959, 959, 1661205773, '19550531 22:13:13.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8615, 295, 295, 1185205741, '19790130 16:24:23.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8616, 759, 759, 188978158, '20070523 04:51:27.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8617, 593, 593, 1960208137, '20041227 15:18:50.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8618, 608, 608, 827291842, '19620117 20:34:46.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8619, 547, 547, 975647416, '19721225 22:25:11.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8620, 998, 998, 1301751769, '20050828 11:52:16.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8621, 988, 988, 513084116, '20050612 05:28:41.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8622, 841, 841, 1982207667, '19531229 14:30:53.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8623, 106, 106, 477653241, '19981010 18:47:06.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8624, 861, 861, 1846123105, '19590724 11:32:41.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8625, 860, 860, 382712887, '19670110 21:37:09.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8626, 509, 509, 1148665801, '20060202 02:07:05.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8627, 286, 286, 1869849243, '19930527 00:07:18.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8628, 693, 693, 470786889, '20001211 08:05:08.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8629, 505, 505, 131109367, '19760612 01:42:39.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8630, 940, 940, 339741877, '19810109 03:48:09.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8631, 675, 675, 830007834, '20011007 09:58:20.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8632, 54, 54, 2022297172, '19600516 08:57:59.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8633, 896, 896, 1045793822, '20001025 02:20:03.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8634, 189, 189, 58728750, '19841230 01:13:28.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8635, 869, 869, 357034778, '20060504 22:29:57.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8636, 446, 446, 1358766258, '19990925 11:19:15.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8637, 256, 256, 742061347, '19680203 16:14:40.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8638, 544, 544, 702348915, '19880713 06:09:41.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8639, 934, 934, 578845721, '19560310 07:11:10.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8640, 368, 368, 546773503, '19600520 18:36:47.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8641, 386, 386, 1582756911, '19791015 09:14:10.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8642, 625, 625, 1509656392, '20001224 02:37:11.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8643, 73, 73, 103755195, '19970610 03:13:16.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8644, 875, 875, 1939706576, '20070418 18:07:27.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8645, 577, 577, 546565786, '19631021 18:30:58.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8646, 8, 8, 1127779815, '19830801 16:09:55.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8647, 530, 530, 851071574, '19960624 09:25:38.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8648, 502, 502, 2074716159, '19971028 03:42:47.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8649, 167, 167, 700781697, '19580918 16:19:17.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8650, 38, 38, 1134255519, '19990219 20:55:03.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8651, 77, 77, 234038325, '19760902 18:26:18.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8652, 263, 263, 629584533, '20040207 22:42:06.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8653, 428, 428, 680841688, '19720222 17:03:41.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8654, 905, 905, 1332424458, '19830217 08:57:35.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8655, 400, 400, 879481083, '19790927 10:36:22.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8656, 543, 543, 1238120128, '19700712 10:54:32.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8657, 437, 437, 204515596, '19960805 11:36:56.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8658, 14, 14, 367523002, '19550503 04:55:54.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8659, 148, 148, 1711002019, '19630115 06:46:40.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8660, 924, 924, 354411838, '19830306 01:08:32.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8661, 311, 311, 1464220974, '19670407 23:15:23.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8662, 558, 558, 374522767, '19540314 18:57:52.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8663, 187, 187, 557535014, '19560825 22:07:32.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8664, 554, 554, 1916660988, '20060120 19:04:20.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8665, 733, 733, 1379640974, '19950822 13:40:05.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8666, 516, 516, 1446445153, '19891203 21:55:19.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8667, 600, 600, 1784453425, '19621228 18:05:33.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8668, 722, 722, 1613378339, '19590604 17:29:20.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8669, 609, 609, 1017557676, '19570125 23:43:15.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8670, 255, 255, 1188122860, '19870507 05:05:07.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8671, 643, 643, 1109459741, '19920419 12:36:46.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8672, 389, 389, 947189898, '19690617 09:09:09.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8673, 800, 800, 1570280918, '19581110 13:23:35.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8674, 489, 489, 2127640928, '19650808 03:48:24.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8675, 763, 763, 619700958, '19781115 02:38:06.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8676, 769, 769, 2071448220, '19570620 02:51:30.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8677, 949, 949, 1721044718, '19640720 11:17:36.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8678, 531, 531, 1050834471, '19990624 00:39:38.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8679, 457, 457, 79491043, '20031002 17:01:43.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8680, 9, 9, 1752060975, '19910611 05:27:14.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8681, 545, 545, 1387640870, '19620811 16:41:26.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8682, 253, 253, 816665216, '20030729 20:24:31.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8683, 563, 563, 1632071796, '19950326 15:45:50.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8684, 344, 344, 539304442, '19850422 04:47:35.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8685, 951, 951, 2036248889, '19570509 09:21:50.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8686, 114, 114, 312791595, '20050829 11:16:14.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8687, 349, 349, 1400634963, '19960323 15:54:17.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8688, 69, 69, 302170469, '19700908 17:22:28.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8689, 836, 836, 506093341, '19580405 14:37:05.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8690, 925, 925, 1706734199, '19881024 11:35:24.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8691, 577, 577, 1602967294, '19560220 23:42:19.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8692, 762, 762, 1938605397, '19651004 11:18:46.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8693, 229, 229, 2033687215, '19780629 23:23:01.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8694, 293, 293, 1702185800, '19810104 06:02:38.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8695, 204, 204, 983067358, '20010211 19:21:23.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8696, 45, 45, 31221880, '19780803 14:16:17.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8697, 443, 443, 1784294928, '19970501 04:29:38.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8698, 167, 167, 1646753016, '19990520 08:08:56.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8699, 196, 196, 1027829436, '19640828 04:27:22.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8700, 433, 433, 1185296409, '19811116 20:35:39.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8701, 400, 400, 1495677099, '19730804 22:04:21.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8702, 952, 952, 94496749, '19900121 15:56:18.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8703, 734, 734, 577110267, '19931003 03:59:31.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8704, 927, 927, 395207190, '19790514 11:14:09.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8705, 21, 21, 1702878541, '19591103 08:18:17.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8706, 967, 967, 1725338255, '19600319 09:17:09.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8707, 850, 850, 1902858916, '19980331 09:18:30.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8708, 177, 177, 645708085, '19590718 13:15:17.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8709, 203, 203, 1296140094, '19570405 06:19:29.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8710, 327, 327, 575852790, '19750410 07:44:52.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8711, 399, 399, 1481412929, '19581223 23:36:55.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8712, 995, 995, 1738711767, '20050211 10:57:17.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8713, 656, 656, 1921895450, '19590731 11:54:11.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8714, 384, 384, 1759060306, '19790806 01:19:25.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8715, 842, 842, 1406892199, '19730726 08:27:05.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8716, 654, 654, 1115319592, '19710909 02:50:52.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8717, 872, 872, 2065263560, '19661219 03:12:01.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8718, 144, 144, 1285382523, '19790505 17:19:56.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8719, 459, 459, 1004336224, '20010914 09:42:30.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8720, 982, 982, 374572365, '19971223 02:23:50.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8721, 121, 121, 1909966395, '20010911 06:01:01.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8722, 722, 722, 1902092284, '20000420 00:43:05.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8723, 366, 366, 257463006, '19540302 02:52:15.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8724, 396, 396, 87558765, '19760402 12:07:50.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8725, 123, 123, 1950313562, '19840317 05:22:48.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8726, 894, 894, 1947283274, '19790718 01:18:00.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8727, 324, 324, 755429707, '19981218 09:46:08.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8728, 786, 786, 947247092, '19851105 07:20:57.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8729, 427, 427, 366715247, '19720625 08:27:00.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8730, 594, 594, 2010180964, '19530414 12:21:48.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8731, 99, 99, 9741848, '19680218 22:21:51.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8732, 630, 630, 592009050, '19730302 03:08:40.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8733, 937, 937, 2041323075, '19871027 20:12:16.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8734, 245, 245, 26605454, '19741116 20:26:04.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8735, 364, 364, 166897524, '19701107 07:22:52.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8736, 238, 238, 804157224, '19800720 00:45:07.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8737, 40, 40, 1583331606, '19621027 16:24:59.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8738, 507, 507, 1428571667, '19681113 04:31:41.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8739, 610, 610, 677658432, '19780620 20:29:18.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8740, 572, 572, 1899578471, '20050219 00:04:40.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8741, 878, 878, 966791157, '19600601 01:57:44.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8742, 386, 386, 1366941396, '19890907 02:39:00.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8743, 420, 420, 2023315654, '19660606 11:02:58.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8744, 197, 197, 1580618864, '19901226 06:52:12.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8745, 562, 562, 2119815455, '19821102 11:58:29.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8746, 532, 532, 1671427846, '19590108 12:45:01.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8747, 413, 413, 1365057137, '19590307 23:24:34.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8748, 472, 472, 1367852430, '20061123 22:03:35.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8749, 365, 365, 1479311701, '19600611 21:35:32.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8750, 616, 616, 812323981, '19820606 16:30:31.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8751, 514, 514, 969532708, '19640816 11:04:15.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8752, 376, 376, 1906449277, '19701228 11:09:42.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8753, 426, 426, 35563604, '20050903 22:26:25.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8754, 508, 508, 598694810, '19740905 02:03:32.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8755, 602, 602, 360587850, '19880306 14:34:37.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8756, 560, 560, 2022643504, '19810415 21:21:16.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8757, 19, 19, 1732518433, '19881122 13:04:02.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8758, 621, 621, 1444030813, '19700703 15:51:40.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8759, 781, 781, 1363047396, '20030226 05:51:20.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8760, 339, 339, 1077673077, '19880417 07:00:16.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8761, 662, 662, 1467013882, '19690331 23:31:01.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8762, 200, 200, 315783076, '19650527 01:57:32.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8763, 33, 33, 400623059, '19950122 04:48:16.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8764, 541, 541, 1924553975, '19561222 17:57:40.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8765, 807, 807, 2066893315, '19600221 09:23:01.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8766, 282, 282, 885201422, '19931022 20:28:15.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8767, 299, 299, 1322036655, '19700418 14:45:01.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8768, 194, 194, 1740082674, '19920912 15:28:07.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8769, 12, 12, 1334284746, '19610929 17:56:33.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8770, 637, 637, 939048837, '20010105 07:41:58.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8771, 501, 501, 344364214, '19611113 10:25:52.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8772, 126, 126, 789616054, '20060203 22:40:20.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8773, 797, 797, 1008504228, '19531115 20:50:38.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8774, 190, 190, 1000242443, '20040725 09:37:49.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8775, 747, 747, 148359186, '19900725 00:26:06.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8776, 698, 698, 35586145, '19650105 03:22:01.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8777, 371, 371, 129085877, '19861115 13:40:06.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8778, 336, 336, 477142339, '19710306 20:00:03.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8779, 765, 765, 502066242, '20010531 00:09:21.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8780, 348, 348, 1067351812, '19780310 16:37:46.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8781, 805, 805, 45656341, '19730513 01:53:26.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8782, 378, 378, 1259123998, '20000124 11:42:32.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8783, 119, 119, 1491327699, '19780527 09:45:25.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8784, 232, 232, 2096502717, '19980118 15:56:29.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8785, 559, 559, 1647031972, '19960828 01:17:33.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8786, 978, 978, 1857690322, '20050220 11:12:09.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8787, 196, 196, 1321652229, '19750628 04:42:14.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8788, 575, 575, 966220978, '19660222 18:08:44.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8789, 792, 792, 1913214504, '19940911 05:51:26.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8790, 445, 445, 172532571, '19970723 10:01:35.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8791, 381, 381, 536190770, '19990827 11:41:04.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8792, 898, 898, 637722656, '20000425 00:33:18.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8793, 215, 215, 937045260, '19730916 22:40:20.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8794, 890, 890, 1724065717, '19980304 13:29:01.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8795, 329, 329, 760634378, '19881121 00:33:39.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8796, 893, 893, 1426866228, '20030105 00:00:11.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8797, 573, 573, 1754784433, '20020815 21:50:43.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8798, 532, 532, 1263816186, '19620615 17:05:17.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8799, 127, 127, 1290444847, '19831105 21:29:10.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8800, 560, 560, 1051380343, '19970109 15:30:13.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8801, 604, 604, 622388771, '19960920 21:59:59.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8802, 727, 727, 447864008, '19740624 07:56:26.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8803, 720, 720, 460587489, '19980225 04:07:01.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8804, 982, 982, 224943919, '19670606 13:53:34.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8805, 564, 564, 683001456, '19730725 06:04:38.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8806, 416, 416, 1362363455, '19660712 12:23:55.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8807, 764, 764, 1034948645, '19700212 14:19:03.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8808, 105, 105, 2037060380, '19540210 12:48:36.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8809, 131, 131, 248632133, '19920211 01:37:25.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8810, 917, 917, 598394831, '19760302 11:12:35.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8811, 800, 800, 1924823420, '20020531 07:41:10.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8812, 943, 943, 1583445883, '19700919 17:03:59.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8813, 325, 325, 907635400, '19770201 15:42:19.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8814, 660, 660, 2142055814, '19771219 13:13:33.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8815, 9, 9, 330561129, '19671207 05:06:50.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8816, 476, 476, 225516162, '19770307 11:48:28.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8817, 673, 673, 778091912, '19941230 16:12:07.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8818, 661, 661, 178606408, '20050102 12:51:47.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8819, 645, 645, 1173250021, '19680425 16:40:07.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8820, 947, 947, 1017105675, '19621230 22:10:52.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8821, 461, 461, 683223988, '19710428 15:46:00.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8822, 23, 23, 1506776900, '19570224 20:36:17.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8823, 744, 744, 750375094, '20060101 09:36:40.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8824, 370, 370, 1429230962, '19720308 07:54:58.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8825, 666, 666, 182011532, '19540512 20:00:53.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8826, 665, 665, 283119114, '19690720 09:52:33.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8827, 309, 309, 1344218869, '19850520 23:59:59.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8828, 889, 889, 139765842, '19630913 07:21:36.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8829, 321, 321, 1962209287, '19680904 09:04:09.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8830, 432, 432, 833696437, '19950720 00:25:54.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8831, 163, 163, 1441710559, '19700522 05:31:17.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8832, 657, 657, 1415703842, '19770602 20:34:48.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8833, 271, 271, 242813037, '19950430 22:30:52.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8834, 151, 151, 2105497925, '19570522 08:39:07.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8835, 630, 630, 2059733424, '19890618 18:37:47.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8836, 732, 732, 1276933095, '20061118 17:56:59.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8837, 872, 872, 1606439192, '19541130 07:35:31.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8838, 383, 383, 138505874, '19631222 19:51:50.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8839, 867, 867, 1913434020, '19770627 09:51:50.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8840, 372, 372, 362064941, '19830217 12:53:37.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8841, 871, 871, 1107918867, '19880109 20:53:05.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8842, 153, 153, 351385084, '19740518 15:53:38.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8843, 755, 755, 1996133031, '19820111 16:31:18.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8844, 550, 550, 1970655245, '19710712 18:38:51.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8845, 701, 701, 744082481, '20030222 02:20:25.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8846, 291, 291, 1472137538, '19811208 18:37:05.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8847, 243, 243, 1937305619, '19760324 08:50:59.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8848, 740, 740, 593813073, '20030928 09:26:47.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8849, 355, 355, 1370702808, '19830330 08:22:10.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8850, 386, 386, 49492140, '19640916 12:45:11.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8851, 435, 435, 1215187760, '19610106 07:48:03.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8852, 118, 118, 38592359, '20050812 08:58:56.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8853, 798, 798, 315816378, '20020219 00:25:09.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8854, 404, 404, 1242155786, '19731106 23:18:18.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8855, 879, 879, 2056339042, '19780914 23:44:13.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8856, 508, 508, 1560159053, '19920728 01:23:42.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8857, 883, 883, 47100853, '19760621 22:19:46.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8858, 178, 178, 884430715, '19781220 20:12:03.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8859, 497, 497, 1733579607, '19660125 17:52:41.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8860, 844, 844, 2050302537, '19570105 20:12:04.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8861, 451, 451, 701940431, '19890220 12:23:55.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8862, 727, 727, 1666723984, '19590603 06:57:27.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8863, 615, 615, 721940335, '19930607 03:44:26.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8864, 345, 345, 68321462, '20040724 01:11:31.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8865, 310, 310, 852479242, '19581013 05:41:17.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8866, 854, 854, 84802392, '19771229 11:06:21.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8867, 110, 110, 1925243202, '19830521 18:33:07.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8868, 647, 647, 2072683918, '19720913 07:03:11.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8869, 728, 728, 1233248933, '19960701 18:35:46.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8870, 18, 18, 975861468, '19690117 11:09:51.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8871, 146, 146, 2126184014, '19750409 04:12:57.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8872, 405, 405, 457296826, '19840109 20:20:16.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8873, 612, 612, 917391239, '19800709 02:59:57.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8874, 300, 300, 517966975, '19930308 03:46:29.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8875, 37, 37, 1791320535, '19821221 01:17:46.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8876, 506, 506, 577772647, '20041209 23:52:22.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8877, 537, 537, 1852489736, '19830213 04:04:59.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8878, 812, 812, 1904075996, '19870623 14:57:49.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8879, 850, 850, 86392700, '19770114 05:34:32.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8880, 701, 701, 1937215247, '19800603 01:23:48.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8881, 496, 496, 1137443662, '20010427 01:01:34.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8882, 363, 363, 651976865, '19890822 14:33:12.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8883, 138, 138, 267122041, '19880614 22:59:25.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8884, 293, 293, 765043152, '19561219 20:18:58.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8885, 994, 994, 1321940730, '19870714 16:37:50.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8886, 817, 817, 986194947, '19721008 20:32:21.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8887, 458, 458, 1535759511, '19830413 20:09:39.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8888, 679, 679, 368627733, '19740624 23:12:33.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8889, 837, 837, 109989286, '19860906 08:54:54.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8890, 240, 240, 886868620, '20041120 17:14:05.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8891, 338, 338, 861257569, '19830530 19:37:12.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8892, 441, 441, 18029338, '19831210 11:23:27.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8893, 785, 785, 1430943460, '20051126 01:59:09.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8894, 337, 337, 1526861129, '19730622 13:39:45.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8895, 597, 597, 247051030, '20011227 00:29:42.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8896, 260, 260, 386019714, '19810809 13:55:38.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8897, 359, 359, 282649772, '19881210 12:09:11.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8898, 717, 717, 1937050816, '19850619 15:19:46.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8899, 184, 184, 167879074, '19650930 12:57:33.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8900, 886, 886, 2022447643, '19780224 15:14:04.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8901, 273, 273, 1076980641, '20060720 00:03:57.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8902, 137, 137, 1515707534, '19560712 01:47:48.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8903, 330, 330, 928413936, '19600329 14:51:00.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8904, 230, 230, 724954745, '19670313 21:12:19.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8905, 802, 802, 364678863, '19970609 08:32:13.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8906, 380, 380, 1892463161, '19841227 11:27:47.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8907, 553, 553, 1578794296, '19780204 05:58:59.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8908, 814, 814, 1924665736, '19611214 20:38:40.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8909, 834, 834, 513106069, '19981116 22:00:31.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8910, 574, 574, 227775231, '19811004 23:51:51.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8911, 490, 490, 124827469, '19620615 21:18:42.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8912, 685, 685, 1764150682, '19961229 07:21:56.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8913, 192, 192, 25998583, '19541207 14:37:31.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8914, 883, 883, 1435405730, '19930823 16:28:53.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8915, 754, 754, 1333724537, '19630910 19:10:30.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8916, 674, 674, 1359536407, '20070630 21:50:43.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8917, 797, 797, 1110176211, '19781218 07:58:01.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8918, 718, 718, 1037116642, '19890619 07:25:27.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8919, 709, 709, 126709965, '19700111 08:33:41.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8920, 358, 358, 1990543578, '19940103 09:08:56.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8921, 901, 901, 1976831233, '20020916 14:56:42.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8922, 671, 671, 197996808, '19611127 19:20:34.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8923, 226, 226, 528511293, '19940106 22:08:16.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8924, 261, 261, 1110012608, '19990811 01:21:41.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8925, 329, 329, 1404773747, '19930819 15:32:38.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8926, 741, 741, 2096683669, '19990429 16:49:29.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8927, 194, 194, 358947999, '19860212 18:21:07.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8928, 511, 511, 2100369866, '19600118 13:20:11.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8929, 627, 627, 577991838, '19990312 03:16:47.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8930, 1000, 1000, 1155595594, '19540514 11:22:08.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8931, 230, 230, 1129032960, '19681231 11:43:10.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8932, 348, 348, 1542957140, '20050825 12:45:12.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8933, 811, 811, 647117744, '19740924 02:00:16.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8934, 930, 930, 34225472, '20061119 14:20:28.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8935, 223, 223, 286757579, '19811114 01:19:50.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8936, 481, 481, 124843938, '19971016 23:13:46.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8937, 521, 521, 1664059033, '19820526 23:42:11.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8938, 674, 674, 274604292, '19740405 01:47:06.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8939, 475, 475, 2143112975, '19670712 11:46:44.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8940, 664, 664, 1560241980, '19550719 05:10:03.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8941, 540, 540, 490047164, '20020903 14:33:22.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8942, 576, 576, 1096513575, '19740430 23:30:58.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8943, 55, 55, 827663531, '19830808 01:12:01.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8944, 6, 6, 1909423768, '19571203 09:03:03.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8945, 410, 410, 667813587, '19950608 19:55:22.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8946, 118, 118, 1751800898, '19940531 12:15:16.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8947, 18, 18, 1991745351, '19820103 20:45:56.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8948, 439, 439, 2032439104, '19650405 09:30:16.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8949, 313, 313, 1402189606, '19621013 18:29:14.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8950, 843, 843, 312351948, '20020628 02:38:58.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8951, 646, 646, 354003157, '19550824 05:57:36.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8952, 825, 825, 819501617, '20070623 04:43:44.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8953, 301, 301, 1239706836, '19680609 06:46:05.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8954, 894, 894, 117831600, '19790927 03:48:37.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8955, 937, 937, 1668566818, '19830610 00:17:21.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8956, 990, 990, 1283718794, '19970823 04:43:22.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8957, 242, 242, 2144960338, '19700705 03:39:31.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8958, 628, 628, 1089151432, '19680818 13:29:18.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8959, 555, 555, 2119695298, '19810724 05:39:40.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8960, 289, 289, 1341448176, '20060209 15:42:44.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8961, 532, 532, 1653789096, '20061113 17:06:39.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8962, 22, 22, 259690217, '19710118 16:38:47.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8963, 757, 757, 1000944220, '19701004 17:21:52.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8964, 792, 792, 283483248, '19970706 10:38:22.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8965, 277, 277, 52977282, '19651005 12:08:40.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8966, 403, 403, 177196527, '19641020 08:33:29.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8967, 586, 586, 1962230501, '19750408 05:21:40.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8968, 398, 398, 357508618, '19560118 00:17:02.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8969, 724, 724, 1213682897, '19641010 15:09:02.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8970, 948, 948, 1126638451, '19731123 19:56:44.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8971, 287, 287, 2078624224, '19780204 22:08:31.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8972, 611, 611, 1268872338, '19570915 06:10:54.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8973, 968, 968, 378958087, '19741209 19:38:42.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8974, 357, 357, 608163503, '19670627 03:23:37.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8975, 367, 367, 771453946, '19990501 18:35:33.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8976, 812, 812, 137847467, '19810519 15:25:43.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8977, 457, 457, 979487807, '19860421 18:08:32.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8978, 441, 441, 376804135, '19890204 13:05:13.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8979, 667, 667, 1223464807, '19570305 05:26:18.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8980, 968, 968, 492622356, '20070321 03:17:22.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8981, 575, 575, 1745646639, '19700426 20:03:33.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8982, 15, 15, 137948890, '19731110 08:50:51.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8983, 340, 340, 21168508, '20050405 18:50:57.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8984, 874, 874, 1178608225, '20040913 00:37:49.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8985, 807, 807, 1150874166, '20060920 05:24:32.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8986, 891, 891, 478831216, '19690710 06:59:26.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8987, 27, 27, 876284776, '19900319 05:59:07.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8988, 678, 678, 1755916351, '20021227 22:11:38.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8989, 957, 957, 1128172150, '19760612 14:03:06.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8990, 835, 835, 1334923449, '19920322 20:36:28.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8991, 921, 921, 1053477269, '19800414 19:34:14.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8992, 693, 693, 2117815493, '19661009 10:12:53.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8993, 658, 658, 1044862831, '20000910 20:07:25.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8994, 164, 164, 1686898589, '19690531 20:04:00.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8995, 632, 632, 1733802979, '19560903 12:03:23.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8996, 591, 591, 529802915, '19840816 21:54:34.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8997, 898, 898, 2008960686, '19560728 06:09:06.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8998, 403, 403, 1333235558, '19940131 14:33:38.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (8999, 129, 129, 1164557008, '20000227 20:54:22.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9000, 78, 78, 1699203604, '19830820 11:21:53.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9001, 759, 759, 420162407, '19720224 06:53:35.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9002, 493, 493, 1121247460, '19781218 20:28:54.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9003, 827, 827, 1776418641, '19530626 18:21:14.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9004, 308, 308, 119362940, '19961118 22:32:30.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9005, 94, 94, 1613383891, '19770524 04:30:26.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9006, 640, 640, 657994325, '20051207 23:46:41.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9007, 272, 272, 1184576866, '19850715 09:05:01.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9008, 678, 678, 340142415, '19531215 03:22:27.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9009, 902, 902, 1786700478, '19880527 09:13:03.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9010, 9, 9, 398827345, '19550122 08:51:38.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9011, 923, 923, 36848753, '19640504 10:34:49.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9012, 792, 792, 945841484, '19890529 14:34:17.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9013, 942, 942, 148449367, '19640616 08:02:59.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9014, 762, 762, 1815621754, '19820505 02:22:18.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9015, 893, 893, 1224297716, '19881016 19:39:10.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9016, 927, 927, 1610107618, '19860104 08:15:48.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9017, 824, 824, 415703445, '19731013 21:47:50.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9018, 918, 918, 1881997840, '19740120 16:44:03.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9019, 303, 303, 1691856782, '19981016 05:13:50.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9020, 830, 830, 1167983947, '20040327 05:09:14.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9021, 302, 302, 135927693, '19820803 02:34:22.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9022, 673, 673, 1274513662, '19800410 07:10:01.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9023, 504, 504, 1977791530, '19870806 10:13:55.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9024, 670, 670, 1804573010, '19800719 18:32:34.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9025, 819, 819, 1786658729, '20010810 00:22:29.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9026, 585, 585, 1756116894, '19640429 11:55:38.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9027, 48, 48, 109835608, '19650103 10:03:29.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9028, 227, 227, 918417346, '19580711 23:34:41.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9029, 915, 915, 1697200133, '19631024 15:20:13.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9030, 703, 703, 1128989528, '19670915 20:40:58.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9031, 66, 66, 1738141103, '19771023 09:16:36.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9032, 519, 519, 466351502, '20000320 03:34:54.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9033, 379, 379, 1680874563, '19961207 16:10:51.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9034, 877, 877, 1429409918, '19810715 18:04:25.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9035, 808, 808, 1532253490, '19880126 20:23:46.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9036, 242, 242, 1535652909, '19990508 23:34:39.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9037, 776, 776, 1118373692, '19730517 14:29:36.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9038, 224, 224, 703692003, '19610518 20:18:26.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9039, 982, 982, 1311164303, '19800422 20:07:22.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9040, 330, 330, 1074510903, '19531014 05:37:50.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9041, 784, 784, 1169896838, '19911225 21:54:24.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9042, 317, 317, 1259249300, '19890405 02:36:39.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9043, 990, 990, 321792443, '19670803 12:58:35.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9044, 355, 355, 400812932, '19740523 05:11:28.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9045, 873, 873, 12618574, '19801119 10:01:38.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9046, 584, 584, 1866673066, '19981116 04:59:57.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9047, 388, 388, 1082027627, '19550426 02:09:53.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9048, 361, 361, 968057673, '19710510 17:45:07.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9049, 311, 311, 752179768, '19880814 00:24:49.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9050, 737, 737, 2130655354, '19780830 03:47:35.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9051, 17, 17, 825435660, '19631105 00:06:44.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9052, 651, 651, 1357423161, '19900707 13:25:03.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9053, 719, 719, 1938782076, '20030417 09:19:48.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9054, 214, 214, 2054792970, '20031202 15:45:07.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9055, 484, 484, 1052451989, '19540118 08:47:31.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9056, 921, 921, 22037382, '19991114 23:43:33.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9057, 44, 44, 1289401418, '19990514 10:14:58.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9058, 134, 134, 1927019248, '19801206 23:48:40.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9059, 3, 3, 2017930672, '20030410 22:10:00.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9060, 672, 672, 572889938, '19660125 16:34:48.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9061, 359, 359, 820274674, '19931205 13:43:12.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9062, 715, 715, 2128231363, '19800105 09:30:19.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9063, 946, 946, 401979562, '19980220 12:02:14.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9064, 53, 53, 432602481, '19730912 12:32:04.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9065, 559, 559, 1946630545, '19850330 23:35:01.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9066, 715, 715, 213621361, '19720214 06:59:55.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9067, 951, 951, 1701907835, '20000620 22:23:26.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9068, 1000, 1000, 1146753857, '19901201 13:58:34.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9069, 633, 633, 1863489237, '20020408 05:58:31.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9070, 895, 895, 1179789772, '19970609 20:04:31.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9071, 686, 686, 1750857103, '19650820 17:46:11.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9072, 706, 706, 1319150713, '19650529 01:29:23.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9073, 290, 290, 1688650126, '20010928 20:36:41.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9074, 609, 609, 887715135, '19980102 23:35:59.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9075, 207, 207, 634725916, '19650403 07:14:50.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9076, 940, 940, 2085154427, '20010428 23:57:42.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9077, 606, 606, 479139486, '19650908 18:11:26.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9078, 668, 668, 1871670165, '19660317 05:02:26.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9079, 930, 930, 1855696153, '20070830 08:30:55.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9080, 925, 925, 1797686612, '19550925 19:22:32.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9081, 370, 370, 1539294230, '19620105 09:45:45.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9082, 998, 998, 887555378, '20010825 16:18:21.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9083, 73, 73, 1334746968, '19771126 23:09:51.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9084, 264, 264, 1531438228, '19930223 11:32:38.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9085, 793, 793, 339082526, '19561111 20:34:14.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9086, 980, 980, 125035060, '19950417 19:51:33.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9087, 731, 731, 1677737817, '20041202 18:15:05.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9088, 430, 430, 654407141, '20010105 00:25:44.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9089, 440, 440, 156439518, '19800628 09:16:54.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9090, 945, 945, 1398216092, '19960312 20:40:12.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9091, 598, 598, 2021681761, '20071225 13:19:40.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9092, 155, 155, 1534548102, '20000609 14:40:56.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9093, 437, 437, 1698003060, '19660206 22:08:26.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9094, 644, 644, 1768591392, '19670330 03:32:33.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9095, 254, 254, 774812929, '19671108 15:54:38.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9096, 326, 326, 1772724112, '19641221 12:24:04.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9097, 914, 914, 860402537, '19990212 14:34:25.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9098, 176, 176, 1327188872, '20011120 00:26:30.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9099, 453, 453, 1575150926, '19970221 05:36:27.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9100, 759, 759, 321007502, '19611007 03:01:42.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9101, 910, 910, 1396856069, '20060527 06:36:31.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9102, 184, 184, 460302999, '19720524 12:11:19.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9103, 562, 562, 433993141, '19770131 11:46:35.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9104, 687, 687, 992477880, '19990306 04:20:17.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9105, 278, 278, 1688612535, '19660110 10:01:23.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9106, 870, 870, 757965512, '20060607 22:37:21.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9107, 83, 83, 1167308145, '19961007 16:48:21.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9108, 938, 938, 143124486, '19580415 09:43:48.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9109, 638, 638, 379577316, '19910901 08:30:16.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9110, 979, 979, 91161593, '19600920 08:49:48.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9111, 467, 467, 1214886613, '19870309 05:32:07.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9112, 261, 261, 106384711, '19860228 05:12:31.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9113, 523, 523, 1894583990, '19810408 15:17:44.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9114, 734, 734, 1941049135, '20000717 02:47:28.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9115, 91, 91, 1675889027, '19570121 06:49:03.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9116, 886, 886, 935537501, '20000410 21:24:50.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9117, 618, 618, 711984246, '19550209 10:20:27.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9118, 733, 733, 432162635, '19571229 00:29:35.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9119, 101, 101, 246655779, '19691101 15:57:50.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9120, 40, 40, 1865910066, '19971213 03:43:27.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9121, 824, 824, 980145241, '19750313 12:44:50.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9122, 798, 798, 1020910093, '20070615 21:57:42.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9123, 861, 861, 1745382577, '19630606 04:41:39.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9124, 371, 371, 795601323, '19590127 09:18:19.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9125, 457, 457, 555643014, '19970615 06:44:51.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9126, 708, 708, 98278188, '19730312 03:05:15.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9127, 281, 281, 196529157, '20070421 03:20:57.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9128, 314, 314, 362230846, '19870703 17:04:07.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9129, 198, 198, 1173609041, '19830226 07:41:21.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9130, 537, 537, 827908891, '19530413 18:50:46.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9131, 575, 575, 936541306, '19550318 09:23:17.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9132, 392, 392, 1284344528, '19711019 17:44:56.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9133, 582, 582, 1093852417, '19770122 23:25:59.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9134, 997, 997, 1453188502, '19981124 05:29:13.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9135, 943, 943, 1265155814, '19570430 12:46:01.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9136, 808, 808, 1419166965, '19970812 21:34:26.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9137, 448, 448, 964156898, '19770726 04:31:46.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9138, 776, 776, 2140031684, '19860922 18:49:34.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9139, 899, 899, 1884645880, '19800215 01:31:14.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9140, 74, 74, 1223085125, '19580605 21:56:53.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9141, 222, 222, 1372601263, '20060710 03:03:11.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9142, 42, 42, 1858406510, '19990821 08:31:16.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9143, 735, 735, 78273446, '19620507 15:55:27.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9144, 841, 841, 745961432, '19721009 00:27:06.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9145, 935, 935, 1981532995, '19620105 15:08:05.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9146, 245, 245, 219151610, '19741028 08:07:08.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9147, 114, 114, 1295781894, '19720303 23:23:12.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9148, 889, 889, 517447110, '19730722 19:20:57.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9149, 41, 41, 328503043, '19630309 20:43:30.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9150, 756, 756, 953238439, '19750729 18:29:47.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9151, 745, 745, 1725815002, '19621113 02:03:36.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9152, 750, 750, 1336415763, '19940215 14:04:49.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9153, 104, 104, 190747454, '19850119 08:28:40.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9154, 110, 110, 1637554656, '20070311 01:52:59.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9155, 205, 205, 1747783489, '19940726 14:16:52.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9156, 85, 85, 277783773, '20061211 08:38:48.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9157, 385, 385, 1790595115, '19611220 07:29:39.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9158, 872, 872, 1911372948, '19710106 02:28:16.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9159, 158, 158, 1215061232, '19540921 21:35:26.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9160, 480, 480, 1943437618, '20001030 06:56:08.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9161, 356, 356, 143255312, '20070216 19:16:23.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9162, 94, 94, 1861440296, '19620407 23:44:14.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9163, 653, 653, 878430307, '19830217 02:02:27.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9164, 54, 54, 1077993256, '19910521 13:39:30.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9165, 643, 643, 267209713, '19580706 23:26:30.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9166, 107, 107, 1661289997, '19680521 11:47:10.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9167, 543, 543, 367502073, '19620206 05:46:32.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9168, 725, 725, 862750868, '19900515 09:48:30.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9169, 52, 52, 923196383, '19960319 14:01:27.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9170, 72, 72, 1711958795, '19670610 09:14:36.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9171, 82, 82, 25264444, '19750917 16:53:04.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9172, 105, 105, 978873791, '19760518 15:30:53.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9173, 907, 907, 687499145, '19851112 22:58:20.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9174, 975, 975, 1391519807, '19640529 18:00:56.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9175, 414, 414, 108211622, '19990605 00:40:16.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9176, 123, 123, 1173776726, '19830723 04:13:33.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9177, 774, 774, 534544588, '19980209 06:02:14.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9178, 24, 24, 1704112811, '19980827 04:14:33.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9179, 28, 28, 2089919678, '20050120 18:10:13.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9180, 76, 76, 727114437, '19750819 22:37:42.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9181, 191, 191, 328821125, '19540109 03:42:02.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9182, 751, 751, 218404442, '19860930 07:59:59.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9183, 106, 106, 2012401321, '19770426 20:20:36.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9184, 46, 46, 1788109806, '19600731 13:11:33.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9185, 703, 703, 1319261490, '19980531 16:47:09.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9186, 737, 737, 426564303, '19690129 19:18:28.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9187, 976, 976, 875083783, '19940929 09:16:16.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9188, 619, 619, 493173349, '19771113 21:33:00.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9189, 967, 967, 1663152595, '19570501 15:12:21.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9190, 401, 401, 303287591, '19580520 04:07:12.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9191, 941, 941, 1006984117, '19880824 14:04:46.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9192, 913, 913, 511622952, '19590722 02:03:29.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9193, 642, 642, 1798642470, '19850101 21:14:07.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9194, 154, 154, 2084039263, '19870416 18:35:06.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9195, 569, 569, 1179340286, '19590419 02:40:29.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9196, 945, 945, 1773772553, '19970404 03:18:57.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9197, 523, 523, 103901601, '19690705 06:28:49.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9198, 436, 436, 498624452, '19781217 02:15:56.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9199, 129, 129, 41942832, '19670406 01:00:35.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9200, 139, 139, 1664149474, '20010816 03:20:54.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9201, 876, 876, 206643676, '19650922 02:20:36.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9202, 486, 486, 479990218, '19891019 13:34:41.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9203, 130, 130, 888605767, '19850504 05:19:30.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9204, 235, 235, 1857189375, '20030929 11:28:53.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9205, 52, 52, 1132114527, '20071111 01:36:42.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9206, 671, 671, 621616397, '19940626 10:32:42.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9207, 126, 126, 1556748269, '19610405 15:06:28.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9208, 147, 147, 642040850, '19730823 14:27:43.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9209, 981, 981, 452213285, '19601007 01:12:43.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9210, 959, 959, 875794530, '19640105 10:03:19.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9211, 897, 897, 1457391408, '19611102 02:36:34.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9212, 818, 818, 2022769026, '19710425 03:15:06.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9213, 888, 888, 504866364, '19731215 08:18:02.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9214, 57, 57, 1076237050, '19870201 22:57:43.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9215, 86, 86, 1122024001, '19991023 03:14:05.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9216, 54, 54, 1232776619, '19730520 11:16:24.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9217, 434, 434, 1889449146, '19921211 03:23:37.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9218, 169, 169, 2041714536, '19750720 12:50:53.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9219, 311, 311, 355029619, '20001219 20:52:21.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9220, 141, 141, 1936566125, '19970606 04:08:02.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9221, 285, 285, 2043220502, '19810822 02:30:54.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9222, 176, 176, 1082667691, '19920324 08:13:32.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9223, 274, 274, 208928844, '19860114 18:36:09.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9224, 685, 685, 512179857, '19901101 09:54:14.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9225, 81, 81, 291942680, '19861015 19:09:50.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9226, 975, 975, 756698567, '19690227 14:49:34.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9227, 971, 971, 1025927201, '19990516 18:16:46.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9228, 481, 481, 317612105, '20060729 04:23:14.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9229, 820, 820, 612133058, '19580211 15:20:27.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9230, 193, 193, 1275232200, '19550303 21:21:18.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9231, 310, 310, 1272687875, '19670118 21:44:43.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9232, 670, 670, 132484491, '19720225 03:46:18.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9233, 93, 93, 1427986052, '19840524 03:13:57.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9234, 192, 192, 131902615, '19560607 14:49:19.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9235, 536, 536, 308932537, '19621128 20:17:06.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9236, 862, 862, 1934970156, '19720321 14:07:21.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9237, 203, 203, 1092036369, '19540530 02:40:28.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9238, 847, 847, 1437480061, '19810728 08:51:43.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9239, 677, 677, 1711885367, '19600919 11:34:50.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9240, 69, 69, 862489893, '19561206 06:14:26.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9241, 379, 379, 958952856, '19601027 04:11:59.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9242, 903, 903, 1424584253, '19740206 18:48:33.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9243, 394, 394, 1472437071, '19700207 20:20:16.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9244, 72, 72, 777907590, '20010426 05:09:02.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9245, 508, 508, 51712964, '20040717 01:30:38.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9246, 206, 206, 731745201, '19700503 10:49:40.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9247, 487, 487, 1737746305, '19930805 17:45:26.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9248, 716, 716, 1398255140, '20051129 22:16:23.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9249, 628, 628, 920499402, '19950624 15:21:01.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9250, 926, 926, 1385558856, '19931128 15:24:05.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9251, 30, 30, 925613184, '19570424 23:55:19.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9252, 146, 146, 1693474408, '20011215 17:37:56.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9253, 675, 675, 792309674, '19851227 05:23:35.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9254, 835, 835, 1206127421, '19771029 20:52:32.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9255, 155, 155, 2114677968, '19721226 00:50:00.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9256, 655, 655, 418114806, '19810629 18:07:04.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9257, 135, 135, 1453918413, '19561005 18:58:31.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9258, 37, 37, 606003844, '20020702 19:25:16.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9259, 512, 512, 1447447197, '19691215 16:19:03.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9260, 513, 513, 1091422600, '19910914 10:47:07.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9261, 223, 223, 1960125985, '20030209 16:15:56.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9262, 108, 108, 169697343, '19620907 10:43:14.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9263, 114, 114, 1433582612, '19680712 23:07:16.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9264, 213, 213, 907970801, '19580807 03:51:24.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9265, 780, 780, 650025391, '20041216 12:18:35.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9266, 900, 900, 1283391056, '19970906 22:50:16.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9267, 15, 15, 28851091, '19941201 00:01:08.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9268, 260, 260, 669221026, '19700710 17:28:43.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9269, 810, 810, 928037405, '19770307 02:40:37.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9270, 549, 549, 302068628, '19800803 13:06:43.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9271, 267, 267, 124030946, '19711223 08:35:55.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9272, 474, 474, 1591322267, '19640516 18:31:54.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9273, 801, 801, 981424864, '19671101 01:16:02.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9274, 843, 843, 252892903, '19970114 14:37:54.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9275, 526, 526, 1960043023, '19890810 23:56:03.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9276, 162, 162, 2018709150, '19600716 07:42:21.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9277, 584, 584, 1722352714, '19750215 11:53:15.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9278, 962, 962, 130047752, '19920919 13:27:07.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9279, 527, 527, 2026411442, '19940416 08:23:36.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9280, 162, 162, 279879055, '19690615 08:20:10.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9281, 607, 607, 1468980377, '19830724 21:04:08.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9282, 61, 61, 50389733, '20010615 20:00:22.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9283, 280, 280, 1257330654, '19640205 13:02:13.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9284, 820, 820, 1281235739, '19720314 23:56:21.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9285, 183, 183, 755679856, '20051107 21:25:59.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9286, 676, 676, 712195369, '19730202 04:06:47.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9287, 976, 976, 883521206, '19940227 22:22:42.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9288, 419, 419, 1543662348, '19590727 06:21:25.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9289, 556, 556, 60515999, '19910612 13:59:18.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9290, 639, 639, 75451649, '19890531 02:10:02.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9291, 428, 428, 86904271, '19680616 19:08:50.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9292, 297, 297, 2072921463, '19591127 07:15:12.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9293, 157, 157, 826159964, '19640813 16:32:39.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9294, 986, 986, 676759014, '19770105 00:47:42.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9295, 6, 6, 1732446921, '19611026 13:58:30.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9296, 799, 799, 1600372546, '20060218 17:28:45.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9297, 805, 805, 1624501576, '19580728 19:41:17.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9298, 687, 687, 1589930765, '19640704 16:28:52.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9299, 873, 873, 27102730, '20040509 16:50:26.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9300, 229, 229, 1518943789, '19591111 02:40:22.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9301, 908, 908, 2103343529, '19830602 10:48:31.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9302, 956, 956, 376640250, '19760128 00:16:43.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9303, 691, 691, 544284008, '19810925 19:35:46.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9304, 885, 885, 1148611640, '19671122 02:14:17.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9305, 621, 621, 503993068, '19741208 06:48:10.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9306, 999, 999, 248419804, '20001208 05:23:24.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9307, 864, 864, 395422899, '19870215 16:21:53.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9308, 588, 588, 1781875421, '19961211 14:45:41.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9309, 218, 218, 604791232, '19960319 20:56:28.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9310, 565, 565, 399194900, '19650612 17:07:38.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9311, 949, 949, 786949492, '19590516 06:13:49.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9312, 93, 93, 1340297793, '19720116 05:31:23.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9313, 588, 588, 1683032387, '19610319 11:01:39.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9314, 781, 781, 1699548571, '19530703 07:58:53.130')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9315, 45, 45, 336193836, '19610221 13:42:59.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9316, 81, 81, 2064721180, '19540827 20:15:33.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9317, 99, 99, 1977535366, '20060801 21:41:01.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9318, 984, 984, 833114715, '20040428 23:10:54.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9319, 978, 978, 1674731381, '19600928 06:25:24.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9320, 900, 900, 996745652, '19841114 08:11:47.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9321, 218, 218, 130311016, '19560711 00:27:33.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9322, 231, 231, 564527301, '19880507 17:39:43.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9323, 939, 939, 111865417, '19870128 03:29:24.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9324, 644, 644, 61739956, '19951007 00:30:34.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9325, 85, 85, 557689727, '19650217 17:57:53.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9326, 183, 183, 987369943, '19650127 01:20:42.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9327, 152, 152, 5353437, '20071003 01:59:14.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9328, 171, 171, 1269233134, '19981026 00:28:19.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9329, 829, 829, 370968117, '19880322 00:39:23.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9330, 992, 992, 858910649, '19910622 06:27:18.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9331, 277, 277, 857405920, '19541220 12:01:04.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9332, 184, 184, 268932201, '19630814 19:24:22.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9333, 534, 534, 1136321736, '19960511 20:36:41.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9334, 828, 828, 1921106039, '19870607 05:43:14.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9335, 390, 390, 716993153, '19940112 21:31:38.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9336, 324, 324, 430100309, '19600627 20:47:24.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9337, 290, 290, 1413643755, '19720921 00:24:36.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9338, 399, 399, 704400775, '20040315 10:47:55.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9339, 125, 125, 1840541588, '20050406 17:08:11.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9340, 966, 966, 1863954431, '19571201 16:02:34.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9341, 234, 234, 1954191090, '19931217 11:44:53.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9342, 81, 81, 476275348, '20050318 07:37:01.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9343, 317, 317, 171773826, '19710508 09:24:56.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9344, 320, 320, 1423127599, '19781231 20:51:40.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9345, 763, 763, 753226603, '19830116 19:56:13.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9346, 190, 190, 685265829, '20040531 13:37:26.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9347, 932, 932, 39773271, '20060908 20:13:33.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9348, 863, 863, 1336586953, '19640212 08:33:46.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9349, 26, 26, 1271041048, '19681114 11:04:42.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9350, 367, 367, 1690049563, '19600301 17:42:57.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9351, 795, 795, 1149334220, '20070719 19:47:43.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9352, 162, 162, 1744118952, '19620330 20:30:22.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9353, 443, 443, 1849141827, '19561007 10:03:27.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9354, 248, 248, 1363580882, '19720626 08:38:38.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9355, 233, 233, 1984314533, '19560503 02:12:48.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9356, 909, 909, 1013212345, '20030124 17:08:48.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9357, 856, 856, 771458224, '19961229 20:47:18.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9358, 284, 284, 95958936, '19931219 19:05:11.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9359, 224, 224, 562373405, '19551005 08:16:23.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9360, 722, 722, 1900666351, '19621112 05:27:27.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9361, 566, 566, 1205645728, '20010307 03:24:10.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9362, 334, 334, 1989280819, '19960421 15:53:34.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9363, 620, 620, 771592096, '19610922 14:06:17.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9364, 1, 1, 740075864, '19570929 14:29:10.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9365, 928, 928, 67479240, '19630625 05:06:34.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9366, 349, 349, 1234867906, '20031001 10:49:26.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9367, 702, 702, 1118844226, '19830905 08:54:42.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9368, 91, 91, 1758225201, '19811012 05:18:25.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9369, 517, 517, 1071696179, '19670620 10:27:14.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9370, 13, 13, 1764331937, '19530827 16:55:35.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9371, 752, 752, 1002342516, '19891205 19:50:57.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9372, 953, 953, 1934532546, '19550520 10:53:05.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9373, 595, 595, 647231612, '20070122 06:02:42.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9374, 216, 216, 793790249, '19551029 14:22:50.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9375, 731, 731, 1535948427, '19981127 20:26:53.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9376, 167, 167, 1470711144, '19590424 16:50:31.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9377, 417, 417, 972860706, '19700101 08:14:46.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9378, 764, 764, 725059002, '19610128 06:37:44.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9379, 115, 115, 65849520, '19650921 04:34:21.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9380, 730, 730, 1670760295, '19680918 04:20:27.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9381, 301, 301, 541186767, '19660520 05:07:09.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9382, 58, 58, 2014243561, '19960821 17:25:28.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9383, 47, 47, 324060079, '19821212 13:23:36.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9384, 194, 194, 2143744809, '19810121 06:56:26.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9385, 360, 360, 155986154, '19911203 10:39:34.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9386, 795, 795, 458132513, '20000920 15:30:41.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9387, 172, 172, 278028783, '19591108 09:20:54.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9388, 22, 22, 370057760, '19761116 11:58:02.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9389, 451, 451, 711048115, '19840205 03:30:25.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9390, 570, 570, 1590682417, '19981219 04:22:50.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9391, 355, 355, 218545234, '19710629 00:00:06.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9392, 386, 386, 1658931310, '19861002 05:19:24.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9393, 165, 165, 935966009, '20010612 02:31:32.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9394, 623, 623, 261886996, '19950527 11:40:44.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9395, 666, 666, 39056939, '19640925 12:38:24.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9396, 496, 496, 977132511, '20050826 19:51:19.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9397, 413, 413, 898720195, '19960626 17:30:44.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9398, 626, 626, 496808074, '19660809 18:55:45.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9399, 121, 121, 390396633, '19680709 15:45:05.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9400, 305, 305, 266232525, '19870417 09:06:46.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9401, 787, 787, 1424593092, '19730927 04:42:44.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9402, 292, 292, 1735199307, '19771128 14:55:07.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9403, 328, 328, 1177064513, '20040823 22:06:32.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9404, 334, 334, 1136508503, '19680320 18:09:07.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9405, 729, 729, 1285023952, '19780326 21:51:59.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9406, 400, 400, 1710639730, '20050302 08:54:38.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9407, 944, 944, 1357469377, '19630308 14:27:39.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9408, 804, 804, 2006280722, '19531210 19:40:37.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9409, 865, 865, 1878478954, '19810729 12:11:44.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9410, 448, 448, 1311793585, '20050108 09:22:17.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9411, 811, 811, 590319468, '19860124 08:54:01.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9412, 236, 236, 1042027166, '19881202 14:09:33.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9413, 740, 740, 1924155606, '19810331 14:30:50.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9414, 680, 680, 435693310, '19950116 03:55:56.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9415, 336, 336, 1287985620, '20040624 00:20:17.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9416, 603, 603, 27804564, '19570717 09:58:41.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9417, 407, 407, 1772064128, '19660512 02:29:57.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9418, 264, 264, 1751669676, '19880831 07:09:50.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9419, 782, 782, 833262853, '19731027 03:49:36.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9420, 163, 163, 889733930, '19701003 13:35:52.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9421, 938, 938, 760120316, '19961124 01:28:31.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9422, 704, 704, 2140079552, '19591020 20:56:40.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9423, 342, 342, 1195477504, '20050906 01:47:59.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9424, 402, 402, 2138875415, '19760701 06:04:23.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9425, 267, 267, 955942448, '19900228 16:55:28.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9426, 752, 752, 1049522616, '19560306 14:31:32.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9427, 950, 950, 826830438, '19611207 08:21:33.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9428, 713, 713, 278447501, '19640828 18:21:58.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9429, 326, 326, 1329321022, '19990202 01:44:25.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9430, 431, 431, 261488450, '20010402 00:35:33.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9431, 64, 64, 1093334313, '19701027 23:19:46.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9432, 376, 376, 904540226, '19560525 13:19:00.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9433, 513, 513, 279053239, '20000720 14:52:38.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9434, 552, 552, 2024963868, '19860605 19:27:34.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9435, 974, 974, 2111536970, '20021221 23:37:42.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9436, 954, 954, 609860368, '19960620 14:12:01.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9437, 74, 74, 604578563, '19991228 19:18:56.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9438, 116, 116, 1014069774, '19670924 19:14:28.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9439, 229, 229, 1947270870, '19551029 09:04:27.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9440, 927, 927, 1036355422, '19941002 01:44:56.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9441, 555, 555, 724643602, '19900717 01:03:02.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9442, 741, 741, 1775059039, '19581129 13:40:17.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9443, 282, 282, 149491482, '20030419 23:46:17.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9444, 212, 212, 369180824, '19870126 18:08:08.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9445, 268, 268, 483328580, '19651125 19:28:48.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9446, 836, 836, 1386065357, '19900726 09:50:32.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9447, 134, 134, 815989218, '19580704 14:48:34.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9448, 25, 25, 539457678, '19590528 22:35:35.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9449, 113, 113, 1332949658, '19981202 11:20:26.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9450, 409, 409, 526887719, '19600312 02:39:42.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9451, 669, 669, 1790810215, '19920417 17:21:22.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9452, 821, 821, 431012868, '19601027 10:20:54.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9453, 10, 10, 591308131, '20001012 15:06:09.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9454, 352, 352, 438989223, '20051006 02:09:13.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9455, 609, 609, 1448285515, '19980523 07:38:15.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9456, 395, 395, 315060904, '19661209 07:46:04.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9457, 148, 148, 1538690824, '19800323 13:07:08.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9458, 120, 120, 1561218128, '19810223 16:02:08.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9459, 657, 657, 1763558346, '19860120 01:13:39.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9460, 952, 952, 1559787710, '19750121 07:20:26.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9461, 494, 494, 1019135476, '19960327 00:33:05.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9462, 760, 760, 618651429, '20060709 20:05:41.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9463, 462, 462, 1021429191, '19621107 17:56:11.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9464, 989, 989, 523021452, '19880427 11:36:11.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9465, 720, 720, 850109018, '19870315 10:02:30.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9466, 57, 57, 1943288736, '19820901 19:35:00.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9467, 43, 43, 782940311, '19960513 23:16:54.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9468, 692, 692, 1686280427, '20021025 19:03:15.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9469, 166, 166, 1536020927, '20000125 04:18:13.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9470, 28, 28, 140763218, '19610104 10:08:16.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9471, 580, 580, 1454752447, '19650719 14:39:45.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9472, 293, 293, 2037046519, '20060817 07:15:28.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9473, 689, 689, 639221207, '19851103 22:05:23.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9474, 471, 471, 1420678216, '19870125 02:04:40.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9475, 240, 240, 66301111, '19880318 12:32:50.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9476, 950, 950, 1453896719, '19901227 11:48:14.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9477, 210, 210, 830004385, '19640701 21:10:22.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9478, 589, 589, 692038012, '19710812 07:39:51.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9479, 393, 393, 1712023208, '19630807 10:35:34.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9480, 345, 345, 1848334225, '20070805 07:04:55.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9481, 561, 561, 1886251332, '20050901 23:42:58.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9482, 990, 990, 1412845277, '19550713 09:45:57.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9483, 139, 139, 129265513, '19730926 07:01:32.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9484, 803, 803, 1274605717, '19911123 23:04:43.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9485, 528, 528, 2091333245, '19611216 07:14:11.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9486, 380, 380, 473952848, '19630101 12:58:52.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9487, 579, 579, 31705657, '19630812 22:12:51.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9488, 454, 454, 1545398385, '20021014 12:43:25.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9489, 95, 95, 276189213, '19960113 11:49:19.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9490, 2, 2, 1557556643, '19890113 15:51:38.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9491, 221, 221, 1452543317, '19690331 01:04:53.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9492, 288, 288, 1188031003, '19711105 03:16:47.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9493, 72, 72, 1830771208, '19890903 18:00:49.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9494, 480, 480, 99991716, '19881007 01:44:00.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9495, 195, 195, 1366856080, '20060707 01:11:50.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9496, 126, 126, 1838670530, '19920107 04:57:21.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9497, 765, 765, 1247303428, '20040121 19:44:06.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9498, 306, 306, 1969736425, '19671224 12:10:06.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9499, 299, 299, 1950204933, '20071113 08:05:37.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9500, 230, 230, 2126881005, '19910325 23:53:48.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9501, 879, 879, 1767277881, '20020313 10:33:37.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9502, 251, 251, 242485193, '19630908 19:45:18.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9503, 451, 451, 2093652562, '19670503 18:17:21.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9504, 778, 778, 1537124192, '19901129 01:12:09.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9505, 678, 678, 1370014005, '20020823 11:59:57.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9506, 197, 197, 767599699, '19930831 10:05:53.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9507, 840, 840, 1388062348, '19821223 12:15:30.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9508, 322, 322, 622105687, '19660919 13:01:29.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9509, 65, 65, 601266471, '19700721 13:36:23.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9510, 815, 815, 1701555498, '19600527 19:50:01.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9511, 407, 407, 358814909, '19550610 10:35:41.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9512, 49, 49, 502983323, '19610813 05:27:16.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9513, 489, 489, 606920668, '19700721 05:26:33.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9514, 403, 403, 1715584559, '19860616 18:08:43.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9515, 810, 810, 1044711001, '19770520 07:37:27.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9516, 892, 892, 963681501, '19930915 14:47:07.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9517, 549, 549, 448180630, '19851014 13:04:08.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9518, 323, 323, 1685602094, '19781214 18:51:28.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9519, 493, 493, 1091841160, '19790514 04:21:59.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9520, 30, 30, 268493999, '19770314 21:03:37.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9521, 359, 359, 2121971481, '19720121 21:22:09.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9522, 965, 965, 1374084234, '20010730 10:33:29.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9523, 267, 267, 797298254, '19591014 07:13:56.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9524, 19, 19, 1497306883, '19640112 12:26:35.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9525, 688, 688, 349095160, '19991006 09:03:22.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9526, 12, 12, 678229259, '20010913 11:22:57.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9527, 869, 869, 1098131608, '19691214 13:14:38.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9528, 433, 433, 1252142152, '20050126 20:21:22.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9529, 327, 327, 2133530607, '19880720 00:52:49.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9530, 410, 410, 1456353343, '20040311 07:35:29.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9531, 185, 185, 3350356, '19941205 16:04:07.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9532, 895, 895, 1437164839, '20040708 09:00:16.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9533, 305, 305, 714879943, '19710928 23:34:14.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9534, 700, 700, 592238358, '19800513 10:41:46.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9535, 520, 520, 68922373, '19580525 20:31:17.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9536, 618, 618, 54945795, '19941226 03:57:40.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9537, 489, 489, 2003664363, '19960310 15:28:36.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9538, 39, 39, 541623548, '19901028 05:49:22.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9539, 436, 436, 696093110, '19970401 11:04:45.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9540, 788, 788, 1232303982, '19760415 21:08:17.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9541, 775, 775, 1585821980, '19880109 00:43:21.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9542, 958, 958, 858893296, '20041122 09:11:21.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9543, 33, 33, 791843918, '19850326 23:07:01.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9544, 103, 103, 952782458, '19880818 15:59:18.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9545, 22, 22, 587912752, '19860807 05:15:56.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9546, 265, 265, 164050925, '19600818 19:37:37.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9547, 715, 715, 44013189, '19540417 21:50:13.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9548, 766, 766, 951854460, '19560320 23:52:06.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9549, 758, 758, 855868569, '19640520 18:06:32.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9550, 795, 795, 1612813821, '19651022 10:24:43.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9551, 515, 515, 1323844230, '19590326 15:53:12.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9552, 1, 1, 300453917, '19780208 00:52:38.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9553, 341, 341, 85954967, '19960811 07:48:06.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9554, 853, 853, 714887968, '19830901 11:01:59.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9555, 411, 411, 1059327026, '19720305 02:31:38.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9556, 396, 396, 362103779, '19530815 00:00:08.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9557, 430, 430, 836053080, '19561126 12:31:21.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9558, 714, 714, 427814632, '19560422 05:50:46.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9559, 413, 413, 690961349, '19990222 16:08:46.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9560, 182, 182, 1484447455, '19531211 00:37:00.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9561, 655, 655, 1033908214, '19760917 20:51:14.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9562, 663, 663, 1311745700, '19851125 15:54:08.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9563, 405, 405, 280050795, '19860301 12:08:39.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9564, 370, 370, 895851612, '19740511 06:00:54.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9565, 213, 213, 686352517, '19730622 03:45:53.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9566, 458, 458, 2063241037, '19581202 01:35:25.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9567, 290, 290, 866247375, '19971114 05:53:02.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9568, 595, 595, 514887123, '19980308 18:44:46.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9569, 172, 172, 1311672945, '19810122 21:37:25.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9570, 432, 432, 817855269, '19900525 03:39:47.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9571, 306, 306, 1683655545, '20050706 23:18:31.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9572, 361, 361, 1009128501, '20021217 07:14:45.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9573, 375, 375, 115322706, '19890913 07:46:42.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9574, 846, 846, 615840307, '19560129 07:13:41.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9575, 372, 372, 998686714, '19970305 20:20:16.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9576, 452, 452, 1141562006, '19750228 12:10:47.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9577, 438, 438, 802966757, '19690506 11:26:28.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9578, 189, 189, 1149558810, '19790224 15:14:38.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9579, 452, 452, 538271765, '19850606 07:10:38.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9580, 732, 732, 1448799181, '19920218 13:25:45.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9581, 735, 735, 1157487154, '20000529 13:32:43.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9582, 555, 555, 872276189, '19660926 13:22:32.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9583, 738, 738, 944729528, '19930909 02:14:50.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9584, 263, 263, 968013935, '19750929 14:28:05.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9585, 724, 724, 1162283360, '19971217 15:42:16.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9586, 29, 29, 178758682, '19691028 15:11:29.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9587, 571, 571, 2085088522, '19601128 01:12:09.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9588, 274, 274, 146077593, '19960127 12:32:14.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9589, 525, 525, 1411603846, '19610311 08:10:08.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9590, 60, 60, 1597568643, '19571011 20:31:09.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9591, 857, 857, 134695746, '19910130 15:26:18.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9592, 20, 20, 1207626953, '19921119 09:37:49.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9593, 634, 634, 1087439260, '19990904 13:40:35.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9594, 35, 35, 1572876596, '19960422 10:27:45.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9595, 170, 170, 2114578224, '20070729 00:17:02.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9596, 227, 227, 1350619811, '19550214 08:49:13.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9597, 129, 129, 255215390, '19710924 21:02:41.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9598, 712, 712, 63482967, '19631117 17:06:07.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9599, 688, 688, 669361209, '19680228 12:13:24.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9600, 912, 912, 761694733, '19800906 03:40:31.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9601, 446, 446, 583001150, '19701004 13:44:35.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9602, 923, 923, 779950962, '19640209 03:05:27.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9603, 795, 795, 832268216, '19830226 02:14:41.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9604, 440, 440, 1160518774, '19811226 14:26:45.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9605, 996, 996, 852428359, '19680416 11:06:11.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9606, 132, 132, 1657248687, '19640408 08:38:26.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9607, 594, 594, 1988440621, '19960526 17:05:55.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9608, 961, 961, 274464989, '19930715 00:34:25.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9609, 717, 717, 1089770309, '19940627 14:41:43.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9610, 712, 712, 2066683389, '20050104 14:20:50.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9611, 635, 635, 1590077081, '19920409 12:33:40.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9612, 686, 686, 1301424427, '19851001 21:16:43.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9613, 239, 239, 1704184806, '19781115 22:40:07.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9614, 534, 534, 1591895721, '19600106 02:43:01.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9615, 684, 684, 1527668060, '19610713 11:04:16.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9616, 55, 55, 1177090393, '19621224 07:28:42.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9617, 601, 601, 515102599, '20000317 13:39:17.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9618, 920, 920, 1853965441, '19630603 21:40:34.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9619, 345, 345, 1649250599, '19840523 14:18:37.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9620, 588, 588, 990541049, '19560825 12:34:23.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9621, 278, 278, 195401386, '20060103 00:23:15.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9622, 993, 993, 89054885, '19541019 17:20:47.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9623, 160, 160, 1201462598, '19891229 10:34:38.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9624, 565, 565, 1105932179, '19760414 01:06:15.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9625, 279, 279, 134229542, '20070424 12:13:28.550')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9626, 281, 281, 629770202, '19650818 13:40:41.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9627, 528, 528, 861225421, '19560414 17:34:10.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9628, 498, 498, 1574097478, '20010522 21:18:57.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9629, 514, 514, 1109183556, '19560702 06:56:38.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9630, 518, 518, 388967018, '19950121 11:31:02.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9631, 599, 599, 1434211425, '19560606 15:08:05.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9632, 63, 63, 573594437, '19580620 18:20:20.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9633, 369, 369, 1612222880, '19631229 03:01:13.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9634, 503, 503, 893855481, '19571001 03:30:07.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9635, 504, 504, 602388726, '19740517 23:41:10.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9636, 209, 209, 1860162389, '19890421 10:27:16.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9637, 371, 371, 1941074126, '19910731 11:07:51.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9638, 369, 369, 1506779942, '19640914 11:48:05.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9639, 355, 355, 1011217628, '19600615 03:21:54.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9640, 16, 16, 1397076771, '19860911 07:03:49.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9641, 348, 348, 2063387280, '19810602 22:05:34.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9642, 239, 239, 1554270774, '19750516 00:37:44.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9643, 49, 49, 1097434326, '19540802 21:50:30.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9644, 478, 478, 6370523, '19640305 17:49:17.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9645, 174, 174, 993219745, '19730703 07:57:29.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9646, 792, 792, 1714221832, '19580501 18:09:35.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9647, 532, 532, 792365637, '19670105 10:57:42.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9648, 543, 543, 1995829807, '19920830 10:57:34.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9649, 83, 83, 955086235, '19871011 23:23:28.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9650, 227, 227, 2008009365, '19970805 16:48:20.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9651, 530, 530, 641533921, '19621128 19:09:56.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9652, 235, 235, 904403377, '19610423 23:22:07.930')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9653, 324, 324, 64349225, '19870626 02:47:29.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9654, 605, 605, 339742940, '19640705 23:39:01.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9655, 512, 512, 358627758, '19820903 03:17:16.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9656, 135, 135, 618994656, '19681216 20:23:47.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9657, 464, 464, 480599808, '19820209 16:30:48.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9658, 188, 188, 1451217810, '19591115 01:08:25.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9659, 21, 21, 1407432240, '19820903 02:13:16.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9660, 638, 638, 270043162, '19550830 21:25:30.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9661, 704, 704, 690010315, '19601225 15:04:16.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9662, 127, 127, 1327040810, '20030103 19:46:57.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9663, 603, 603, 1630358732, '19900113 17:37:46.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9664, 165, 165, 157756867, '20070606 03:10:41.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9665, 955, 955, 2009521469, '20010731 23:12:45.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9666, 741, 741, 221123305, '19861021 18:13:19.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9667, 311, 311, 31849888, '19741005 18:15:30.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9668, 455, 455, 413859164, '19740215 19:10:00.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9669, 158, 158, 1855607556, '19930820 03:01:50.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9670, 406, 406, 1943687770, '19800323 00:36:59.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9671, 705, 705, 84497849, '19790525 20:20:50.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9672, 53, 53, 1916598448, '19880704 01:51:11.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9673, 363, 363, 191794911, '19551220 18:18:40.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9674, 578, 578, 1213373445, '20050911 07:14:47.790')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9675, 320, 320, 1559828816, '19830324 14:28:48.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9676, 702, 702, 1877914301, '19830821 23:45:30.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9677, 707, 707, 646402886, '19530319 19:30:17.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9678, 584, 584, 1514111929, '19781026 16:45:20.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9679, 851, 851, 554929696, '19551013 17:08:46.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9680, 281, 281, 814201544, '20011224 18:03:53.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9681, 198, 198, 1551850993, '20060813 02:42:59.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9682, 965, 965, 433647223, '19710815 06:36:35.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9683, 978, 978, 1897688500, '19660812 21:55:28.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9684, 383, 383, 1312357615, '19661126 14:08:18.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9685, 566, 566, 1080285231, '19850223 16:21:06.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9686, 346, 346, 1848625161, '20030212 15:45:57.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9687, 130, 130, 2045830189, '19781225 15:32:50.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9688, 190, 190, 2120961521, '20070624 03:22:12.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9689, 387, 387, 73457322, '19830128 00:12:50.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9690, 254, 254, 1473937577, '19580602 03:17:23.240')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9691, 434, 434, 2061498954, '19600312 17:56:28.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9692, 903, 903, 1665165978, '19840916 09:59:26.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9693, 478, 478, 1712228217, '19900111 09:34:48.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9694, 402, 402, 1625373132, '19571017 05:56:23.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9695, 624, 624, 941136771, '19780917 15:59:33.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9696, 537, 537, 301277784, '19860530 02:18:37.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9697, 286, 286, 595168517, '19930501 06:59:58.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9698, 59, 59, 774768968, '19550226 18:39:48.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9699, 500, 500, 74855720, '19700804 18:36:32.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9700, 800, 800, 272506145, '19940911 13:44:10.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9701, 970, 970, 1268840137, '19910726 23:54:05.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9702, 813, 813, 391593288, '20001118 15:47:42.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9703, 890, 890, 335802600, '20070110 07:55:43.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9704, 934, 934, 547983441, '19600721 22:46:28.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9705, 440, 440, 2145551468, '19710314 20:27:30.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9706, 692, 692, 1883749571, '19820526 17:18:44.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9707, 465, 465, 439258148, '19580506 05:03:27.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9708, 181, 181, 32011460, '19891013 19:32:42.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9709, 549, 549, 934936418, '19890413 09:10:12.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9710, 936, 936, 292289376, '20070112 03:31:45.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9711, 530, 530, 2145371084, '19680930 00:53:29.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9712, 415, 415, 2035045451, '19560416 23:45:27.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9713, 106, 106, 612522393, '19570202 07:59:39.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9714, 541, 541, 882996475, '19880908 08:09:23.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9715, 550, 550, 272356218, '19570116 18:42:31.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9716, 809, 809, 2040831997, '19970512 08:27:41.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9717, 18, 18, 2121848250, '19890524 21:51:28.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9718, 675, 675, 2075288137, '19760219 03:29:28.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9719, 266, 266, 454855520, '19750413 10:49:35.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9720, 234, 234, 1906243333, '20060618 07:26:47.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9721, 378, 378, 2050238860, '19601028 02:40:28.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9722, 382, 382, 2146020243, '19750413 14:53:17.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9723, 161, 161, 1260808134, '19990118 18:57:09.380')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9724, 671, 671, 887297370, '19880320 23:44:27.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9725, 694, 694, 53788261, '19730111 06:40:31.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9726, 370, 370, 953532815, '20020907 10:21:24.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9727, 676, 676, 1862690807, '20060622 16:16:23.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9728, 636, 636, 1408282552, '20060304 12:22:16.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9729, 590, 590, 841838862, '19791226 15:15:14.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9730, 656, 656, 135074081, '20041024 12:10:11.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9731, 161, 161, 55891525, '19980421 16:45:32.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9732, 641, 641, 1746820342, '20060121 00:50:28.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9733, 367, 367, 345212696, '19610324 22:08:48.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9734, 687, 687, 231152755, '19690131 03:24:36.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9735, 145, 145, 864908837, '19630601 18:09:48.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9736, 391, 391, 368070146, '19580925 10:55:17.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9737, 624, 624, 1458377829, '19720803 22:40:51.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9738, 905, 905, 581063954, '19590122 23:08:59.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9739, 379, 379, 830267045, '20030913 17:40:48.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9740, 637, 637, 2043628510, '19551001 23:02:21.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9741, 524, 524, 232489243, '19971010 10:42:29.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9742, 983, 983, 1035807297, '19970312 20:00:08.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9743, 313, 313, 405745548, '19710313 18:12:00.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9744, 428, 428, 1862159557, '19840115 20:41:05.770')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9745, 891, 891, 1880710506, '19970901 02:23:53.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9746, 450, 450, 1710131227, '19561126 18:11:00.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9747, 133, 133, 1474094711, '19800815 01:59:47.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9748, 570, 570, 1633265920, '19540506 01:25:25.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9749, 539, 539, 522169509, '19531001 11:13:52.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9750, 470, 470, 144059727, '19890507 07:31:51.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9751, 354, 354, 887078700, '20050104 04:27:08.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9752, 747, 747, 940638979, '19700314 03:30:30.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9753, 257, 257, 657514977, '19871115 07:50:13.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9754, 574, 574, 602887416, '19720216 11:09:45.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9755, 498, 498, 2060608374, '19861115 11:03:41.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9756, 719, 719, 290387839, '19690415 09:00:47.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9757, 671, 671, 1776534322, '19541101 20:50:32.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9758, 806, 806, 57268566, '19711023 08:11:16.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9759, 449, 449, 570064232, '19950710 16:05:57.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9760, 825, 825, 384391912, '19760706 10:06:05.650')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9761, 444, 444, 1665448768, '19831204 01:02:21.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9762, 441, 441, 240252594, '19600302 16:41:11.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9763, 753, 753, 1684532242, '19621020 04:17:27.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9764, 112, 112, 1620657359, '19920618 21:00:00.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9765, 581, 581, 1865779428, '19610923 10:46:12.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9766, 613, 613, 704593884, '19700909 00:03:01.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9767, 557, 557, 1594635779, '20030123 01:36:39.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9768, 425, 425, 1392787761, '19960102 04:35:02.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9769, 399, 399, 1863190322, '19780410 13:59:35.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9770, 228, 228, 1410743884, '20060423 07:47:14.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9771, 141, 141, 1788229001, '19771009 09:46:49.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9772, 832, 832, 1802920928, '19830503 22:42:28.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9773, 562, 562, 1338847214, '19800606 09:48:40.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9774, 736, 736, 1464640091, '19720713 11:47:13.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9775, 486, 486, 967955760, '19610908 20:44:17.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9776, 608, 608, 68792671, '19710817 06:40:20.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9777, 408, 408, 974417676, '19570131 20:41:17.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9778, 373, 373, 1436460160, '19680104 22:16:03.610')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9779, 469, 469, 2007464609, '19980719 21:20:34.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9780, 481, 481, 1309466969, '19690215 12:29:31.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9781, 775, 775, 1847250521, '19750124 08:21:37.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9782, 761, 761, 1866575150, '20050217 14:50:57.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9783, 724, 724, 955327461, '20050604 01:08:23.840')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9784, 36, 36, 1481658425, '19980820 07:43:22.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9785, 89, 89, 1187654566, '20071020 07:43:03.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9786, 427, 427, 1453334105, '19810208 13:15:02.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9787, 511, 511, 417152447, '19710309 17:00:15.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9788, 500, 500, 2049925836, '19970205 10:59:02.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9789, 186, 186, 528893866, '19900317 16:20:54.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9790, 339, 339, 2116493692, '20020215 09:09:00.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9791, 256, 256, 1368566292, '19561124 14:04:44.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9792, 759, 759, 843409378, '19531013 14:29:35.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9793, 398, 398, 947156202, '19710715 07:03:02.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9794, 75, 75, 1502960220, '19800310 07:34:42.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9795, 318, 318, 347256929, '19791028 22:00:00.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9796, 858, 858, 785940395, '19900810 18:01:18.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9797, 599, 599, 1103177463, '19870525 15:42:41.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9798, 257, 257, 352194831, '19860923 21:11:59.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9799, 896, 896, 515544699, '19750425 09:54:53.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9800, 362, 362, 1274733843, '19791225 02:20:51.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9801, 765, 765, 700337755, '19611103 16:34:20.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9802, 733, 733, 146518535, '19920812 21:24:44.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9803, 542, 542, 1158374682, '19840125 11:25:50.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9804, 993, 993, 24866674, '19550610 03:26:38.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9805, 289, 289, 1484052365, '19640729 21:45:02.460')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9806, 734, 734, 558470070, '19740904 05:44:39.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9807, 207, 207, 2077461012, '19971006 17:41:50.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9808, 153, 153, 1583163289, '19680504 20:02:59.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9809, 29, 29, 2051887100, '19630610 14:25:28.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9810, 441, 441, 1876229803, '19680401 04:23:21.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9811, 318, 318, 174406956, '19650315 12:19:29.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9812, 215, 215, 39153802, '19941027 22:34:29.160')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9813, 622, 622, 492852383, '19810404 10:50:42.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9814, 427, 427, 864660658, '19790526 03:36:25.870')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9815, 725, 725, 453510946, '19540614 01:44:28.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9816, 710, 710, 1903704841, '19861015 10:11:24.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9817, 242, 242, 1386832651, '19620928 15:32:47.680')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9818, 804, 804, 543934318, '19720229 20:34:04.490')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9819, 582, 582, 347701959, '19920829 13:16:57.440')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9820, 763, 763, 830562218, '19880813 21:31:10.180')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9821, 741, 741, 237708087, '20070702 07:02:46.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9822, 50, 50, 1945314331, '19581218 14:37:36.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9823, 816, 816, 1964879947, '19581018 12:14:08.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9824, 975, 975, 113258098, '19840223 04:50:34.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9825, 14, 14, 1170818309, '20020530 17:42:29.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9826, 742, 742, 398393997, '19761227 19:17:13.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9827, 463, 463, 1145101171, '19641020 15:39:26.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9828, 758, 758, 889040916, '19530330 02:13:57.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9829, 11, 11, 257279021, '20000914 13:47:13.370')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9830, 123, 123, 270052420, '19790129 02:42:59.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9831, 259, 259, 516390642, '19920323 14:57:38.700')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9832, 152, 152, 1358540873, '19780509 23:29:18.200')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9833, 491, 491, 1365820648, '20000910 12:21:10.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9834, 673, 673, 967192713, '19710727 18:59:42.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9835, 600, 600, 2023931539, '19600414 19:55:10.290')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9836, 344, 344, 532052495, '19900612 10:56:52.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9837, 677, 677, 1295583080, '19740125 03:25:06.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9838, 796, 796, 455044610, '20021226 21:41:45.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9839, 332, 332, 1475686516, '19870122 09:58:20.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9840, 46, 46, 2111332038, '19860216 01:58:23.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9841, 157, 157, 397610708, '19910504 19:33:12.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9842, 529, 529, 46985060, '19551105 20:57:15.950')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9843, 760, 760, 1868568554, '19860829 20:33:34.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9844, 541, 541, 1747124930, '19741217 11:57:33.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9845, 561, 561, 1921498834, '19891203 20:49:30.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9846, 35, 35, 225205508, '19700127 15:30:15.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9847, 436, 436, 884193976, '19800710 03:38:53.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9848, 458, 458, 123456082, '20000219 03:26:36.900')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9849, 1000, 1000, 760610579, '19780928 05:50:13.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9850, 709, 709, 1459847875, '20010112 11:48:36.430')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9851, 646, 646, 238387620, '19801113 02:28:30.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9852, 750, 750, 1894215999, '19680326 19:08:36.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9853, 149, 149, 2133353943, '20020124 07:55:01.980')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9854, 780, 780, 2018818651, '19940910 12:23:43.010')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9855, 461, 461, 1348789329, '19800623 19:18:05.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9856, 258, 258, 228315999, '19551118 01:56:44.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9857, 737, 737, 1182130206, '19861027 09:10:36.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9858, 422, 422, 2066854198, '20071202 06:35:16.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9859, 435, 435, 2119510608, '19610109 09:44:08.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9860, 312, 312, 1366534725, '19950802 02:27:48.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9861, 854, 854, 1244629840, '19621115 14:05:13.710')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9862, 138, 138, 876495740, '19970710 15:27:52.220')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9863, 614, 614, 1217111504, '19750820 06:15:46.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9864, 777, 777, 1942843827, '19920511 11:42:29.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9865, 853, 853, 581692858, '19840108 13:25:42.620')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9866, 985, 985, 2135756976, '19941105 12:50:11.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9867, 55, 55, 29702905, '20020215 10:13:18.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9868, 768, 768, 2024857394, '19620909 15:50:59.850')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9869, 822, 822, 1912910394, '19720211 07:41:15.570')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9870, 69, 69, 966940866, '19720101 14:47:35.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9871, 629, 629, 208290070, '19650921 06:46:17.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9872, 689, 689, 1291782250, '19671002 17:51:02.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9873, 263, 263, 489247010, '19930206 10:35:44.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9874, 197, 197, 1619156531, '19590715 11:18:33.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9875, 863, 863, 1875791036, '20050411 01:57:57.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9876, 249, 249, 134776949, '20040827 10:05:30.050')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9877, 482, 482, 508840025, '19800419 18:04:02.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9878, 316, 316, 2023462631, '19911101 00:16:34.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9879, 88, 88, 281765452, '20020321 08:01:03.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9880, 813, 813, 1109284456, '19850504 02:12:14.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9881, 385, 385, 178662203, '20040618 15:38:20.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9882, 945, 945, 1358201148, '19720831 12:12:49.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9883, 934, 934, 104976994, '19820701 20:23:43.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9884, 331, 331, 1563504385, '20070902 01:58:36.940')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9885, 488, 488, 860154116, '20060317 00:14:28.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9886, 678, 678, 867718311, '19761228 19:49:01.860')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9887, 47, 47, 1159068662, '19840414 15:34:16.230')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9888, 201, 201, 236417860, '19590101 23:57:27.420')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9889, 241, 241, 1542904399, '19990201 23:41:36.340')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9890, 746, 746, 1064225733, '19570529 17:58:26.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9891, 217, 217, 1489955178, '19560817 01:46:16.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9892, 364, 364, 1919597753, '19740222 20:49:50.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9893, 751, 751, 157517672, '19941218 11:57:37.080')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9894, 340, 340, 1020651804, '19990622 07:30:31.830')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9895, 812, 812, 1114744230, '19760403 11:53:09.730')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9896, 865, 865, 2116157761, '20011024 04:05:20.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9897, 772, 772, 261440006, '19880317 14:41:29.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9898, 322, 322, 1356790958, '20020419 08:51:05.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9899, 570, 570, 1759899139, '19981208 22:31:50.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9900, 702, 702, 2143179495, '20030128 07:59:18.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9901, 409, 409, 2046197745, '19751212 05:16:56.350')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9902, 105, 105, 115001313, '19701101 11:47:53.530')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9903, 596, 596, 300929853, '19810108 19:45:21.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9904, 77, 77, 1038065214, '19590928 15:02:38.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9905, 38, 38, 2140973668, '19880424 05:02:19.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9906, 783, 783, 321011373, '19660212 08:37:28.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9907, 303, 303, 1290689005, '19830217 08:32:52.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9908, 90, 90, 1829254350, '19950713 20:36:27.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9909, 171, 171, 474942418, '19970531 10:25:45.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9910, 833, 833, 348882021, '19831027 09:12:35.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9911, 601, 601, 1248085811, '19830730 07:52:42.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9912, 906, 906, 923337237, '20021226 08:54:01.820')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9913, 654, 654, 809954047, '19580913 22:34:12.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9914, 753, 753, 614377481, '19830907 07:31:53.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9915, 744, 744, 94643163, '19990212 10:49:28.070')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9916, 924, 924, 412144114, '19980316 01:52:24.600')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9917, 871, 871, 1384573206, '19680110 19:04:08.280')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9918, 501, 501, 143821075, '19751218 04:17:09.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9919, 788, 788, 188224900, '19940224 11:28:00.670')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9920, 745, 745, 1961626984, '19600111 17:36:40.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9921, 273, 273, 1979076919, '19630724 21:15:55.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9922, 390, 390, 1293476738, '19960215 10:15:51.310')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9923, 361, 361, 1123975742, '19710807 16:09:23.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9924, 711, 711, 677983706, '19670915 13:42:49.500')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9925, 922, 922, 746695626, '19680517 13:01:19.330')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9926, 143, 143, 2043222485, '19990729 09:56:26.890')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9927, 279, 279, 419679, '19801014 05:53:25.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9928, 895, 895, 1256458641, '20010817 03:05:12.270')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9929, 864, 864, 870138695, '19910411 23:25:24.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9930, 540, 540, 1379541369, '19560618 21:52:37.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9931, 128, 128, 1355418020, '19690612 19:24:00.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9932, 699, 699, 602602832, '19851231 09:12:56.410')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9933, 411, 411, 1087851722, '20001122 01:44:43.660')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9934, 529, 529, 1248359580, '20070221 00:01:45.030')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9935, 154, 154, 1498212511, '19620524 20:55:18.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9936, 419, 419, 1386530734, '19860819 03:50:26.970')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9937, 193, 193, 116639318, '19990822 16:27:27.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9938, 596, 596, 877581970, '19751005 05:21:05.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9939, 439, 439, 1016939111, '19720510 20:56:17.910')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9940, 771, 771, 639610579, '19930202 15:36:59.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9941, 748, 748, 1345043743, '20011110 11:16:09.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9942, 697, 697, 1896947389, '19961001 18:57:48.250')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9943, 451, 451, 1520788706, '19690802 13:31:42.210')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9944, 649, 649, 1892625781, '19680408 14:29:00.450')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9945, 684, 684, 530261532, '19811030 10:05:43.920')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9946, 642, 642, 141397488, '19610821 16:52:14.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9947, 792, 792, 29530938, '19680611 22:15:38.320')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9948, 272, 272, 106492585, '19640413 04:25:43.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9949, 884, 884, 794544799, '19530510 20:41:03.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9950, 534, 534, 1498668547, '19860118 10:00:45.140')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9951, 8, 8, 691392782, '19861015 09:01:11.880')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9952, 836, 836, 966560306, '19650401 10:24:20.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9953, 133, 133, 154303786, '19610223 21:23:05.000')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9954, 488, 488, 1105292615, '19911129 04:55:10.120')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9955, 321, 321, 228217130, '19920707 10:43:23.390')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9956, 614, 614, 434626240, '19871026 19:01:05.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9957, 314, 314, 1113985928, '20070326 19:38:30.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9958, 495, 495, 165549987, '19660427 06:02:31.810')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9959, 423, 423, 864038895, '19990513 02:01:18.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9960, 405, 405, 796342581, '19960925 19:05:52.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9961, 583, 583, 884617784, '19930430 02:44:03.360')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9962, 967, 967, 1094753429, '19890702 05:27:40.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9963, 707, 707, 1908446199, '19570403 21:11:03.720')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9964, 799, 799, 721680087, '19931213 12:33:07.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9965, 807, 807, 417573047, '19670517 13:48:35.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9966, 664, 664, 1292294595, '20050728 22:39:45.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9967, 885, 885, 713558759, '19550204 07:09:18.150')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9968, 774, 774, 1471955585, '19590723 22:32:27.170')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9969, 561, 561, 1968014267, '19740416 10:36:35.040')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9970, 20, 20, 2048659146, '19650627 06:59:01.090')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9971, 743, 743, 1788877268, '20060724 09:24:56.800')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9972, 829, 829, 1644847615, '20000406 13:43:02.480')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9973, 861, 861, 405113129, '19560809 07:20:51.190')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9974, 666, 666, 702844352, '19540122 19:51:00.780')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9975, 354, 354, 126058607, '19660302 06:20:31.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9976, 961, 961, 337276326, '19741022 02:18:07.260')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9977, 929, 929, 1365373195, '19790716 20:44:09.100')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9978, 831, 831, 73156293, '19560501 01:40:23.060')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9979, 875, 875, 1957738262, '19931115 03:37:05.580')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9980, 461, 461, 2020260406, '19590927 20:09:04.740')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9981, 770, 770, 778853696, '19840217 11:40:48.560')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9982, 718, 718, 2014960825, '19690704 01:27:41.590')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9983, 700, 700, 533585749, '20010409 06:24:08.510')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9984, 177, 177, 941689722, '19580325 13:24:39.110')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9985, 32, 32, 1274506629, '19770903 12:51:25.750')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9986, 959, 959, 1607175264, '19570314 08:59:39.300')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9987, 168, 168, 250650043, '19771107 11:49:51.400')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9988, 39, 39, 1773513690, '19611225 20:49:33.540')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9989, 784, 784, 1408106952, '19670817 13:18:21.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9990, 109, 109, 478160459, '19820729 01:54:13.470')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9991, 849, 849, 469250134, '19870525 08:11:56.960')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9992, 951, 951, 1374769708, '19860428 10:24:55.990')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9993, 617, 617, 412732244, '19840524 03:19:47.020')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9994, 437, 437, 803859274, '19830815 01:50:25.760')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9995, 84, 84, 682422031, '20071005 12:52:56.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9996, 27, 27, 443960738, '19650512 05:48:28.640')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9997, 790, 790, 1316689496, '19920630 21:46:44.520')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9998, 825, 825, 1652692538, '19830820 00:58:34.630')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (9999, 868, 868, 645980997, '19531124 00:40:24.690')
INSERT INTO [dbo].[Highscores] ([ID], [SpielerID], [VideospielID], [Punkte], [Zeitpunkt]) VALUES (10000, 993, 993, 755384627, '19840402 11:25:58.180')
SET IDENTITY_INSERT [dbo].[Highscores] OFF
ALTER TABLE [dbo].[Highscores]
    WITH NOCHECK ADD CONSTRAINT [FK_Highscores_Spieler] FOREIGN KEY ([SpielerID]) REFERENCES [dbo].[Spieler] ([ID]) ON DELETE CASCADE ON UPDATE CASCADE
ALTER TABLE [dbo].[Highscores]
    WITH NOCHECK ADD CONSTRAINT [FK_Highscores_Games] FOREIGN KEY ([VideospielID]) REFERENCES [dbo].[Videospiele] ([ID]) ON DELETE CASCADE ON UPDATE CASCADE
ALTER TABLE [dbo].[Spieler]
    WITH NOCHECK ADD CONSTRAINT [FK_Spieler_Laender] FOREIGN KEY ([HerkunftslandID]) REFERENCES [dbo].[Laender] ([ID]) ON DELETE CASCADE ON UPDATE CASCADE
COMMIT TRANSACTION

PRINT 'Daten eingefügt.'
GO
